const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["assets/codeMirrorModule-BoWUGj0J.js","assets/codeMirrorModule-C3UTv-Ge.css"])))=>i.map(i=>d[i]);
(function(){const l=document.createElement("link").relList;if(l&&l.supports&&l.supports("modulepreload"))return;for(const r of document.querySelectorAll('link[rel="modulepreload"]'))s(r);new MutationObserver(r=>{for(const o of r)if(o.type==="childList")for(const h of o.addedNodes)h.tagName==="LINK"&&h.rel==="modulepreload"&&s(h)}).observe(document,{childList:!0,subtree:!0});function a(r){const o={};return r.integrity&&(o.integrity=r.integrity),r.referrerPolicy&&(o.referrerPolicy=r.referrerPolicy),r.crossOrigin==="use-credentials"?o.credentials="include":r.crossOrigin==="anonymous"?o.credentials="omit":o.credentials="same-origin",o}function s(r){if(r.ep)return;r.ep=!0;const o=a(r);fetch(r.href,o)}})();function W0(u){return u&&u.__esModule&&Object.prototype.hasOwnProperty.call(u,"default")?u.default:u}var Zr={exports:{}},wa={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Cg;function I0(){if(Cg)return wa;Cg=1;var u=Symbol.for("react.transitional.element"),l=Symbol.for("react.fragment");function a(s,r,o){var h=null;if(o!==void 0&&(h=""+o),r.key!==void 0&&(h=""+r.key),"key"in r){o={};for(var d in r)d!=="key"&&(o[d]=r[d])}else o=r;return r=o.ref,{$$typeof:u,type:s,key:h,ref:r!==void 0?r:null,props:o}}return wa.Fragment=l,wa.jsx=a,wa.jsxs=a,wa}var Rg;function F0(){return Rg||(Rg=1,Zr.exports=I0()),Zr.exports}var X=F0(),Jr={exports:{}},fe={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var kg;function P0(){if(kg)return fe;kg=1;var u=Symbol.for("react.transitional.element"),l=Symbol.for("react.portal"),a=Symbol.for("react.fragment"),s=Symbol.for("react.strict_mode"),r=Symbol.for("react.profiler"),o=Symbol.for("react.consumer"),h=Symbol.for("react.context"),d=Symbol.for("react.forward_ref"),m=Symbol.for("react.suspense"),p=Symbol.for("react.memo"),b=Symbol.for("react.lazy"),S=Symbol.iterator;function N(A){return A===null||typeof A!="object"?null:(A=S&&A[S]||A["@@iterator"],typeof A=="function"?A:null)}var E={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},D=Object.assign,T={};function w(A,Y,W){this.props=A,this.context=Y,this.refs=T,this.updater=W||E}w.prototype.isReactComponent={},w.prototype.setState=function(A,Y){if(typeof A!="object"&&typeof A!="function"&&A!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,A,Y,"setState")},w.prototype.forceUpdate=function(A){this.updater.enqueueForceUpdate(this,A,"forceUpdate")};function k(){}k.prototype=w.prototype;function L(A,Y,W){this.props=A,this.context=Y,this.refs=T,this.updater=W||E}var H=L.prototype=new k;H.constructor=L,D(H,w.prototype),H.isPureReactComponent=!0;var Q=Array.isArray,G={H:null,A:null,T:null,S:null,V:null},K=Object.prototype.hasOwnProperty;function ee(A,Y,W,J,ne,ye){return W=ye.ref,{$$typeof:u,type:A,key:Y,ref:W!==void 0?W:null,props:ye}}function V(A,Y){return ee(A.type,Y,void 0,void 0,void 0,A.props)}function z(A){return typeof A=="object"&&A!==null&&A.$$typeof===u}function F(A){var Y={"=":"=0",":":"=2"};return"$"+A.replace(/[=:]/g,function(W){return Y[W]})}var se=/\/+/g;function U(A,Y){return typeof A=="object"&&A!==null&&A.key!=null?F(""+A.key):Y.toString(36)}function te(){}function xe(A){switch(A.status){case"fulfilled":return A.value;case"rejected":throw A.reason;default:switch(typeof A.status=="string"?A.then(te,te):(A.status="pending",A.then(function(Y){A.status==="pending"&&(A.status="fulfilled",A.value=Y)},function(Y){A.status==="pending"&&(A.status="rejected",A.reason=Y)})),A.status){case"fulfilled":return A.value;case"rejected":throw A.reason}}throw A}function qe(A,Y,W,J,ne){var ye=typeof A;(ye==="undefined"||ye==="boolean")&&(A=null);var ce=!1;if(A===null)ce=!0;else switch(ye){case"bigint":case"string":case"number":ce=!0;break;case"object":switch(A.$$typeof){case u:case l:ce=!0;break;case b:return ce=A._init,qe(ce(A._payload),Y,W,J,ne)}}if(ce)return ne=ne(A),ce=J===""?"."+U(A,0):J,Q(ne)?(W="",ce!=null&&(W=ce.replace(se,"$&/")+"/"),qe(ne,Y,W,"",function(vn){return vn})):ne!=null&&(z(ne)&&(ne=V(ne,W+(ne.key==null||A&&A.key===ne.key?"":(""+ne.key).replace(se,"$&/")+"/")+ce)),Y.push(ne)),1;ce=0;var yt=J===""?".":J+":";if(Q(A))for(var De=0;De<A.length;De++)J=A[De],ye=yt+U(J,De),ce+=qe(J,Y,W,ye,ne);else if(De=N(A),typeof De=="function")for(A=De.call(A),De=0;!(J=A.next()).done;)J=J.value,ye=yt+U(J,De++),ce+=qe(J,Y,W,ye,ne);else if(ye==="object"){if(typeof A.then=="function")return qe(xe(A),Y,W,J,ne);throw Y=String(A),Error("Objects are not valid as a React child (found: "+(Y==="[object Object]"?"object with keys {"+Object.keys(A).join(", ")+"}":Y)+"). If you meant to render a collection of children, use an array instead.")}return ce}function B(A,Y,W){if(A==null)return A;var J=[],ne=0;return qe(A,J,"","",function(ye){return Y.call(W,ye,ne++)}),J}function Z(A){if(A._status===-1){var Y=A._result;Y=Y(),Y.then(function(W){(A._status===0||A._status===-1)&&(A._status=1,A._result=W)},function(W){(A._status===0||A._status===-1)&&(A._status=2,A._result=W)}),A._status===-1&&(A._status=0,A._result=Y)}if(A._status===1)return A._result.default;throw A._result}var ae=typeof reportError=="function"?reportError:function(A){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var Y=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof A=="object"&&A!==null&&typeof A.message=="string"?String(A.message):String(A),error:A});if(!window.dispatchEvent(Y))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",A);return}console.error(A)};function Oe(){}return fe.Children={map:B,forEach:function(A,Y,W){B(A,function(){Y.apply(this,arguments)},W)},count:function(A){var Y=0;return B(A,function(){Y++}),Y},toArray:function(A){return B(A,function(Y){return Y})||[]},only:function(A){if(!z(A))throw Error("React.Children.only expected to receive a single React element child.");return A}},fe.Component=w,fe.Fragment=a,fe.Profiler=r,fe.PureComponent=L,fe.StrictMode=s,fe.Suspense=m,fe.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=G,fe.__COMPILER_RUNTIME={__proto__:null,c:function(A){return G.H.useMemoCache(A)}},fe.cache=function(A){return function(){return A.apply(null,arguments)}},fe.cloneElement=function(A,Y,W){if(A==null)throw Error("The argument must be a React element, but you passed "+A+".");var J=D({},A.props),ne=A.key,ye=void 0;if(Y!=null)for(ce in Y.ref!==void 0&&(ye=void 0),Y.key!==void 0&&(ne=""+Y.key),Y)!K.call(Y,ce)||ce==="key"||ce==="__self"||ce==="__source"||ce==="ref"&&Y.ref===void 0||(J[ce]=Y[ce]);var ce=arguments.length-2;if(ce===1)J.children=W;else if(1<ce){for(var yt=Array(ce),De=0;De<ce;De++)yt[De]=arguments[De+2];J.children=yt}return ee(A.type,ne,void 0,void 0,ye,J)},fe.createContext=function(A){return A={$$typeof:h,_currentValue:A,_currentValue2:A,_threadCount:0,Provider:null,Consumer:null},A.Provider=A,A.Consumer={$$typeof:o,_context:A},A},fe.createElement=function(A,Y,W){var J,ne={},ye=null;if(Y!=null)for(J in Y.key!==void 0&&(ye=""+Y.key),Y)K.call(Y,J)&&J!=="key"&&J!=="__self"&&J!=="__source"&&(ne[J]=Y[J]);var ce=arguments.length-2;if(ce===1)ne.children=W;else if(1<ce){for(var yt=Array(ce),De=0;De<ce;De++)yt[De]=arguments[De+2];ne.children=yt}if(A&&A.defaultProps)for(J in ce=A.defaultProps,ce)ne[J]===void 0&&(ne[J]=ce[J]);return ee(A,ye,void 0,void 0,null,ne)},fe.createRef=function(){return{current:null}},fe.forwardRef=function(A){return{$$typeof:d,render:A}},fe.isValidElement=z,fe.lazy=function(A){return{$$typeof:b,_payload:{_status:-1,_result:A},_init:Z}},fe.memo=function(A,Y){return{$$typeof:p,type:A,compare:Y===void 0?null:Y}},fe.startTransition=function(A){var Y=G.T,W={};G.T=W;try{var J=A(),ne=G.S;ne!==null&&ne(W,J),typeof J=="object"&&J!==null&&typeof J.then=="function"&&J.then(Oe,ae)}catch(ye){ae(ye)}finally{G.T=Y}},fe.unstable_useCacheRefresh=function(){return G.H.useCacheRefresh()},fe.use=function(A){return G.H.use(A)},fe.useActionState=function(A,Y,W){return G.H.useActionState(A,Y,W)},fe.useCallback=function(A,Y){return G.H.useCallback(A,Y)},fe.useContext=function(A){return G.H.useContext(A)},fe.useDebugValue=function(){},fe.useDeferredValue=function(A,Y){return G.H.useDeferredValue(A,Y)},fe.useEffect=function(A,Y,W){var J=G.H;if(typeof W=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return J.useEffect(A,Y)},fe.useId=function(){return G.H.useId()},fe.useImperativeHandle=function(A,Y,W){return G.H.useImperativeHandle(A,Y,W)},fe.useInsertionEffect=function(A,Y){return G.H.useInsertionEffect(A,Y)},fe.useLayoutEffect=function(A,Y){return G.H.useLayoutEffect(A,Y)},fe.useMemo=function(A,Y){return G.H.useMemo(A,Y)},fe.useOptimistic=function(A,Y){return G.H.useOptimistic(A,Y)},fe.useReducer=function(A,Y,W){return G.H.useReducer(A,Y,W)},fe.useRef=function(A){return G.H.useRef(A)},fe.useState=function(A){return G.H.useState(A)},fe.useSyncExternalStore=function(A,Y,W){return G.H.useSyncExternalStore(A,Y,W)},fe.useTransition=function(){return G.H.useTransition()},fe.version="19.1.1",fe}var Lg;function Tf(){return Lg||(Lg=1,Jr.exports=P0()),Jr.exports}var me=Tf();const Gn=W0(me);function rm(){const u=Gn.useRef(null),[l]=of(u);return[l,u]}function of(u){const[l,a]=Gn.useState(new DOMRect(0,0,10,10)),s=Gn.useCallback(()=>{const r=u==null?void 0:u.current;r&&a(r.getBoundingClientRect())},[u]);return Gn.useLayoutEffect(()=>{const r=u==null?void 0:u.current;if(!r)return;s();const o=new ResizeObserver(s);return o.observe(r),window.addEventListener("resize",s),()=>{o.disconnect(),window.removeEventListener("resize",s)}},[s,u]),[l,s]}function e1(u){if(u<0||!isFinite(u))return"-";if(u===0)return"0";if(u<1e3)return u.toFixed(0)+"ms";const l=u/1e3;if(l<60)return l.toFixed(1)+"s";const a=l/60;if(a<60)return a.toFixed(1)+"m";const s=a/60;return s<24?s.toFixed(1)+"h":(s/24).toFixed(1)+"d"}function zg(u){const l=document.createElement("textarea");l.style.position="absolute",l.style.zIndex="-1000",l.value=u,document.body.appendChild(l),l.select(),document.execCommand("copy"),l.remove()}function fu(u,l){u&&(l=ol.getObject(u,l));const[a,s]=Gn.useState(l),r=Gn.useCallback(o=>{u?ol.setObject(u,o):s(o)},[u,s]);return Gn.useEffect(()=>{if(u){const o=()=>s(ol.getObject(u,l));return ol.onChangeEmitter.addEventListener(u,o),()=>ol.onChangeEmitter.removeEventListener(u,o)}},[l,u]),[a,r]}class t1{constructor(){this.onChangeEmitter=new EventTarget}getString(l,a){return localStorage[l]||a}setString(l,a){var s;localStorage[l]=a,this.onChangeEmitter.dispatchEvent(new Event(l)),(s=window.saveSettings)==null||s.call(window)}getObject(l,a){if(!localStorage[l])return a;try{return JSON.parse(localStorage[l])}catch{return a}}setObject(l,a){var s;localStorage[l]=JSON.stringify(a),this.onChangeEmitter.dispatchEvent(new Event(l)),(s=window.saveSettings)==null||s.call(window)}}const ol=new t1;function pl(...u){return u.filter(Boolean).join(" ")}const Ug="\\u0000-\\u0020\\u007f-\\u009f",n1=new RegExp("(?:[a-zA-Z][a-zA-Z0-9+.-]{2,}:\\/\\/|www\\.)[^\\s"+Ug+'"]{2,}[^\\s'+Ug+`"')}\\],:;.!?]`,"ug");function l1(){if(document.playwrightThemeInitialized)return;document.playwrightThemeInitialized=!0,document.defaultView.addEventListener("focus",s=>{s.target.document.nodeType===Node.DOCUMENT_NODE&&document.body.classList.remove("inactive")},!1),document.defaultView.addEventListener("blur",s=>{document.body.classList.add("inactive")},!1);const l=window.matchMedia("(prefers-color-scheme: dark)").matches?"dark-mode":"light-mode";ol.getString("theme",l)==="dark-mode"?document.documentElement.classList.add("dark-mode"):document.documentElement.classList.add("light-mode")}const i1=new Set;function a1(){const u=hf(),l=u==="dark-mode"?"light-mode":"dark-mode";document.documentElement.classList.remove(u),document.documentElement.classList.add(l),ol.setString("theme",l);for(const a of i1)a(l)}function hf(){return document.documentElement.classList.contains("dark-mode")?"dark-mode":"light-mode"}function s1(){const[u,l]=Gn.useState(hf()==="dark-mode");return[u,a=>{hf()==="dark-mode"!==a&&a1(),l(a)}]}var Wr={exports:{}},Ea={},Ir={exports:{}},Fr={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Bg;function u1(){return Bg||(Bg=1,(function(u){function l(B,Z){var ae=B.length;B.push(Z);e:for(;0<ae;){var Oe=ae-1>>>1,A=B[Oe];if(0<r(A,Z))B[Oe]=Z,B[ae]=A,ae=Oe;else break e}}function a(B){return B.length===0?null:B[0]}function s(B){if(B.length===0)return null;var Z=B[0],ae=B.pop();if(ae!==Z){B[0]=ae;e:for(var Oe=0,A=B.length,Y=A>>>1;Oe<Y;){var W=2*(Oe+1)-1,J=B[W],ne=W+1,ye=B[ne];if(0>r(J,ae))ne<A&&0>r(ye,J)?(B[Oe]=ye,B[ne]=ae,Oe=ne):(B[Oe]=J,B[W]=ae,Oe=W);else if(ne<A&&0>r(ye,ae))B[Oe]=ye,B[ne]=ae,Oe=ne;else break e}}return Z}function r(B,Z){var ae=B.sortIndex-Z.sortIndex;return ae!==0?ae:B.id-Z.id}if(u.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var o=performance;u.unstable_now=function(){return o.now()}}else{var h=Date,d=h.now();u.unstable_now=function(){return h.now()-d}}var m=[],p=[],b=1,S=null,N=3,E=!1,D=!1,T=!1,w=!1,k=typeof setTimeout=="function"?setTimeout:null,L=typeof clearTimeout=="function"?clearTimeout:null,H=typeof setImmediate<"u"?setImmediate:null;function Q(B){for(var Z=a(p);Z!==null;){if(Z.callback===null)s(p);else if(Z.startTime<=B)s(p),Z.sortIndex=Z.expirationTime,l(m,Z);else break;Z=a(p)}}function G(B){if(T=!1,Q(B),!D)if(a(m)!==null)D=!0,K||(K=!0,U());else{var Z=a(p);Z!==null&&qe(G,Z.startTime-B)}}var K=!1,ee=-1,V=5,z=-1;function F(){return w?!0:!(u.unstable_now()-z<V)}function se(){if(w=!1,K){var B=u.unstable_now();z=B;var Z=!0;try{e:{D=!1,T&&(T=!1,L(ee),ee=-1),E=!0;var ae=N;try{t:{for(Q(B),S=a(m);S!==null&&!(S.expirationTime>B&&F());){var Oe=S.callback;if(typeof Oe=="function"){S.callback=null,N=S.priorityLevel;var A=Oe(S.expirationTime<=B);if(B=u.unstable_now(),typeof A=="function"){S.callback=A,Q(B),Z=!0;break t}S===a(m)&&s(m),Q(B)}else s(m);S=a(m)}if(S!==null)Z=!0;else{var Y=a(p);Y!==null&&qe(G,Y.startTime-B),Z=!1}}break e}finally{S=null,N=ae,E=!1}Z=void 0}}finally{Z?U():K=!1}}}var U;if(typeof H=="function")U=function(){H(se)};else if(typeof MessageChannel<"u"){var te=new MessageChannel,xe=te.port2;te.port1.onmessage=se,U=function(){xe.postMessage(null)}}else U=function(){k(se,0)};function qe(B,Z){ee=k(function(){B(u.unstable_now())},Z)}u.unstable_IdlePriority=5,u.unstable_ImmediatePriority=1,u.unstable_LowPriority=4,u.unstable_NormalPriority=3,u.unstable_Profiling=null,u.unstable_UserBlockingPriority=2,u.unstable_cancelCallback=function(B){B.callback=null},u.unstable_forceFrameRate=function(B){0>B||125<B?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):V=0<B?Math.floor(1e3/B):5},u.unstable_getCurrentPriorityLevel=function(){return N},u.unstable_next=function(B){switch(N){case 1:case 2:case 3:var Z=3;break;default:Z=N}var ae=N;N=Z;try{return B()}finally{N=ae}},u.unstable_requestPaint=function(){w=!0},u.unstable_runWithPriority=function(B,Z){switch(B){case 1:case 2:case 3:case 4:case 5:break;default:B=3}var ae=N;N=B;try{return Z()}finally{N=ae}},u.unstable_scheduleCallback=function(B,Z,ae){var Oe=u.unstable_now();switch(typeof ae=="object"&&ae!==null?(ae=ae.delay,ae=typeof ae=="number"&&0<ae?Oe+ae:Oe):ae=Oe,B){case 1:var A=-1;break;case 2:A=250;break;case 5:A=1073741823;break;case 4:A=1e4;break;default:A=5e3}return A=ae+A,B={id:b++,callback:Z,priorityLevel:B,startTime:ae,expirationTime:A,sortIndex:-1},ae>Oe?(B.sortIndex=ae,l(p,B),a(m)===null&&B===a(p)&&(T?(L(ee),ee=-1):T=!0,qe(G,ae-Oe))):(B.sortIndex=A,l(m,B),D||E||(D=!0,K||(K=!0,U()))),B},u.unstable_shouldYield=F,u.unstable_wrapCallback=function(B){var Z=N;return function(){var ae=N;N=Z;try{return B.apply(this,arguments)}finally{N=ae}}}})(Fr)),Fr}var qg;function c1(){return qg||(qg=1,Ir.exports=u1()),Ir.exports}var Pr={exports:{}},it={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var jg;function r1(){if(jg)return it;jg=1;var u=Tf();function l(m){var p="https://react.dev/errors/"+m;if(1<arguments.length){p+="?args[]="+encodeURIComponent(arguments[1]);for(var b=2;b<arguments.length;b++)p+="&args[]="+encodeURIComponent(arguments[b])}return"Minified React error #"+m+"; visit "+p+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function a(){}var s={d:{f:a,r:function(){throw Error(l(522))},D:a,C:a,L:a,m:a,X:a,S:a,M:a},p:0,findDOMNode:null},r=Symbol.for("react.portal");function o(m,p,b){var S=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:r,key:S==null?null:""+S,children:m,containerInfo:p,implementation:b}}var h=u.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function d(m,p){if(m==="font")return"";if(typeof p=="string")return p==="use-credentials"?p:""}return it.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=s,it.createPortal=function(m,p){var b=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!p||p.nodeType!==1&&p.nodeType!==9&&p.nodeType!==11)throw Error(l(299));return o(m,p,null,b)},it.flushSync=function(m){var p=h.T,b=s.p;try{if(h.T=null,s.p=2,m)return m()}finally{h.T=p,s.p=b,s.d.f()}},it.preconnect=function(m,p){typeof m=="string"&&(p?(p=p.crossOrigin,p=typeof p=="string"?p==="use-credentials"?p:"":void 0):p=null,s.d.C(m,p))},it.prefetchDNS=function(m){typeof m=="string"&&s.d.D(m)},it.preinit=function(m,p){if(typeof m=="string"&&p&&typeof p.as=="string"){var b=p.as,S=d(b,p.crossOrigin),N=typeof p.integrity=="string"?p.integrity:void 0,E=typeof p.fetchPriority=="string"?p.fetchPriority:void 0;b==="style"?s.d.S(m,typeof p.precedence=="string"?p.precedence:void 0,{crossOrigin:S,integrity:N,fetchPriority:E}):b==="script"&&s.d.X(m,{crossOrigin:S,integrity:N,fetchPriority:E,nonce:typeof p.nonce=="string"?p.nonce:void 0})}},it.preinitModule=function(m,p){if(typeof m=="string")if(typeof p=="object"&&p!==null){if(p.as==null||p.as==="script"){var b=d(p.as,p.crossOrigin);s.d.M(m,{crossOrigin:b,integrity:typeof p.integrity=="string"?p.integrity:void 0,nonce:typeof p.nonce=="string"?p.nonce:void 0})}}else p==null&&s.d.M(m)},it.preload=function(m,p){if(typeof m=="string"&&typeof p=="object"&&p!==null&&typeof p.as=="string"){var b=p.as,S=d(b,p.crossOrigin);s.d.L(m,b,{crossOrigin:S,integrity:typeof p.integrity=="string"?p.integrity:void 0,nonce:typeof p.nonce=="string"?p.nonce:void 0,type:typeof p.type=="string"?p.type:void 0,fetchPriority:typeof p.fetchPriority=="string"?p.fetchPriority:void 0,referrerPolicy:typeof p.referrerPolicy=="string"?p.referrerPolicy:void 0,imageSrcSet:typeof p.imageSrcSet=="string"?p.imageSrcSet:void 0,imageSizes:typeof p.imageSizes=="string"?p.imageSizes:void 0,media:typeof p.media=="string"?p.media:void 0})}},it.preloadModule=function(m,p){if(typeof m=="string")if(p){var b=d(p.as,p.crossOrigin);s.d.m(m,{as:typeof p.as=="string"&&p.as!=="script"?p.as:void 0,crossOrigin:b,integrity:typeof p.integrity=="string"?p.integrity:void 0})}else s.d.m(m)},it.requestFormReset=function(m){s.d.r(m)},it.unstable_batchedUpdates=function(m,p){return m(p)},it.useFormState=function(m,p,b){return h.H.useFormState(m,p,b)},it.useFormStatus=function(){return h.H.useHostTransitionStatus()},it.version="19.1.1",it}var Hg;function f1(){if(Hg)return Pr.exports;Hg=1;function u(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(u)}catch(l){console.error(l)}}return u(),Pr.exports=r1(),Pr.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var $g;function o1(){if($g)return Ea;$g=1;var u=c1(),l=Tf(),a=f1();function s(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function r(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function o(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function h(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function d(e){if(o(e)!==e)throw Error(s(188))}function m(e){var t=e.alternate;if(!t){if(t=o(e),t===null)throw Error(s(188));return t!==e?null:e}for(var n=e,i=t;;){var c=n.return;if(c===null)break;var f=c.alternate;if(f===null){if(i=c.return,i!==null){n=i;continue}break}if(c.child===f.child){for(f=c.child;f;){if(f===n)return d(c),e;if(f===i)return d(c),t;f=f.sibling}throw Error(s(188))}if(n.return!==i.return)n=c,i=f;else{for(var g=!1,y=c.child;y;){if(y===n){g=!0,n=c,i=f;break}if(y===i){g=!0,i=c,n=f;break}y=y.sibling}if(!g){for(y=f.child;y;){if(y===n){g=!0,n=f,i=c;break}if(y===i){g=!0,i=f,n=c;break}y=y.sibling}if(!g)throw Error(s(189))}}if(n.alternate!==i)throw Error(s(190))}if(n.tag!==3)throw Error(s(188));return n.stateNode.current===n?e:t}function p(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=p(e),t!==null)return t;e=e.sibling}return null}var b=Object.assign,S=Symbol.for("react.element"),N=Symbol.for("react.transitional.element"),E=Symbol.for("react.portal"),D=Symbol.for("react.fragment"),T=Symbol.for("react.strict_mode"),w=Symbol.for("react.profiler"),k=Symbol.for("react.provider"),L=Symbol.for("react.consumer"),H=Symbol.for("react.context"),Q=Symbol.for("react.forward_ref"),G=Symbol.for("react.suspense"),K=Symbol.for("react.suspense_list"),ee=Symbol.for("react.memo"),V=Symbol.for("react.lazy"),z=Symbol.for("react.activity"),F=Symbol.for("react.memo_cache_sentinel"),se=Symbol.iterator;function U(e){return e===null||typeof e!="object"?null:(e=se&&e[se]||e["@@iterator"],typeof e=="function"?e:null)}var te=Symbol.for("react.client.reference");function xe(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===te?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case D:return"Fragment";case w:return"Profiler";case T:return"StrictMode";case G:return"Suspense";case K:return"SuspenseList";case z:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case E:return"Portal";case H:return(e.displayName||"Context")+".Provider";case L:return(e._context.displayName||"Context")+".Consumer";case Q:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case ee:return t=e.displayName||null,t!==null?t:xe(e.type)||"Memo";case V:t=e._payload,e=e._init;try{return xe(e(t))}catch{}}return null}var qe=Array.isArray,B=l.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,Z=a.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,ae={pending:!1,data:null,method:null,action:null},Oe=[],A=-1;function Y(e){return{current:e}}function W(e){0>A||(e.current=Oe[A],Oe[A]=null,A--)}function J(e,t){A++,Oe[A]=e.current,e.current=t}var ne=Y(null),ye=Y(null),ce=Y(null),yt=Y(null);function De(e,t){switch(J(ce,t),J(ye,e),J(ne,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?ag(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=ag(t),e=sg(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}W(ne),J(ne,e)}function vn(){W(ne),W(ye),W(ce)}function ku(e){e.memoizedState!==null&&J(yt,e);var t=ne.current,n=sg(t,e.type);t!==n&&(J(ye,e),J(ne,n))}function za(e){ye.current===e&&(W(ne),W(ye)),yt.current===e&&(W(yt),ya._currentValue=ae)}var Lu=Object.prototype.hasOwnProperty,zu=u.unstable_scheduleCallback,Uu=u.unstable_cancelCallback,Mp=u.unstable_shouldYield,xp=u.unstable_requestPaint,Vt=u.unstable_now,Dp=u.unstable_getCurrentPriorityLevel,qf=u.unstable_ImmediatePriority,jf=u.unstable_UserBlockingPriority,Ua=u.unstable_NormalPriority,Cp=u.unstable_LowPriority,Hf=u.unstable_IdlePriority,Rp=u.log,kp=u.unstable_setDisableYieldValue,Oi=null,vt=null;function bn(e){if(typeof Rp=="function"&&kp(e),vt&&typeof vt.setStrictMode=="function")try{vt.setStrictMode(Oi,e)}catch{}}var bt=Math.clz32?Math.clz32:Up,Lp=Math.log,zp=Math.LN2;function Up(e){return e>>>=0,e===0?32:31-(Lp(e)/zp|0)|0}var Ba=256,qa=4194304;function Xn(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return e&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function ja(e,t,n){var i=e.pendingLanes;if(i===0)return 0;var c=0,f=e.suspendedLanes,g=e.pingedLanes;e=e.warmLanes;var y=i&134217727;return y!==0?(i=y&~f,i!==0?c=Xn(i):(g&=y,g!==0?c=Xn(g):n||(n=y&~e,n!==0&&(c=Xn(n))))):(y=i&~f,y!==0?c=Xn(y):g!==0?c=Xn(g):n||(n=i&~e,n!==0&&(c=Xn(n)))),c===0?0:t!==0&&t!==c&&(t&f)===0&&(f=c&-c,n=t&-t,f>=n||f===32&&(n&4194048)!==0)?t:c}function Ni(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function Bp(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function $f(){var e=Ba;return Ba<<=1,(Ba&4194048)===0&&(Ba=256),e}function Yf(){var e=qa;return qa<<=1,(qa&62914560)===0&&(qa=4194304),e}function Bu(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function _i(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function qp(e,t,n,i,c,f){var g=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var y=e.entanglements,v=e.expirationTimes,x=e.hiddenUpdates;for(n=g&~n;0<n;){var q=31-bt(n),$=1<<q;y[q]=0,v[q]=-1;var C=x[q];if(C!==null)for(x[q]=null,q=0;q<C.length;q++){var R=C[q];R!==null&&(R.lane&=-536870913)}n&=~$}i!==0&&Gf(e,i,0),f!==0&&c===0&&e.tag!==0&&(e.suspendedLanes|=f&~(g&~t))}function Gf(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var i=31-bt(t);e.entangledLanes|=t,e.entanglements[i]=e.entanglements[i]|1073741824|n&4194090}function Kf(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var i=31-bt(n),c=1<<i;c&t|e[i]&t&&(e[i]|=t),n&=~c}}function qu(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function ju(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function Vf(){var e=Z.p;return e!==0?e:(e=window.event,e===void 0?32:Og(e.type))}function jp(e,t){var n=Z.p;try{return Z.p=e,t()}finally{Z.p=n}}var Sn=Math.random().toString(36).slice(2),nt="__reactFiber$"+Sn,rt="__reactProps$"+Sn,Sl="__reactContainer$"+Sn,Hu="__reactEvents$"+Sn,Hp="__reactListeners$"+Sn,$p="__reactHandles$"+Sn,Qf="__reactResources$"+Sn,Mi="__reactMarker$"+Sn;function $u(e){delete e[nt],delete e[rt],delete e[Hu],delete e[Hp],delete e[$p]}function Tl(e){var t=e[nt];if(t)return t;for(var n=e.parentNode;n;){if(t=n[Sl]||n[nt]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=fg(e);e!==null;){if(n=e[nt])return n;e=fg(e)}return t}e=n,n=e.parentNode}return null}function wl(e){if(e=e[nt]||e[Sl]){var t=e.tag;if(t===5||t===6||t===13||t===26||t===27||t===3)return e}return null}function xi(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(s(33))}function El(e){var t=e[Qf];return t||(t=e[Qf]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function Ze(e){e[Mi]=!0}var Xf=new Set,Zf={};function Zn(e,t){Al(e,t),Al(e+"Capture",t)}function Al(e,t){for(Zf[e]=t,e=0;e<t.length;e++)Xf.add(t[e])}var Yp=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),Jf={},Wf={};function Gp(e){return Lu.call(Wf,e)?!0:Lu.call(Jf,e)?!1:Yp.test(e)?Wf[e]=!0:(Jf[e]=!0,!1)}function Ha(e,t,n){if(Gp(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var i=t.toLowerCase().slice(0,5);if(i!=="data-"&&i!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function $a(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function Pt(e,t,n,i){if(i===null)e.removeAttribute(n);else{switch(typeof i){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+i)}}var Yu,If;function Ol(e){if(Yu===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);Yu=t&&t[1]||"",If=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Yu+e+If}var Gu=!1;function Ku(e,t){if(!e||Gu)return"";Gu=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var i={DetermineComponentFrameRoot:function(){try{if(t){var $=function(){throw Error()};if(Object.defineProperty($.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct($,[])}catch(R){var C=R}Reflect.construct(e,[],$)}else{try{$.call()}catch(R){C=R}e.call($.prototype)}}else{try{throw Error()}catch(R){C=R}($=e())&&typeof $.catch=="function"&&$.catch(function(){})}}catch(R){if(R&&C&&typeof R.stack=="string")return[R.stack,C.stack]}return[null,null]}};i.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var c=Object.getOwnPropertyDescriptor(i.DetermineComponentFrameRoot,"name");c&&c.configurable&&Object.defineProperty(i.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var f=i.DetermineComponentFrameRoot(),g=f[0],y=f[1];if(g&&y){var v=g.split(`
`),x=y.split(`
`);for(c=i=0;i<v.length&&!v[i].includes("DetermineComponentFrameRoot");)i++;for(;c<x.length&&!x[c].includes("DetermineComponentFrameRoot");)c++;if(i===v.length||c===x.length)for(i=v.length-1,c=x.length-1;1<=i&&0<=c&&v[i]!==x[c];)c--;for(;1<=i&&0<=c;i--,c--)if(v[i]!==x[c]){if(i!==1||c!==1)do if(i--,c--,0>c||v[i]!==x[c]){var q=`
`+v[i].replace(" at new "," at ");return e.displayName&&q.includes("<anonymous>")&&(q=q.replace("<anonymous>",e.displayName)),q}while(1<=i&&0<=c);break}}}finally{Gu=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?Ol(n):""}function Kp(e){switch(e.tag){case 26:case 27:case 5:return Ol(e.type);case 16:return Ol("Lazy");case 13:return Ol("Suspense");case 19:return Ol("SuspenseList");case 0:case 15:return Ku(e.type,!1);case 11:return Ku(e.type.render,!1);case 1:return Ku(e.type,!0);case 31:return Ol("Activity");default:return""}}function Ff(e){try{var t="";do t+=Kp(e),e=e.return;while(e);return t}catch(n){return`
Error generating stack: `+n.message+`
`+n.stack}}function xt(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function Pf(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function Vp(e){var t=Pf(e)?"checked":"value",n=Object.getOwnPropertyDescriptor(e.constructor.prototype,t),i=""+e[t];if(!e.hasOwnProperty(t)&&typeof n<"u"&&typeof n.get=="function"&&typeof n.set=="function"){var c=n.get,f=n.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return c.call(this)},set:function(g){i=""+g,f.call(this,g)}}),Object.defineProperty(e,t,{enumerable:n.enumerable}),{getValue:function(){return i},setValue:function(g){i=""+g},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function Ya(e){e._valueTracker||(e._valueTracker=Vp(e))}function eo(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),i="";return e&&(i=Pf(e)?e.checked?"true":"false":e.value),e=i,e!==n?(t.setValue(e),!0):!1}function Ga(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var Qp=/[\n"\\]/g;function Dt(e){return e.replace(Qp,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function Vu(e,t,n,i,c,f,g,y){e.name="",g!=null&&typeof g!="function"&&typeof g!="symbol"&&typeof g!="boolean"?e.type=g:e.removeAttribute("type"),t!=null?g==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+xt(t)):e.value!==""+xt(t)&&(e.value=""+xt(t)):g!=="submit"&&g!=="reset"||e.removeAttribute("value"),t!=null?Qu(e,g,xt(t)):n!=null?Qu(e,g,xt(n)):i!=null&&e.removeAttribute("value"),c==null&&f!=null&&(e.defaultChecked=!!f),c!=null&&(e.checked=c&&typeof c!="function"&&typeof c!="symbol"),y!=null&&typeof y!="function"&&typeof y!="symbol"&&typeof y!="boolean"?e.name=""+xt(y):e.removeAttribute("name")}function to(e,t,n,i,c,f,g,y){if(f!=null&&typeof f!="function"&&typeof f!="symbol"&&typeof f!="boolean"&&(e.type=f),t!=null||n!=null){if(!(f!=="submit"&&f!=="reset"||t!=null))return;n=n!=null?""+xt(n):"",t=t!=null?""+xt(t):n,y||t===e.value||(e.value=t),e.defaultValue=t}i=i??c,i=typeof i!="function"&&typeof i!="symbol"&&!!i,e.checked=y?e.checked:!!i,e.defaultChecked=!!i,g!=null&&typeof g!="function"&&typeof g!="symbol"&&typeof g!="boolean"&&(e.name=g)}function Qu(e,t,n){t==="number"&&Ga(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function Nl(e,t,n,i){if(e=e.options,t){t={};for(var c=0;c<n.length;c++)t["$"+n[c]]=!0;for(n=0;n<e.length;n++)c=t.hasOwnProperty("$"+e[n].value),e[n].selected!==c&&(e[n].selected=c),c&&i&&(e[n].defaultSelected=!0)}else{for(n=""+xt(n),t=null,c=0;c<e.length;c++){if(e[c].value===n){e[c].selected=!0,i&&(e[c].defaultSelected=!0);return}t!==null||e[c].disabled||(t=e[c])}t!==null&&(t.selected=!0)}}function no(e,t,n){if(t!=null&&(t=""+xt(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+xt(n):""}function lo(e,t,n,i){if(t==null){if(i!=null){if(n!=null)throw Error(s(92));if(qe(i)){if(1<i.length)throw Error(s(93));i=i[0]}n=i}n==null&&(n=""),t=n}n=xt(t),e.defaultValue=n,i=e.textContent,i===n&&i!==""&&i!==null&&(e.value=i)}function _l(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var Xp=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function io(e,t,n){var i=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?i?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":i?e.setProperty(t,n):typeof n!="number"||n===0||Xp.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function ao(e,t,n){if(t!=null&&typeof t!="object")throw Error(s(62));if(e=e.style,n!=null){for(var i in n)!n.hasOwnProperty(i)||t!=null&&t.hasOwnProperty(i)||(i.indexOf("--")===0?e.setProperty(i,""):i==="float"?e.cssFloat="":e[i]="");for(var c in t)i=t[c],t.hasOwnProperty(c)&&n[c]!==i&&io(e,c,i)}else for(var f in t)t.hasOwnProperty(f)&&io(e,f,t[f])}function Xu(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Zp=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Jp=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function Ka(e){return Jp.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}var Zu=null;function Ju(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var Ml=null,xl=null;function so(e){var t=wl(e);if(t&&(e=t.stateNode)){var n=e[rt]||null;e:switch(e=t.stateNode,t.type){case"input":if(Vu(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+Dt(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var i=n[t];if(i!==e&&i.form===e.form){var c=i[rt]||null;if(!c)throw Error(s(90));Vu(i,c.value,c.defaultValue,c.defaultValue,c.checked,c.defaultChecked,c.type,c.name)}}for(t=0;t<n.length;t++)i=n[t],i.form===e.form&&eo(i)}break e;case"textarea":no(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&Nl(e,!!n.multiple,t,!1)}}}var Wu=!1;function uo(e,t,n){if(Wu)return e(t,n);Wu=!0;try{var i=e(t);return i}finally{if(Wu=!1,(Ml!==null||xl!==null)&&(xs(),Ml&&(t=Ml,e=xl,xl=Ml=null,so(t),e)))for(t=0;t<e.length;t++)so(e[t])}}function Di(e,t){var n=e.stateNode;if(n===null)return null;var i=n[rt]||null;if(i===null)return null;n=i[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(i=!i.disabled)||(e=e.type,i=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!i;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(s(231,t,typeof n));return n}var en=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),Iu=!1;if(en)try{var Ci={};Object.defineProperty(Ci,"passive",{get:function(){Iu=!0}}),window.addEventListener("test",Ci,Ci),window.removeEventListener("test",Ci,Ci)}catch{Iu=!1}var Tn=null,Fu=null,Va=null;function co(){if(Va)return Va;var e,t=Fu,n=t.length,i,c="value"in Tn?Tn.value:Tn.textContent,f=c.length;for(e=0;e<n&&t[e]===c[e];e++);var g=n-e;for(i=1;i<=g&&t[n-i]===c[f-i];i++);return Va=c.slice(e,1<i?1-i:void 0)}function Qa(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function Xa(){return!0}function ro(){return!1}function ft(e){function t(n,i,c,f,g){this._reactName=n,this._targetInst=c,this.type=i,this.nativeEvent=f,this.target=g,this.currentTarget=null;for(var y in e)e.hasOwnProperty(y)&&(n=e[y],this[y]=n?n(f):f[y]);return this.isDefaultPrevented=(f.defaultPrevented!=null?f.defaultPrevented:f.returnValue===!1)?Xa:ro,this.isPropagationStopped=ro,this}return b(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=Xa)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=Xa)},persist:function(){},isPersistent:Xa}),t}var Jn={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},Za=ft(Jn),Ri=b({},Jn,{view:0,detail:0}),Wp=ft(Ri),Pu,ec,ki,Ja=b({},Ri,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:nc,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==ki&&(ki&&e.type==="mousemove"?(Pu=e.screenX-ki.screenX,ec=e.screenY-ki.screenY):ec=Pu=0,ki=e),Pu)},movementY:function(e){return"movementY"in e?e.movementY:ec}}),fo=ft(Ja),Ip=b({},Ja,{dataTransfer:0}),Fp=ft(Ip),Pp=b({},Ri,{relatedTarget:0}),tc=ft(Pp),ey=b({},Jn,{animationName:0,elapsedTime:0,pseudoElement:0}),ty=ft(ey),ny=b({},Jn,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),ly=ft(ny),iy=b({},Jn,{data:0}),oo=ft(iy),ay={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},sy={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},uy={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function cy(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=uy[e])?!!t[e]:!1}function nc(){return cy}var ry=b({},Ri,{key:function(e){if(e.key){var t=ay[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=Qa(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?sy[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:nc,charCode:function(e){return e.type==="keypress"?Qa(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?Qa(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),fy=ft(ry),oy=b({},Ja,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),ho=ft(oy),hy=b({},Ri,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:nc}),dy=ft(hy),gy=b({},Jn,{propertyName:0,elapsedTime:0,pseudoElement:0}),my=ft(gy),py=b({},Ja,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),yy=ft(py),vy=b({},Jn,{newState:0,oldState:0}),by=ft(vy),Sy=[9,13,27,32],lc=en&&"CompositionEvent"in window,Li=null;en&&"documentMode"in document&&(Li=document.documentMode);var Ty=en&&"TextEvent"in window&&!Li,go=en&&(!lc||Li&&8<Li&&11>=Li),mo=" ",po=!1;function yo(e,t){switch(e){case"keyup":return Sy.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function vo(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Dl=!1;function wy(e,t){switch(e){case"compositionend":return vo(t);case"keypress":return t.which!==32?null:(po=!0,mo);case"textInput":return e=t.data,e===mo&&po?null:e;default:return null}}function Ey(e,t){if(Dl)return e==="compositionend"||!lc&&yo(e,t)?(e=co(),Va=Fu=Tn=null,Dl=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return go&&t.locale!=="ko"?null:t.data;default:return null}}var Ay={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function bo(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!Ay[e.type]:t==="textarea"}function So(e,t,n,i){Ml?xl?xl.push(i):xl=[i]:Ml=i,t=zs(t,"onChange"),0<t.length&&(n=new Za("onChange","change",null,n,i),e.push({event:n,listeners:t}))}var zi=null,Ui=null;function Oy(e){eg(e,0)}function Wa(e){var t=xi(e);if(eo(t))return e}function To(e,t){if(e==="change")return t}var wo=!1;if(en){var ic;if(en){var ac="oninput"in document;if(!ac){var Eo=document.createElement("div");Eo.setAttribute("oninput","return;"),ac=typeof Eo.oninput=="function"}ic=ac}else ic=!1;wo=ic&&(!document.documentMode||9<document.documentMode)}function Ao(){zi&&(zi.detachEvent("onpropertychange",Oo),Ui=zi=null)}function Oo(e){if(e.propertyName==="value"&&Wa(Ui)){var t=[];So(t,Ui,e,Ju(e)),uo(Oy,t)}}function Ny(e,t,n){e==="focusin"?(Ao(),zi=t,Ui=n,zi.attachEvent("onpropertychange",Oo)):e==="focusout"&&Ao()}function _y(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return Wa(Ui)}function My(e,t){if(e==="click")return Wa(t)}function xy(e,t){if(e==="input"||e==="change")return Wa(t)}function Dy(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var St=typeof Object.is=="function"?Object.is:Dy;function Bi(e,t){if(St(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),i=Object.keys(t);if(n.length!==i.length)return!1;for(i=0;i<n.length;i++){var c=n[i];if(!Lu.call(t,c)||!St(e[c],t[c]))return!1}return!0}function No(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function _o(e,t){var n=No(e);e=0;for(var i;n;){if(n.nodeType===3){if(i=e+n.textContent.length,e<=t&&i>=t)return{node:n,offset:t-e};e=i}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=No(n)}}function Mo(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?Mo(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function xo(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=Ga(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=Ga(e.document)}return t}function sc(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var Cy=en&&"documentMode"in document&&11>=document.documentMode,Cl=null,uc=null,qi=null,cc=!1;function Do(e,t,n){var i=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;cc||Cl==null||Cl!==Ga(i)||(i=Cl,"selectionStart"in i&&sc(i)?i={start:i.selectionStart,end:i.selectionEnd}:(i=(i.ownerDocument&&i.ownerDocument.defaultView||window).getSelection(),i={anchorNode:i.anchorNode,anchorOffset:i.anchorOffset,focusNode:i.focusNode,focusOffset:i.focusOffset}),qi&&Bi(qi,i)||(qi=i,i=zs(uc,"onSelect"),0<i.length&&(t=new Za("onSelect","select",null,t,n),e.push({event:t,listeners:i}),t.target=Cl)))}function Wn(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var Rl={animationend:Wn("Animation","AnimationEnd"),animationiteration:Wn("Animation","AnimationIteration"),animationstart:Wn("Animation","AnimationStart"),transitionrun:Wn("Transition","TransitionRun"),transitionstart:Wn("Transition","TransitionStart"),transitioncancel:Wn("Transition","TransitionCancel"),transitionend:Wn("Transition","TransitionEnd")},rc={},Co={};en&&(Co=document.createElement("div").style,"AnimationEvent"in window||(delete Rl.animationend.animation,delete Rl.animationiteration.animation,delete Rl.animationstart.animation),"TransitionEvent"in window||delete Rl.transitionend.transition);function In(e){if(rc[e])return rc[e];if(!Rl[e])return e;var t=Rl[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in Co)return rc[e]=t[n];return e}var Ro=In("animationend"),ko=In("animationiteration"),Lo=In("animationstart"),Ry=In("transitionrun"),ky=In("transitionstart"),Ly=In("transitioncancel"),zo=In("transitionend"),Uo=new Map,fc="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");fc.push("scrollEnd");function Ht(e,t){Uo.set(e,t),Zn(t,[e])}var Bo=new WeakMap;function Ct(e,t){if(typeof e=="object"&&e!==null){var n=Bo.get(e);return n!==void 0?n:(t={value:e,source:t,stack:Ff(t)},Bo.set(e,t),t)}return{value:e,source:t,stack:Ff(t)}}var Rt=[],kl=0,oc=0;function Ia(){for(var e=kl,t=oc=kl=0;t<e;){var n=Rt[t];Rt[t++]=null;var i=Rt[t];Rt[t++]=null;var c=Rt[t];Rt[t++]=null;var f=Rt[t];if(Rt[t++]=null,i!==null&&c!==null){var g=i.pending;g===null?c.next=c:(c.next=g.next,g.next=c),i.pending=c}f!==0&&qo(n,c,f)}}function Fa(e,t,n,i){Rt[kl++]=e,Rt[kl++]=t,Rt[kl++]=n,Rt[kl++]=i,oc|=i,e.lanes|=i,e=e.alternate,e!==null&&(e.lanes|=i)}function hc(e,t,n,i){return Fa(e,t,n,i),Pa(e)}function Ll(e,t){return Fa(e,null,null,t),Pa(e)}function qo(e,t,n){e.lanes|=n;var i=e.alternate;i!==null&&(i.lanes|=n);for(var c=!1,f=e.return;f!==null;)f.childLanes|=n,i=f.alternate,i!==null&&(i.childLanes|=n),f.tag===22&&(e=f.stateNode,e===null||e._visibility&1||(c=!0)),e=f,f=f.return;return e.tag===3?(f=e.stateNode,c&&t!==null&&(c=31-bt(n),e=f.hiddenUpdates,i=e[c],i===null?e[c]=[t]:i.push(t),t.lane=n|536870912),f):null}function Pa(e){if(50<ra)throw ra=0,vr=null,Error(s(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var zl={};function zy(e,t,n,i){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=i,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function Tt(e,t,n,i){return new zy(e,t,n,i)}function dc(e){return e=e.prototype,!(!e||!e.isReactComponent)}function tn(e,t){var n=e.alternate;return n===null?(n=Tt(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&65011712,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function jo(e,t){e.flags&=65011714;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function es(e,t,n,i,c,f){var g=0;if(i=e,typeof e=="function")dc(e)&&(g=1);else if(typeof e=="string")g=B0(e,n,ne.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case z:return e=Tt(31,n,t,c),e.elementType=z,e.lanes=f,e;case D:return Fn(n.children,c,f,t);case T:g=8,c|=24;break;case w:return e=Tt(12,n,t,c|2),e.elementType=w,e.lanes=f,e;case G:return e=Tt(13,n,t,c),e.elementType=G,e.lanes=f,e;case K:return e=Tt(19,n,t,c),e.elementType=K,e.lanes=f,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case k:case H:g=10;break e;case L:g=9;break e;case Q:g=11;break e;case ee:g=14;break e;case V:g=16,i=null;break e}g=29,n=Error(s(130,e===null?"null":typeof e,"")),i=null}return t=Tt(g,n,t,c),t.elementType=e,t.type=i,t.lanes=f,t}function Fn(e,t,n,i){return e=Tt(7,e,i,t),e.lanes=n,e}function gc(e,t,n){return e=Tt(6,e,null,t),e.lanes=n,e}function mc(e,t,n){return t=Tt(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var Ul=[],Bl=0,ts=null,ns=0,kt=[],Lt=0,Pn=null,nn=1,ln="";function el(e,t){Ul[Bl++]=ns,Ul[Bl++]=ts,ts=e,ns=t}function Ho(e,t,n){kt[Lt++]=nn,kt[Lt++]=ln,kt[Lt++]=Pn,Pn=e;var i=nn;e=ln;var c=32-bt(i)-1;i&=~(1<<c),n+=1;var f=32-bt(t)+c;if(30<f){var g=c-c%5;f=(i&(1<<g)-1).toString(32),i>>=g,c-=g,nn=1<<32-bt(t)+c|n<<c|i,ln=f+e}else nn=1<<f|n<<c|i,ln=e}function pc(e){e.return!==null&&(el(e,1),Ho(e,1,0))}function yc(e){for(;e===ts;)ts=Ul[--Bl],Ul[Bl]=null,ns=Ul[--Bl],Ul[Bl]=null;for(;e===Pn;)Pn=kt[--Lt],kt[Lt]=null,ln=kt[--Lt],kt[Lt]=null,nn=kt[--Lt],kt[Lt]=null}var ut=null,Ue=null,be=!1,tl=null,Qt=!1,vc=Error(s(519));function nl(e){var t=Error(s(418,""));throw $i(Ct(t,e)),vc}function $o(e){var t=e.stateNode,n=e.type,i=e.memoizedProps;switch(t[nt]=e,t[rt]=i,n){case"dialog":ge("cancel",t),ge("close",t);break;case"iframe":case"object":case"embed":ge("load",t);break;case"video":case"audio":for(n=0;n<oa.length;n++)ge(oa[n],t);break;case"source":ge("error",t);break;case"img":case"image":case"link":ge("error",t),ge("load",t);break;case"details":ge("toggle",t);break;case"input":ge("invalid",t),to(t,i.value,i.defaultValue,i.checked,i.defaultChecked,i.type,i.name,!0),Ya(t);break;case"select":ge("invalid",t);break;case"textarea":ge("invalid",t),lo(t,i.value,i.defaultValue,i.children),Ya(t)}n=i.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||i.suppressHydrationWarning===!0||ig(t.textContent,n)?(i.popover!=null&&(ge("beforetoggle",t),ge("toggle",t)),i.onScroll!=null&&ge("scroll",t),i.onScrollEnd!=null&&ge("scrollend",t),i.onClick!=null&&(t.onclick=Us),t=!0):t=!1,t||nl(e)}function Yo(e){for(ut=e.return;ut;)switch(ut.tag){case 5:case 13:Qt=!1;return;case 27:case 3:Qt=!0;return;default:ut=ut.return}}function ji(e){if(e!==ut)return!1;if(!be)return Yo(e),be=!0,!1;var t=e.tag,n;if((n=t!==3&&t!==27)&&((n=t===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||Lr(e.type,e.memoizedProps)),n=!n),n&&Ue&&nl(e),Yo(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(s(317));e:{for(e=e.nextSibling,t=0;e;){if(e.nodeType===8)if(n=e.data,n==="/$"){if(t===0){Ue=Yt(e.nextSibling);break e}t--}else n!=="$"&&n!=="$!"&&n!=="$?"||t++;e=e.nextSibling}Ue=null}}else t===27?(t=Ue,Bn(e.type)?(e=qr,qr=null,Ue=e):Ue=t):Ue=ut?Yt(e.stateNode.nextSibling):null;return!0}function Hi(){Ue=ut=null,be=!1}function Go(){var e=tl;return e!==null&&(dt===null?dt=e:dt.push.apply(dt,e),tl=null),e}function $i(e){tl===null?tl=[e]:tl.push(e)}var bc=Y(null),ll=null,an=null;function wn(e,t,n){J(bc,t._currentValue),t._currentValue=n}function sn(e){e._currentValue=bc.current,W(bc)}function Sc(e,t,n){for(;e!==null;){var i=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,i!==null&&(i.childLanes|=t)):i!==null&&(i.childLanes&t)!==t&&(i.childLanes|=t),e===n)break;e=e.return}}function Tc(e,t,n,i){var c=e.child;for(c!==null&&(c.return=e);c!==null;){var f=c.dependencies;if(f!==null){var g=c.child;f=f.firstContext;e:for(;f!==null;){var y=f;f=c;for(var v=0;v<t.length;v++)if(y.context===t[v]){f.lanes|=n,y=f.alternate,y!==null&&(y.lanes|=n),Sc(f.return,n,e),i||(g=null);break e}f=y.next}}else if(c.tag===18){if(g=c.return,g===null)throw Error(s(341));g.lanes|=n,f=g.alternate,f!==null&&(f.lanes|=n),Sc(g,n,e),g=null}else g=c.child;if(g!==null)g.return=c;else for(g=c;g!==null;){if(g===e){g=null;break}if(c=g.sibling,c!==null){c.return=g.return,g=c;break}g=g.return}c=g}}function Yi(e,t,n,i){e=null;for(var c=t,f=!1;c!==null;){if(!f){if((c.flags&524288)!==0)f=!0;else if((c.flags&262144)!==0)break}if(c.tag===10){var g=c.alternate;if(g===null)throw Error(s(387));if(g=g.memoizedProps,g!==null){var y=c.type;St(c.pendingProps.value,g.value)||(e!==null?e.push(y):e=[y])}}else if(c===yt.current){if(g=c.alternate,g===null)throw Error(s(387));g.memoizedState.memoizedState!==c.memoizedState.memoizedState&&(e!==null?e.push(ya):e=[ya])}c=c.return}e!==null&&Tc(t,e,n,i),t.flags|=262144}function ls(e){for(e=e.firstContext;e!==null;){if(!St(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function il(e){ll=e,an=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function lt(e){return Ko(ll,e)}function is(e,t){return ll===null&&il(e),Ko(e,t)}function Ko(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},an===null){if(e===null)throw Error(s(308));an=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else an=an.next=t;return n}var Uy=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,i){e.push(i)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},By=u.unstable_scheduleCallback,qy=u.unstable_NormalPriority,Ve={$$typeof:H,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function wc(){return{controller:new Uy,data:new Map,refCount:0}}function Gi(e){e.refCount--,e.refCount===0&&By(qy,function(){e.controller.abort()})}var Ki=null,Ec=0,ql=0,jl=null;function jy(e,t){if(Ki===null){var n=Ki=[];Ec=0,ql=Or(),jl={status:"pending",value:void 0,then:function(i){n.push(i)}}}return Ec++,t.then(Vo,Vo),t}function Vo(){if(--Ec===0&&Ki!==null){jl!==null&&(jl.status="fulfilled");var e=Ki;Ki=null,ql=0,jl=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function Hy(e,t){var n=[],i={status:"pending",value:null,reason:null,then:function(c){n.push(c)}};return e.then(function(){i.status="fulfilled",i.value=t;for(var c=0;c<n.length;c++)(0,n[c])(t)},function(c){for(i.status="rejected",i.reason=c,c=0;c<n.length;c++)(0,n[c])(void 0)}),i}var Qo=B.S;B.S=function(e,t){typeof t=="object"&&t!==null&&typeof t.then=="function"&&jy(e,t),Qo!==null&&Qo(e,t)};var al=Y(null);function Ac(){var e=al.current;return e!==null?e:Me.pooledCache}function as(e,t){t===null?J(al,al.current):J(al,t.pool)}function Xo(){var e=Ac();return e===null?null:{parent:Ve._currentValue,pool:e}}var Vi=Error(s(460)),Zo=Error(s(474)),ss=Error(s(542)),Oc={then:function(){}};function Jo(e){return e=e.status,e==="fulfilled"||e==="rejected"}function us(){}function Wo(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(us,us),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Fo(e),e;default:if(typeof t.status=="string")t.then(us,us);else{if(e=Me,e!==null&&100<e.shellSuspendCounter)throw Error(s(482));e=t,e.status="pending",e.then(function(i){if(t.status==="pending"){var c=t;c.status="fulfilled",c.value=i}},function(i){if(t.status==="pending"){var c=t;c.status="rejected",c.reason=i}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,Fo(e),e}throw Qi=t,Vi}}var Qi=null;function Io(){if(Qi===null)throw Error(s(459));var e=Qi;return Qi=null,e}function Fo(e){if(e===Vi||e===ss)throw Error(s(483))}var En=!1;function Nc(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function _c(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function An(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function On(e,t,n){var i=e.updateQueue;if(i===null)return null;if(i=i.shared,(Se&2)!==0){var c=i.pending;return c===null?t.next=t:(t.next=c.next,c.next=t),i.pending=t,t=Pa(e),qo(e,null,n),t}return Fa(e,i,t,n),Pa(e)}function Xi(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194048)!==0)){var i=t.lanes;i&=e.pendingLanes,n|=i,t.lanes=n,Kf(e,n)}}function Mc(e,t){var n=e.updateQueue,i=e.alternate;if(i!==null&&(i=i.updateQueue,n===i)){var c=null,f=null;if(n=n.firstBaseUpdate,n!==null){do{var g={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};f===null?c=f=g:f=f.next=g,n=n.next}while(n!==null);f===null?c=f=t:f=f.next=t}else c=f=t;n={baseState:i.baseState,firstBaseUpdate:c,lastBaseUpdate:f,shared:i.shared,callbacks:i.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var xc=!1;function Zi(){if(xc){var e=jl;if(e!==null)throw e}}function Ji(e,t,n,i){xc=!1;var c=e.updateQueue;En=!1;var f=c.firstBaseUpdate,g=c.lastBaseUpdate,y=c.shared.pending;if(y!==null){c.shared.pending=null;var v=y,x=v.next;v.next=null,g===null?f=x:g.next=x,g=v;var q=e.alternate;q!==null&&(q=q.updateQueue,y=q.lastBaseUpdate,y!==g&&(y===null?q.firstBaseUpdate=x:y.next=x,q.lastBaseUpdate=v))}if(f!==null){var $=c.baseState;g=0,q=x=v=null,y=f;do{var C=y.lane&-536870913,R=C!==y.lane;if(R?(pe&C)===C:(i&C)===C){C!==0&&C===ql&&(xc=!0),q!==null&&(q=q.next={lane:0,tag:y.tag,payload:y.payload,callback:null,next:null});e:{var ue=e,le=y;C=t;var Ae=n;switch(le.tag){case 1:if(ue=le.payload,typeof ue=="function"){$=ue.call(Ae,$,C);break e}$=ue;break e;case 3:ue.flags=ue.flags&-65537|128;case 0:if(ue=le.payload,C=typeof ue=="function"?ue.call(Ae,$,C):ue,C==null)break e;$=b({},$,C);break e;case 2:En=!0}}C=y.callback,C!==null&&(e.flags|=64,R&&(e.flags|=8192),R=c.callbacks,R===null?c.callbacks=[C]:R.push(C))}else R={lane:C,tag:y.tag,payload:y.payload,callback:y.callback,next:null},q===null?(x=q=R,v=$):q=q.next=R,g|=C;if(y=y.next,y===null){if(y=c.shared.pending,y===null)break;R=y,y=R.next,R.next=null,c.lastBaseUpdate=R,c.shared.pending=null}}while(!0);q===null&&(v=$),c.baseState=v,c.firstBaseUpdate=x,c.lastBaseUpdate=q,f===null&&(c.shared.lanes=0),kn|=g,e.lanes=g,e.memoizedState=$}}function Po(e,t){if(typeof e!="function")throw Error(s(191,e));e.call(t)}function eh(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)Po(n[e],t)}var Hl=Y(null),cs=Y(0);function th(e,t){e=dn,J(cs,e),J(Hl,t),dn=e|t.baseLanes}function Dc(){J(cs,dn),J(Hl,Hl.current)}function Cc(){dn=cs.current,W(Hl),W(cs)}var Nn=0,oe=null,we=null,Ye=null,rs=!1,$l=!1,sl=!1,fs=0,Wi=0,Yl=null,$y=0;function je(){throw Error(s(321))}function Rc(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!St(e[n],t[n]))return!1;return!0}function kc(e,t,n,i,c,f){return Nn=f,oe=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,B.H=e===null||e.memoizedState===null?qh:jh,sl=!1,f=n(i,c),sl=!1,$l&&(f=lh(t,n,i,c)),nh(e),f}function nh(e){B.H=ps;var t=we!==null&&we.next!==null;if(Nn=0,Ye=we=oe=null,rs=!1,Wi=0,Yl=null,t)throw Error(s(300));e===null||Je||(e=e.dependencies,e!==null&&ls(e)&&(Je=!0))}function lh(e,t,n,i){oe=e;var c=0;do{if($l&&(Yl=null),Wi=0,$l=!1,25<=c)throw Error(s(301));if(c+=1,Ye=we=null,e.updateQueue!=null){var f=e.updateQueue;f.lastEffect=null,f.events=null,f.stores=null,f.memoCache!=null&&(f.memoCache.index=0)}B.H=Zy,f=t(n,i)}while($l);return f}function Yy(){var e=B.H,t=e.useState()[0];return t=typeof t.then=="function"?Ii(t):t,e=e.useState()[0],(we!==null?we.memoizedState:null)!==e&&(oe.flags|=1024),t}function Lc(){var e=fs!==0;return fs=0,e}function zc(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function Uc(e){if(rs){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}rs=!1}Nn=0,Ye=we=oe=null,$l=!1,Wi=fs=0,Yl=null}function ot(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Ye===null?oe.memoizedState=Ye=e:Ye=Ye.next=e,Ye}function Ge(){if(we===null){var e=oe.alternate;e=e!==null?e.memoizedState:null}else e=we.next;var t=Ye===null?oe.memoizedState:Ye.next;if(t!==null)Ye=t,we=e;else{if(e===null)throw oe.alternate===null?Error(s(467)):Error(s(310));we=e,e={memoizedState:we.memoizedState,baseState:we.baseState,baseQueue:we.baseQueue,queue:we.queue,next:null},Ye===null?oe.memoizedState=Ye=e:Ye=Ye.next=e}return Ye}function Bc(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Ii(e){var t=Wi;return Wi+=1,Yl===null&&(Yl=[]),e=Wo(Yl,e,t),t=oe,(Ye===null?t.memoizedState:Ye.next)===null&&(t=t.alternate,B.H=t===null||t.memoizedState===null?qh:jh),e}function os(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return Ii(e);if(e.$$typeof===H)return lt(e)}throw Error(s(438,String(e)))}function qc(e){var t=null,n=oe.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var i=oe.alternate;i!==null&&(i=i.updateQueue,i!==null&&(i=i.memoCache,i!=null&&(t={data:i.data.map(function(c){return c.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=Bc(),oe.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),i=0;i<e;i++)n[i]=F;return t.index++,n}function un(e,t){return typeof t=="function"?t(e):t}function hs(e){var t=Ge();return jc(t,we,e)}function jc(e,t,n){var i=e.queue;if(i===null)throw Error(s(311));i.lastRenderedReducer=n;var c=e.baseQueue,f=i.pending;if(f!==null){if(c!==null){var g=c.next;c.next=f.next,f.next=g}t.baseQueue=c=f,i.pending=null}if(f=e.baseState,c===null)e.memoizedState=f;else{t=c.next;var y=g=null,v=null,x=t,q=!1;do{var $=x.lane&-536870913;if($!==x.lane?(pe&$)===$:(Nn&$)===$){var C=x.revertLane;if(C===0)v!==null&&(v=v.next={lane:0,revertLane:0,action:x.action,hasEagerState:x.hasEagerState,eagerState:x.eagerState,next:null}),$===ql&&(q=!0);else if((Nn&C)===C){x=x.next,C===ql&&(q=!0);continue}else $={lane:0,revertLane:x.revertLane,action:x.action,hasEagerState:x.hasEagerState,eagerState:x.eagerState,next:null},v===null?(y=v=$,g=f):v=v.next=$,oe.lanes|=C,kn|=C;$=x.action,sl&&n(f,$),f=x.hasEagerState?x.eagerState:n(f,$)}else C={lane:$,revertLane:x.revertLane,action:x.action,hasEagerState:x.hasEagerState,eagerState:x.eagerState,next:null},v===null?(y=v=C,g=f):v=v.next=C,oe.lanes|=$,kn|=$;x=x.next}while(x!==null&&x!==t);if(v===null?g=f:v.next=y,!St(f,e.memoizedState)&&(Je=!0,q&&(n=jl,n!==null)))throw n;e.memoizedState=f,e.baseState=g,e.baseQueue=v,i.lastRenderedState=f}return c===null&&(i.lanes=0),[e.memoizedState,i.dispatch]}function Hc(e){var t=Ge(),n=t.queue;if(n===null)throw Error(s(311));n.lastRenderedReducer=e;var i=n.dispatch,c=n.pending,f=t.memoizedState;if(c!==null){n.pending=null;var g=c=c.next;do f=e(f,g.action),g=g.next;while(g!==c);St(f,t.memoizedState)||(Je=!0),t.memoizedState=f,t.baseQueue===null&&(t.baseState=f),n.lastRenderedState=f}return[f,i]}function ih(e,t,n){var i=oe,c=Ge(),f=be;if(f){if(n===void 0)throw Error(s(407));n=n()}else n=t();var g=!St((we||c).memoizedState,n);g&&(c.memoizedState=n,Je=!0),c=c.queue;var y=uh.bind(null,i,c,e);if(Fi(2048,8,y,[e]),c.getSnapshot!==t||g||Ye!==null&&Ye.memoizedState.tag&1){if(i.flags|=2048,Gl(9,ds(),sh.bind(null,i,c,n,t),null),Me===null)throw Error(s(349));f||(Nn&124)!==0||ah(i,t,n)}return n}function ah(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=oe.updateQueue,t===null?(t=Bc(),oe.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function sh(e,t,n,i){t.value=n,t.getSnapshot=i,ch(t)&&rh(e)}function uh(e,t,n){return n(function(){ch(t)&&rh(e)})}function ch(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!St(e,n)}catch{return!0}}function rh(e){var t=Ll(e,2);t!==null&&Nt(t,e,2)}function $c(e){var t=ot();if(typeof e=="function"){var n=e;if(e=n(),sl){bn(!0);try{n()}finally{bn(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:un,lastRenderedState:e},t}function fh(e,t,n,i){return e.baseState=n,jc(e,we,typeof i=="function"?i:un)}function Gy(e,t,n,i,c){if(ms(e))throw Error(s(485));if(e=t.action,e!==null){var f={payload:c,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(g){f.listeners.push(g)}};B.T!==null?n(!0):f.isTransition=!1,i(f),n=t.pending,n===null?(f.next=t.pending=f,oh(t,f)):(f.next=n.next,t.pending=n.next=f)}}function oh(e,t){var n=t.action,i=t.payload,c=e.state;if(t.isTransition){var f=B.T,g={};B.T=g;try{var y=n(c,i),v=B.S;v!==null&&v(g,y),hh(e,t,y)}catch(x){Yc(e,t,x)}finally{B.T=f}}else try{f=n(c,i),hh(e,t,f)}catch(x){Yc(e,t,x)}}function hh(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(i){dh(e,t,i)},function(i){return Yc(e,t,i)}):dh(e,t,n)}function dh(e,t,n){t.status="fulfilled",t.value=n,gh(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,oh(e,n)))}function Yc(e,t,n){var i=e.pending;if(e.pending=null,i!==null){i=i.next;do t.status="rejected",t.reason=n,gh(t),t=t.next;while(t!==i)}e.action=null}function gh(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function mh(e,t){return t}function ph(e,t){if(be){var n=Me.formState;if(n!==null){e:{var i=oe;if(be){if(Ue){t:{for(var c=Ue,f=Qt;c.nodeType!==8;){if(!f){c=null;break t}if(c=Yt(c.nextSibling),c===null){c=null;break t}}f=c.data,c=f==="F!"||f==="F"?c:null}if(c){Ue=Yt(c.nextSibling),i=c.data==="F!";break e}}nl(i)}i=!1}i&&(t=n[0])}}return n=ot(),n.memoizedState=n.baseState=t,i={pending:null,lanes:0,dispatch:null,lastRenderedReducer:mh,lastRenderedState:t},n.queue=i,n=zh.bind(null,oe,i),i.dispatch=n,i=$c(!1),f=Xc.bind(null,oe,!1,i.queue),i=ot(),c={state:t,dispatch:null,action:e,pending:null},i.queue=c,n=Gy.bind(null,oe,c,f,n),c.dispatch=n,i.memoizedState=e,[t,n,!1]}function yh(e){var t=Ge();return vh(t,we,e)}function vh(e,t,n){if(t=jc(e,t,mh)[0],e=hs(un)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var i=Ii(t)}catch(g){throw g===Vi?ss:g}else i=t;t=Ge();var c=t.queue,f=c.dispatch;return n!==t.memoizedState&&(oe.flags|=2048,Gl(9,ds(),Ky.bind(null,c,n),null)),[i,f,e]}function Ky(e,t){e.action=t}function bh(e){var t=Ge(),n=we;if(n!==null)return vh(t,n,e);Ge(),t=t.memoizedState,n=Ge();var i=n.queue.dispatch;return n.memoizedState=e,[t,i,!1]}function Gl(e,t,n,i){return e={tag:e,create:n,deps:i,inst:t,next:null},t=oe.updateQueue,t===null&&(t=Bc(),oe.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(i=n.next,n.next=e,e.next=i,t.lastEffect=e),e}function ds(){return{destroy:void 0,resource:void 0}}function Sh(){return Ge().memoizedState}function gs(e,t,n,i){var c=ot();i=i===void 0?null:i,oe.flags|=e,c.memoizedState=Gl(1|t,ds(),n,i)}function Fi(e,t,n,i){var c=Ge();i=i===void 0?null:i;var f=c.memoizedState.inst;we!==null&&i!==null&&Rc(i,we.memoizedState.deps)?c.memoizedState=Gl(t,f,n,i):(oe.flags|=e,c.memoizedState=Gl(1|t,f,n,i))}function Th(e,t){gs(8390656,8,e,t)}function wh(e,t){Fi(2048,8,e,t)}function Eh(e,t){return Fi(4,2,e,t)}function Ah(e,t){return Fi(4,4,e,t)}function Oh(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function Nh(e,t,n){n=n!=null?n.concat([e]):null,Fi(4,4,Oh.bind(null,t,e),n)}function Gc(){}function _h(e,t){var n=Ge();t=t===void 0?null:t;var i=n.memoizedState;return t!==null&&Rc(t,i[1])?i[0]:(n.memoizedState=[e,t],e)}function Mh(e,t){var n=Ge();t=t===void 0?null:t;var i=n.memoizedState;if(t!==null&&Rc(t,i[1]))return i[0];if(i=e(),sl){bn(!0);try{e()}finally{bn(!1)}}return n.memoizedState=[i,t],i}function Kc(e,t,n){return n===void 0||(Nn&1073741824)!==0?e.memoizedState=t:(e.memoizedState=n,e=Cd(),oe.lanes|=e,kn|=e,n)}function xh(e,t,n,i){return St(n,t)?n:Hl.current!==null?(e=Kc(e,n,i),St(e,t)||(Je=!0),e):(Nn&42)===0?(Je=!0,e.memoizedState=n):(e=Cd(),oe.lanes|=e,kn|=e,t)}function Dh(e,t,n,i,c){var f=Z.p;Z.p=f!==0&&8>f?f:8;var g=B.T,y={};B.T=y,Xc(e,!1,t,n);try{var v=c(),x=B.S;if(x!==null&&x(y,v),v!==null&&typeof v=="object"&&typeof v.then=="function"){var q=Hy(v,i);Pi(e,t,q,Ot(e))}else Pi(e,t,i,Ot(e))}catch($){Pi(e,t,{then:function(){},status:"rejected",reason:$},Ot())}finally{Z.p=f,B.T=g}}function Vy(){}function Vc(e,t,n,i){if(e.tag!==5)throw Error(s(476));var c=Ch(e).queue;Dh(e,c,t,ae,n===null?Vy:function(){return Rh(e),n(i)})}function Ch(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:ae,baseState:ae,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:un,lastRenderedState:ae},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:un,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function Rh(e){var t=Ch(e).next.queue;Pi(e,t,{},Ot())}function Qc(){return lt(ya)}function kh(){return Ge().memoizedState}function Lh(){return Ge().memoizedState}function Qy(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=Ot();e=An(n);var i=On(t,e,n);i!==null&&(Nt(i,t,n),Xi(i,t,n)),t={cache:wc()},e.payload=t;return}t=t.return}}function Xy(e,t,n){var i=Ot();n={lane:i,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null},ms(e)?Uh(t,n):(n=hc(e,t,n,i),n!==null&&(Nt(n,e,i),Bh(n,t,i)))}function zh(e,t,n){var i=Ot();Pi(e,t,n,i)}function Pi(e,t,n,i){var c={lane:i,revertLane:0,action:n,hasEagerState:!1,eagerState:null,next:null};if(ms(e))Uh(t,c);else{var f=e.alternate;if(e.lanes===0&&(f===null||f.lanes===0)&&(f=t.lastRenderedReducer,f!==null))try{var g=t.lastRenderedState,y=f(g,n);if(c.hasEagerState=!0,c.eagerState=y,St(y,g))return Fa(e,t,c,0),Me===null&&Ia(),!1}catch{}finally{}if(n=hc(e,t,c,i),n!==null)return Nt(n,e,i),Bh(n,t,i),!0}return!1}function Xc(e,t,n,i){if(i={lane:2,revertLane:Or(),action:i,hasEagerState:!1,eagerState:null,next:null},ms(e)){if(t)throw Error(s(479))}else t=hc(e,n,i,2),t!==null&&Nt(t,e,2)}function ms(e){var t=e.alternate;return e===oe||t!==null&&t===oe}function Uh(e,t){$l=rs=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function Bh(e,t,n){if((n&4194048)!==0){var i=t.lanes;i&=e.pendingLanes,n|=i,t.lanes=n,Kf(e,n)}}var ps={readContext:lt,use:os,useCallback:je,useContext:je,useEffect:je,useImperativeHandle:je,useLayoutEffect:je,useInsertionEffect:je,useMemo:je,useReducer:je,useRef:je,useState:je,useDebugValue:je,useDeferredValue:je,useTransition:je,useSyncExternalStore:je,useId:je,useHostTransitionStatus:je,useFormState:je,useActionState:je,useOptimistic:je,useMemoCache:je,useCacheRefresh:je},qh={readContext:lt,use:os,useCallback:function(e,t){return ot().memoizedState=[e,t===void 0?null:t],e},useContext:lt,useEffect:Th,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,gs(4194308,4,Oh.bind(null,t,e),n)},useLayoutEffect:function(e,t){return gs(4194308,4,e,t)},useInsertionEffect:function(e,t){gs(4,2,e,t)},useMemo:function(e,t){var n=ot();t=t===void 0?null:t;var i=e();if(sl){bn(!0);try{e()}finally{bn(!1)}}return n.memoizedState=[i,t],i},useReducer:function(e,t,n){var i=ot();if(n!==void 0){var c=n(t);if(sl){bn(!0);try{n(t)}finally{bn(!1)}}}else c=t;return i.memoizedState=i.baseState=c,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:c},i.queue=e,e=e.dispatch=Xy.bind(null,oe,e),[i.memoizedState,e]},useRef:function(e){var t=ot();return e={current:e},t.memoizedState=e},useState:function(e){e=$c(e);var t=e.queue,n=zh.bind(null,oe,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:Gc,useDeferredValue:function(e,t){var n=ot();return Kc(n,e,t)},useTransition:function(){var e=$c(!1);return e=Dh.bind(null,oe,e.queue,!0,!1),ot().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var i=oe,c=ot();if(be){if(n===void 0)throw Error(s(407));n=n()}else{if(n=t(),Me===null)throw Error(s(349));(pe&124)!==0||ah(i,t,n)}c.memoizedState=n;var f={value:n,getSnapshot:t};return c.queue=f,Th(uh.bind(null,i,f,e),[e]),i.flags|=2048,Gl(9,ds(),sh.bind(null,i,f,n,t),null),n},useId:function(){var e=ot(),t=Me.identifierPrefix;if(be){var n=ln,i=nn;n=(i&~(1<<32-bt(i)-1)).toString(32)+n,t="«"+t+"R"+n,n=fs++,0<n&&(t+="H"+n.toString(32)),t+="»"}else n=$y++,t="«"+t+"r"+n.toString(32)+"»";return e.memoizedState=t},useHostTransitionStatus:Qc,useFormState:ph,useActionState:ph,useOptimistic:function(e){var t=ot();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=Xc.bind(null,oe,!0,n),n.dispatch=t,[e,t]},useMemoCache:qc,useCacheRefresh:function(){return ot().memoizedState=Qy.bind(null,oe)}},jh={readContext:lt,use:os,useCallback:_h,useContext:lt,useEffect:wh,useImperativeHandle:Nh,useInsertionEffect:Eh,useLayoutEffect:Ah,useMemo:Mh,useReducer:hs,useRef:Sh,useState:function(){return hs(un)},useDebugValue:Gc,useDeferredValue:function(e,t){var n=Ge();return xh(n,we.memoizedState,e,t)},useTransition:function(){var e=hs(un)[0],t=Ge().memoizedState;return[typeof e=="boolean"?e:Ii(e),t]},useSyncExternalStore:ih,useId:kh,useHostTransitionStatus:Qc,useFormState:yh,useActionState:yh,useOptimistic:function(e,t){var n=Ge();return fh(n,we,e,t)},useMemoCache:qc,useCacheRefresh:Lh},Zy={readContext:lt,use:os,useCallback:_h,useContext:lt,useEffect:wh,useImperativeHandle:Nh,useInsertionEffect:Eh,useLayoutEffect:Ah,useMemo:Mh,useReducer:Hc,useRef:Sh,useState:function(){return Hc(un)},useDebugValue:Gc,useDeferredValue:function(e,t){var n=Ge();return we===null?Kc(n,e,t):xh(n,we.memoizedState,e,t)},useTransition:function(){var e=Hc(un)[0],t=Ge().memoizedState;return[typeof e=="boolean"?e:Ii(e),t]},useSyncExternalStore:ih,useId:kh,useHostTransitionStatus:Qc,useFormState:bh,useActionState:bh,useOptimistic:function(e,t){var n=Ge();return we!==null?fh(n,we,e,t):(n.baseState=e,[e,n.queue.dispatch])},useMemoCache:qc,useCacheRefresh:Lh},Kl=null,ea=0;function ys(e){var t=ea;return ea+=1,Kl===null&&(Kl=[]),Wo(Kl,e,t)}function ta(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function vs(e,t){throw t.$$typeof===S?Error(s(525)):(e=Object.prototype.toString.call(t),Error(s(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function Hh(e){var t=e._init;return t(e._payload)}function $h(e){function t(_,O){if(e){var M=_.deletions;M===null?(_.deletions=[O],_.flags|=16):M.push(O)}}function n(_,O){if(!e)return null;for(;O!==null;)t(_,O),O=O.sibling;return null}function i(_){for(var O=new Map;_!==null;)_.key!==null?O.set(_.key,_):O.set(_.index,_),_=_.sibling;return O}function c(_,O){return _=tn(_,O),_.index=0,_.sibling=null,_}function f(_,O,M){return _.index=M,e?(M=_.alternate,M!==null?(M=M.index,M<O?(_.flags|=67108866,O):M):(_.flags|=67108866,O)):(_.flags|=1048576,O)}function g(_){return e&&_.alternate===null&&(_.flags|=67108866),_}function y(_,O,M,j){return O===null||O.tag!==6?(O=gc(M,_.mode,j),O.return=_,O):(O=c(O,M),O.return=_,O)}function v(_,O,M,j){var I=M.type;return I===D?q(_,O,M.props.children,j,M.key):O!==null&&(O.elementType===I||typeof I=="object"&&I!==null&&I.$$typeof===V&&Hh(I)===O.type)?(O=c(O,M.props),ta(O,M),O.return=_,O):(O=es(M.type,M.key,M.props,null,_.mode,j),ta(O,M),O.return=_,O)}function x(_,O,M,j){return O===null||O.tag!==4||O.stateNode.containerInfo!==M.containerInfo||O.stateNode.implementation!==M.implementation?(O=mc(M,_.mode,j),O.return=_,O):(O=c(O,M.children||[]),O.return=_,O)}function q(_,O,M,j,I){return O===null||O.tag!==7?(O=Fn(M,_.mode,j,I),O.return=_,O):(O=c(O,M),O.return=_,O)}function $(_,O,M){if(typeof O=="string"&&O!==""||typeof O=="number"||typeof O=="bigint")return O=gc(""+O,_.mode,M),O.return=_,O;if(typeof O=="object"&&O!==null){switch(O.$$typeof){case N:return M=es(O.type,O.key,O.props,null,_.mode,M),ta(M,O),M.return=_,M;case E:return O=mc(O,_.mode,M),O.return=_,O;case V:var j=O._init;return O=j(O._payload),$(_,O,M)}if(qe(O)||U(O))return O=Fn(O,_.mode,M,null),O.return=_,O;if(typeof O.then=="function")return $(_,ys(O),M);if(O.$$typeof===H)return $(_,is(_,O),M);vs(_,O)}return null}function C(_,O,M,j){var I=O!==null?O.key:null;if(typeof M=="string"&&M!==""||typeof M=="number"||typeof M=="bigint")return I!==null?null:y(_,O,""+M,j);if(typeof M=="object"&&M!==null){switch(M.$$typeof){case N:return M.key===I?v(_,O,M,j):null;case E:return M.key===I?x(_,O,M,j):null;case V:return I=M._init,M=I(M._payload),C(_,O,M,j)}if(qe(M)||U(M))return I!==null?null:q(_,O,M,j,null);if(typeof M.then=="function")return C(_,O,ys(M),j);if(M.$$typeof===H)return C(_,O,is(_,M),j);vs(_,M)}return null}function R(_,O,M,j,I){if(typeof j=="string"&&j!==""||typeof j=="number"||typeof j=="bigint")return _=_.get(M)||null,y(O,_,""+j,I);if(typeof j=="object"&&j!==null){switch(j.$$typeof){case N:return _=_.get(j.key===null?M:j.key)||null,v(O,_,j,I);case E:return _=_.get(j.key===null?M:j.key)||null,x(O,_,j,I);case V:var he=j._init;return j=he(j._payload),R(_,O,M,j,I)}if(qe(j)||U(j))return _=_.get(M)||null,q(O,_,j,I,null);if(typeof j.then=="function")return R(_,O,M,ys(j),I);if(j.$$typeof===H)return R(_,O,M,is(O,j),I);vs(O,j)}return null}function ue(_,O,M,j){for(var I=null,he=null,P=O,ie=O=0,Ie=null;P!==null&&ie<M.length;ie++){P.index>ie?(Ie=P,P=null):Ie=P.sibling;var ve=C(_,P,M[ie],j);if(ve===null){P===null&&(P=Ie);break}e&&P&&ve.alternate===null&&t(_,P),O=f(ve,O,ie),he===null?I=ve:he.sibling=ve,he=ve,P=Ie}if(ie===M.length)return n(_,P),be&&el(_,ie),I;if(P===null){for(;ie<M.length;ie++)P=$(_,M[ie],j),P!==null&&(O=f(P,O,ie),he===null?I=P:he.sibling=P,he=P);return be&&el(_,ie),I}for(P=i(P);ie<M.length;ie++)Ie=R(P,_,ie,M[ie],j),Ie!==null&&(e&&Ie.alternate!==null&&P.delete(Ie.key===null?ie:Ie.key),O=f(Ie,O,ie),he===null?I=Ie:he.sibling=Ie,he=Ie);return e&&P.forEach(function(Yn){return t(_,Yn)}),be&&el(_,ie),I}function le(_,O,M,j){if(M==null)throw Error(s(151));for(var I=null,he=null,P=O,ie=O=0,Ie=null,ve=M.next();P!==null&&!ve.done;ie++,ve=M.next()){P.index>ie?(Ie=P,P=null):Ie=P.sibling;var Yn=C(_,P,ve.value,j);if(Yn===null){P===null&&(P=Ie);break}e&&P&&Yn.alternate===null&&t(_,P),O=f(Yn,O,ie),he===null?I=Yn:he.sibling=Yn,he=Yn,P=Ie}if(ve.done)return n(_,P),be&&el(_,ie),I;if(P===null){for(;!ve.done;ie++,ve=M.next())ve=$(_,ve.value,j),ve!==null&&(O=f(ve,O,ie),he===null?I=ve:he.sibling=ve,he=ve);return be&&el(_,ie),I}for(P=i(P);!ve.done;ie++,ve=M.next())ve=R(P,_,ie,ve.value,j),ve!==null&&(e&&ve.alternate!==null&&P.delete(ve.key===null?ie:ve.key),O=f(ve,O,ie),he===null?I=ve:he.sibling=ve,he=ve);return e&&P.forEach(function(J0){return t(_,J0)}),be&&el(_,ie),I}function Ae(_,O,M,j){if(typeof M=="object"&&M!==null&&M.type===D&&M.key===null&&(M=M.props.children),typeof M=="object"&&M!==null){switch(M.$$typeof){case N:e:{for(var I=M.key;O!==null;){if(O.key===I){if(I=M.type,I===D){if(O.tag===7){n(_,O.sibling),j=c(O,M.props.children),j.return=_,_=j;break e}}else if(O.elementType===I||typeof I=="object"&&I!==null&&I.$$typeof===V&&Hh(I)===O.type){n(_,O.sibling),j=c(O,M.props),ta(j,M),j.return=_,_=j;break e}n(_,O);break}else t(_,O);O=O.sibling}M.type===D?(j=Fn(M.props.children,_.mode,j,M.key),j.return=_,_=j):(j=es(M.type,M.key,M.props,null,_.mode,j),ta(j,M),j.return=_,_=j)}return g(_);case E:e:{for(I=M.key;O!==null;){if(O.key===I)if(O.tag===4&&O.stateNode.containerInfo===M.containerInfo&&O.stateNode.implementation===M.implementation){n(_,O.sibling),j=c(O,M.children||[]),j.return=_,_=j;break e}else{n(_,O);break}else t(_,O);O=O.sibling}j=mc(M,_.mode,j),j.return=_,_=j}return g(_);case V:return I=M._init,M=I(M._payload),Ae(_,O,M,j)}if(qe(M))return ue(_,O,M,j);if(U(M)){if(I=U(M),typeof I!="function")throw Error(s(150));return M=I.call(M),le(_,O,M,j)}if(typeof M.then=="function")return Ae(_,O,ys(M),j);if(M.$$typeof===H)return Ae(_,O,is(_,M),j);vs(_,M)}return typeof M=="string"&&M!==""||typeof M=="number"||typeof M=="bigint"?(M=""+M,O!==null&&O.tag===6?(n(_,O.sibling),j=c(O,M),j.return=_,_=j):(n(_,O),j=gc(M,_.mode,j),j.return=_,_=j),g(_)):n(_,O)}return function(_,O,M,j){try{ea=0;var I=Ae(_,O,M,j);return Kl=null,I}catch(P){if(P===Vi||P===ss)throw P;var he=Tt(29,P,null,_.mode);return he.lanes=j,he.return=_,he}finally{}}}var Vl=$h(!0),Yh=$h(!1),zt=Y(null),Xt=null;function _n(e){var t=e.alternate;J(Qe,Qe.current&1),J(zt,e),Xt===null&&(t===null||Hl.current!==null||t.memoizedState!==null)&&(Xt=e)}function Gh(e){if(e.tag===22){if(J(Qe,Qe.current),J(zt,e),Xt===null){var t=e.alternate;t!==null&&t.memoizedState!==null&&(Xt=e)}}else Mn()}function Mn(){J(Qe,Qe.current),J(zt,zt.current)}function cn(e){W(zt),Xt===e&&(Xt=null),W(Qe)}var Qe=Y(0);function bs(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||n.data==="$?"||Br(n)))return t}else if(t.tag===19&&t.memoizedProps.revealOrder!==void 0){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}function Zc(e,t,n,i){t=e.memoizedState,n=n(i,t),n=n==null?t:b({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var Jc={enqueueSetState:function(e,t,n){e=e._reactInternals;var i=Ot(),c=An(i);c.payload=t,n!=null&&(c.callback=n),t=On(e,c,i),t!==null&&(Nt(t,e,i),Xi(t,e,i))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var i=Ot(),c=An(i);c.tag=1,c.payload=t,n!=null&&(c.callback=n),t=On(e,c,i),t!==null&&(Nt(t,e,i),Xi(t,e,i))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=Ot(),i=An(n);i.tag=2,t!=null&&(i.callback=t),t=On(e,i,n),t!==null&&(Nt(t,e,n),Xi(t,e,n))}};function Kh(e,t,n,i,c,f,g){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(i,f,g):t.prototype&&t.prototype.isPureReactComponent?!Bi(n,i)||!Bi(c,f):!0}function Vh(e,t,n,i){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,i),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,i),t.state!==e&&Jc.enqueueReplaceState(t,t.state,null)}function ul(e,t){var n=t;if("ref"in t){n={};for(var i in t)i!=="ref"&&(n[i]=t[i])}if(e=e.defaultProps){n===t&&(n=b({},n));for(var c in e)n[c]===void 0&&(n[c]=e[c])}return n}var Ss=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)};function Qh(e){Ss(e)}function Xh(e){console.error(e)}function Zh(e){Ss(e)}function Ts(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(i){setTimeout(function(){throw i})}}function Jh(e,t,n){try{var i=e.onCaughtError;i(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(c){setTimeout(function(){throw c})}}function Wc(e,t,n){return n=An(n),n.tag=3,n.payload={element:null},n.callback=function(){Ts(e,t)},n}function Wh(e){return e=An(e),e.tag=3,e}function Ih(e,t,n,i){var c=n.type.getDerivedStateFromError;if(typeof c=="function"){var f=i.value;e.payload=function(){return c(f)},e.callback=function(){Jh(t,n,i)}}var g=n.stateNode;g!==null&&typeof g.componentDidCatch=="function"&&(e.callback=function(){Jh(t,n,i),typeof c!="function"&&(Ln===null?Ln=new Set([this]):Ln.add(this));var y=i.stack;this.componentDidCatch(i.value,{componentStack:y!==null?y:""})})}function Jy(e,t,n,i,c){if(n.flags|=32768,i!==null&&typeof i=="object"&&typeof i.then=="function"){if(t=n.alternate,t!==null&&Yi(t,n,c,!0),n=zt.current,n!==null){switch(n.tag){case 13:return Xt===null?Sr():n.alternate===null&&Be===0&&(Be=3),n.flags&=-257,n.flags|=65536,n.lanes=c,i===Oc?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([i]):t.add(i),wr(e,i,c)),!1;case 22:return n.flags|=65536,i===Oc?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([i])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([i]):n.add(i)),wr(e,i,c)),!1}throw Error(s(435,n.tag))}return wr(e,i,c),Sr(),!1}if(be)return t=zt.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=c,i!==vc&&(e=Error(s(422),{cause:i}),$i(Ct(e,n)))):(i!==vc&&(t=Error(s(423),{cause:i}),$i(Ct(t,n))),e=e.current.alternate,e.flags|=65536,c&=-c,e.lanes|=c,i=Ct(i,n),c=Wc(e.stateNode,i,c),Mc(e,c),Be!==4&&(Be=2)),!1;var f=Error(s(520),{cause:i});if(f=Ct(f,n),ca===null?ca=[f]:ca.push(f),Be!==4&&(Be=2),t===null)return!0;i=Ct(i,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=c&-c,n.lanes|=e,e=Wc(n.stateNode,i,e),Mc(n,e),!1;case 1:if(t=n.type,f=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||f!==null&&typeof f.componentDidCatch=="function"&&(Ln===null||!Ln.has(f))))return n.flags|=65536,c&=-c,n.lanes|=c,c=Wh(c),Ih(c,e,n,i),Mc(n,c),!1}n=n.return}while(n!==null);return!1}var Fh=Error(s(461)),Je=!1;function Fe(e,t,n,i){t.child=e===null?Yh(t,null,n,i):Vl(t,e.child,n,i)}function Ph(e,t,n,i,c){n=n.render;var f=t.ref;if("ref"in i){var g={};for(var y in i)y!=="ref"&&(g[y]=i[y])}else g=i;return il(t),i=kc(e,t,n,g,f,c),y=Lc(),e!==null&&!Je?(zc(e,t,c),rn(e,t,c)):(be&&y&&pc(t),t.flags|=1,Fe(e,t,i,c),t.child)}function ed(e,t,n,i,c){if(e===null){var f=n.type;return typeof f=="function"&&!dc(f)&&f.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=f,td(e,t,f,i,c)):(e=es(n.type,null,i,t,t.mode,c),e.ref=t.ref,e.return=t,t.child=e)}if(f=e.child,!ir(e,c)){var g=f.memoizedProps;if(n=n.compare,n=n!==null?n:Bi,n(g,i)&&e.ref===t.ref)return rn(e,t,c)}return t.flags|=1,e=tn(f,i),e.ref=t.ref,e.return=t,t.child=e}function td(e,t,n,i,c){if(e!==null){var f=e.memoizedProps;if(Bi(f,i)&&e.ref===t.ref)if(Je=!1,t.pendingProps=i=f,ir(e,c))(e.flags&131072)!==0&&(Je=!0);else return t.lanes=e.lanes,rn(e,t,c)}return Ic(e,t,n,i,c)}function nd(e,t,n){var i=t.pendingProps,c=i.children,f=e!==null?e.memoizedState:null;if(i.mode==="hidden"){if((t.flags&128)!==0){if(i=f!==null?f.baseLanes|n:n,e!==null){for(c=t.child=e.child,f=0;c!==null;)f=f|c.lanes|c.childLanes,c=c.sibling;t.childLanes=f&~i}else t.childLanes=0,t.child=null;return ld(e,t,i,n)}if((n&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&as(t,f!==null?f.cachePool:null),f!==null?th(t,f):Dc(),Gh(t);else return t.lanes=t.childLanes=536870912,ld(e,t,f!==null?f.baseLanes|n:n,n)}else f!==null?(as(t,f.cachePool),th(t,f),Mn(),t.memoizedState=null):(e!==null&&as(t,null),Dc(),Mn());return Fe(e,t,c,n),t.child}function ld(e,t,n,i){var c=Ac();return c=c===null?null:{parent:Ve._currentValue,pool:c},t.memoizedState={baseLanes:n,cachePool:c},e!==null&&as(t,null),Dc(),Gh(t),e!==null&&Yi(e,t,i,!0),null}function ws(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof n!="function"&&typeof n!="object")throw Error(s(284));(e===null||e.ref!==n)&&(t.flags|=4194816)}}function Ic(e,t,n,i,c){return il(t),n=kc(e,t,n,i,void 0,c),i=Lc(),e!==null&&!Je?(zc(e,t,c),rn(e,t,c)):(be&&i&&pc(t),t.flags|=1,Fe(e,t,n,c),t.child)}function id(e,t,n,i,c,f){return il(t),t.updateQueue=null,n=lh(t,i,n,c),nh(e),i=Lc(),e!==null&&!Je?(zc(e,t,f),rn(e,t,f)):(be&&i&&pc(t),t.flags|=1,Fe(e,t,n,f),t.child)}function ad(e,t,n,i,c){if(il(t),t.stateNode===null){var f=zl,g=n.contextType;typeof g=="object"&&g!==null&&(f=lt(g)),f=new n(i,f),t.memoizedState=f.state!==null&&f.state!==void 0?f.state:null,f.updater=Jc,t.stateNode=f,f._reactInternals=t,f=t.stateNode,f.props=i,f.state=t.memoizedState,f.refs={},Nc(t),g=n.contextType,f.context=typeof g=="object"&&g!==null?lt(g):zl,f.state=t.memoizedState,g=n.getDerivedStateFromProps,typeof g=="function"&&(Zc(t,n,g,i),f.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof f.getSnapshotBeforeUpdate=="function"||typeof f.UNSAFE_componentWillMount!="function"&&typeof f.componentWillMount!="function"||(g=f.state,typeof f.componentWillMount=="function"&&f.componentWillMount(),typeof f.UNSAFE_componentWillMount=="function"&&f.UNSAFE_componentWillMount(),g!==f.state&&Jc.enqueueReplaceState(f,f.state,null),Ji(t,i,f,c),Zi(),f.state=t.memoizedState),typeof f.componentDidMount=="function"&&(t.flags|=4194308),i=!0}else if(e===null){f=t.stateNode;var y=t.memoizedProps,v=ul(n,y);f.props=v;var x=f.context,q=n.contextType;g=zl,typeof q=="object"&&q!==null&&(g=lt(q));var $=n.getDerivedStateFromProps;q=typeof $=="function"||typeof f.getSnapshotBeforeUpdate=="function",y=t.pendingProps!==y,q||typeof f.UNSAFE_componentWillReceiveProps!="function"&&typeof f.componentWillReceiveProps!="function"||(y||x!==g)&&Vh(t,f,i,g),En=!1;var C=t.memoizedState;f.state=C,Ji(t,i,f,c),Zi(),x=t.memoizedState,y||C!==x||En?(typeof $=="function"&&(Zc(t,n,$,i),x=t.memoizedState),(v=En||Kh(t,n,v,i,C,x,g))?(q||typeof f.UNSAFE_componentWillMount!="function"&&typeof f.componentWillMount!="function"||(typeof f.componentWillMount=="function"&&f.componentWillMount(),typeof f.UNSAFE_componentWillMount=="function"&&f.UNSAFE_componentWillMount()),typeof f.componentDidMount=="function"&&(t.flags|=4194308)):(typeof f.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=i,t.memoizedState=x),f.props=i,f.state=x,f.context=g,i=v):(typeof f.componentDidMount=="function"&&(t.flags|=4194308),i=!1)}else{f=t.stateNode,_c(e,t),g=t.memoizedProps,q=ul(n,g),f.props=q,$=t.pendingProps,C=f.context,x=n.contextType,v=zl,typeof x=="object"&&x!==null&&(v=lt(x)),y=n.getDerivedStateFromProps,(x=typeof y=="function"||typeof f.getSnapshotBeforeUpdate=="function")||typeof f.UNSAFE_componentWillReceiveProps!="function"&&typeof f.componentWillReceiveProps!="function"||(g!==$||C!==v)&&Vh(t,f,i,v),En=!1,C=t.memoizedState,f.state=C,Ji(t,i,f,c),Zi();var R=t.memoizedState;g!==$||C!==R||En||e!==null&&e.dependencies!==null&&ls(e.dependencies)?(typeof y=="function"&&(Zc(t,n,y,i),R=t.memoizedState),(q=En||Kh(t,n,q,i,C,R,v)||e!==null&&e.dependencies!==null&&ls(e.dependencies))?(x||typeof f.UNSAFE_componentWillUpdate!="function"&&typeof f.componentWillUpdate!="function"||(typeof f.componentWillUpdate=="function"&&f.componentWillUpdate(i,R,v),typeof f.UNSAFE_componentWillUpdate=="function"&&f.UNSAFE_componentWillUpdate(i,R,v)),typeof f.componentDidUpdate=="function"&&(t.flags|=4),typeof f.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof f.componentDidUpdate!="function"||g===e.memoizedProps&&C===e.memoizedState||(t.flags|=4),typeof f.getSnapshotBeforeUpdate!="function"||g===e.memoizedProps&&C===e.memoizedState||(t.flags|=1024),t.memoizedProps=i,t.memoizedState=R),f.props=i,f.state=R,f.context=v,i=q):(typeof f.componentDidUpdate!="function"||g===e.memoizedProps&&C===e.memoizedState||(t.flags|=4),typeof f.getSnapshotBeforeUpdate!="function"||g===e.memoizedProps&&C===e.memoizedState||(t.flags|=1024),i=!1)}return f=i,ws(e,t),i=(t.flags&128)!==0,f||i?(f=t.stateNode,n=i&&typeof n.getDerivedStateFromError!="function"?null:f.render(),t.flags|=1,e!==null&&i?(t.child=Vl(t,e.child,null,c),t.child=Vl(t,null,n,c)):Fe(e,t,n,c),t.memoizedState=f.state,e=t.child):e=rn(e,t,c),e}function sd(e,t,n,i){return Hi(),t.flags|=256,Fe(e,t,n,i),t.child}var Fc={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function Pc(e){return{baseLanes:e,cachePool:Xo()}}function er(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=Ut),e}function ud(e,t,n){var i=t.pendingProps,c=!1,f=(t.flags&128)!==0,g;if((g=f)||(g=e!==null&&e.memoizedState===null?!1:(Qe.current&2)!==0),g&&(c=!0,t.flags&=-129),g=(t.flags&32)!==0,t.flags&=-33,e===null){if(be){if(c?_n(t):Mn(),be){var y=Ue,v;if(v=y){e:{for(v=y,y=Qt;v.nodeType!==8;){if(!y){y=null;break e}if(v=Yt(v.nextSibling),v===null){y=null;break e}}y=v}y!==null?(t.memoizedState={dehydrated:y,treeContext:Pn!==null?{id:nn,overflow:ln}:null,retryLane:536870912,hydrationErrors:null},v=Tt(18,null,null,0),v.stateNode=y,v.return=t,t.child=v,ut=t,Ue=null,v=!0):v=!1}v||nl(t)}if(y=t.memoizedState,y!==null&&(y=y.dehydrated,y!==null))return Br(y)?t.lanes=32:t.lanes=536870912,null;cn(t)}return y=i.children,i=i.fallback,c?(Mn(),c=t.mode,y=Es({mode:"hidden",children:y},c),i=Fn(i,c,n,null),y.return=t,i.return=t,y.sibling=i,t.child=y,c=t.child,c.memoizedState=Pc(n),c.childLanes=er(e,g,n),t.memoizedState=Fc,i):(_n(t),tr(t,y))}if(v=e.memoizedState,v!==null&&(y=v.dehydrated,y!==null)){if(f)t.flags&256?(_n(t),t.flags&=-257,t=nr(e,t,n)):t.memoizedState!==null?(Mn(),t.child=e.child,t.flags|=128,t=null):(Mn(),c=i.fallback,y=t.mode,i=Es({mode:"visible",children:i.children},y),c=Fn(c,y,n,null),c.flags|=2,i.return=t,c.return=t,i.sibling=c,t.child=i,Vl(t,e.child,null,n),i=t.child,i.memoizedState=Pc(n),i.childLanes=er(e,g,n),t.memoizedState=Fc,t=c);else if(_n(t),Br(y)){if(g=y.nextSibling&&y.nextSibling.dataset,g)var x=g.dgst;g=x,i=Error(s(419)),i.stack="",i.digest=g,$i({value:i,source:null,stack:null}),t=nr(e,t,n)}else if(Je||Yi(e,t,n,!1),g=(n&e.childLanes)!==0,Je||g){if(g=Me,g!==null&&(i=n&-n,i=(i&42)!==0?1:qu(i),i=(i&(g.suspendedLanes|n))!==0?0:i,i!==0&&i!==v.retryLane))throw v.retryLane=i,Ll(e,i),Nt(g,e,i),Fh;y.data==="$?"||Sr(),t=nr(e,t,n)}else y.data==="$?"?(t.flags|=192,t.child=e.child,t=null):(e=v.treeContext,Ue=Yt(y.nextSibling),ut=t,be=!0,tl=null,Qt=!1,e!==null&&(kt[Lt++]=nn,kt[Lt++]=ln,kt[Lt++]=Pn,nn=e.id,ln=e.overflow,Pn=t),t=tr(t,i.children),t.flags|=4096);return t}return c?(Mn(),c=i.fallback,y=t.mode,v=e.child,x=v.sibling,i=tn(v,{mode:"hidden",children:i.children}),i.subtreeFlags=v.subtreeFlags&65011712,x!==null?c=tn(x,c):(c=Fn(c,y,n,null),c.flags|=2),c.return=t,i.return=t,i.sibling=c,t.child=i,i=c,c=t.child,y=e.child.memoizedState,y===null?y=Pc(n):(v=y.cachePool,v!==null?(x=Ve._currentValue,v=v.parent!==x?{parent:x,pool:x}:v):v=Xo(),y={baseLanes:y.baseLanes|n,cachePool:v}),c.memoizedState=y,c.childLanes=er(e,g,n),t.memoizedState=Fc,i):(_n(t),n=e.child,e=n.sibling,n=tn(n,{mode:"visible",children:i.children}),n.return=t,n.sibling=null,e!==null&&(g=t.deletions,g===null?(t.deletions=[e],t.flags|=16):g.push(e)),t.child=n,t.memoizedState=null,n)}function tr(e,t){return t=Es({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function Es(e,t){return e=Tt(22,e,null,t),e.lanes=0,e.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},e}function nr(e,t,n){return Vl(t,e.child,null,n),e=tr(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function cd(e,t,n){e.lanes|=t;var i=e.alternate;i!==null&&(i.lanes|=t),Sc(e.return,t,n)}function lr(e,t,n,i,c){var f=e.memoizedState;f===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:i,tail:n,tailMode:c}:(f.isBackwards=t,f.rendering=null,f.renderingStartTime=0,f.last=i,f.tail=n,f.tailMode=c)}function rd(e,t,n){var i=t.pendingProps,c=i.revealOrder,f=i.tail;if(Fe(e,t,i.children,n),i=Qe.current,(i&2)!==0)i=i&1|2,t.flags|=128;else{if(e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&cd(e,n,t);else if(e.tag===19)cd(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}i&=1}switch(J(Qe,i),c){case"forwards":for(n=t.child,c=null;n!==null;)e=n.alternate,e!==null&&bs(e)===null&&(c=n),n=n.sibling;n=c,n===null?(c=t.child,t.child=null):(c=n.sibling,n.sibling=null),lr(t,!1,c,n,f);break;case"backwards":for(n=null,c=t.child,t.child=null;c!==null;){if(e=c.alternate,e!==null&&bs(e)===null){t.child=c;break}e=c.sibling,c.sibling=n,n=c,c=e}lr(t,!0,n,null,f);break;case"together":lr(t,!1,null,null,void 0);break;default:t.memoizedState=null}return t.child}function rn(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),kn|=t.lanes,(n&t.childLanes)===0)if(e!==null){if(Yi(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(s(153));if(t.child!==null){for(e=t.child,n=tn(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=tn(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function ir(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&ls(e)))}function Wy(e,t,n){switch(t.tag){case 3:De(t,t.stateNode.containerInfo),wn(t,Ve,e.memoizedState.cache),Hi();break;case 27:case 5:ku(t);break;case 4:De(t,t.stateNode.containerInfo);break;case 10:wn(t,t.type,t.memoizedProps.value);break;case 13:var i=t.memoizedState;if(i!==null)return i.dehydrated!==null?(_n(t),t.flags|=128,null):(n&t.child.childLanes)!==0?ud(e,t,n):(_n(t),e=rn(e,t,n),e!==null?e.sibling:null);_n(t);break;case 19:var c=(e.flags&128)!==0;if(i=(n&t.childLanes)!==0,i||(Yi(e,t,n,!1),i=(n&t.childLanes)!==0),c){if(i)return rd(e,t,n);t.flags|=128}if(c=t.memoizedState,c!==null&&(c.rendering=null,c.tail=null,c.lastEffect=null),J(Qe,Qe.current),i)break;return null;case 22:case 23:return t.lanes=0,nd(e,t,n);case 24:wn(t,Ve,e.memoizedState.cache)}return rn(e,t,n)}function fd(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)Je=!0;else{if(!ir(e,n)&&(t.flags&128)===0)return Je=!1,Wy(e,t,n);Je=(e.flags&131072)!==0}else Je=!1,be&&(t.flags&1048576)!==0&&Ho(t,ns,t.index);switch(t.lanes=0,t.tag){case 16:e:{e=t.pendingProps;var i=t.elementType,c=i._init;if(i=c(i._payload),t.type=i,typeof i=="function")dc(i)?(e=ul(i,e),t.tag=1,t=ad(null,t,i,e,n)):(t.tag=0,t=Ic(null,t,i,e,n));else{if(i!=null){if(c=i.$$typeof,c===Q){t.tag=11,t=Ph(null,t,i,e,n);break e}else if(c===ee){t.tag=14,t=ed(null,t,i,e,n);break e}}throw t=xe(i)||i,Error(s(306,t,""))}}return t;case 0:return Ic(e,t,t.type,t.pendingProps,n);case 1:return i=t.type,c=ul(i,t.pendingProps),ad(e,t,i,c,n);case 3:e:{if(De(t,t.stateNode.containerInfo),e===null)throw Error(s(387));i=t.pendingProps;var f=t.memoizedState;c=f.element,_c(e,t),Ji(t,i,null,n);var g=t.memoizedState;if(i=g.cache,wn(t,Ve,i),i!==f.cache&&Tc(t,[Ve],n,!0),Zi(),i=g.element,f.isDehydrated)if(f={element:i,isDehydrated:!1,cache:g.cache},t.updateQueue.baseState=f,t.memoizedState=f,t.flags&256){t=sd(e,t,i,n);break e}else if(i!==c){c=Ct(Error(s(424)),t),$i(c),t=sd(e,t,i,n);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(Ue=Yt(e.firstChild),ut=t,be=!0,tl=null,Qt=!0,n=Yh(t,null,i,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling}else{if(Hi(),i===c){t=rn(e,t,n);break e}Fe(e,t,i,n)}t=t.child}return t;case 26:return ws(e,t),e===null?(n=gg(t.type,null,t.pendingProps,null))?t.memoizedState=n:be||(n=t.type,e=t.pendingProps,i=Bs(ce.current).createElement(n),i[nt]=t,i[rt]=e,et(i,n,e),Ze(i),t.stateNode=i):t.memoizedState=gg(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return ku(t),e===null&&be&&(i=t.stateNode=og(t.type,t.pendingProps,ce.current),ut=t,Qt=!0,c=Ue,Bn(t.type)?(qr=c,Ue=Yt(i.firstChild)):Ue=c),Fe(e,t,t.pendingProps.children,n),ws(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&be&&((c=i=Ue)&&(i=A0(i,t.type,t.pendingProps,Qt),i!==null?(t.stateNode=i,ut=t,Ue=Yt(i.firstChild),Qt=!1,c=!0):c=!1),c||nl(t)),ku(t),c=t.type,f=t.pendingProps,g=e!==null?e.memoizedProps:null,i=f.children,Lr(c,f)?i=null:g!==null&&Lr(c,g)&&(t.flags|=32),t.memoizedState!==null&&(c=kc(e,t,Yy,null,null,n),ya._currentValue=c),ws(e,t),Fe(e,t,i,n),t.child;case 6:return e===null&&be&&((e=n=Ue)&&(n=O0(n,t.pendingProps,Qt),n!==null?(t.stateNode=n,ut=t,Ue=null,e=!0):e=!1),e||nl(t)),null;case 13:return ud(e,t,n);case 4:return De(t,t.stateNode.containerInfo),i=t.pendingProps,e===null?t.child=Vl(t,null,i,n):Fe(e,t,i,n),t.child;case 11:return Ph(e,t,t.type,t.pendingProps,n);case 7:return Fe(e,t,t.pendingProps,n),t.child;case 8:return Fe(e,t,t.pendingProps.children,n),t.child;case 12:return Fe(e,t,t.pendingProps.children,n),t.child;case 10:return i=t.pendingProps,wn(t,t.type,i.value),Fe(e,t,i.children,n),t.child;case 9:return c=t.type._context,i=t.pendingProps.children,il(t),c=lt(c),i=i(c),t.flags|=1,Fe(e,t,i,n),t.child;case 14:return ed(e,t,t.type,t.pendingProps,n);case 15:return td(e,t,t.type,t.pendingProps,n);case 19:return rd(e,t,n);case 31:return i=t.pendingProps,n=t.mode,i={mode:i.mode,children:i.children},e===null?(n=Es(i,n),n.ref=t.ref,t.child=n,n.return=t,t=n):(n=tn(e.child,i),n.ref=t.ref,t.child=n,n.return=t,t=n),t;case 22:return nd(e,t,n);case 24:return il(t),i=lt(Ve),e===null?(c=Ac(),c===null&&(c=Me,f=wc(),c.pooledCache=f,f.refCount++,f!==null&&(c.pooledCacheLanes|=n),c=f),t.memoizedState={parent:i,cache:c},Nc(t),wn(t,Ve,c)):((e.lanes&n)!==0&&(_c(e,t),Ji(t,null,null,n),Zi()),c=e.memoizedState,f=t.memoizedState,c.parent!==i?(c={parent:i,cache:i},t.memoizedState=c,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=c),wn(t,Ve,i)):(i=f.cache,wn(t,Ve,i),i!==c.cache&&Tc(t,[Ve],n,!0))),Fe(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(s(156,t.tag))}function fn(e){e.flags|=4}function od(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!bg(t)){if(t=zt.current,t!==null&&((pe&4194048)===pe?Xt!==null:(pe&62914560)!==pe&&(pe&536870912)===0||t!==Xt))throw Qi=Oc,Zo;e.flags|=8192}}function As(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?Yf():536870912,e.lanes|=t,Jl|=t)}function na(e,t){if(!be)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var i=null;n!==null;)n.alternate!==null&&(i=n),n=n.sibling;i===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:i.sibling=null}}function ke(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,i=0;if(t)for(var c=e.child;c!==null;)n|=c.lanes|c.childLanes,i|=c.subtreeFlags&65011712,i|=c.flags&65011712,c.return=e,c=c.sibling;else for(c=e.child;c!==null;)n|=c.lanes|c.childLanes,i|=c.subtreeFlags,i|=c.flags,c.return=e,c=c.sibling;return e.subtreeFlags|=i,e.childLanes=n,t}function Iy(e,t,n){var i=t.pendingProps;switch(yc(t),t.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return ke(t),null;case 1:return ke(t),null;case 3:return n=t.stateNode,i=null,e!==null&&(i=e.memoizedState.cache),t.memoizedState.cache!==i&&(t.flags|=2048),sn(Ve),vn(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(ji(t)?fn(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,Go())),ke(t),null;case 26:return n=t.memoizedState,e===null?(fn(t),n!==null?(ke(t),od(t,n)):(ke(t),t.flags&=-16777217)):n?n!==e.memoizedState?(fn(t),ke(t),od(t,n)):(ke(t),t.flags&=-16777217):(e.memoizedProps!==i&&fn(t),ke(t),t.flags&=-16777217),null;case 27:za(t),n=ce.current;var c=t.type;if(e!==null&&t.stateNode!=null)e.memoizedProps!==i&&fn(t);else{if(!i){if(t.stateNode===null)throw Error(s(166));return ke(t),null}e=ne.current,ji(t)?$o(t):(e=og(c,i,n),t.stateNode=e,fn(t))}return ke(t),null;case 5:if(za(t),n=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==i&&fn(t);else{if(!i){if(t.stateNode===null)throw Error(s(166));return ke(t),null}if(e=ne.current,ji(t))$o(t);else{switch(c=Bs(ce.current),e){case 1:e=c.createElementNS("http://www.w3.org/2000/svg",n);break;case 2:e=c.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;default:switch(n){case"svg":e=c.createElementNS("http://www.w3.org/2000/svg",n);break;case"math":e=c.createElementNS("http://www.w3.org/1998/Math/MathML",n);break;case"script":e=c.createElement("div"),e.innerHTML="<script><\/script>",e=e.removeChild(e.firstChild);break;case"select":e=typeof i.is=="string"?c.createElement("select",{is:i.is}):c.createElement("select"),i.multiple?e.multiple=!0:i.size&&(e.size=i.size);break;default:e=typeof i.is=="string"?c.createElement(n,{is:i.is}):c.createElement(n)}}e[nt]=t,e[rt]=i;e:for(c=t.child;c!==null;){if(c.tag===5||c.tag===6)e.appendChild(c.stateNode);else if(c.tag!==4&&c.tag!==27&&c.child!==null){c.child.return=c,c=c.child;continue}if(c===t)break e;for(;c.sibling===null;){if(c.return===null||c.return===t)break e;c=c.return}c.sibling.return=c.return,c=c.sibling}t.stateNode=e;e:switch(et(e,n,i),n){case"button":case"input":case"select":case"textarea":e=!!i.autoFocus;break e;case"img":e=!0;break e;default:e=!1}e&&fn(t)}}return ke(t),t.flags&=-16777217,null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==i&&fn(t);else{if(typeof i!="string"&&t.stateNode===null)throw Error(s(166));if(e=ce.current,ji(t)){if(e=t.stateNode,n=t.memoizedProps,i=null,c=ut,c!==null)switch(c.tag){case 27:case 5:i=c.memoizedProps}e[nt]=t,e=!!(e.nodeValue===n||i!==null&&i.suppressHydrationWarning===!0||ig(e.nodeValue,n)),e||nl(t)}else e=Bs(e).createTextNode(i),e[nt]=t,t.stateNode=e}return ke(t),null;case 13:if(i=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(c=ji(t),i!==null&&i.dehydrated!==null){if(e===null){if(!c)throw Error(s(318));if(c=t.memoizedState,c=c!==null?c.dehydrated:null,!c)throw Error(s(317));c[nt]=t}else Hi(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;ke(t),c=!1}else c=Go(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=c),c=!0;if(!c)return t.flags&256?(cn(t),t):(cn(t),null)}if(cn(t),(t.flags&128)!==0)return t.lanes=n,t;if(n=i!==null,e=e!==null&&e.memoizedState!==null,n){i=t.child,c=null,i.alternate!==null&&i.alternate.memoizedState!==null&&i.alternate.memoizedState.cachePool!==null&&(c=i.alternate.memoizedState.cachePool.pool);var f=null;i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(f=i.memoizedState.cachePool.pool),f!==c&&(i.flags|=2048)}return n!==e&&n&&(t.child.flags|=8192),As(t,t.updateQueue),ke(t),null;case 4:return vn(),e===null&&xr(t.stateNode.containerInfo),ke(t),null;case 10:return sn(t.type),ke(t),null;case 19:if(W(Qe),c=t.memoizedState,c===null)return ke(t),null;if(i=(t.flags&128)!==0,f=c.rendering,f===null)if(i)na(c,!1);else{if(Be!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(f=bs(e),f!==null){for(t.flags|=128,na(c,!1),e=f.updateQueue,t.updateQueue=e,As(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)jo(n,e),n=n.sibling;return J(Qe,Qe.current&1|2),t.child}e=e.sibling}c.tail!==null&&Vt()>_s&&(t.flags|=128,i=!0,na(c,!1),t.lanes=4194304)}else{if(!i)if(e=bs(f),e!==null){if(t.flags|=128,i=!0,e=e.updateQueue,t.updateQueue=e,As(t,e),na(c,!0),c.tail===null&&c.tailMode==="hidden"&&!f.alternate&&!be)return ke(t),null}else 2*Vt()-c.renderingStartTime>_s&&n!==536870912&&(t.flags|=128,i=!0,na(c,!1),t.lanes=4194304);c.isBackwards?(f.sibling=t.child,t.child=f):(e=c.last,e!==null?e.sibling=f:t.child=f,c.last=f)}return c.tail!==null?(t=c.tail,c.rendering=t,c.tail=t.sibling,c.renderingStartTime=Vt(),t.sibling=null,e=Qe.current,J(Qe,i?e&1|2:e&1),t):(ke(t),null);case 22:case 23:return cn(t),Cc(),i=t.memoizedState!==null,e!==null?e.memoizedState!==null!==i&&(t.flags|=8192):i&&(t.flags|=8192),i?(n&536870912)!==0&&(t.flags&128)===0&&(ke(t),t.subtreeFlags&6&&(t.flags|=8192)):ke(t),n=t.updateQueue,n!==null&&As(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),i=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(i=t.memoizedState.cachePool.pool),i!==n&&(t.flags|=2048),e!==null&&W(al),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),sn(Ve),ke(t),null;case 25:return null;case 30:return null}throw Error(s(156,t.tag))}function Fy(e,t){switch(yc(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return sn(Ve),vn(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return za(t),null;case 13:if(cn(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(s(340));Hi()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return W(Qe),null;case 4:return vn(),null;case 10:return sn(t.type),null;case 22:case 23:return cn(t),Cc(),e!==null&&W(al),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return sn(Ve),null;case 25:return null;default:return null}}function hd(e,t){switch(yc(t),t.tag){case 3:sn(Ve),vn();break;case 26:case 27:case 5:za(t);break;case 4:vn();break;case 13:cn(t);break;case 19:W(Qe);break;case 10:sn(t.type);break;case 22:case 23:cn(t),Cc(),e!==null&&W(al);break;case 24:sn(Ve)}}function la(e,t){try{var n=t.updateQueue,i=n!==null?n.lastEffect:null;if(i!==null){var c=i.next;n=c;do{if((n.tag&e)===e){i=void 0;var f=n.create,g=n.inst;i=f(),g.destroy=i}n=n.next}while(n!==c)}}catch(y){Ne(t,t.return,y)}}function xn(e,t,n){try{var i=t.updateQueue,c=i!==null?i.lastEffect:null;if(c!==null){var f=c.next;i=f;do{if((i.tag&e)===e){var g=i.inst,y=g.destroy;if(y!==void 0){g.destroy=void 0,c=t;var v=n,x=y;try{x()}catch(q){Ne(c,v,q)}}}i=i.next}while(i!==f)}}catch(q){Ne(t,t.return,q)}}function dd(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{eh(t,n)}catch(i){Ne(e,e.return,i)}}}function gd(e,t,n){n.props=ul(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(i){Ne(e,t,i)}}function ia(e,t){try{var n=e.ref;if(n!==null){switch(e.tag){case 26:case 27:case 5:var i=e.stateNode;break;case 30:i=e.stateNode;break;default:i=e.stateNode}typeof n=="function"?e.refCleanup=n(i):n.current=i}}catch(c){Ne(e,t,c)}}function Zt(e,t){var n=e.ref,i=e.refCleanup;if(n!==null)if(typeof i=="function")try{i()}catch(c){Ne(e,t,c)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(c){Ne(e,t,c)}else n.current=null}function md(e){var t=e.type,n=e.memoizedProps,i=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&i.focus();break e;case"img":n.src?i.src=n.src:n.srcSet&&(i.srcset=n.srcSet)}}catch(c){Ne(e,e.return,c)}}function ar(e,t,n){try{var i=e.stateNode;b0(i,e.type,n,t),i[rt]=t}catch(c){Ne(e,e.return,c)}}function pd(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&Bn(e.type)||e.tag===4}function sr(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||pd(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&Bn(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function ur(e,t,n){var i=e.tag;if(i===5||i===6)e=e.stateNode,t?(n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n).insertBefore(e,t):(t=n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n,t.appendChild(e),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=Us));else if(i!==4&&(i===27&&Bn(e.type)&&(n=e.stateNode,t=null),e=e.child,e!==null))for(ur(e,t,n),e=e.sibling;e!==null;)ur(e,t,n),e=e.sibling}function Os(e,t,n){var i=e.tag;if(i===5||i===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(i!==4&&(i===27&&Bn(e.type)&&(n=e.stateNode),e=e.child,e!==null))for(Os(e,t,n),e=e.sibling;e!==null;)Os(e,t,n),e=e.sibling}function yd(e){var t=e.stateNode,n=e.memoizedProps;try{for(var i=e.type,c=t.attributes;c.length;)t.removeAttributeNode(c[0]);et(t,i,n),t[nt]=e,t[rt]=n}catch(f){Ne(e,e.return,f)}}var on=!1,He=!1,cr=!1,vd=typeof WeakSet=="function"?WeakSet:Set,We=null;function Py(e,t){if(e=e.containerInfo,Rr=Gs,e=xo(e),sc(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var i=n.getSelection&&n.getSelection();if(i&&i.rangeCount!==0){n=i.anchorNode;var c=i.anchorOffset,f=i.focusNode;i=i.focusOffset;try{n.nodeType,f.nodeType}catch{n=null;break e}var g=0,y=-1,v=-1,x=0,q=0,$=e,C=null;t:for(;;){for(var R;$!==n||c!==0&&$.nodeType!==3||(y=g+c),$!==f||i!==0&&$.nodeType!==3||(v=g+i),$.nodeType===3&&(g+=$.nodeValue.length),(R=$.firstChild)!==null;)C=$,$=R;for(;;){if($===e)break t;if(C===n&&++x===c&&(y=g),C===f&&++q===i&&(v=g),(R=$.nextSibling)!==null)break;$=C,C=$.parentNode}$=R}n=y===-1||v===-1?null:{start:y,end:v}}else n=null}n=n||{start:0,end:0}}else n=null;for(kr={focusedElem:e,selectionRange:n},Gs=!1,We=t;We!==null;)if(t=We,e=t.child,(t.subtreeFlags&1024)!==0&&e!==null)e.return=t,We=e;else for(;We!==null;){switch(t=We,f=t.alternate,e=t.flags,t.tag){case 0:break;case 11:case 15:break;case 1:if((e&1024)!==0&&f!==null){e=void 0,n=t,c=f.memoizedProps,f=f.memoizedState,i=n.stateNode;try{var ue=ul(n.type,c,n.elementType===n.type);e=i.getSnapshotBeforeUpdate(ue,f),i.__reactInternalSnapshotBeforeUpdate=e}catch(le){Ne(n,n.return,le)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)Ur(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":Ur(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(s(163))}if(e=t.sibling,e!==null){e.return=t.return,We=e;break}We=t.return}}function bd(e,t,n){var i=n.flags;switch(n.tag){case 0:case 11:case 15:Dn(e,n),i&4&&la(5,n);break;case 1:if(Dn(e,n),i&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(g){Ne(n,n.return,g)}else{var c=ul(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(c,t,e.__reactInternalSnapshotBeforeUpdate)}catch(g){Ne(n,n.return,g)}}i&64&&dd(n),i&512&&ia(n,n.return);break;case 3:if(Dn(e,n),i&64&&(e=n.updateQueue,e!==null)){if(t=null,n.child!==null)switch(n.child.tag){case 27:case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}try{eh(e,t)}catch(g){Ne(n,n.return,g)}}break;case 27:t===null&&i&4&&yd(n);case 26:case 5:Dn(e,n),t===null&&i&4&&md(n),i&512&&ia(n,n.return);break;case 12:Dn(e,n);break;case 13:Dn(e,n),i&4&&wd(e,n),i&64&&(e=n.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(n=c0.bind(null,n),N0(e,n))));break;case 22:if(i=n.memoizedState!==null||on,!i){t=t!==null&&t.memoizedState!==null||He,c=on;var f=He;on=i,(He=t)&&!f?Cn(e,n,(n.subtreeFlags&8772)!==0):Dn(e,n),on=c,He=f}break;case 30:break;default:Dn(e,n)}}function Sd(e){var t=e.alternate;t!==null&&(e.alternate=null,Sd(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&$u(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var Ce=null,ht=!1;function hn(e,t,n){for(n=n.child;n!==null;)Td(e,t,n),n=n.sibling}function Td(e,t,n){if(vt&&typeof vt.onCommitFiberUnmount=="function")try{vt.onCommitFiberUnmount(Oi,n)}catch{}switch(n.tag){case 26:He||Zt(n,t),hn(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:He||Zt(n,t);var i=Ce,c=ht;Bn(n.type)&&(Ce=n.stateNode,ht=!1),hn(e,t,n),da(n.stateNode),Ce=i,ht=c;break;case 5:He||Zt(n,t);case 6:if(i=Ce,c=ht,Ce=null,hn(e,t,n),Ce=i,ht=c,Ce!==null)if(ht)try{(Ce.nodeType===9?Ce.body:Ce.nodeName==="HTML"?Ce.ownerDocument.body:Ce).removeChild(n.stateNode)}catch(f){Ne(n,t,f)}else try{Ce.removeChild(n.stateNode)}catch(f){Ne(n,t,f)}break;case 18:Ce!==null&&(ht?(e=Ce,rg(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,n.stateNode),Ta(e)):rg(Ce,n.stateNode));break;case 4:i=Ce,c=ht,Ce=n.stateNode.containerInfo,ht=!0,hn(e,t,n),Ce=i,ht=c;break;case 0:case 11:case 14:case 15:He||xn(2,n,t),He||xn(4,n,t),hn(e,t,n);break;case 1:He||(Zt(n,t),i=n.stateNode,typeof i.componentWillUnmount=="function"&&gd(n,t,i)),hn(e,t,n);break;case 21:hn(e,t,n);break;case 22:He=(i=He)||n.memoizedState!==null,hn(e,t,n),He=i;break;default:hn(e,t,n)}}function wd(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{Ta(e)}catch(n){Ne(t,t.return,n)}}function e0(e){switch(e.tag){case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new vd),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new vd),t;default:throw Error(s(435,e.tag))}}function rr(e,t){var n=e0(e);t.forEach(function(i){var c=r0.bind(null,e,i);n.has(i)||(n.add(i),i.then(c,c))})}function wt(e,t){var n=t.deletions;if(n!==null)for(var i=0;i<n.length;i++){var c=n[i],f=e,g=t,y=g;e:for(;y!==null;){switch(y.tag){case 27:if(Bn(y.type)){Ce=y.stateNode,ht=!1;break e}break;case 5:Ce=y.stateNode,ht=!1;break e;case 3:case 4:Ce=y.stateNode.containerInfo,ht=!0;break e}y=y.return}if(Ce===null)throw Error(s(160));Td(f,g,c),Ce=null,ht=!1,f=c.alternate,f!==null&&(f.return=null),c.return=null}if(t.subtreeFlags&13878)for(t=t.child;t!==null;)Ed(t,e),t=t.sibling}var $t=null;function Ed(e,t){var n=e.alternate,i=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:wt(t,e),Et(e),i&4&&(xn(3,e,e.return),la(3,e),xn(5,e,e.return));break;case 1:wt(t,e),Et(e),i&512&&(He||n===null||Zt(n,n.return)),i&64&&on&&(e=e.updateQueue,e!==null&&(i=e.callbacks,i!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?i:n.concat(i))));break;case 26:var c=$t;if(wt(t,e),Et(e),i&512&&(He||n===null||Zt(n,n.return)),i&4){var f=n!==null?n.memoizedState:null;if(i=e.memoizedState,n===null)if(i===null)if(e.stateNode===null){e:{i=e.type,n=e.memoizedProps,c=c.ownerDocument||c;t:switch(i){case"title":f=c.getElementsByTagName("title")[0],(!f||f[Mi]||f[nt]||f.namespaceURI==="http://www.w3.org/2000/svg"||f.hasAttribute("itemprop"))&&(f=c.createElement(i),c.head.insertBefore(f,c.querySelector("head > title"))),et(f,i,n),f[nt]=e,Ze(f),i=f;break e;case"link":var g=yg("link","href",c).get(i+(n.href||""));if(g){for(var y=0;y<g.length;y++)if(f=g[y],f.getAttribute("href")===(n.href==null||n.href===""?null:n.href)&&f.getAttribute("rel")===(n.rel==null?null:n.rel)&&f.getAttribute("title")===(n.title==null?null:n.title)&&f.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){g.splice(y,1);break t}}f=c.createElement(i),et(f,i,n),c.head.appendChild(f);break;case"meta":if(g=yg("meta","content",c).get(i+(n.content||""))){for(y=0;y<g.length;y++)if(f=g[y],f.getAttribute("content")===(n.content==null?null:""+n.content)&&f.getAttribute("name")===(n.name==null?null:n.name)&&f.getAttribute("property")===(n.property==null?null:n.property)&&f.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&f.getAttribute("charset")===(n.charSet==null?null:n.charSet)){g.splice(y,1);break t}}f=c.createElement(i),et(f,i,n),c.head.appendChild(f);break;default:throw Error(s(468,i))}f[nt]=e,Ze(f),i=f}e.stateNode=i}else vg(c,e.type,e.stateNode);else e.stateNode=pg(c,i,e.memoizedProps);else f!==i?(f===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):f.count--,i===null?vg(c,e.type,e.stateNode):pg(c,i,e.memoizedProps)):i===null&&e.stateNode!==null&&ar(e,e.memoizedProps,n.memoizedProps)}break;case 27:wt(t,e),Et(e),i&512&&(He||n===null||Zt(n,n.return)),n!==null&&i&4&&ar(e,e.memoizedProps,n.memoizedProps);break;case 5:if(wt(t,e),Et(e),i&512&&(He||n===null||Zt(n,n.return)),e.flags&32){c=e.stateNode;try{_l(c,"")}catch(R){Ne(e,e.return,R)}}i&4&&e.stateNode!=null&&(c=e.memoizedProps,ar(e,c,n!==null?n.memoizedProps:c)),i&1024&&(cr=!0);break;case 6:if(wt(t,e),Et(e),i&4){if(e.stateNode===null)throw Error(s(162));i=e.memoizedProps,n=e.stateNode;try{n.nodeValue=i}catch(R){Ne(e,e.return,R)}}break;case 3:if(Hs=null,c=$t,$t=qs(t.containerInfo),wt(t,e),$t=c,Et(e),i&4&&n!==null&&n.memoizedState.isDehydrated)try{Ta(t.containerInfo)}catch(R){Ne(e,e.return,R)}cr&&(cr=!1,Ad(e));break;case 4:i=$t,$t=qs(e.stateNode.containerInfo),wt(t,e),Et(e),$t=i;break;case 12:wt(t,e),Et(e);break;case 13:wt(t,e),Et(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(mr=Vt()),i&4&&(i=e.updateQueue,i!==null&&(e.updateQueue=null,rr(e,i)));break;case 22:c=e.memoizedState!==null;var v=n!==null&&n.memoizedState!==null,x=on,q=He;if(on=x||c,He=q||v,wt(t,e),He=q,on=x,Et(e),i&8192)e:for(t=e.stateNode,t._visibility=c?t._visibility&-2:t._visibility|1,c&&(n===null||v||on||He||cl(e)),n=null,t=e;;){if(t.tag===5||t.tag===26){if(n===null){v=n=t;try{if(f=v.stateNode,c)g=f.style,typeof g.setProperty=="function"?g.setProperty("display","none","important"):g.display="none";else{y=v.stateNode;var $=v.memoizedProps.style,C=$!=null&&$.hasOwnProperty("display")?$.display:null;y.style.display=C==null||typeof C=="boolean"?"":(""+C).trim()}}catch(R){Ne(v,v.return,R)}}}else if(t.tag===6){if(n===null){v=t;try{v.stateNode.nodeValue=c?"":v.memoizedProps}catch(R){Ne(v,v.return,R)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}i&4&&(i=e.updateQueue,i!==null&&(n=i.retryQueue,n!==null&&(i.retryQueue=null,rr(e,n))));break;case 19:wt(t,e),Et(e),i&4&&(i=e.updateQueue,i!==null&&(e.updateQueue=null,rr(e,i)));break;case 30:break;case 21:break;default:wt(t,e),Et(e)}}function Et(e){var t=e.flags;if(t&2){try{for(var n,i=e.return;i!==null;){if(pd(i)){n=i;break}i=i.return}if(n==null)throw Error(s(160));switch(n.tag){case 27:var c=n.stateNode,f=sr(e);Os(e,f,c);break;case 5:var g=n.stateNode;n.flags&32&&(_l(g,""),n.flags&=-33);var y=sr(e);Os(e,y,g);break;case 3:case 4:var v=n.stateNode.containerInfo,x=sr(e);ur(e,x,v);break;default:throw Error(s(161))}}catch(q){Ne(e,e.return,q)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function Ad(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;Ad(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function Dn(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)bd(e,t.alternate,t),t=t.sibling}function cl(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:xn(4,t,t.return),cl(t);break;case 1:Zt(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&gd(t,t.return,n),cl(t);break;case 27:da(t.stateNode);case 26:case 5:Zt(t,t.return),cl(t);break;case 22:t.memoizedState===null&&cl(t);break;case 30:cl(t);break;default:cl(t)}e=e.sibling}}function Cn(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var i=t.alternate,c=e,f=t,g=f.flags;switch(f.tag){case 0:case 11:case 15:Cn(c,f,n),la(4,f);break;case 1:if(Cn(c,f,n),i=f,c=i.stateNode,typeof c.componentDidMount=="function")try{c.componentDidMount()}catch(x){Ne(i,i.return,x)}if(i=f,c=i.updateQueue,c!==null){var y=i.stateNode;try{var v=c.shared.hiddenCallbacks;if(v!==null)for(c.shared.hiddenCallbacks=null,c=0;c<v.length;c++)Po(v[c],y)}catch(x){Ne(i,i.return,x)}}n&&g&64&&dd(f),ia(f,f.return);break;case 27:yd(f);case 26:case 5:Cn(c,f,n),n&&i===null&&g&4&&md(f),ia(f,f.return);break;case 12:Cn(c,f,n);break;case 13:Cn(c,f,n),n&&g&4&&wd(c,f);break;case 22:f.memoizedState===null&&Cn(c,f,n),ia(f,f.return);break;case 30:break;default:Cn(c,f,n)}t=t.sibling}}function fr(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&Gi(n))}function or(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&Gi(e))}function Jt(e,t,n,i){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)Od(e,t,n,i),t=t.sibling}function Od(e,t,n,i){var c=t.flags;switch(t.tag){case 0:case 11:case 15:Jt(e,t,n,i),c&2048&&la(9,t);break;case 1:Jt(e,t,n,i);break;case 3:Jt(e,t,n,i),c&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&Gi(e)));break;case 12:if(c&2048){Jt(e,t,n,i),e=t.stateNode;try{var f=t.memoizedProps,g=f.id,y=f.onPostCommit;typeof y=="function"&&y(g,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(v){Ne(t,t.return,v)}}else Jt(e,t,n,i);break;case 13:Jt(e,t,n,i);break;case 23:break;case 22:f=t.stateNode,g=t.alternate,t.memoizedState!==null?f._visibility&2?Jt(e,t,n,i):aa(e,t):f._visibility&2?Jt(e,t,n,i):(f._visibility|=2,Ql(e,t,n,i,(t.subtreeFlags&10256)!==0)),c&2048&&fr(g,t);break;case 24:Jt(e,t,n,i),c&2048&&or(t.alternate,t);break;default:Jt(e,t,n,i)}}function Ql(e,t,n,i,c){for(c=c&&(t.subtreeFlags&10256)!==0,t=t.child;t!==null;){var f=e,g=t,y=n,v=i,x=g.flags;switch(g.tag){case 0:case 11:case 15:Ql(f,g,y,v,c),la(8,g);break;case 23:break;case 22:var q=g.stateNode;g.memoizedState!==null?q._visibility&2?Ql(f,g,y,v,c):aa(f,g):(q._visibility|=2,Ql(f,g,y,v,c)),c&&x&2048&&fr(g.alternate,g);break;case 24:Ql(f,g,y,v,c),c&&x&2048&&or(g.alternate,g);break;default:Ql(f,g,y,v,c)}t=t.sibling}}function aa(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,i=t,c=i.flags;switch(i.tag){case 22:aa(n,i),c&2048&&fr(i.alternate,i);break;case 24:aa(n,i),c&2048&&or(i.alternate,i);break;default:aa(n,i)}t=t.sibling}}var sa=8192;function Xl(e){if(e.subtreeFlags&sa)for(e=e.child;e!==null;)Nd(e),e=e.sibling}function Nd(e){switch(e.tag){case 26:Xl(e),e.flags&sa&&e.memoizedState!==null&&j0($t,e.memoizedState,e.memoizedProps);break;case 5:Xl(e);break;case 3:case 4:var t=$t;$t=qs(e.stateNode.containerInfo),Xl(e),$t=t;break;case 22:e.memoizedState===null&&(t=e.alternate,t!==null&&t.memoizedState!==null?(t=sa,sa=16777216,Xl(e),sa=t):Xl(e));break;default:Xl(e)}}function _d(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function ua(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var i=t[n];We=i,xd(i,e)}_d(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)Md(e),e=e.sibling}function Md(e){switch(e.tag){case 0:case 11:case 15:ua(e),e.flags&2048&&xn(9,e,e.return);break;case 3:ua(e);break;case 12:ua(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,Ns(e)):ua(e);break;default:ua(e)}}function Ns(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var i=t[n];We=i,xd(i,e)}_d(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:xn(8,t,t.return),Ns(t);break;case 22:n=t.stateNode,n._visibility&2&&(n._visibility&=-3,Ns(t));break;default:Ns(t)}e=e.sibling}}function xd(e,t){for(;We!==null;){var n=We;switch(n.tag){case 0:case 11:case 15:xn(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var i=n.memoizedState.cachePool.pool;i!=null&&i.refCount++}break;case 24:Gi(n.memoizedState.cache)}if(i=n.child,i!==null)i.return=n,We=i;else e:for(n=e;We!==null;){i=We;var c=i.sibling,f=i.return;if(Sd(i),i===n){We=null;break e}if(c!==null){c.return=f,We=c;break e}We=f}}}var t0={getCacheForType:function(e){var t=lt(Ve),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n}},n0=typeof WeakMap=="function"?WeakMap:Map,Se=0,Me=null,de=null,pe=0,Te=0,At=null,Rn=!1,Zl=!1,hr=!1,dn=0,Be=0,kn=0,rl=0,dr=0,Ut=0,Jl=0,ca=null,dt=null,gr=!1,mr=0,_s=1/0,Ms=null,Ln=null,Pe=0,zn=null,Wl=null,Il=0,pr=0,yr=null,Dd=null,ra=0,vr=null;function Ot(){if((Se&2)!==0&&pe!==0)return pe&-pe;if(B.T!==null){var e=ql;return e!==0?e:Or()}return Vf()}function Cd(){Ut===0&&(Ut=(pe&536870912)===0||be?$f():536870912);var e=zt.current;return e!==null&&(e.flags|=32),Ut}function Nt(e,t,n){(e===Me&&(Te===2||Te===9)||e.cancelPendingCommit!==null)&&(Fl(e,0),Un(e,pe,Ut,!1)),_i(e,n),((Se&2)===0||e!==Me)&&(e===Me&&((Se&2)===0&&(rl|=n),Be===4&&Un(e,pe,Ut,!1)),Wt(e))}function Rd(e,t,n){if((Se&6)!==0)throw Error(s(327));var i=!n&&(t&124)===0&&(t&e.expiredLanes)===0||Ni(e,t),c=i?a0(e,t):Tr(e,t,!0),f=i;do{if(c===0){Zl&&!i&&Un(e,t,0,!1);break}else{if(n=e.current.alternate,f&&!l0(n)){c=Tr(e,t,!1),f=!1;continue}if(c===2){if(f=t,e.errorRecoveryDisabledLanes&f)var g=0;else g=e.pendingLanes&-536870913,g=g!==0?g:g&536870912?536870912:0;if(g!==0){t=g;e:{var y=e;c=ca;var v=y.current.memoizedState.isDehydrated;if(v&&(Fl(y,g).flags|=256),g=Tr(y,g,!1),g!==2){if(hr&&!v){y.errorRecoveryDisabledLanes|=f,rl|=f,c=4;break e}f=dt,dt=c,f!==null&&(dt===null?dt=f:dt.push.apply(dt,f))}c=g}if(f=!1,c!==2)continue}}if(c===1){Fl(e,0),Un(e,t,0,!0);break}e:{switch(i=e,f=c,f){case 0:case 1:throw Error(s(345));case 4:if((t&4194048)!==t)break;case 6:Un(i,t,Ut,!Rn);break e;case 2:dt=null;break;case 3:case 5:break;default:throw Error(s(329))}if((t&62914560)===t&&(c=mr+300-Vt(),10<c)){if(Un(i,t,Ut,!Rn),ja(i,0,!0)!==0)break e;i.timeoutHandle=ug(kd.bind(null,i,n,dt,Ms,gr,t,Ut,rl,Jl,Rn,f,2,-0,0),c);break e}kd(i,n,dt,Ms,gr,t,Ut,rl,Jl,Rn,f,0,-0,0)}}break}while(!0);Wt(e)}function kd(e,t,n,i,c,f,g,y,v,x,q,$,C,R){if(e.timeoutHandle=-1,$=t.subtreeFlags,($&8192||($&16785408)===16785408)&&(pa={stylesheets:null,count:0,unsuspend:q0},Nd(t),$=H0(),$!==null)){e.cancelPendingCommit=$(Hd.bind(null,e,t,f,n,i,c,g,y,v,q,1,C,R)),Un(e,f,g,!x);return}Hd(e,t,f,n,i,c,g,y,v)}function l0(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var i=0;i<n.length;i++){var c=n[i],f=c.getSnapshot;c=c.value;try{if(!St(f(),c))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function Un(e,t,n,i){t&=~dr,t&=~rl,e.suspendedLanes|=t,e.pingedLanes&=~t,i&&(e.warmLanes|=t),i=e.expirationTimes;for(var c=t;0<c;){var f=31-bt(c),g=1<<f;i[f]=-1,c&=~g}n!==0&&Gf(e,n,t)}function xs(){return(Se&6)===0?(fa(0),!1):!0}function br(){if(de!==null){if(Te===0)var e=de.return;else e=de,an=ll=null,Uc(e),Kl=null,ea=0,e=de;for(;e!==null;)hd(e.alternate,e),e=e.return;de=null}}function Fl(e,t){var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,T0(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),br(),Me=e,de=n=tn(e.current,null),pe=t,Te=0,At=null,Rn=!1,Zl=Ni(e,t),hr=!1,Jl=Ut=dr=rl=kn=Be=0,dt=ca=null,gr=!1,(t&8)!==0&&(t|=t&32);var i=e.entangledLanes;if(i!==0)for(e=e.entanglements,i&=t;0<i;){var c=31-bt(i),f=1<<c;t|=e[c],i&=~f}return dn=t,Ia(),n}function Ld(e,t){oe=null,B.H=ps,t===Vi||t===ss?(t=Io(),Te=3):t===Zo?(t=Io(),Te=4):Te=t===Fh?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,At=t,de===null&&(Be=1,Ts(e,Ct(t,e.current)))}function zd(){var e=B.H;return B.H=ps,e===null?ps:e}function Ud(){var e=B.A;return B.A=t0,e}function Sr(){Be=4,Rn||(pe&4194048)!==pe&&zt.current!==null||(Zl=!0),(kn&134217727)===0&&(rl&134217727)===0||Me===null||Un(Me,pe,Ut,!1)}function Tr(e,t,n){var i=Se;Se|=2;var c=zd(),f=Ud();(Me!==e||pe!==t)&&(Ms=null,Fl(e,t)),t=!1;var g=Be;e:do try{if(Te!==0&&de!==null){var y=de,v=At;switch(Te){case 8:br(),g=6;break e;case 3:case 2:case 9:case 6:zt.current===null&&(t=!0);var x=Te;if(Te=0,At=null,Pl(e,y,v,x),n&&Zl){g=0;break e}break;default:x=Te,Te=0,At=null,Pl(e,y,v,x)}}i0(),g=Be;break}catch(q){Ld(e,q)}while(!0);return t&&e.shellSuspendCounter++,an=ll=null,Se=i,B.H=c,B.A=f,de===null&&(Me=null,pe=0,Ia()),g}function i0(){for(;de!==null;)Bd(de)}function a0(e,t){var n=Se;Se|=2;var i=zd(),c=Ud();Me!==e||pe!==t?(Ms=null,_s=Vt()+500,Fl(e,t)):Zl=Ni(e,t);e:do try{if(Te!==0&&de!==null){t=de;var f=At;t:switch(Te){case 1:Te=0,At=null,Pl(e,t,f,1);break;case 2:case 9:if(Jo(f)){Te=0,At=null,qd(t);break}t=function(){Te!==2&&Te!==9||Me!==e||(Te=7),Wt(e)},f.then(t,t);break e;case 3:Te=7;break e;case 4:Te=5;break e;case 7:Jo(f)?(Te=0,At=null,qd(t)):(Te=0,At=null,Pl(e,t,f,7));break;case 5:var g=null;switch(de.tag){case 26:g=de.memoizedState;case 5:case 27:var y=de;if(!g||bg(g)){Te=0,At=null;var v=y.sibling;if(v!==null)de=v;else{var x=y.return;x!==null?(de=x,Ds(x)):de=null}break t}}Te=0,At=null,Pl(e,t,f,5);break;case 6:Te=0,At=null,Pl(e,t,f,6);break;case 8:br(),Be=6;break e;default:throw Error(s(462))}}s0();break}catch(q){Ld(e,q)}while(!0);return an=ll=null,B.H=i,B.A=c,Se=n,de!==null?0:(Me=null,pe=0,Ia(),Be)}function s0(){for(;de!==null&&!Mp();)Bd(de)}function Bd(e){var t=fd(e.alternate,e,dn);e.memoizedProps=e.pendingProps,t===null?Ds(e):de=t}function qd(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=id(n,t,t.pendingProps,t.type,void 0,pe);break;case 11:t=id(n,t,t.pendingProps,t.type.render,t.ref,pe);break;case 5:Uc(t);default:hd(n,t),t=de=jo(t,dn),t=fd(n,t,dn)}e.memoizedProps=e.pendingProps,t===null?Ds(e):de=t}function Pl(e,t,n,i){an=ll=null,Uc(t),Kl=null,ea=0;var c=t.return;try{if(Jy(e,c,t,n,pe)){Be=1,Ts(e,Ct(n,e.current)),de=null;return}}catch(f){if(c!==null)throw de=c,f;Be=1,Ts(e,Ct(n,e.current)),de=null;return}t.flags&32768?(be||i===1?e=!0:Zl||(pe&536870912)!==0?e=!1:(Rn=e=!0,(i===2||i===9||i===3||i===6)&&(i=zt.current,i!==null&&i.tag===13&&(i.flags|=16384))),jd(t,e)):Ds(t)}function Ds(e){var t=e;do{if((t.flags&32768)!==0){jd(t,Rn);return}e=t.return;var n=Iy(t.alternate,t,dn);if(n!==null){de=n;return}if(t=t.sibling,t!==null){de=t;return}de=t=e}while(t!==null);Be===0&&(Be=5)}function jd(e,t){do{var n=Fy(e.alternate,e);if(n!==null){n.flags&=32767,de=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){de=e;return}de=e=n}while(e!==null);Be=6,de=null}function Hd(e,t,n,i,c,f,g,y,v){e.cancelPendingCommit=null;do Cs();while(Pe!==0);if((Se&6)!==0)throw Error(s(327));if(t!==null){if(t===e.current)throw Error(s(177));if(f=t.lanes|t.childLanes,f|=oc,qp(e,n,f,g,y,v),e===Me&&(de=Me=null,pe=0),Wl=t,zn=e,Il=n,pr=f,yr=c,Dd=i,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,f0(Ua,function(){return Vd(),null})):(e.callbackNode=null,e.callbackPriority=0),i=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||i){i=B.T,B.T=null,c=Z.p,Z.p=2,g=Se,Se|=4;try{Py(e,t,n)}finally{Se=g,Z.p=c,B.T=i}}Pe=1,$d(),Yd(),Gd()}}function $d(){if(Pe===1){Pe=0;var e=zn,t=Wl,n=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||n){n=B.T,B.T=null;var i=Z.p;Z.p=2;var c=Se;Se|=4;try{Ed(t,e);var f=kr,g=xo(e.containerInfo),y=f.focusedElem,v=f.selectionRange;if(g!==y&&y&&y.ownerDocument&&Mo(y.ownerDocument.documentElement,y)){if(v!==null&&sc(y)){var x=v.start,q=v.end;if(q===void 0&&(q=x),"selectionStart"in y)y.selectionStart=x,y.selectionEnd=Math.min(q,y.value.length);else{var $=y.ownerDocument||document,C=$&&$.defaultView||window;if(C.getSelection){var R=C.getSelection(),ue=y.textContent.length,le=Math.min(v.start,ue),Ae=v.end===void 0?le:Math.min(v.end,ue);!R.extend&&le>Ae&&(g=Ae,Ae=le,le=g);var _=_o(y,le),O=_o(y,Ae);if(_&&O&&(R.rangeCount!==1||R.anchorNode!==_.node||R.anchorOffset!==_.offset||R.focusNode!==O.node||R.focusOffset!==O.offset)){var M=$.createRange();M.setStart(_.node,_.offset),R.removeAllRanges(),le>Ae?(R.addRange(M),R.extend(O.node,O.offset)):(M.setEnd(O.node,O.offset),R.addRange(M))}}}}for($=[],R=y;R=R.parentNode;)R.nodeType===1&&$.push({element:R,left:R.scrollLeft,top:R.scrollTop});for(typeof y.focus=="function"&&y.focus(),y=0;y<$.length;y++){var j=$[y];j.element.scrollLeft=j.left,j.element.scrollTop=j.top}}Gs=!!Rr,kr=Rr=null}finally{Se=c,Z.p=i,B.T=n}}e.current=t,Pe=2}}function Yd(){if(Pe===2){Pe=0;var e=zn,t=Wl,n=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||n){n=B.T,B.T=null;var i=Z.p;Z.p=2;var c=Se;Se|=4;try{bd(e,t.alternate,t)}finally{Se=c,Z.p=i,B.T=n}}Pe=3}}function Gd(){if(Pe===4||Pe===3){Pe=0,xp();var e=zn,t=Wl,n=Il,i=Dd;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?Pe=5:(Pe=0,Wl=zn=null,Kd(e,e.pendingLanes));var c=e.pendingLanes;if(c===0&&(Ln=null),ju(n),t=t.stateNode,vt&&typeof vt.onCommitFiberRoot=="function")try{vt.onCommitFiberRoot(Oi,t,void 0,(t.current.flags&128)===128)}catch{}if(i!==null){t=B.T,c=Z.p,Z.p=2,B.T=null;try{for(var f=e.onRecoverableError,g=0;g<i.length;g++){var y=i[g];f(y.value,{componentStack:y.stack})}}finally{B.T=t,Z.p=c}}(Il&3)!==0&&Cs(),Wt(e),c=e.pendingLanes,(n&4194090)!==0&&(c&42)!==0?e===vr?ra++:(ra=0,vr=e):ra=0,fa(0)}}function Kd(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,Gi(t)))}function Cs(e){return $d(),Yd(),Gd(),Vd()}function Vd(){if(Pe!==5)return!1;var e=zn,t=pr;pr=0;var n=ju(Il),i=B.T,c=Z.p;try{Z.p=32>n?32:n,B.T=null,n=yr,yr=null;var f=zn,g=Il;if(Pe=0,Wl=zn=null,Il=0,(Se&6)!==0)throw Error(s(331));var y=Se;if(Se|=4,Md(f.current),Od(f,f.current,g,n),Se=y,fa(0,!1),vt&&typeof vt.onPostCommitFiberRoot=="function")try{vt.onPostCommitFiberRoot(Oi,f)}catch{}return!0}finally{Z.p=c,B.T=i,Kd(e,t)}}function Qd(e,t,n){t=Ct(n,t),t=Wc(e.stateNode,t,2),e=On(e,t,2),e!==null&&(_i(e,2),Wt(e))}function Ne(e,t,n){if(e.tag===3)Qd(e,e,n);else for(;t!==null;){if(t.tag===3){Qd(t,e,n);break}else if(t.tag===1){var i=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof i.componentDidCatch=="function"&&(Ln===null||!Ln.has(i))){e=Ct(n,e),n=Wh(2),i=On(t,n,2),i!==null&&(Ih(n,i,t,e),_i(i,2),Wt(i));break}}t=t.return}}function wr(e,t,n){var i=e.pingCache;if(i===null){i=e.pingCache=new n0;var c=new Set;i.set(t,c)}else c=i.get(t),c===void 0&&(c=new Set,i.set(t,c));c.has(n)||(hr=!0,c.add(n),e=u0.bind(null,e,t,n),t.then(e,e))}function u0(e,t,n){var i=e.pingCache;i!==null&&i.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,Me===e&&(pe&n)===n&&(Be===4||Be===3&&(pe&62914560)===pe&&300>Vt()-mr?(Se&2)===0&&Fl(e,0):dr|=n,Jl===pe&&(Jl=0)),Wt(e)}function Xd(e,t){t===0&&(t=Yf()),e=Ll(e,t),e!==null&&(_i(e,t),Wt(e))}function c0(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),Xd(e,n)}function r0(e,t){var n=0;switch(e.tag){case 13:var i=e.stateNode,c=e.memoizedState;c!==null&&(n=c.retryLane);break;case 19:i=e.stateNode;break;case 22:i=e.stateNode._retryCache;break;default:throw Error(s(314))}i!==null&&i.delete(t),Xd(e,n)}function f0(e,t){return zu(e,t)}var Rs=null,ei=null,Er=!1,ks=!1,Ar=!1,fl=0;function Wt(e){e!==ei&&e.next===null&&(ei===null?Rs=ei=e:ei=ei.next=e),ks=!0,Er||(Er=!0,h0())}function fa(e,t){if(!Ar&&ks){Ar=!0;do for(var n=!1,i=Rs;i!==null;){if(e!==0){var c=i.pendingLanes;if(c===0)var f=0;else{var g=i.suspendedLanes,y=i.pingedLanes;f=(1<<31-bt(42|e)+1)-1,f&=c&~(g&~y),f=f&201326741?f&201326741|1:f?f|2:0}f!==0&&(n=!0,Id(i,f))}else f=pe,f=ja(i,i===Me?f:0,i.cancelPendingCommit!==null||i.timeoutHandle!==-1),(f&3)===0||Ni(i,f)||(n=!0,Id(i,f));i=i.next}while(n);Ar=!1}}function o0(){Zd()}function Zd(){ks=Er=!1;var e=0;fl!==0&&(S0()&&(e=fl),fl=0);for(var t=Vt(),n=null,i=Rs;i!==null;){var c=i.next,f=Jd(i,t);f===0?(i.next=null,n===null?Rs=c:n.next=c,c===null&&(ei=n)):(n=i,(e!==0||(f&3)!==0)&&(ks=!0)),i=c}fa(e)}function Jd(e,t){for(var n=e.suspendedLanes,i=e.pingedLanes,c=e.expirationTimes,f=e.pendingLanes&-62914561;0<f;){var g=31-bt(f),y=1<<g,v=c[g];v===-1?((y&n)===0||(y&i)!==0)&&(c[g]=Bp(y,t)):v<=t&&(e.expiredLanes|=y),f&=~y}if(t=Me,n=pe,n=ja(e,e===t?n:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),i=e.callbackNode,n===0||e===t&&(Te===2||Te===9)||e.cancelPendingCommit!==null)return i!==null&&i!==null&&Uu(i),e.callbackNode=null,e.callbackPriority=0;if((n&3)===0||Ni(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(i!==null&&Uu(i),ju(n)){case 2:case 8:n=jf;break;case 32:n=Ua;break;case 268435456:n=Hf;break;default:n=Ua}return i=Wd.bind(null,e),n=zu(n,i),e.callbackPriority=t,e.callbackNode=n,t}return i!==null&&i!==null&&Uu(i),e.callbackPriority=2,e.callbackNode=null,2}function Wd(e,t){if(Pe!==0&&Pe!==5)return e.callbackNode=null,e.callbackPriority=0,null;var n=e.callbackNode;if(Cs()&&e.callbackNode!==n)return null;var i=pe;return i=ja(e,e===Me?i:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),i===0?null:(Rd(e,i,t),Jd(e,Vt()),e.callbackNode!=null&&e.callbackNode===n?Wd.bind(null,e):null)}function Id(e,t){if(Cs())return null;Rd(e,t,!0)}function h0(){w0(function(){(Se&6)!==0?zu(qf,o0):Zd()})}function Or(){return fl===0&&(fl=$f()),fl}function Fd(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:Ka(""+e)}function Pd(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function d0(e,t,n,i,c){if(t==="submit"&&n&&n.stateNode===c){var f=Fd((c[rt]||null).action),g=i.submitter;g&&(t=(t=g[rt]||null)?Fd(t.formAction):g.getAttribute("formAction"),t!==null&&(f=t,g=null));var y=new Za("action","action",null,i,c);e.push({event:y,listeners:[{instance:null,listener:function(){if(i.defaultPrevented){if(fl!==0){var v=g?Pd(c,g):new FormData(c);Vc(n,{pending:!0,data:v,method:c.method,action:f},null,v)}}else typeof f=="function"&&(y.preventDefault(),v=g?Pd(c,g):new FormData(c),Vc(n,{pending:!0,data:v,method:c.method,action:f},f,v))},currentTarget:c}]})}}for(var Nr=0;Nr<fc.length;Nr++){var _r=fc[Nr],g0=_r.toLowerCase(),m0=_r[0].toUpperCase()+_r.slice(1);Ht(g0,"on"+m0)}Ht(Ro,"onAnimationEnd"),Ht(ko,"onAnimationIteration"),Ht(Lo,"onAnimationStart"),Ht("dblclick","onDoubleClick"),Ht("focusin","onFocus"),Ht("focusout","onBlur"),Ht(Ry,"onTransitionRun"),Ht(ky,"onTransitionStart"),Ht(Ly,"onTransitionCancel"),Ht(zo,"onTransitionEnd"),Al("onMouseEnter",["mouseout","mouseover"]),Al("onMouseLeave",["mouseout","mouseover"]),Al("onPointerEnter",["pointerout","pointerover"]),Al("onPointerLeave",["pointerout","pointerover"]),Zn("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),Zn("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),Zn("onBeforeInput",["compositionend","keypress","textInput","paste"]),Zn("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),Zn("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),Zn("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var oa="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),p0=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(oa));function eg(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var i=e[n],c=i.event;i=i.listeners;e:{var f=void 0;if(t)for(var g=i.length-1;0<=g;g--){var y=i[g],v=y.instance,x=y.currentTarget;if(y=y.listener,v!==f&&c.isPropagationStopped())break e;f=y,c.currentTarget=x;try{f(c)}catch(q){Ss(q)}c.currentTarget=null,f=v}else for(g=0;g<i.length;g++){if(y=i[g],v=y.instance,x=y.currentTarget,y=y.listener,v!==f&&c.isPropagationStopped())break e;f=y,c.currentTarget=x;try{f(c)}catch(q){Ss(q)}c.currentTarget=null,f=v}}}}function ge(e,t){var n=t[Hu];n===void 0&&(n=t[Hu]=new Set);var i=e+"__bubble";n.has(i)||(tg(t,e,2,!1),n.add(i))}function Mr(e,t,n){var i=0;t&&(i|=4),tg(n,e,i,t)}var Ls="_reactListening"+Math.random().toString(36).slice(2);function xr(e){if(!e[Ls]){e[Ls]=!0,Xf.forEach(function(n){n!=="selectionchange"&&(p0.has(n)||Mr(n,!1,e),Mr(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[Ls]||(t[Ls]=!0,Mr("selectionchange",!1,t))}}function tg(e,t,n,i){switch(Og(t)){case 2:var c=G0;break;case 8:c=K0;break;default:c=Gr}n=c.bind(null,t,n,e),c=void 0,!Iu||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(c=!0),i?c!==void 0?e.addEventListener(t,n,{capture:!0,passive:c}):e.addEventListener(t,n,!0):c!==void 0?e.addEventListener(t,n,{passive:c}):e.addEventListener(t,n,!1)}function Dr(e,t,n,i,c){var f=i;if((t&1)===0&&(t&2)===0&&i!==null)e:for(;;){if(i===null)return;var g=i.tag;if(g===3||g===4){var y=i.stateNode.containerInfo;if(y===c)break;if(g===4)for(g=i.return;g!==null;){var v=g.tag;if((v===3||v===4)&&g.stateNode.containerInfo===c)return;g=g.return}for(;y!==null;){if(g=Tl(y),g===null)return;if(v=g.tag,v===5||v===6||v===26||v===27){i=f=g;continue e}y=y.parentNode}}i=i.return}uo(function(){var x=f,q=Ju(n),$=[];e:{var C=Uo.get(e);if(C!==void 0){var R=Za,ue=e;switch(e){case"keypress":if(Qa(n)===0)break e;case"keydown":case"keyup":R=fy;break;case"focusin":ue="focus",R=tc;break;case"focusout":ue="blur",R=tc;break;case"beforeblur":case"afterblur":R=tc;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":R=fo;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":R=Fp;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":R=dy;break;case Ro:case ko:case Lo:R=ty;break;case zo:R=my;break;case"scroll":case"scrollend":R=Wp;break;case"wheel":R=yy;break;case"copy":case"cut":case"paste":R=ly;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":R=ho;break;case"toggle":case"beforetoggle":R=by}var le=(t&4)!==0,Ae=!le&&(e==="scroll"||e==="scrollend"),_=le?C!==null?C+"Capture":null:C;le=[];for(var O=x,M;O!==null;){var j=O;if(M=j.stateNode,j=j.tag,j!==5&&j!==26&&j!==27||M===null||_===null||(j=Di(O,_),j!=null&&le.push(ha(O,j,M))),Ae)break;O=O.return}0<le.length&&(C=new R(C,ue,null,n,q),$.push({event:C,listeners:le}))}}if((t&7)===0){e:{if(C=e==="mouseover"||e==="pointerover",R=e==="mouseout"||e==="pointerout",C&&n!==Zu&&(ue=n.relatedTarget||n.fromElement)&&(Tl(ue)||ue[Sl]))break e;if((R||C)&&(C=q.window===q?q:(C=q.ownerDocument)?C.defaultView||C.parentWindow:window,R?(ue=n.relatedTarget||n.toElement,R=x,ue=ue?Tl(ue):null,ue!==null&&(Ae=o(ue),le=ue.tag,ue!==Ae||le!==5&&le!==27&&le!==6)&&(ue=null)):(R=null,ue=x),R!==ue)){if(le=fo,j="onMouseLeave",_="onMouseEnter",O="mouse",(e==="pointerout"||e==="pointerover")&&(le=ho,j="onPointerLeave",_="onPointerEnter",O="pointer"),Ae=R==null?C:xi(R),M=ue==null?C:xi(ue),C=new le(j,O+"leave",R,n,q),C.target=Ae,C.relatedTarget=M,j=null,Tl(q)===x&&(le=new le(_,O+"enter",ue,n,q),le.target=M,le.relatedTarget=Ae,j=le),Ae=j,R&&ue)t:{for(le=R,_=ue,O=0,M=le;M;M=ti(M))O++;for(M=0,j=_;j;j=ti(j))M++;for(;0<O-M;)le=ti(le),O--;for(;0<M-O;)_=ti(_),M--;for(;O--;){if(le===_||_!==null&&le===_.alternate)break t;le=ti(le),_=ti(_)}le=null}else le=null;R!==null&&ng($,C,R,le,!1),ue!==null&&Ae!==null&&ng($,Ae,ue,le,!0)}}e:{if(C=x?xi(x):window,R=C.nodeName&&C.nodeName.toLowerCase(),R==="select"||R==="input"&&C.type==="file")var I=To;else if(bo(C))if(wo)I=xy;else{I=_y;var he=Ny}else R=C.nodeName,!R||R.toLowerCase()!=="input"||C.type!=="checkbox"&&C.type!=="radio"?x&&Xu(x.elementType)&&(I=To):I=My;if(I&&(I=I(e,x))){So($,I,n,q);break e}he&&he(e,C,x),e==="focusout"&&x&&C.type==="number"&&x.memoizedProps.value!=null&&Qu(C,"number",C.value)}switch(he=x?xi(x):window,e){case"focusin":(bo(he)||he.contentEditable==="true")&&(Cl=he,uc=x,qi=null);break;case"focusout":qi=uc=Cl=null;break;case"mousedown":cc=!0;break;case"contextmenu":case"mouseup":case"dragend":cc=!1,Do($,n,q);break;case"selectionchange":if(Cy)break;case"keydown":case"keyup":Do($,n,q)}var P;if(lc)e:{switch(e){case"compositionstart":var ie="onCompositionStart";break e;case"compositionend":ie="onCompositionEnd";break e;case"compositionupdate":ie="onCompositionUpdate";break e}ie=void 0}else Dl?yo(e,n)&&(ie="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(ie="onCompositionStart");ie&&(go&&n.locale!=="ko"&&(Dl||ie!=="onCompositionStart"?ie==="onCompositionEnd"&&Dl&&(P=co()):(Tn=q,Fu="value"in Tn?Tn.value:Tn.textContent,Dl=!0)),he=zs(x,ie),0<he.length&&(ie=new oo(ie,e,null,n,q),$.push({event:ie,listeners:he}),P?ie.data=P:(P=vo(n),P!==null&&(ie.data=P)))),(P=Ty?wy(e,n):Ey(e,n))&&(ie=zs(x,"onBeforeInput"),0<ie.length&&(he=new oo("onBeforeInput","beforeinput",null,n,q),$.push({event:he,listeners:ie}),he.data=P)),d0($,e,x,n,q)}eg($,t)})}function ha(e,t,n){return{instance:e,listener:t,currentTarget:n}}function zs(e,t){for(var n=t+"Capture",i=[];e!==null;){var c=e,f=c.stateNode;if(c=c.tag,c!==5&&c!==26&&c!==27||f===null||(c=Di(e,n),c!=null&&i.unshift(ha(e,c,f)),c=Di(e,t),c!=null&&i.push(ha(e,c,f))),e.tag===3)return i;e=e.return}return[]}function ti(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function ng(e,t,n,i,c){for(var f=t._reactName,g=[];n!==null&&n!==i;){var y=n,v=y.alternate,x=y.stateNode;if(y=y.tag,v!==null&&v===i)break;y!==5&&y!==26&&y!==27||x===null||(v=x,c?(x=Di(n,f),x!=null&&g.unshift(ha(n,x,v))):c||(x=Di(n,f),x!=null&&g.push(ha(n,x,v)))),n=n.return}g.length!==0&&e.push({event:t,listeners:g})}var y0=/\r\n?/g,v0=/\u0000|\uFFFD/g;function lg(e){return(typeof e=="string"?e:""+e).replace(y0,`
`).replace(v0,"")}function ig(e,t){return t=lg(t),lg(e)===t}function Us(){}function Ee(e,t,n,i,c,f){switch(n){case"children":typeof i=="string"?t==="body"||t==="textarea"&&i===""||_l(e,i):(typeof i=="number"||typeof i=="bigint")&&t!=="body"&&_l(e,""+i);break;case"className":$a(e,"class",i);break;case"tabIndex":$a(e,"tabindex",i);break;case"dir":case"role":case"viewBox":case"width":case"height":$a(e,n,i);break;case"style":ao(e,i,f);break;case"data":if(t!=="object"){$a(e,"data",i);break}case"src":case"href":if(i===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(i==null||typeof i=="function"||typeof i=="symbol"||typeof i=="boolean"){e.removeAttribute(n);break}i=Ka(""+i),e.setAttribute(n,i);break;case"action":case"formAction":if(typeof i=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof f=="function"&&(n==="formAction"?(t!=="input"&&Ee(e,t,"name",c.name,c,null),Ee(e,t,"formEncType",c.formEncType,c,null),Ee(e,t,"formMethod",c.formMethod,c,null),Ee(e,t,"formTarget",c.formTarget,c,null)):(Ee(e,t,"encType",c.encType,c,null),Ee(e,t,"method",c.method,c,null),Ee(e,t,"target",c.target,c,null)));if(i==null||typeof i=="symbol"||typeof i=="boolean"){e.removeAttribute(n);break}i=Ka(""+i),e.setAttribute(n,i);break;case"onClick":i!=null&&(e.onclick=Us);break;case"onScroll":i!=null&&ge("scroll",e);break;case"onScrollEnd":i!=null&&ge("scrollend",e);break;case"dangerouslySetInnerHTML":if(i!=null){if(typeof i!="object"||!("__html"in i))throw Error(s(61));if(n=i.__html,n!=null){if(c.children!=null)throw Error(s(60));e.innerHTML=n}}break;case"multiple":e.multiple=i&&typeof i!="function"&&typeof i!="symbol";break;case"muted":e.muted=i&&typeof i!="function"&&typeof i!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(i==null||typeof i=="function"||typeof i=="boolean"||typeof i=="symbol"){e.removeAttribute("xlink:href");break}n=Ka(""+i),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":i!=null&&typeof i!="function"&&typeof i!="symbol"?e.setAttribute(n,""+i):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":i&&typeof i!="function"&&typeof i!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":i===!0?e.setAttribute(n,""):i!==!1&&i!=null&&typeof i!="function"&&typeof i!="symbol"?e.setAttribute(n,i):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":i!=null&&typeof i!="function"&&typeof i!="symbol"&&!isNaN(i)&&1<=i?e.setAttribute(n,i):e.removeAttribute(n);break;case"rowSpan":case"start":i==null||typeof i=="function"||typeof i=="symbol"||isNaN(i)?e.removeAttribute(n):e.setAttribute(n,i);break;case"popover":ge("beforetoggle",e),ge("toggle",e),Ha(e,"popover",i);break;case"xlinkActuate":Pt(e,"http://www.w3.org/1999/xlink","xlink:actuate",i);break;case"xlinkArcrole":Pt(e,"http://www.w3.org/1999/xlink","xlink:arcrole",i);break;case"xlinkRole":Pt(e,"http://www.w3.org/1999/xlink","xlink:role",i);break;case"xlinkShow":Pt(e,"http://www.w3.org/1999/xlink","xlink:show",i);break;case"xlinkTitle":Pt(e,"http://www.w3.org/1999/xlink","xlink:title",i);break;case"xlinkType":Pt(e,"http://www.w3.org/1999/xlink","xlink:type",i);break;case"xmlBase":Pt(e,"http://www.w3.org/XML/1998/namespace","xml:base",i);break;case"xmlLang":Pt(e,"http://www.w3.org/XML/1998/namespace","xml:lang",i);break;case"xmlSpace":Pt(e,"http://www.w3.org/XML/1998/namespace","xml:space",i);break;case"is":Ha(e,"is",i);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=Zp.get(n)||n,Ha(e,n,i))}}function Cr(e,t,n,i,c,f){switch(n){case"style":ao(e,i,f);break;case"dangerouslySetInnerHTML":if(i!=null){if(typeof i!="object"||!("__html"in i))throw Error(s(61));if(n=i.__html,n!=null){if(c.children!=null)throw Error(s(60));e.innerHTML=n}}break;case"children":typeof i=="string"?_l(e,i):(typeof i=="number"||typeof i=="bigint")&&_l(e,""+i);break;case"onScroll":i!=null&&ge("scroll",e);break;case"onScrollEnd":i!=null&&ge("scrollend",e);break;case"onClick":i!=null&&(e.onclick=Us);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Zf.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(c=n.endsWith("Capture"),t=n.slice(2,c?n.length-7:void 0),f=e[rt]||null,f=f!=null?f[n]:null,typeof f=="function"&&e.removeEventListener(t,f,c),typeof i=="function")){typeof f!="function"&&f!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,i,c);break e}n in e?e[n]=i:i===!0?e.setAttribute(n,""):Ha(e,n,i)}}}function et(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":ge("error",e),ge("load",e);var i=!1,c=!1,f;for(f in n)if(n.hasOwnProperty(f)){var g=n[f];if(g!=null)switch(f){case"src":i=!0;break;case"srcSet":c=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(s(137,t));default:Ee(e,t,f,g,n,null)}}c&&Ee(e,t,"srcSet",n.srcSet,n,null),i&&Ee(e,t,"src",n.src,n,null);return;case"input":ge("invalid",e);var y=f=g=c=null,v=null,x=null;for(i in n)if(n.hasOwnProperty(i)){var q=n[i];if(q!=null)switch(i){case"name":c=q;break;case"type":g=q;break;case"checked":v=q;break;case"defaultChecked":x=q;break;case"value":f=q;break;case"defaultValue":y=q;break;case"children":case"dangerouslySetInnerHTML":if(q!=null)throw Error(s(137,t));break;default:Ee(e,t,i,q,n,null)}}to(e,f,y,v,x,g,c,!1),Ya(e);return;case"select":ge("invalid",e),i=g=f=null;for(c in n)if(n.hasOwnProperty(c)&&(y=n[c],y!=null))switch(c){case"value":f=y;break;case"defaultValue":g=y;break;case"multiple":i=y;default:Ee(e,t,c,y,n,null)}t=f,n=g,e.multiple=!!i,t!=null?Nl(e,!!i,t,!1):n!=null&&Nl(e,!!i,n,!0);return;case"textarea":ge("invalid",e),f=c=i=null;for(g in n)if(n.hasOwnProperty(g)&&(y=n[g],y!=null))switch(g){case"value":i=y;break;case"defaultValue":c=y;break;case"children":f=y;break;case"dangerouslySetInnerHTML":if(y!=null)throw Error(s(91));break;default:Ee(e,t,g,y,n,null)}lo(e,i,c,f),Ya(e);return;case"option":for(v in n)if(n.hasOwnProperty(v)&&(i=n[v],i!=null))switch(v){case"selected":e.selected=i&&typeof i!="function"&&typeof i!="symbol";break;default:Ee(e,t,v,i,n,null)}return;case"dialog":ge("beforetoggle",e),ge("toggle",e),ge("cancel",e),ge("close",e);break;case"iframe":case"object":ge("load",e);break;case"video":case"audio":for(i=0;i<oa.length;i++)ge(oa[i],e);break;case"image":ge("error",e),ge("load",e);break;case"details":ge("toggle",e);break;case"embed":case"source":case"link":ge("error",e),ge("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(x in n)if(n.hasOwnProperty(x)&&(i=n[x],i!=null))switch(x){case"children":case"dangerouslySetInnerHTML":throw Error(s(137,t));default:Ee(e,t,x,i,n,null)}return;default:if(Xu(t)){for(q in n)n.hasOwnProperty(q)&&(i=n[q],i!==void 0&&Cr(e,t,q,i,n,void 0));return}}for(y in n)n.hasOwnProperty(y)&&(i=n[y],i!=null&&Ee(e,t,y,i,n,null))}function b0(e,t,n,i){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var c=null,f=null,g=null,y=null,v=null,x=null,q=null;for(R in n){var $=n[R];if(n.hasOwnProperty(R)&&$!=null)switch(R){case"checked":break;case"value":break;case"defaultValue":v=$;default:i.hasOwnProperty(R)||Ee(e,t,R,null,i,$)}}for(var C in i){var R=i[C];if($=n[C],i.hasOwnProperty(C)&&(R!=null||$!=null))switch(C){case"type":f=R;break;case"name":c=R;break;case"checked":x=R;break;case"defaultChecked":q=R;break;case"value":g=R;break;case"defaultValue":y=R;break;case"children":case"dangerouslySetInnerHTML":if(R!=null)throw Error(s(137,t));break;default:R!==$&&Ee(e,t,C,R,i,$)}}Vu(e,g,y,v,x,q,f,c);return;case"select":R=g=y=C=null;for(f in n)if(v=n[f],n.hasOwnProperty(f)&&v!=null)switch(f){case"value":break;case"multiple":R=v;default:i.hasOwnProperty(f)||Ee(e,t,f,null,i,v)}for(c in i)if(f=i[c],v=n[c],i.hasOwnProperty(c)&&(f!=null||v!=null))switch(c){case"value":C=f;break;case"defaultValue":y=f;break;case"multiple":g=f;default:f!==v&&Ee(e,t,c,f,i,v)}t=y,n=g,i=R,C!=null?Nl(e,!!n,C,!1):!!i!=!!n&&(t!=null?Nl(e,!!n,t,!0):Nl(e,!!n,n?[]:"",!1));return;case"textarea":R=C=null;for(y in n)if(c=n[y],n.hasOwnProperty(y)&&c!=null&&!i.hasOwnProperty(y))switch(y){case"value":break;case"children":break;default:Ee(e,t,y,null,i,c)}for(g in i)if(c=i[g],f=n[g],i.hasOwnProperty(g)&&(c!=null||f!=null))switch(g){case"value":C=c;break;case"defaultValue":R=c;break;case"children":break;case"dangerouslySetInnerHTML":if(c!=null)throw Error(s(91));break;default:c!==f&&Ee(e,t,g,c,i,f)}no(e,C,R);return;case"option":for(var ue in n)if(C=n[ue],n.hasOwnProperty(ue)&&C!=null&&!i.hasOwnProperty(ue))switch(ue){case"selected":e.selected=!1;break;default:Ee(e,t,ue,null,i,C)}for(v in i)if(C=i[v],R=n[v],i.hasOwnProperty(v)&&C!==R&&(C!=null||R!=null))switch(v){case"selected":e.selected=C&&typeof C!="function"&&typeof C!="symbol";break;default:Ee(e,t,v,C,i,R)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var le in n)C=n[le],n.hasOwnProperty(le)&&C!=null&&!i.hasOwnProperty(le)&&Ee(e,t,le,null,i,C);for(x in i)if(C=i[x],R=n[x],i.hasOwnProperty(x)&&C!==R&&(C!=null||R!=null))switch(x){case"children":case"dangerouslySetInnerHTML":if(C!=null)throw Error(s(137,t));break;default:Ee(e,t,x,C,i,R)}return;default:if(Xu(t)){for(var Ae in n)C=n[Ae],n.hasOwnProperty(Ae)&&C!==void 0&&!i.hasOwnProperty(Ae)&&Cr(e,t,Ae,void 0,i,C);for(q in i)C=i[q],R=n[q],!i.hasOwnProperty(q)||C===R||C===void 0&&R===void 0||Cr(e,t,q,C,i,R);return}}for(var _ in n)C=n[_],n.hasOwnProperty(_)&&C!=null&&!i.hasOwnProperty(_)&&Ee(e,t,_,null,i,C);for($ in i)C=i[$],R=n[$],!i.hasOwnProperty($)||C===R||C==null&&R==null||Ee(e,t,$,C,i,R)}var Rr=null,kr=null;function Bs(e){return e.nodeType===9?e:e.ownerDocument}function ag(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function sg(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function Lr(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var zr=null;function S0(){var e=window.event;return e&&e.type==="popstate"?e===zr?!1:(zr=e,!0):(zr=null,!1)}var ug=typeof setTimeout=="function"?setTimeout:void 0,T0=typeof clearTimeout=="function"?clearTimeout:void 0,cg=typeof Promise=="function"?Promise:void 0,w0=typeof queueMicrotask=="function"?queueMicrotask:typeof cg<"u"?function(e){return cg.resolve(null).then(e).catch(E0)}:ug;function E0(e){setTimeout(function(){throw e})}function Bn(e){return e==="head"}function rg(e,t){var n=t,i=0,c=0;do{var f=n.nextSibling;if(e.removeChild(n),f&&f.nodeType===8)if(n=f.data,n==="/$"){if(0<i&&8>i){n=i;var g=e.ownerDocument;if(n&1&&da(g.documentElement),n&2&&da(g.body),n&4)for(n=g.head,da(n),g=n.firstChild;g;){var y=g.nextSibling,v=g.nodeName;g[Mi]||v==="SCRIPT"||v==="STYLE"||v==="LINK"&&g.rel.toLowerCase()==="stylesheet"||n.removeChild(g),g=y}}if(c===0){e.removeChild(f),Ta(t);return}c--}else n==="$"||n==="$?"||n==="$!"?c++:i=n.charCodeAt(0)-48;else i=0;n=f}while(n);Ta(t)}function Ur(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":Ur(n),$u(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function A0(e,t,n,i){for(;e.nodeType===1;){var c=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!i&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(i){if(!e[Mi])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(f=e.getAttribute("rel"),f==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(f!==c.rel||e.getAttribute("href")!==(c.href==null||c.href===""?null:c.href)||e.getAttribute("crossorigin")!==(c.crossOrigin==null?null:c.crossOrigin)||e.getAttribute("title")!==(c.title==null?null:c.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(f=e.getAttribute("src"),(f!==(c.src==null?null:c.src)||e.getAttribute("type")!==(c.type==null?null:c.type)||e.getAttribute("crossorigin")!==(c.crossOrigin==null?null:c.crossOrigin))&&f&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var f=c.name==null?null:""+c.name;if(c.type==="hidden"&&e.getAttribute("name")===f)return e}else return e;if(e=Yt(e.nextSibling),e===null)break}return null}function O0(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Yt(e.nextSibling),e===null))return null;return e}function Br(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState==="complete"}function N0(e,t){var n=e.ownerDocument;if(e.data!=="$?"||n.readyState==="complete")t();else{var i=function(){t(),n.removeEventListener("DOMContentLoaded",i)};n.addEventListener("DOMContentLoaded",i),e._reactRetry=i}}function Yt(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="F!"||t==="F")break;if(t==="/$")return null}}return e}var qr=null;function fg(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"){if(t===0)return e;t--}else n==="/$"&&t++}e=e.previousSibling}return null}function og(e,t,n){switch(t=Bs(n),e){case"html":if(e=t.documentElement,!e)throw Error(s(452));return e;case"head":if(e=t.head,!e)throw Error(s(453));return e;case"body":if(e=t.body,!e)throw Error(s(454));return e;default:throw Error(s(451))}}function da(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);$u(e)}var Bt=new Map,hg=new Set;function qs(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var gn=Z.d;Z.d={f:_0,r:M0,D:x0,C:D0,L:C0,m:R0,X:L0,S:k0,M:z0};function _0(){var e=gn.f(),t=xs();return e||t}function M0(e){var t=wl(e);t!==null&&t.tag===5&&t.type==="form"?Rh(t):gn.r(e)}var ni=typeof document>"u"?null:document;function dg(e,t,n){var i=ni;if(i&&typeof t=="string"&&t){var c=Dt(t);c='link[rel="'+e+'"][href="'+c+'"]',typeof n=="string"&&(c+='[crossorigin="'+n+'"]'),hg.has(c)||(hg.add(c),e={rel:e,crossOrigin:n,href:t},i.querySelector(c)===null&&(t=i.createElement("link"),et(t,"link",e),Ze(t),i.head.appendChild(t)))}}function x0(e){gn.D(e),dg("dns-prefetch",e,null)}function D0(e,t){gn.C(e,t),dg("preconnect",e,t)}function C0(e,t,n){gn.L(e,t,n);var i=ni;if(i&&e&&t){var c='link[rel="preload"][as="'+Dt(t)+'"]';t==="image"&&n&&n.imageSrcSet?(c+='[imagesrcset="'+Dt(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(c+='[imagesizes="'+Dt(n.imageSizes)+'"]')):c+='[href="'+Dt(e)+'"]';var f=c;switch(t){case"style":f=li(e);break;case"script":f=ii(e)}Bt.has(f)||(e=b({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),Bt.set(f,e),i.querySelector(c)!==null||t==="style"&&i.querySelector(ga(f))||t==="script"&&i.querySelector(ma(f))||(t=i.createElement("link"),et(t,"link",e),Ze(t),i.head.appendChild(t)))}}function R0(e,t){gn.m(e,t);var n=ni;if(n&&e){var i=t&&typeof t.as=="string"?t.as:"script",c='link[rel="modulepreload"][as="'+Dt(i)+'"][href="'+Dt(e)+'"]',f=c;switch(i){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":f=ii(e)}if(!Bt.has(f)&&(e=b({rel:"modulepreload",href:e},t),Bt.set(f,e),n.querySelector(c)===null)){switch(i){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(ma(f)))return}i=n.createElement("link"),et(i,"link",e),Ze(i),n.head.appendChild(i)}}}function k0(e,t,n){gn.S(e,t,n);var i=ni;if(i&&e){var c=El(i).hoistableStyles,f=li(e);t=t||"default";var g=c.get(f);if(!g){var y={loading:0,preload:null};if(g=i.querySelector(ga(f)))y.loading=5;else{e=b({rel:"stylesheet",href:e,"data-precedence":t},n),(n=Bt.get(f))&&jr(e,n);var v=g=i.createElement("link");Ze(v),et(v,"link",e),v._p=new Promise(function(x,q){v.onload=x,v.onerror=q}),v.addEventListener("load",function(){y.loading|=1}),v.addEventListener("error",function(){y.loading|=2}),y.loading|=4,js(g,t,i)}g={type:"stylesheet",instance:g,count:1,state:y},c.set(f,g)}}}function L0(e,t){gn.X(e,t);var n=ni;if(n&&e){var i=El(n).hoistableScripts,c=ii(e),f=i.get(c);f||(f=n.querySelector(ma(c)),f||(e=b({src:e,async:!0},t),(t=Bt.get(c))&&Hr(e,t),f=n.createElement("script"),Ze(f),et(f,"link",e),n.head.appendChild(f)),f={type:"script",instance:f,count:1,state:null},i.set(c,f))}}function z0(e,t){gn.M(e,t);var n=ni;if(n&&e){var i=El(n).hoistableScripts,c=ii(e),f=i.get(c);f||(f=n.querySelector(ma(c)),f||(e=b({src:e,async:!0,type:"module"},t),(t=Bt.get(c))&&Hr(e,t),f=n.createElement("script"),Ze(f),et(f,"link",e),n.head.appendChild(f)),f={type:"script",instance:f,count:1,state:null},i.set(c,f))}}function gg(e,t,n,i){var c=(c=ce.current)?qs(c):null;if(!c)throw Error(s(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=li(n.href),n=El(c).hoistableStyles,i=n.get(t),i||(i={type:"style",instance:null,count:0,state:null},n.set(t,i)),i):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=li(n.href);var f=El(c).hoistableStyles,g=f.get(e);if(g||(c=c.ownerDocument||c,g={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},f.set(e,g),(f=c.querySelector(ga(e)))&&!f._p&&(g.instance=f,g.state.loading=5),Bt.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},Bt.set(e,n),f||U0(c,e,n,g.state))),t&&i===null)throw Error(s(528,""));return g}if(t&&i!==null)throw Error(s(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=ii(n),n=El(c).hoistableScripts,i=n.get(t),i||(i={type:"script",instance:null,count:0,state:null},n.set(t,i)),i):{type:"void",instance:null,count:0,state:null};default:throw Error(s(444,e))}}function li(e){return'href="'+Dt(e)+'"'}function ga(e){return'link[rel="stylesheet"]['+e+"]"}function mg(e){return b({},e,{"data-precedence":e.precedence,precedence:null})}function U0(e,t,n,i){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?i.loading=1:(t=e.createElement("link"),i.preload=t,t.addEventListener("load",function(){return i.loading|=1}),t.addEventListener("error",function(){return i.loading|=2}),et(t,"link",n),Ze(t),e.head.appendChild(t))}function ii(e){return'[src="'+Dt(e)+'"]'}function ma(e){return"script[async]"+e}function pg(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var i=e.querySelector('style[data-href~="'+Dt(n.href)+'"]');if(i)return t.instance=i,Ze(i),i;var c=b({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return i=(e.ownerDocument||e).createElement("style"),Ze(i),et(i,"style",c),js(i,n.precedence,e),t.instance=i;case"stylesheet":c=li(n.href);var f=e.querySelector(ga(c));if(f)return t.state.loading|=4,t.instance=f,Ze(f),f;i=mg(n),(c=Bt.get(c))&&jr(i,c),f=(e.ownerDocument||e).createElement("link"),Ze(f);var g=f;return g._p=new Promise(function(y,v){g.onload=y,g.onerror=v}),et(f,"link",i),t.state.loading|=4,js(f,n.precedence,e),t.instance=f;case"script":return f=ii(n.src),(c=e.querySelector(ma(f)))?(t.instance=c,Ze(c),c):(i=n,(c=Bt.get(f))&&(i=b({},n),Hr(i,c)),e=e.ownerDocument||e,c=e.createElement("script"),Ze(c),et(c,"link",i),e.head.appendChild(c),t.instance=c);case"void":return null;default:throw Error(s(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(i=t.instance,t.state.loading|=4,js(i,n.precedence,e));return t.instance}function js(e,t,n){for(var i=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),c=i.length?i[i.length-1]:null,f=c,g=0;g<i.length;g++){var y=i[g];if(y.dataset.precedence===t)f=y;else if(f!==c)break}f?f.parentNode.insertBefore(e,f.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function jr(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function Hr(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var Hs=null;function yg(e,t,n){if(Hs===null){var i=new Map,c=Hs=new Map;c.set(n,i)}else c=Hs,i=c.get(n),i||(i=new Map,c.set(n,i));if(i.has(e))return i;for(i.set(e,null),n=n.getElementsByTagName(e),c=0;c<n.length;c++){var f=n[c];if(!(f[Mi]||f[nt]||e==="link"&&f.getAttribute("rel")==="stylesheet")&&f.namespaceURI!=="http://www.w3.org/2000/svg"){var g=f.getAttribute(t)||"";g=e+g;var y=i.get(g);y?y.push(f):i.set(g,[f])}}return i}function vg(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function B0(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function bg(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}var pa=null;function q0(){}function j0(e,t,n){if(pa===null)throw Error(s(475));var i=pa;if(t.type==="stylesheet"&&(typeof n.media!="string"||matchMedia(n.media).matches!==!1)&&(t.state.loading&4)===0){if(t.instance===null){var c=li(n.href),f=e.querySelector(ga(c));if(f){e=f._p,e!==null&&typeof e=="object"&&typeof e.then=="function"&&(i.count++,i=$s.bind(i),e.then(i,i)),t.state.loading|=4,t.instance=f,Ze(f);return}f=e.ownerDocument||e,n=mg(n),(c=Bt.get(c))&&jr(n,c),f=f.createElement("link"),Ze(f);var g=f;g._p=new Promise(function(y,v){g.onload=y,g.onerror=v}),et(f,"link",n),t.instance=f}i.stylesheets===null&&(i.stylesheets=new Map),i.stylesheets.set(t,e),(e=t.state.preload)&&(t.state.loading&3)===0&&(i.count++,t=$s.bind(i),e.addEventListener("load",t),e.addEventListener("error",t))}}function H0(){if(pa===null)throw Error(s(475));var e=pa;return e.stylesheets&&e.count===0&&$r(e,e.stylesheets),0<e.count?function(t){var n=setTimeout(function(){if(e.stylesheets&&$r(e,e.stylesheets),e.unsuspend){var i=e.unsuspend;e.unsuspend=null,i()}},6e4);return e.unsuspend=t,function(){e.unsuspend=null,clearTimeout(n)}}:null}function $s(){if(this.count--,this.count===0){if(this.stylesheets)$r(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var Ys=null;function $r(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,Ys=new Map,t.forEach($0,e),Ys=null,$s.call(e))}function $0(e,t){if(!(t.state.loading&4)){var n=Ys.get(e);if(n)var i=n.get(null);else{n=new Map,Ys.set(e,n);for(var c=e.querySelectorAll("link[data-precedence],style[data-precedence]"),f=0;f<c.length;f++){var g=c[f];(g.nodeName==="LINK"||g.getAttribute("media")!=="not all")&&(n.set(g.dataset.precedence,g),i=g)}i&&n.set(null,i)}c=t.instance,g=c.getAttribute("data-precedence"),f=n.get(g)||i,f===i&&n.set(null,c),n.set(g,c),this.count++,i=$s.bind(this),c.addEventListener("load",i),c.addEventListener("error",i),f?f.parentNode.insertBefore(c,f.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(c,e.firstChild)),t.state.loading|=4}}var ya={$$typeof:H,Provider:null,Consumer:null,_currentValue:ae,_currentValue2:ae,_threadCount:0};function Y0(e,t,n,i,c,f,g,y){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=Bu(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=Bu(0),this.hiddenUpdates=Bu(null),this.identifierPrefix=i,this.onUncaughtError=c,this.onCaughtError=f,this.onRecoverableError=g,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=y,this.incompleteTransitions=new Map}function Sg(e,t,n,i,c,f,g,y,v,x,q,$){return e=new Y0(e,t,n,g,y,v,x,$),t=1,f===!0&&(t|=24),f=Tt(3,null,null,t),e.current=f,f.stateNode=e,t=wc(),t.refCount++,e.pooledCache=t,t.refCount++,f.memoizedState={element:i,isDehydrated:n,cache:t},Nc(f),e}function Tg(e){return e?(e=zl,e):zl}function wg(e,t,n,i,c,f){c=Tg(c),i.context===null?i.context=c:i.pendingContext=c,i=An(t),i.payload={element:n},f=f===void 0?null:f,f!==null&&(i.callback=f),n=On(e,i,t),n!==null&&(Nt(n,e,t),Xi(n,e,t))}function Eg(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function Yr(e,t){Eg(e,t),(e=e.alternate)&&Eg(e,t)}function Ag(e){if(e.tag===13){var t=Ll(e,67108864);t!==null&&Nt(t,e,67108864),Yr(e,67108864)}}var Gs=!0;function G0(e,t,n,i){var c=B.T;B.T=null;var f=Z.p;try{Z.p=2,Gr(e,t,n,i)}finally{Z.p=f,B.T=c}}function K0(e,t,n,i){var c=B.T;B.T=null;var f=Z.p;try{Z.p=8,Gr(e,t,n,i)}finally{Z.p=f,B.T=c}}function Gr(e,t,n,i){if(Gs){var c=Kr(i);if(c===null)Dr(e,t,i,Ks,n),Ng(e,i);else if(Q0(c,e,t,n,i))i.stopPropagation();else if(Ng(e,i),t&4&&-1<V0.indexOf(e)){for(;c!==null;){var f=wl(c);if(f!==null)switch(f.tag){case 3:if(f=f.stateNode,f.current.memoizedState.isDehydrated){var g=Xn(f.pendingLanes);if(g!==0){var y=f;for(y.pendingLanes|=2,y.entangledLanes|=2;g;){var v=1<<31-bt(g);y.entanglements[1]|=v,g&=~v}Wt(f),(Se&6)===0&&(_s=Vt()+500,fa(0))}}break;case 13:y=Ll(f,2),y!==null&&Nt(y,f,2),xs(),Yr(f,2)}if(f=Kr(i),f===null&&Dr(e,t,i,Ks,n),f===c)break;c=f}c!==null&&i.stopPropagation()}else Dr(e,t,i,null,n)}}function Kr(e){return e=Ju(e),Vr(e)}var Ks=null;function Vr(e){if(Ks=null,e=Tl(e),e!==null){var t=o(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=h(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return Ks=e,null}function Og(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Dp()){case qf:return 2;case jf:return 8;case Ua:case Cp:return 32;case Hf:return 268435456;default:return 32}default:return 32}}var Qr=!1,qn=null,jn=null,Hn=null,va=new Map,ba=new Map,$n=[],V0="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function Ng(e,t){switch(e){case"focusin":case"focusout":qn=null;break;case"dragenter":case"dragleave":jn=null;break;case"mouseover":case"mouseout":Hn=null;break;case"pointerover":case"pointerout":va.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":ba.delete(t.pointerId)}}function Sa(e,t,n,i,c,f){return e===null||e.nativeEvent!==f?(e={blockedOn:t,domEventName:n,eventSystemFlags:i,nativeEvent:f,targetContainers:[c]},t!==null&&(t=wl(t),t!==null&&Ag(t)),e):(e.eventSystemFlags|=i,t=e.targetContainers,c!==null&&t.indexOf(c)===-1&&t.push(c),e)}function Q0(e,t,n,i,c){switch(t){case"focusin":return qn=Sa(qn,e,t,n,i,c),!0;case"dragenter":return jn=Sa(jn,e,t,n,i,c),!0;case"mouseover":return Hn=Sa(Hn,e,t,n,i,c),!0;case"pointerover":var f=c.pointerId;return va.set(f,Sa(va.get(f)||null,e,t,n,i,c)),!0;case"gotpointercapture":return f=c.pointerId,ba.set(f,Sa(ba.get(f)||null,e,t,n,i,c)),!0}return!1}function _g(e){var t=Tl(e.target);if(t!==null){var n=o(t);if(n!==null){if(t=n.tag,t===13){if(t=h(n),t!==null){e.blockedOn=t,jp(e.priority,function(){if(n.tag===13){var i=Ot();i=qu(i);var c=Ll(n,i);c!==null&&Nt(c,n,i),Yr(n,i)}});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function Vs(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=Kr(e.nativeEvent);if(n===null){n=e.nativeEvent;var i=new n.constructor(n.type,n);Zu=i,n.target.dispatchEvent(i),Zu=null}else return t=wl(n),t!==null&&Ag(t),e.blockedOn=n,!1;t.shift()}return!0}function Mg(e,t,n){Vs(e)&&n.delete(t)}function X0(){Qr=!1,qn!==null&&Vs(qn)&&(qn=null),jn!==null&&Vs(jn)&&(jn=null),Hn!==null&&Vs(Hn)&&(Hn=null),va.forEach(Mg),ba.forEach(Mg)}function Qs(e,t){e.blockedOn===t&&(e.blockedOn=null,Qr||(Qr=!0,u.unstable_scheduleCallback(u.unstable_NormalPriority,X0)))}var Xs=null;function xg(e){Xs!==e&&(Xs=e,u.unstable_scheduleCallback(u.unstable_NormalPriority,function(){Xs===e&&(Xs=null);for(var t=0;t<e.length;t+=3){var n=e[t],i=e[t+1],c=e[t+2];if(typeof i!="function"){if(Vr(i||n)===null)continue;break}var f=wl(n);f!==null&&(e.splice(t,3),t-=3,Vc(f,{pending:!0,data:c,method:n.method,action:i},i,c))}}))}function Ta(e){function t(v){return Qs(v,e)}qn!==null&&Qs(qn,e),jn!==null&&Qs(jn,e),Hn!==null&&Qs(Hn,e),va.forEach(t),ba.forEach(t);for(var n=0;n<$n.length;n++){var i=$n[n];i.blockedOn===e&&(i.blockedOn=null)}for(;0<$n.length&&(n=$n[0],n.blockedOn===null);)_g(n),n.blockedOn===null&&$n.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(i=0;i<n.length;i+=3){var c=n[i],f=n[i+1],g=c[rt]||null;if(typeof f=="function")g||xg(n);else if(g){var y=null;if(f&&f.hasAttribute("formAction")){if(c=f,g=f[rt]||null)y=g.formAction;else if(Vr(c)!==null)continue}else y=g.action;typeof y=="function"?n[i+1]=y:(n.splice(i,3),i-=3),xg(n)}}}function Xr(e){this._internalRoot=e}Zs.prototype.render=Xr.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(s(409));var n=t.current,i=Ot();wg(n,i,e,t,null,null)},Zs.prototype.unmount=Xr.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;wg(e.current,2,null,e,null,null),xs(),t[Sl]=null}};function Zs(e){this._internalRoot=e}Zs.prototype.unstable_scheduleHydration=function(e){if(e){var t=Vf();e={blockedOn:null,target:e,priority:t};for(var n=0;n<$n.length&&t!==0&&t<$n[n].priority;n++);$n.splice(n,0,e),n===0&&_g(e)}};var Dg=l.version;if(Dg!=="19.1.1")throw Error(s(527,Dg,"19.1.1"));Z.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(s(188)):(e=Object.keys(e).join(","),Error(s(268,e)));return e=m(t),e=e!==null?p(e):null,e=e===null?null:e.stateNode,e};var Z0={bundleType:0,version:"19.1.1",rendererPackageName:"react-dom",currentDispatcherRef:B,reconcilerVersion:"19.1.1"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var Js=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!Js.isDisabled&&Js.supportsFiber)try{Oi=Js.inject(Z0),vt=Js}catch{}}return Ea.createRoot=function(e,t){if(!r(e))throw Error(s(299));var n=!1,i="",c=Qh,f=Xh,g=Zh,y=null;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(i=t.identifierPrefix),t.onUncaughtError!==void 0&&(c=t.onUncaughtError),t.onCaughtError!==void 0&&(f=t.onCaughtError),t.onRecoverableError!==void 0&&(g=t.onRecoverableError),t.unstable_transitionCallbacks!==void 0&&(y=t.unstable_transitionCallbacks)),t=Sg(e,1,!1,null,null,n,i,c,f,g,y,null),e[Sl]=t.current,xr(e),new Xr(t)},Ea.hydrateRoot=function(e,t,n){if(!r(e))throw Error(s(299));var i=!1,c="",f=Qh,g=Xh,y=Zh,v=null,x=null;return n!=null&&(n.unstable_strictMode===!0&&(i=!0),n.identifierPrefix!==void 0&&(c=n.identifierPrefix),n.onUncaughtError!==void 0&&(f=n.onUncaughtError),n.onCaughtError!==void 0&&(g=n.onCaughtError),n.onRecoverableError!==void 0&&(y=n.onRecoverableError),n.unstable_transitionCallbacks!==void 0&&(v=n.unstable_transitionCallbacks),n.formState!==void 0&&(x=n.formState)),t=Sg(e,1,!0,t,n??null,i,c,f,g,y,v,x),t.context=Tg(null),n=t.current,i=Ot(),i=qu(i),c=An(i),c.callback=null,On(n,c,i),n=i,t.current.lanes=n,_i(t,n),Wt(t),e[Sl]=t.current,xr(e),new Zs(t)},Ea.version="19.1.1",Ea}var Yg;function h1(){if(Yg)return Wr.exports;Yg=1;function u(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(u)}catch(l){console.error(l)}}return u(),Wr.exports=o1(),Wr.exports}var d1=h1();const g1="modulepreload",m1=function(u){return"/"+u},Gg={},p1=function(l,a,s){let r=Promise.resolve();if(a&&a.length>0){let h=function(p){return Promise.all(p.map(b=>Promise.resolve(b).then(S=>({status:"fulfilled",value:S}),S=>({status:"rejected",reason:S}))))};document.getElementsByTagName("link");const d=document.querySelector("meta[property=csp-nonce]"),m=(d==null?void 0:d.nonce)||(d==null?void 0:d.getAttribute("nonce"));r=h(a.map(p=>{if(p=m1(p),p in Gg)return;Gg[p]=!0;const b=p.endsWith(".css"),S=b?'[rel="stylesheet"]':"";if(document.querySelector(`link[href="${p}"]${S}`))return;const N=document.createElement("link");if(N.rel=b?"stylesheet":g1,b||(N.as="script"),N.crossOrigin="",N.href=p,m&&N.setAttribute("nonce",m),document.head.appendChild(N),b)return new Promise((E,D)=>{N.addEventListener("load",E),N.addEventListener("error",()=>D(new Error(`Unable to preload CSS for ${p}`)))})}))}function o(h){const d=new Event("vite:preloadError",{cancelable:!0});if(d.payload=h,window.dispatchEvent(d),!d.defaultPrevented)throw h}return r.then(h=>{for(const d of h||[])d.status==="rejected"&&o(d.reason);return l().catch(o)})};function y1(u,l){const a=/(\x1b\[(\d+(;\d+)*)m)|([^\x1b]+)/g,s=[];let r,o={},h=!1,d=l==null?void 0:l.fg,m=l==null?void 0:l.bg;for(;(r=a.exec(u))!==null;){const[,,p,,b]=r;if(p){const S=+p;switch(S){case 0:o={};break;case 1:o["font-weight"]="bold";break;case 2:o.opacity="0.8";break;case 3:o["font-style"]="italic";break;case 4:o["text-decoration"]="underline";break;case 7:h=!0;break;case 8:o.display="none";break;case 9:o["text-decoration"]="line-through";break;case 22:delete o["font-weight"],delete o["font-style"],delete o.opacity,delete o["text-decoration"];break;case 23:delete o["font-weight"],delete o["font-style"],delete o.opacity;break;case 24:delete o["text-decoration"];break;case 27:h=!1;break;case 30:case 31:case 32:case 33:case 34:case 35:case 36:case 37:d=Kg[S-30];break;case 39:d=l==null?void 0:l.fg;break;case 40:case 41:case 42:case 43:case 44:case 45:case 46:case 47:m=Kg[S-40];break;case 49:m=l==null?void 0:l.bg;break;case 53:o["text-decoration"]="overline";break;case 90:case 91:case 92:case 93:case 94:case 95:case 96:case 97:d=Vg[S-90];break;case 100:case 101:case 102:case 103:case 104:case 105:case 106:case 107:m=Vg[S-100];break}}else if(b){const S={...o},N=h?m:d;N!==void 0&&(S.color=N);const E=h?d:m;E!==void 0&&(S["background-color"]=E),s.push(`<span style="${b1(S)}">${v1(b)}</span>`)}}return s.join("")}const Kg={0:"var(--vscode-terminal-ansiBlack)",1:"var(--vscode-terminal-ansiRed)",2:"var(--vscode-terminal-ansiGreen)",3:"var(--vscode-terminal-ansiYellow)",4:"var(--vscode-terminal-ansiBlue)",5:"var(--vscode-terminal-ansiMagenta)",6:"var(--vscode-terminal-ansiCyan)",7:"var(--vscode-terminal-ansiWhite)"},Vg={0:"var(--vscode-terminal-ansiBrightBlack)",1:"var(--vscode-terminal-ansiBrightRed)",2:"var(--vscode-terminal-ansiBrightGreen)",3:"var(--vscode-terminal-ansiBrightYellow)",4:"var(--vscode-terminal-ansiBrightBlue)",5:"var(--vscode-terminal-ansiBrightMagenta)",6:"var(--vscode-terminal-ansiBrightCyan)",7:"var(--vscode-terminal-ansiBrightWhite)"};function v1(u){return u.replace(/[&"<>]/g,l=>({"&":"&amp;",'"':"&quot;","<":"&lt;",">":"&gt;"})[l])}function b1(u){return Object.entries(u).map(([l,a])=>`${l}: ${a}`).join("; ")}const ef=({text:u,highlighter:l,mimeType:a,linkify:s,readOnly:r,highlight:o,revealLine:h,lineNumbers:d,isFocused:m,focusOnChange:p,wrapLines:b,onChange:S,dataTestId:N,placeholder:E})=>{const[D,T]=rm(),[w]=me.useState(p1(()=>import("./codeMirrorModule-BoWUGj0J.js"),__vite__mapDeps([0,1])).then(Q=>Q.default)),k=me.useRef(null),[L,H]=me.useState();return me.useEffect(()=>{(async()=>{var V,z;const Q=await w;T1(Q);const G=T.current;if(!G)return;const K=E1(l)||w1(a)||(s?"text/linkified":"");if(k.current&&K===k.current.cm.getOption("mode")&&!!r===k.current.cm.getOption("readOnly")&&d===k.current.cm.getOption("lineNumbers")&&b===k.current.cm.getOption("lineWrapping")&&E===k.current.cm.getOption("placeholder"))return;(z=(V=k.current)==null?void 0:V.cm)==null||z.getWrapperElement().remove();const ee=Q(G,{value:"",mode:K,readOnly:!!r,lineNumbers:d,lineWrapping:b,placeholder:E,matchBrackets:!0,autoCloseBrackets:!0});return k.current={cm:ee},m&&ee.focus(),H(ee),ee})()},[w,L,T,l,a,s,d,b,r,m,E]),me.useEffect(()=>{k.current&&k.current.cm.setSize(D.width,D.height)},[D]),me.useLayoutEffect(()=>{var K;if(!L)return;let Q=!1;if(L.getValue()!==u&&(L.setValue(u),Q=!0,p&&(L.execCommand("selectAll"),L.focus())),Q||JSON.stringify(o)!==JSON.stringify(k.current.highlight)){for(const z of k.current.highlight||[])L.removeLineClass(z.line-1,"wrap");for(const z of o||[])L.addLineClass(z.line-1,"wrap",`source-line-${z.type}`);for(const z of k.current.widgets||[])L.removeLineWidget(z);for(const z of k.current.markers||[])z.clear();const ee=[],V=[];for(const z of o||[]){if(z.type!=="subtle-error"&&z.type!=="error")continue;const F=(K=k.current)==null?void 0:K.cm.getLine(z.line-1);if(F){const se={};se.title=z.message||"",V.push(L.markText({line:z.line-1,ch:0},{line:z.line-1,ch:z.column||F.length},{className:"source-line-error-underline",attributes:se}))}if(z.type==="error"){const se=document.createElement("div");se.innerHTML=y1(z.message||""),se.className="source-line-error-widget",ee.push(L.addLineWidget(z.line,se,{above:!0,coverGutter:!1}))}}k.current.highlight=o,k.current.widgets=ee,k.current.markers=V}typeof h=="number"&&k.current.cm.lineCount()>=h&&L.scrollIntoView({line:Math.max(0,h-1),ch:0},50);let G;return S&&(G=()=>S(L.getValue()),L.on("change",G)),()=>{G&&L.off("change",G)}},[L,u,o,h,p,S]),X.jsx("div",{"data-testid":N,className:"cm-wrapper",ref:T,onClick:S1})};function S1(u){var a;if(!(u.target instanceof HTMLElement))return;let l;u.target.classList.contains("cm-linkified")?l=u.target.textContent:u.target.classList.contains("cm-link")&&((a=u.target.nextElementSibling)!=null&&a.classList.contains("cm-url"))&&(l=u.target.nextElementSibling.textContent.slice(1,-1)),l&&(u.preventDefault(),u.stopPropagation(),window.open(l,"_blank"))}let Qg=!1;function T1(u){Qg||(Qg=!0,u.defineSimpleMode("text/linkified",{start:[{regex:n1,token:"linkified"}]}))}function w1(u){if(u){if(u.includes("javascript")||u.includes("json"))return"javascript";if(u.includes("python"))return"python";if(u.includes("csharp"))return"text/x-csharp";if(u.includes("java"))return"text/x-java";if(u.includes("markdown"))return"markdown";if(u.includes("html")||u.includes("svg"))return"htmlmixed";if(u.includes("css"))return"css"}}function E1(u){if(u)return{javascript:"javascript",jsonl:"javascript",python:"python",csharp:"text/x-csharp",java:"text/x-java",markdown:"markdown",html:"htmlmixed",css:"css",yaml:"yaml"}[u]}const A1=50,O1=({sidebarSize:u,sidebarHidden:l=!1,sidebarIsFirst:a=!1,orientation:s="vertical",minSidebarSize:r=A1,settingName:o,sidebar:h,main:d})=>{const m=Math.max(r,u)*window.devicePixelRatio,[p,b]=fu(o?o+"."+s+":size":void 0,m),[S,N]=fu(o?o+"."+s+":size":void 0,m),[E,D]=me.useState(null),[T,w]=rm();let k;s==="vertical"?(k=S/window.devicePixelRatio,T&&T.height<k&&(k=T.height-10)):(k=p/window.devicePixelRatio,T&&T.width<k&&(k=T.width-10)),document.body.style.userSelect=E?"none":"inherit";let L={};return s==="vertical"?a?L={top:E?0:k-4,bottom:E?0:void 0,height:E?"initial":8}:L={bottom:E?0:k-4,top:E?0:void 0,height:E?"initial":8}:a?L={left:E?0:k-4,right:E?0:void 0,width:E?"initial":8}:L={right:E?0:k-4,left:E?0:void 0,width:E?"initial":8},X.jsxs("div",{className:pl("split-view",s,a&&"sidebar-first"),ref:w,children:[X.jsx("div",{className:"split-view-main",children:d}),!l&&X.jsx("div",{style:{flexBasis:k},className:"split-view-sidebar",children:h}),!l&&X.jsx("div",{style:L,className:"split-view-resizer",onMouseDown:H=>D({offset:s==="vertical"?H.clientY:H.clientX,size:k}),onMouseUp:()=>D(null),onMouseMove:H=>{if(!H.buttons)D(null);else if(E){const G=(s==="vertical"?H.clientY:H.clientX)-E.offset,K=a?E.size+G:E.size-G,V=H.target.parentElement.getBoundingClientRect(),z=Math.min(Math.max(r,K),(s==="vertical"?V.height:V.width)-r);s==="vertical"?N(z*window.devicePixelRatio):b(z*window.devicePixelRatio)}}})]})},fm=({noShadow:u,children:l,noMinHeight:a,className:s,sidebarBackground:r,onClick:o})=>X.jsx("div",{className:pl("toolbar",u&&"no-shadow",a&&"no-min-height",s,r&&"toolbar-sidebar-background"),onClick:o,children:l}),N1=({tabs:u,selectedTab:l,setSelectedTab:a,leftToolbar:s,rightToolbar:r,dataTestId:o,mode:h})=>{const d=me.useId();return l||(l=u[0].id),h||(h="default"),X.jsx("div",{className:"tabbed-pane","data-testid":o,children:X.jsxs("div",{className:"vbox",children:[X.jsxs(fm,{children:[s&&X.jsxs("div",{style:{flex:"none",display:"flex",margin:"0 4px",alignItems:"center"},children:[...s]}),h==="default"&&X.jsx("div",{style:{flex:"auto",display:"flex",height:"100%",overflow:"hidden"},role:"tablist",children:[...u.map(m=>X.jsx(_1,{id:m.id,ariaControls:`${d}-${m.id}`,title:m.title,count:m.count,errorCount:m.errorCount,selected:l===m.id,onSelect:a},m.id))]}),h==="select"&&X.jsx("div",{style:{flex:"auto",display:"flex",height:"100%",overflow:"hidden"},role:"tablist",children:X.jsx("select",{style:{width:"100%",background:"none",cursor:"pointer"},value:l,onChange:m=>{a==null||a(u[m.currentTarget.selectedIndex].id)},children:u.map(m=>{let p="";return m.count&&(p=` (${m.count})`),m.errorCount&&(p=` (${m.errorCount})`),X.jsxs("option",{value:m.id,role:"tab","aria-controls":`${d}-${m.id}`,children:[m.title,p]},m.id)})})}),r&&X.jsxs("div",{style:{flex:"none",display:"flex",alignItems:"center"},children:[...r]})]}),u.map(m=>{const p="tab-content tab-"+m.id;if(m.component)return X.jsx("div",{id:`${d}-${m.id}`,role:"tabpanel","aria-label":m.title,className:p,style:{display:l===m.id?"inherit":"none"},children:m.component},m.id);if(l===m.id)return X.jsx("div",{id:`${d}-${m.id}`,role:"tabpanel","aria-label":m.title,className:p,children:m.render()},m.id)})]})})},_1=({id:u,title:l,count:a,errorCount:s,selected:r,onSelect:o,ariaControls:h})=>X.jsxs("div",{className:pl("tabbed-pane-tab",r&&"selected"),onClick:()=>o==null?void 0:o(u),role:"tab",title:l,"aria-controls":h,"aria-selected":r,children:[X.jsx("div",{className:"tabbed-pane-tab-label",children:l}),!!a&&X.jsx("div",{className:"tabbed-pane-tab-counter",children:a}),!!s&&X.jsx("div",{className:"tabbed-pane-tab-counter error",children:s})]}),M1=({sources:u,fileId:l,setFileId:a})=>X.jsx("select",{className:"source-chooser",hidden:!u.length,title:"Source chooser",value:l,onChange:s=>{a(s.target.selectedOptions[0].value)},children:x1(u)});function x1(u){const l=r=>r.replace(/.*[/\\]([^/\\]+)/,"$1"),a=r=>X.jsx("option",{value:r.id,children:l(r.label)},r.id),s=new Map;for(const r of u){let o=s.get(r.group||"Debugger");o||(o=[],s.set(r.group||"Debugger",o)),o.push(r)}return[...s.entries()].map(([r,o])=>X.jsx("optgroup",{label:r,children:o.filter(h=>(h.group||"Debugger")===r).map(h=>a(h))},r))}function D1(){return{id:"default",isRecorded:!1,text:"",language:"javascript",label:"",highlight:[]}}const _t=me.forwardRef(function({children:l,title:a="",icon:s,disabled:r=!1,toggled:o=!1,onClick:h=()=>{},style:d,testId:m,className:p,ariaLabel:b},S){return X.jsxs("button",{ref:S,className:pl(p,"toolbar-button",s,o&&"toggled"),onMouseDown:Zg,onClick:h,onDoubleClick:Zg,title:a,disabled:!!r,style:d,"data-testid":m,"aria-label":b||a,children:[s&&X.jsx("span",{className:`codicon codicon-${s}`,style:l?{marginRight:5}:{}}),l]})}),Xg=({style:u})=>X.jsx("div",{className:"toolbar-separator",style:u}),Zg=u=>{u.stopPropagation(),u.preventDefault()},Ke=function(u,l,a){return u>=l&&u<=a};function gt(u){return Ke(u,48,57)}function Jg(u){return gt(u)||Ke(u,65,70)||Ke(u,97,102)}function C1(u){return Ke(u,65,90)}function R1(u){return Ke(u,97,122)}function k1(u){return C1(u)||R1(u)}function L1(u){return u>=128}function tu(u){return k1(u)||L1(u)||u===95}function Wg(u){return tu(u)||gt(u)||u===45}function z1(u){return Ke(u,0,8)||u===11||Ke(u,14,31)||u===127}function nu(u){return u===10}function mn(u){return nu(u)||u===9||u===32}const U1=1114111;class wf extends Error{constructor(l){super(l),this.name="InvalidCharacterError"}}function B1(u){const l=[];for(let a=0;a<u.length;a++){let s=u.charCodeAt(a);if(s===13&&u.charCodeAt(a+1)===10&&(s=10,a++),(s===13||s===12)&&(s=10),s===0&&(s=65533),Ke(s,55296,56319)&&Ke(u.charCodeAt(a+1),56320,57343)){const r=s-55296,o=u.charCodeAt(a+1)-56320;s=Math.pow(2,16)+r*Math.pow(2,10)+o,a++}l.push(s)}return l}function Xe(u){if(u<=65535)return String.fromCharCode(u);u-=Math.pow(2,16);const l=Math.floor(u/Math.pow(2,10))+55296,a=u%Math.pow(2,10)+56320;return String.fromCharCode(l)+String.fromCharCode(a)}function q1(u){const l=B1(u);let a=-1;const s=[];let r;const o=function(U){return U>=l.length?-1:l[U]},h=function(U){if(U===void 0&&(U=1),U>3)throw"Spec Error: no more than three codepoints of lookahead.";return o(a+U)},d=function(U){return U===void 0&&(U=1),a+=U,r=o(a),!0},m=function(){return a-=1,!0},p=function(U){return U===void 0&&(U=r),U===-1},b=function(){if(S(),d(),mn(r)){for(;mn(h());)d();return new df}else{if(r===34)return D();if(r===35)if(Wg(h())||k(h(1),h(2))){const U=new Om("");return H(h(1),h(2),h(3))&&(U.type="id"),U.value=ee(),U}else return new at(r);else return r===36?h()===61?(d(),new Y1):new at(r):r===39?D():r===40?new Sm:r===41?new Tm:r===42?h()===61?(d(),new G1):new at(r):r===43?K()?(m(),N()):new at(r):r===44?new pm:r===45?K()?(m(),N()):h(1)===45&&h(2)===62?(d(2),new dm):Q()?(m(),E()):new at(r):r===46?K()?(m(),N()):new at(r):r===58?new gm:r===59?new mm:r===60?h(1)===33&&h(2)===45&&h(3)===45?(d(3),new hm):new at(r):r===64?H(h(1),h(2),h(3))?new Am(ee()):new at(r):r===91?new bm:r===92?L()?(m(),E()):new at(r):r===93?new gf:r===94?h()===61?(d(),new $1):new at(r):r===123?new ym:r===124?h()===61?(d(),new H1):h()===124?(d(),new wm):new at(r):r===125?new vm:r===126?h()===61?(d(),new j1):new at(r):gt(r)?(m(),N()):tu(r)?(m(),E()):p()?new iu:new at(r)}},S=function(){for(;h(1)===47&&h(2)===42;)for(d(2);;)if(d(),r===42&&h()===47){d();break}else if(p())return},N=function(){const U=V();if(H(h(1),h(2),h(3))){const te=new K1;return te.value=U.value,te.repr=U.repr,te.type=U.type,te.unit=ee(),te}else if(h()===37){d();const te=new xm;return te.value=U.value,te.repr=U.repr,te}else{const te=new Mm;return te.value=U.value,te.repr=U.repr,te.type=U.type,te}},E=function(){const U=ee();if(U.toLowerCase()==="url"&&h()===40){for(d();mn(h(1))&&mn(h(2));)d();return h()===34||h()===39?new au(U):mn(h())&&(h(2)===34||h(2)===39)?new au(U):T()}else return h()===40?(d(),new au(U)):new Em(U)},D=function(U){U===void 0&&(U=r);let te="";for(;d();){if(r===U||p())return new Nm(te);if(nu(r))return m(),new om;r===92?p(h())||(nu(h())?d():te+=Xe(w())):te+=Xe(r)}throw new Error("Internal error")},T=function(){const U=new _m("");for(;mn(h());)d();if(p(h()))return U;for(;d();){if(r===41||p())return U;if(mn(r)){for(;mn(h());)d();return h()===41||p(h())?(d(),U):(F(),new lu)}else{if(r===34||r===39||r===40||z1(r))return F(),new lu;if(r===92)if(L())U.value+=Xe(w());else return F(),new lu;else U.value+=Xe(r)}}throw new Error("Internal error")},w=function(){if(d(),Jg(r)){const U=[r];for(let xe=0;xe<5&&Jg(h());xe++)d(),U.push(r);mn(h())&&d();let te=parseInt(U.map(function(xe){return String.fromCharCode(xe)}).join(""),16);return te>U1&&(te=65533),te}else return p()?65533:r},k=function(U,te){return!(U!==92||nu(te))},L=function(){return k(r,h())},H=function(U,te,xe){return U===45?tu(te)||te===45||k(te,xe):tu(U)?!0:U===92?k(U,te):!1},Q=function(){return H(r,h(1),h(2))},G=function(U,te,xe){return U===43||U===45?!!(gt(te)||te===46&&gt(xe)):U===46?!!gt(te):!!gt(U)},K=function(){return G(r,h(1),h(2))},ee=function(){let U="";for(;d();)if(Wg(r))U+=Xe(r);else if(L())U+=Xe(w());else return m(),U;throw new Error("Internal parse error")},V=function(){let U="",te="integer";for((h()===43||h()===45)&&(d(),U+=Xe(r));gt(h());)d(),U+=Xe(r);if(h(1)===46&&gt(h(2)))for(d(),U+=Xe(r),d(),U+=Xe(r),te="number";gt(h());)d(),U+=Xe(r);const xe=h(1),qe=h(2),B=h(3);if((xe===69||xe===101)&&gt(qe))for(d(),U+=Xe(r),d(),U+=Xe(r),te="number";gt(h());)d(),U+=Xe(r);else if((xe===69||xe===101)&&(qe===43||qe===45)&&gt(B))for(d(),U+=Xe(r),d(),U+=Xe(r),d(),U+=Xe(r),te="number";gt(h());)d(),U+=Xe(r);const Z=z(U);return{type:te,value:Z,repr:U}},z=function(U){return+U},F=function(){for(;d();){if(r===41||p())return;L()&&w()}};let se=0;for(;!p(h());)if(s.push(b()),se++,se>l.length*2)throw new Error("I'm infinite-looping!");return s}class $e{constructor(){this.tokenType=""}toJSON(){return{token:this.tokenType}}toString(){return this.tokenType}toSource(){return""+this}}class om extends $e{constructor(){super(...arguments),this.tokenType="BADSTRING"}}class lu extends $e{constructor(){super(...arguments),this.tokenType="BADURL"}}class df extends $e{constructor(){super(...arguments),this.tokenType="WHITESPACE"}toString(){return"WS"}toSource(){return" "}}class hm extends $e{constructor(){super(...arguments),this.tokenType="CDO"}toSource(){return"<!--"}}class dm extends $e{constructor(){super(...arguments),this.tokenType="CDC"}toSource(){return"-->"}}class gm extends $e{constructor(){super(...arguments),this.tokenType=":"}}class mm extends $e{constructor(){super(...arguments),this.tokenType=";"}}class pm extends $e{constructor(){super(...arguments),this.tokenType=","}}class yi extends $e{constructor(){super(...arguments),this.value="",this.mirror=""}}class ym extends yi{constructor(){super(),this.tokenType="{",this.value="{",this.mirror="}"}}class vm extends yi{constructor(){super(),this.tokenType="}",this.value="}",this.mirror="{"}}class bm extends yi{constructor(){super(),this.tokenType="[",this.value="[",this.mirror="]"}}class gf extends yi{constructor(){super(),this.tokenType="]",this.value="]",this.mirror="["}}class Sm extends yi{constructor(){super(),this.tokenType="(",this.value="(",this.mirror=")"}}class Tm extends yi{constructor(){super(),this.tokenType=")",this.value=")",this.mirror="("}}class j1 extends $e{constructor(){super(...arguments),this.tokenType="~="}}class H1 extends $e{constructor(){super(...arguments),this.tokenType="|="}}class $1 extends $e{constructor(){super(...arguments),this.tokenType="^="}}class Y1 extends $e{constructor(){super(...arguments),this.tokenType="$="}}class G1 extends $e{constructor(){super(...arguments),this.tokenType="*="}}class wm extends $e{constructor(){super(...arguments),this.tokenType="||"}}class iu extends $e{constructor(){super(...arguments),this.tokenType="EOF"}toSource(){return""}}class at extends $e{constructor(l){super(),this.tokenType="DELIM",this.value="",this.value=Xe(l)}toString(){return"DELIM("+this.value+")"}toJSON(){const l=this.constructor.prototype.constructor.prototype.toJSON.call(this);return l.value=this.value,l}toSource(){return this.value==="\\"?`\\
`:this.value}}class vi extends $e{constructor(){super(...arguments),this.value=""}ASCIIMatch(l){return this.value.toLowerCase()===l.toLowerCase()}toJSON(){const l=this.constructor.prototype.constructor.prototype.toJSON.call(this);return l.value=this.value,l}}class Em extends vi{constructor(l){super(),this.tokenType="IDENT",this.value=l}toString(){return"IDENT("+this.value+")"}toSource(){return Ca(this.value)}}class au extends vi{constructor(l){super(),this.tokenType="FUNCTION",this.value=l,this.mirror=")"}toString(){return"FUNCTION("+this.value+")"}toSource(){return Ca(this.value)+"("}}class Am extends vi{constructor(l){super(),this.tokenType="AT-KEYWORD",this.value=l}toString(){return"AT("+this.value+")"}toSource(){return"@"+Ca(this.value)}}class Om extends vi{constructor(l){super(),this.tokenType="HASH",this.value=l,this.type="unrestricted"}toString(){return"HASH("+this.value+")"}toJSON(){const l=this.constructor.prototype.constructor.prototype.toJSON.call(this);return l.value=this.value,l.type=this.type,l}toSource(){return this.type==="id"?"#"+Ca(this.value):"#"+V1(this.value)}}class Nm extends vi{constructor(l){super(),this.tokenType="STRING",this.value=l}toString(){return'"'+Dm(this.value)+'"'}}class _m extends vi{constructor(l){super(),this.tokenType="URL",this.value=l}toString(){return"URL("+this.value+")"}toSource(){return'url("'+Dm(this.value)+'")'}}class Mm extends $e{constructor(){super(),this.tokenType="NUMBER",this.type="integer",this.repr=""}toString(){return this.type==="integer"?"INT("+this.value+")":"NUMBER("+this.value+")"}toJSON(){const l=super.toJSON();return l.value=this.value,l.type=this.type,l.repr=this.repr,l}toSource(){return this.repr}}class xm extends $e{constructor(){super(),this.tokenType="PERCENTAGE",this.repr=""}toString(){return"PERCENTAGE("+this.value+")"}toJSON(){const l=this.constructor.prototype.constructor.prototype.toJSON.call(this);return l.value=this.value,l.repr=this.repr,l}toSource(){return this.repr+"%"}}class K1 extends $e{constructor(){super(),this.tokenType="DIMENSION",this.type="integer",this.repr="",this.unit=""}toString(){return"DIM("+this.value+","+this.unit+")"}toJSON(){const l=this.constructor.prototype.constructor.prototype.toJSON.call(this);return l.value=this.value,l.type=this.type,l.repr=this.repr,l.unit=this.unit,l}toSource(){const l=this.repr;let a=Ca(this.unit);return a[0].toLowerCase()==="e"&&(a[1]==="-"||Ke(a.charCodeAt(1),48,57))&&(a="\\65 "+a.slice(1,a.length)),l+a}}function Ca(u){u=""+u;let l="";const a=u.charCodeAt(0);for(let s=0;s<u.length;s++){const r=u.charCodeAt(s);if(r===0)throw new wf("Invalid character: the input contains U+0000.");Ke(r,1,31)||r===127||s===0&&Ke(r,48,57)||s===1&&Ke(r,48,57)&&a===45?l+="\\"+r.toString(16)+" ":r>=128||r===45||r===95||Ke(r,48,57)||Ke(r,65,90)||Ke(r,97,122)?l+=u[s]:l+="\\"+u[s]}return l}function V1(u){u=""+u;let l="";for(let a=0;a<u.length;a++){const s=u.charCodeAt(a);if(s===0)throw new wf("Invalid character: the input contains U+0000.");s>=128||s===45||s===95||Ke(s,48,57)||Ke(s,65,90)||Ke(s,97,122)?l+=u[a]:l+="\\"+s.toString(16)+" "}return l}function Dm(u){u=""+u;let l="";for(let a=0;a<u.length;a++){const s=u.charCodeAt(a);if(s===0)throw new wf("Invalid character: the input contains U+0000.");Ke(s,1,31)||s===127?l+="\\"+s.toString(16)+" ":s===34||s===92?l+="\\"+u[a]:l+=u[a]}return l}class mt extends Error{}function Q1(u,l){let a;try{a=q1(u),a[a.length-1]instanceof iu||a.push(new iu)}catch(z){const F=z.message+` while parsing css selector "${u}". Did you mean to CSS.escape it?`,se=(z.stack||"").indexOf(z.message);throw se!==-1&&(z.stack=z.stack.substring(0,se)+F+z.stack.substring(se+z.message.length)),z.message=F,z}const s=a.find(z=>z instanceof Am||z instanceof om||z instanceof lu||z instanceof wm||z instanceof hm||z instanceof dm||z instanceof mm||z instanceof ym||z instanceof vm||z instanceof _m||z instanceof xm);if(s)throw new mt(`Unsupported token "${s.toSource()}" while parsing css selector "${u}". Did you mean to CSS.escape it?`);let r=0;const o=new Set;function h(){return new mt(`Unexpected token "${a[r].toSource()}" while parsing css selector "${u}". Did you mean to CSS.escape it?`)}function d(){for(;a[r]instanceof df;)r++}function m(z=r){return a[z]instanceof Em}function p(z=r){return a[z]instanceof Nm}function b(z=r){return a[z]instanceof Mm}function S(z=r){return a[z]instanceof pm}function N(z=r){return a[z]instanceof Sm}function E(z=r){return a[z]instanceof Tm}function D(z=r){return a[z]instanceof au}function T(z=r){return a[z]instanceof at&&a[z].value==="*"}function w(z=r){return a[z]instanceof iu}function k(z=r){return a[z]instanceof at&&[">","+","~"].includes(a[z].value)}function L(z=r){return S(z)||E(z)||w(z)||k(z)||a[z]instanceof df}function H(){const z=[Q()];for(;d(),!!S();)r++,z.push(Q());return z}function Q(){return d(),b()||p()?a[r++].value:G()}function G(){const z={simples:[]};for(d(),k()?z.simples.push({selector:{functions:[{name:"scope",args:[]}]},combinator:""}):z.simples.push({selector:K(),combinator:""});;){if(d(),k())z.simples[z.simples.length-1].combinator=a[r++].value,d();else if(L())break;z.simples.push({combinator:"",selector:K()})}return z}function K(){let z="";const F=[];for(;!L();)if(m()||T())z+=a[r++].toSource();else if(a[r]instanceof Om)z+=a[r++].toSource();else if(a[r]instanceof at&&a[r].value===".")if(r++,m())z+="."+a[r++].toSource();else throw h();else if(a[r]instanceof gm)if(r++,m())if(!l.has(a[r].value.toLowerCase()))z+=":"+a[r++].toSource();else{const se=a[r++].value.toLowerCase();F.push({name:se,args:[]}),o.add(se)}else if(D()){const se=a[r++].value.toLowerCase();if(l.has(se)?(F.push({name:se,args:H()}),o.add(se)):z+=`:${se}(${ee()})`,d(),!E())throw h();r++}else throw h();else if(a[r]instanceof bm){for(z+="[",r++;!(a[r]instanceof gf)&&!w();)z+=a[r++].toSource();if(!(a[r]instanceof gf))throw h();z+="]",r++}else throw h();if(!z&&!F.length)throw h();return{css:z||void 0,functions:F}}function ee(){let z="",F=1;for(;!w()&&((N()||D())&&F++,E()&&F--,!!F);)z+=a[r++].toSource();return z}const V=H();if(!w())throw h();if(V.some(z=>typeof z!="object"||!("simples"in z)))throw new mt(`Error while parsing css selector "${u}". Did you mean to CSS.escape it?`);return{selector:V,names:Array.from(o)}}const Ig=new Set(["internal:has","internal:has-not","internal:and","internal:or","internal:chain","left-of","right-of","above","below","near"]),X1=new Set(["left-of","right-of","above","below","near"]),Z1=new Set(["not","is","where","has","scope","light","visible","text","text-matches","text-is","has-text","above","below","right-of","left-of","near","nth-match"]);function Cm(u){const l=W1(u),a=[];for(const s of l.parts){if(s.name==="css"||s.name==="css:light"){s.name==="css:light"&&(s.body=":light("+s.body+")");const r=Q1(s.body,Z1);a.push({name:"css",body:r.selector,source:s.body});continue}if(Ig.has(s.name)){let r,o;try{const p=JSON.parse("["+s.body+"]");if(!Array.isArray(p)||p.length<1||p.length>2||typeof p[0]!="string")throw new mt(`Malformed selector: ${s.name}=`+s.body);if(r=p[0],p.length===2){if(typeof p[1]!="number"||!X1.has(s.name))throw new mt(`Malformed selector: ${s.name}=`+s.body);o=p[1]}}catch{throw new mt(`Malformed selector: ${s.name}=`+s.body)}const h={name:s.name,source:s.body,body:{parsed:Cm(r),distance:o}},d=[...h.body.parsed.parts].reverse().find(p=>p.name==="internal:control"&&p.body==="enter-frame"),m=d?h.body.parsed.parts.indexOf(d):-1;m!==-1&&J1(h.body.parsed.parts.slice(0,m+1),a.slice(0,m+1))&&h.body.parsed.parts.splice(0,m+1),a.push(h);continue}a.push({...s,source:s.body})}if(Ig.has(a[0].name))throw new mt(`"${a[0].name}" selector cannot be first`);return{capture:l.capture,parts:a}}function J1(u,l){return ci({parts:u})===ci({parts:l})}function ci(u,l){return typeof u=="string"?u:u.parts.map((a,s)=>{let r=!0;!l&&s!==u.capture&&(a.name==="css"||a.name==="xpath"&&a.source.startsWith("//")||a.source.startsWith(".."))&&(r=!1);const o=r?a.name+"=":"";return`${s===u.capture?"*":""}${o}${a.source}`}).join(" >> ")}function W1(u){let l=0,a,s=0;const r={parts:[]},o=()=>{const d=u.substring(s,l).trim(),m=d.indexOf("=");let p,b;m!==-1&&d.substring(0,m).trim().match(/^[a-zA-Z_0-9-+:*]+$/)?(p=d.substring(0,m).trim(),b=d.substring(m+1)):d.length>1&&d[0]==='"'&&d[d.length-1]==='"'||d.length>1&&d[0]==="'"&&d[d.length-1]==="'"?(p="text",b=d):/^\(*\/\//.test(d)||d.startsWith("..")?(p="xpath",b=d):(p="css",b=d);let S=!1;if(p[0]==="*"&&(S=!0,p=p.substring(1)),r.parts.push({name:p,body:b}),S){if(r.capture!==void 0)throw new mt("Only one of the selectors can capture using * modifier");r.capture=r.parts.length-1}};if(!u.includes(">>"))return l=u.length,o(),r;const h=()=>{const m=u.substring(s,l).match(/^\s*text\s*=(.*)$/);return!!m&&!!m[1]};for(;l<u.length;){const d=u[l];d==="\\"&&l+1<u.length?l+=2:d===a?(a=void 0,l++):!a&&(d==='"'||d==="'"||d==="`")&&!h()?(a=d,l++):!a&&d===">"&&u[l+1]===">"?(o(),l+=2,s=l):l++}return o(),r}function tf(u,l){let a=0,s=u.length===0;const r=()=>u[a]||"",o=()=>{const w=r();return++a,s=a>=u.length,w},h=w=>{throw s?new mt(`Unexpected end of selector while parsing selector \`${u}\``):new mt(`Error while parsing selector \`${u}\` - unexpected symbol "${r()}" at position ${a}`+(w?" during "+w:""))};function d(){for(;!s&&/\s/.test(r());)o()}function m(w){return w>=""||w>="0"&&w<="9"||w>="A"&&w<="Z"||w>="a"&&w<="z"||w>="0"&&w<="9"||w==="_"||w==="-"}function p(){let w="";for(d();!s&&m(r());)w+=o();return w}function b(w){let k=o();for(k!==w&&h("parsing quoted string");!s&&r()!==w;)r()==="\\"&&o(),k+=o();return r()!==w&&h("parsing quoted string"),k+=o(),k}function S(){o()!=="/"&&h("parsing regular expression");let w="",k=!1;for(;!s;){if(r()==="\\")w+=o(),s&&h("parsing regular expression");else if(k&&r()==="]")k=!1;else if(!k&&r()==="[")k=!0;else if(!k&&r()==="/")break;w+=o()}o()!=="/"&&h("parsing regular expression");let L="";for(;!s&&r().match(/[dgimsuy]/);)L+=o();try{return new RegExp(w,L)}catch(H){throw new mt(`Error while parsing selector \`${u}\`: ${H.message}`)}}function N(){let w="";return d(),r()==="'"||r()==='"'?w=b(r()).slice(1,-1):w=p(),w||h("parsing property path"),w}function E(){d();let w="";return s||(w+=o()),!s&&w!=="="&&(w+=o()),["=","*=","^=","$=","|=","~="].includes(w)||h("parsing operator"),w}function D(){o();const w=[];for(w.push(N()),d();r()===".";)o(),w.push(N()),d();if(r()==="]")return o(),{name:w.join("."),jsonPath:w,op:"<truthy>",value:null,caseSensitive:!1};const k=E();let L,H=!0;if(d(),r()==="/"){if(k!=="=")throw new mt(`Error while parsing selector \`${u}\` - cannot use ${k} in attribute with regular expression`);L=S()}else if(r()==="'"||r()==='"')L=b(r()).slice(1,-1),d(),r()==="i"||r()==="I"?(H=!1,o()):(r()==="s"||r()==="S")&&(H=!0,o());else{for(L="";!s&&(m(r())||r()==="+"||r()===".");)L+=o();L==="true"?L=!0:L==="false"&&(L=!1)}if(d(),r()!=="]"&&h("parsing attribute value"),o(),k!=="="&&typeof L!="string")throw new mt(`Error while parsing selector \`${u}\` - cannot use ${k} in attribute with non-string matching value - ${L}`);return{name:w.join("."),jsonPath:w,op:k,value:L,caseSensitive:H}}const T={name:"",attributes:[]};for(T.name=p(),d();r()==="[";)T.attributes.push(D()),d();if(s||h(void 0),!T.name&&!T.attributes.length)throw new mt(`Error while parsing selector \`${u}\` - selector cannot be empty`);return T}function yu(u,l="'"){const a=JSON.stringify(u),s=a.substring(1,a.length-1).replace(/\\"/g,'"');if(l==="'")return l+s.replace(/[']/g,"\\'")+l;if(l==='"')return l+s.replace(/["]/g,'\\"')+l;if(l==="`")return l+s.replace(/[`]/g,"\\`")+l;throw new Error("Invalid escape char")}function ou(u){return u.charAt(0).toUpperCase()+u.substring(1)}function Rm(u){return u.replace(/([a-z0-9])([A-Z])/g,"$1_$2").replace(/([A-Z])([A-Z][a-z])/g,"$1_$2").toLowerCase()}function vu(u){return u.replace(/(^|[^\\])(\\\\)*\\(['"`])/g,"$1$2$3")}function km(u,l,a=!1){return I1(u,l,a,1)[0]}function I1(u,l,a=!1,s=20,r){try{return ui(new iv[u](r),Cm(l),a,s)}catch{return[l]}}function ui(u,l,a=!1,s=20){const r=[...l.parts],o=[];let h=a?"frame-locator":"page";for(let d=0;d<r.length;d++){const m=r[d],p=h;if(h="locator",m.name==="internal:describe")continue;if(m.name==="nth"){m.body==="0"?o.push([u.generateLocator(p,"first",""),u.generateLocator(p,"nth","0")]):m.body==="-1"?o.push([u.generateLocator(p,"last",""),u.generateLocator(p,"nth","-1")]):o.push([u.generateLocator(p,"nth",m.body)]);continue}if(m.name==="visible"){o.push([u.generateLocator(p,"visible",m.body),u.generateLocator(p,"default",`visible=${m.body}`)]);continue}if(m.name==="internal:text"){const{exact:D,text:T}=Aa(m.body);o.push([u.generateLocator(p,"text",T,{exact:D})]);continue}if(m.name==="internal:has-text"){const{exact:D,text:T}=Aa(m.body);if(!D){o.push([u.generateLocator(p,"has-text",T,{exact:D})]);continue}}if(m.name==="internal:has-not-text"){const{exact:D,text:T}=Aa(m.body);if(!D){o.push([u.generateLocator(p,"has-not-text",T,{exact:D})]);continue}}if(m.name==="internal:has"){const D=ui(u,m.body.parsed,!1,s);o.push(D.map(T=>u.generateLocator(p,"has",T)));continue}if(m.name==="internal:has-not"){const D=ui(u,m.body.parsed,!1,s);o.push(D.map(T=>u.generateLocator(p,"hasNot",T)));continue}if(m.name==="internal:and"){const D=ui(u,m.body.parsed,!1,s);o.push(D.map(T=>u.generateLocator(p,"and",T)));continue}if(m.name==="internal:or"){const D=ui(u,m.body.parsed,!1,s);o.push(D.map(T=>u.generateLocator(p,"or",T)));continue}if(m.name==="internal:chain"){const D=ui(u,m.body.parsed,!1,s);o.push(D.map(T=>u.generateLocator(p,"chain",T)));continue}if(m.name==="internal:label"){const{exact:D,text:T}=Aa(m.body);o.push([u.generateLocator(p,"label",T,{exact:D})]);continue}if(m.name==="internal:role"){const D=tf(m.body),T={attrs:[]};for(const w of D.attributes)w.name==="name"?(T.exact=w.caseSensitive,T.name=w.value):(w.name==="level"&&typeof w.value=="string"&&(w.value=+w.value),T.attrs.push({name:w.name==="include-hidden"?"includeHidden":w.name,value:w.value}));o.push([u.generateLocator(p,"role",D.name,T)]);continue}if(m.name==="internal:testid"){const D=tf(m.body),{value:T}=D.attributes[0];o.push([u.generateLocator(p,"test-id",T)]);continue}if(m.name==="internal:attr"){const D=tf(m.body),{name:T,value:w,caseSensitive:k}=D.attributes[0],L=w,H=!!k;if(T==="placeholder"){o.push([u.generateLocator(p,"placeholder",L,{exact:H})]);continue}if(T==="alt"){o.push([u.generateLocator(p,"alt",L,{exact:H})]);continue}if(T==="title"){o.push([u.generateLocator(p,"title",L,{exact:H})]);continue}}if(m.name==="internal:control"&&m.body==="enter-frame"){const D=o[o.length-1],T=r[d-1],w=D.map(k=>u.chainLocators([k,u.generateLocator(p,"frame","")]));["xpath","css"].includes(T.name)&&w.push(u.generateLocator(p,"frame-locator",ci({parts:[T]})),u.generateLocator(p,"frame-locator",ci({parts:[T]},!0))),D.splice(0,D.length,...w),h="frame-locator";continue}const b=r[d+1],S=ci({parts:[m]}),N=u.generateLocator(p,"default",S);if(b&&["internal:has-text","internal:has-not-text"].includes(b.name)){const{exact:D,text:T}=Aa(b.body);if(!D){const w=u.generateLocator("locator",b.name==="internal:has-text"?"has-text":"has-not-text",T,{exact:D}),k={};b.name==="internal:has-text"?k.hasText=T:k.hasNotText=T;const L=u.generateLocator(p,"default",S,k);o.push([u.chainLocators([N,w]),L]),d++;continue}}let E;if(["xpath","css"].includes(m.name)){const D=ci({parts:[m]},!0);E=u.generateLocator(p,"default",D)}o.push([N,E].filter(Boolean))}return F1(u,o,s)}function F1(u,l,a){const s=l.map(()=>""),r=[],o=h=>{if(h===l.length)return r.push(u.chainLocators(s)),r.length<a;for(const d of l[h])if(s[h]=d,!o(h+1))return!1;return!0};return o(0),r}function Aa(u){let l=!1;const a=u.match(/^\/(.*)\/([igm]*)$/);return a?{text:new RegExp(a[1],a[2])}:(u.endsWith('"')?(u=JSON.parse(u),l=!0):u.endsWith('"s')?(u=JSON.parse(u.substring(0,u.length-1)),l=!0):u.endsWith('"i')&&(u=JSON.parse(u.substring(0,u.length-1)),l=!1),{exact:l,text:u})}class P1{constructor(l){this.preferredQuote=l}generateLocator(l,a,s,r={}){switch(a){case"default":return r.hasText!==void 0?`locator(${this.quote(s)}, { hasText: ${this.toHasText(r.hasText)} })`:r.hasNotText!==void 0?`locator(${this.quote(s)}, { hasNotText: ${this.toHasText(r.hasNotText)} })`:`locator(${this.quote(s)})`;case"frame-locator":return`frameLocator(${this.quote(s)})`;case"frame":return"contentFrame()";case"nth":return`nth(${s})`;case"first":return"first()";case"last":return"last()";case"visible":return`filter({ visible: ${s==="true"?"true":"false"} })`;case"role":const o=[];tt(r.name)?o.push(`name: ${this.regexToSourceString(r.name)}`):typeof r.name=="string"&&(o.push(`name: ${this.quote(r.name)}`),r.exact&&o.push("exact: true"));for(const{name:d,value:m}of r.attrs)o.push(`${d}: ${typeof m=="string"?this.quote(m):m}`);const h=o.length?`, { ${o.join(", ")} }`:"";return`getByRole(${this.quote(s)}${h})`;case"has-text":return`filter({ hasText: ${this.toHasText(s)} })`;case"has-not-text":return`filter({ hasNotText: ${this.toHasText(s)} })`;case"has":return`filter({ has: ${s} })`;case"hasNot":return`filter({ hasNot: ${s} })`;case"and":return`and(${s})`;case"or":return`or(${s})`;case"chain":return`locator(${s})`;case"test-id":return`getByTestId(${this.toTestIdValue(s)})`;case"text":return this.toCallWithExact("getByText",s,!!r.exact);case"alt":return this.toCallWithExact("getByAltText",s,!!r.exact);case"placeholder":return this.toCallWithExact("getByPlaceholder",s,!!r.exact);case"label":return this.toCallWithExact("getByLabel",s,!!r.exact);case"title":return this.toCallWithExact("getByTitle",s,!!r.exact);default:throw new Error("Unknown selector kind "+a)}}chainLocators(l){return l.join(".")}regexToSourceString(l){return vu(String(l))}toCallWithExact(l,a,s){return tt(a)?`${l}(${this.regexToSourceString(a)})`:s?`${l}(${this.quote(a)}, { exact: true })`:`${l}(${this.quote(a)})`}toHasText(l){return tt(l)?this.regexToSourceString(l):this.quote(l)}toTestIdValue(l){return tt(l)?this.regexToSourceString(l):this.quote(l)}quote(l){return yu(l,this.preferredQuote??"'")}}class ev{generateLocator(l,a,s,r={}){switch(a){case"default":return r.hasText!==void 0?`locator(${this.quote(s)}, has_text=${this.toHasText(r.hasText)})`:r.hasNotText!==void 0?`locator(${this.quote(s)}, has_not_text=${this.toHasText(r.hasNotText)})`:`locator(${this.quote(s)})`;case"frame-locator":return`frame_locator(${this.quote(s)})`;case"frame":return"content_frame";case"nth":return`nth(${s})`;case"first":return"first";case"last":return"last";case"visible":return`filter(visible=${s==="true"?"True":"False"})`;case"role":const o=[];tt(r.name)?o.push(`name=${this.regexToString(r.name)}`):typeof r.name=="string"&&(o.push(`name=${this.quote(r.name)}`),r.exact&&o.push("exact=True"));for(const{name:d,value:m}of r.attrs){let p=typeof m=="string"?this.quote(m):m;typeof m=="boolean"&&(p=m?"True":"False"),o.push(`${Rm(d)}=${p}`)}const h=o.length?`, ${o.join(", ")}`:"";return`get_by_role(${this.quote(s)}${h})`;case"has-text":return`filter(has_text=${this.toHasText(s)})`;case"has-not-text":return`filter(has_not_text=${this.toHasText(s)})`;case"has":return`filter(has=${s})`;case"hasNot":return`filter(has_not=${s})`;case"and":return`and_(${s})`;case"or":return`or_(${s})`;case"chain":return`locator(${s})`;case"test-id":return`get_by_test_id(${this.toTestIdValue(s)})`;case"text":return this.toCallWithExact("get_by_text",s,!!r.exact);case"alt":return this.toCallWithExact("get_by_alt_text",s,!!r.exact);case"placeholder":return this.toCallWithExact("get_by_placeholder",s,!!r.exact);case"label":return this.toCallWithExact("get_by_label",s,!!r.exact);case"title":return this.toCallWithExact("get_by_title",s,!!r.exact);default:throw new Error("Unknown selector kind "+a)}}chainLocators(l){return l.join(".")}regexToString(l){const a=l.flags.includes("i")?", re.IGNORECASE":"";return`re.compile(r"${vu(l.source).replace(/\\\//,"/").replace(/"/g,'\\"')}"${a})`}toCallWithExact(l,a,s){return tt(a)?`${l}(${this.regexToString(a)})`:s?`${l}(${this.quote(a)}, exact=True)`:`${l}(${this.quote(a)})`}toHasText(l){return tt(l)?this.regexToString(l):`${this.quote(l)}`}toTestIdValue(l){return tt(l)?this.regexToString(l):this.quote(l)}quote(l){return yu(l,'"')}}class tv{generateLocator(l,a,s,r={}){let o;switch(l){case"page":o="Page";break;case"frame-locator":o="FrameLocator";break;case"locator":o="Locator";break}switch(a){case"default":return r.hasText!==void 0?`locator(${this.quote(s)}, new ${o}.LocatorOptions().setHasText(${this.toHasText(r.hasText)}))`:r.hasNotText!==void 0?`locator(${this.quote(s)}, new ${o}.LocatorOptions().setHasNotText(${this.toHasText(r.hasNotText)}))`:`locator(${this.quote(s)})`;case"frame-locator":return`frameLocator(${this.quote(s)})`;case"frame":return"contentFrame()";case"nth":return`nth(${s})`;case"first":return"first()";case"last":return"last()";case"visible":return`filter(new ${o}.FilterOptions().setVisible(${s==="true"?"true":"false"}))`;case"role":const h=[];tt(r.name)?h.push(`.setName(${this.regexToString(r.name)})`):typeof r.name=="string"&&(h.push(`.setName(${this.quote(r.name)})`),r.exact&&h.push(".setExact(true)"));for(const{name:m,value:p}of r.attrs)h.push(`.set${ou(m)}(${typeof p=="string"?this.quote(p):p})`);const d=h.length?`, new ${o}.GetByRoleOptions()${h.join("")}`:"";return`getByRole(AriaRole.${Rm(s).toUpperCase()}${d})`;case"has-text":return`filter(new ${o}.FilterOptions().setHasText(${this.toHasText(s)}))`;case"has-not-text":return`filter(new ${o}.FilterOptions().setHasNotText(${this.toHasText(s)}))`;case"has":return`filter(new ${o}.FilterOptions().setHas(${s}))`;case"hasNot":return`filter(new ${o}.FilterOptions().setHasNot(${s}))`;case"and":return`and(${s})`;case"or":return`or(${s})`;case"chain":return`locator(${s})`;case"test-id":return`getByTestId(${this.toTestIdValue(s)})`;case"text":return this.toCallWithExact(o,"getByText",s,!!r.exact);case"alt":return this.toCallWithExact(o,"getByAltText",s,!!r.exact);case"placeholder":return this.toCallWithExact(o,"getByPlaceholder",s,!!r.exact);case"label":return this.toCallWithExact(o,"getByLabel",s,!!r.exact);case"title":return this.toCallWithExact(o,"getByTitle",s,!!r.exact);default:throw new Error("Unknown selector kind "+a)}}chainLocators(l){return l.join(".")}regexToString(l){const a=l.flags.includes("i")?", Pattern.CASE_INSENSITIVE":"";return`Pattern.compile(${this.quote(vu(l.source))}${a})`}toCallWithExact(l,a,s,r){return tt(s)?`${a}(${this.regexToString(s)})`:r?`${a}(${this.quote(s)}, new ${l}.${ou(a)}Options().setExact(true))`:`${a}(${this.quote(s)})`}toHasText(l){return tt(l)?this.regexToString(l):this.quote(l)}toTestIdValue(l){return tt(l)?this.regexToString(l):this.quote(l)}quote(l){return yu(l,'"')}}class nv{generateLocator(l,a,s,r={}){switch(a){case"default":return r.hasText!==void 0?`Locator(${this.quote(s)}, new() { ${this.toHasText(r.hasText)} })`:r.hasNotText!==void 0?`Locator(${this.quote(s)}, new() { ${this.toHasNotText(r.hasNotText)} })`:`Locator(${this.quote(s)})`;case"frame-locator":return`FrameLocator(${this.quote(s)})`;case"frame":return"ContentFrame";case"nth":return`Nth(${s})`;case"first":return"First";case"last":return"Last";case"visible":return`Filter(new() { Visible = ${s==="true"?"true":"false"} })`;case"role":const o=[];tt(r.name)?o.push(`NameRegex = ${this.regexToString(r.name)}`):typeof r.name=="string"&&(o.push(`Name = ${this.quote(r.name)}`),r.exact&&o.push("Exact = true"));for(const{name:d,value:m}of r.attrs)o.push(`${ou(d)} = ${typeof m=="string"?this.quote(m):m}`);const h=o.length?`, new() { ${o.join(", ")} }`:"";return`GetByRole(AriaRole.${ou(s)}${h})`;case"has-text":return`Filter(new() { ${this.toHasText(s)} })`;case"has-not-text":return`Filter(new() { ${this.toHasNotText(s)} })`;case"has":return`Filter(new() { Has = ${s} })`;case"hasNot":return`Filter(new() { HasNot = ${s} })`;case"and":return`And(${s})`;case"or":return`Or(${s})`;case"chain":return`Locator(${s})`;case"test-id":return`GetByTestId(${this.toTestIdValue(s)})`;case"text":return this.toCallWithExact("GetByText",s,!!r.exact);case"alt":return this.toCallWithExact("GetByAltText",s,!!r.exact);case"placeholder":return this.toCallWithExact("GetByPlaceholder",s,!!r.exact);case"label":return this.toCallWithExact("GetByLabel",s,!!r.exact);case"title":return this.toCallWithExact("GetByTitle",s,!!r.exact);default:throw new Error("Unknown selector kind "+a)}}chainLocators(l){return l.join(".")}regexToString(l){const a=l.flags.includes("i")?", RegexOptions.IgnoreCase":"";return`new Regex(${this.quote(vu(l.source))}${a})`}toCallWithExact(l,a,s){return tt(a)?`${l}(${this.regexToString(a)})`:s?`${l}(${this.quote(a)}, new() { Exact = true })`:`${l}(${this.quote(a)})`}toHasText(l){return tt(l)?`HasTextRegex = ${this.regexToString(l)}`:`HasText = ${this.quote(l)}`}toTestIdValue(l){return tt(l)?this.regexToString(l):this.quote(l)}toHasNotText(l){return tt(l)?`HasNotTextRegex = ${this.regexToString(l)}`:`HasNotText = ${this.quote(l)}`}quote(l){return yu(l,'"')}}class lv{generateLocator(l,a,s,r={}){return JSON.stringify({kind:a,body:s,options:r})}chainLocators(l){const a=l.map(s=>JSON.parse(s));for(let s=0;s<a.length-1;++s)a[s].next=a[s+1];return JSON.stringify(a[0])}}const iv={javascript:P1,python:ev,java:tv,csharp:nv,jsonl:lv};function tt(u){return u instanceof RegExp}const av=({language:u,log:l})=>{const a=me.useRef(null),[s,r]=me.useState(new Map);return me.useLayoutEffect(()=>{var o;l.find(h=>h.reveal)&&((o=a.current)==null||o.scrollIntoView({block:"center",inline:"nearest"}))},[a,l]),X.jsxs("div",{className:"call-log",style:{flex:"auto"},children:[l.map(o=>{const h=s.get(o.id),d=typeof h=="boolean"?h:o.status!=="done",m=o.params.selector?km(u,o.params.selector):null;let p=o.title,b="";return o.title.startsWith("expect.to")||o.title.startsWith("expect.not.to")?(p="expect(",b=`).${o.title.substring(7)}()`):o.title.startsWith("locator.")?(p="",b=`.${o.title.substring(8)}()`):(m||o.params.url)&&(p=o.title+"(",b=")"),X.jsxs("div",{className:pl("call-log-call",o.status),children:[X.jsxs("div",{className:"call-log-call-header",children:[X.jsx("span",{className:pl("codicon",`codicon-chevron-${d?"down":"right"}`),style:{cursor:"pointer"},onClick:()=>{const S=new Map(s);S.set(o.id,!d),r(S)}}),p,o.params.url?X.jsx("span",{className:"call-log-details",children:X.jsx("span",{className:"call-log-url",title:o.params.url,children:o.params.url})}):void 0,m?X.jsx("span",{className:"call-log-details",children:X.jsx("span",{className:"call-log-selector",title:`page.${m}`,children:`page.${m}`})}):void 0,b,X.jsx("span",{className:pl("codicon",sv(o))}),typeof o.duration=="number"?X.jsxs("span",{className:"call-log-time",children:["— ",e1(o.duration)]}):void 0]}),(d?o.messages:[]).map((S,N)=>X.jsx("div",{className:"call-log-message",children:S.trim()},N)),!!o.error&&X.jsx("div",{className:"call-log-message error",hidden:!d,children:o.error})]},o.id)}),X.jsx("div",{ref:a})]})};function sv(u){switch(u.status){case"done":return"codicon-check";case"in-progress":return"codicon-clock";case"paused":return"codicon-debug-pause";case"error":return"codicon-error"}}const Ef=Symbol.for("yaml.alias"),mf=Symbol.for("yaml.document"),Kn=Symbol.for("yaml.map"),Lm=Symbol.for("yaml.pair"),Ft=Symbol.for("yaml.scalar"),bi=Symbol.for("yaml.seq"),jt=Symbol.for("yaml.node.type"),vl=u=>!!u&&typeof u=="object"&&u[jt]===Ef,bl=u=>!!u&&typeof u=="object"&&u[jt]===mf,Si=u=>!!u&&typeof u=="object"&&u[jt]===Kn,Re=u=>!!u&&typeof u=="object"&&u[jt]===Lm,_e=u=>!!u&&typeof u=="object"&&u[jt]===Ft,Ti=u=>!!u&&typeof u=="object"&&u[jt]===bi;function Le(u){if(u&&typeof u=="object")switch(u[jt]){case Kn:case bi:return!0}return!1}function ze(u){if(u&&typeof u=="object")switch(u[jt]){case Ef:case Kn:case Ft:case bi:return!0}return!1}const uv=u=>(_e(u)||Le(u))&&!!u.anchor,pt=Symbol("break visit"),zm=Symbol("skip children"),It=Symbol("remove node");function Vn(u,l){const a=Um(l);bl(u)?ri(null,u.contents,a,Object.freeze([u]))===It&&(u.contents=null):ri(null,u,a,Object.freeze([]))}Vn.BREAK=pt;Vn.SKIP=zm;Vn.REMOVE=It;function ri(u,l,a,s){const r=Bm(u,l,a,s);if(ze(r)||Re(r))return qm(u,s,r),ri(u,r,a,s);if(typeof r!="symbol"){if(Le(l)){s=Object.freeze(s.concat(l));for(let o=0;o<l.items.length;++o){const h=ri(o,l.items[o],a,s);if(typeof h=="number")o=h-1;else{if(h===pt)return pt;h===It&&(l.items.splice(o,1),o-=1)}}}else if(Re(l)){s=Object.freeze(s.concat(l));const o=ri("key",l.key,a,s);if(o===pt)return pt;o===It&&(l.key=null);const h=ri("value",l.value,a,s);if(h===pt)return pt;h===It&&(l.value=null)}}return r}async function bu(u,l){const a=Um(l);bl(u)?await fi(null,u.contents,a,Object.freeze([u]))===It&&(u.contents=null):await fi(null,u,a,Object.freeze([]))}bu.BREAK=pt;bu.SKIP=zm;bu.REMOVE=It;async function fi(u,l,a,s){const r=await Bm(u,l,a,s);if(ze(r)||Re(r))return qm(u,s,r),fi(u,r,a,s);if(typeof r!="symbol"){if(Le(l)){s=Object.freeze(s.concat(l));for(let o=0;o<l.items.length;++o){const h=await fi(o,l.items[o],a,s);if(typeof h=="number")o=h-1;else{if(h===pt)return pt;h===It&&(l.items.splice(o,1),o-=1)}}}else if(Re(l)){s=Object.freeze(s.concat(l));const o=await fi("key",l.key,a,s);if(o===pt)return pt;o===It&&(l.key=null);const h=await fi("value",l.value,a,s);if(h===pt)return pt;h===It&&(l.value=null)}}return r}function Um(u){return typeof u=="object"&&(u.Collection||u.Node||u.Value)?Object.assign({Alias:u.Node,Map:u.Node,Scalar:u.Node,Seq:u.Node},u.Value&&{Map:u.Value,Scalar:u.Value,Seq:u.Value},u.Collection&&{Map:u.Collection,Seq:u.Collection},u):u}function Bm(u,l,a,s){var r,o,h,d,m;if(typeof a=="function")return a(u,l,s);if(Si(l))return(r=a.Map)==null?void 0:r.call(a,u,l,s);if(Ti(l))return(o=a.Seq)==null?void 0:o.call(a,u,l,s);if(Re(l))return(h=a.Pair)==null?void 0:h.call(a,u,l,s);if(_e(l))return(d=a.Scalar)==null?void 0:d.call(a,u,l,s);if(vl(l))return(m=a.Alias)==null?void 0:m.call(a,u,l,s)}function qm(u,l,a){const s=l[l.length-1];if(Le(s))s.items[u]=a;else if(Re(s))u==="key"?s.key=a:s.value=a;else if(bl(s))s.contents=a;else{const r=vl(s)?"alias":"scalar";throw new Error(`Cannot replace node with ${r} parent`)}}const cv={"!":"%21",",":"%2C","[":"%5B","]":"%5D","{":"%7B","}":"%7D"},rv=u=>u.replace(/[!,[\]{}]/g,l=>cv[l]);class ct{constructor(l,a){this.docStart=null,this.docEnd=!1,this.yaml=Object.assign({},ct.defaultYaml,l),this.tags=Object.assign({},ct.defaultTags,a)}clone(){const l=new ct(this.yaml,this.tags);return l.docStart=this.docStart,l}atDocument(){const l=new ct(this.yaml,this.tags);switch(this.yaml.version){case"1.1":this.atNextDocument=!0;break;case"1.2":this.atNextDocument=!1,this.yaml={explicit:ct.defaultYaml.explicit,version:"1.2"},this.tags=Object.assign({},ct.defaultTags);break}return l}add(l,a){this.atNextDocument&&(this.yaml={explicit:ct.defaultYaml.explicit,version:"1.1"},this.tags=Object.assign({},ct.defaultTags),this.atNextDocument=!1);const s=l.trim().split(/[ \t]+/),r=s.shift();switch(r){case"%TAG":{if(s.length!==2&&(a(0,"%TAG directive should contain exactly two parts"),s.length<2))return!1;const[o,h]=s;return this.tags[o]=h,!0}case"%YAML":{if(this.yaml.explicit=!0,s.length!==1)return a(0,"%YAML directive should contain exactly one part"),!1;const[o]=s;if(o==="1.1"||o==="1.2")return this.yaml.version=o,!0;{const h=/^\d+\.\d+$/.test(o);return a(6,`Unsupported YAML version ${o}`,h),!1}}default:return a(0,`Unknown directive ${r}`,!0),!1}}tagName(l,a){if(l==="!")return"!";if(l[0]!=="!")return a(`Not a valid tag: ${l}`),null;if(l[1]==="<"){const h=l.slice(2,-1);return h==="!"||h==="!!"?(a(`Verbatim tags aren't resolved, so ${l} is invalid.`),null):(l[l.length-1]!==">"&&a("Verbatim tags must end with a >"),h)}const[,s,r]=l.match(/^(.*!)([^!]*)$/s);r||a(`The ${l} tag has no suffix`);const o=this.tags[s];if(o)try{return o+decodeURIComponent(r)}catch(h){return a(String(h)),null}return s==="!"?l:(a(`Could not resolve tag: ${l}`),null)}tagString(l){for(const[a,s]of Object.entries(this.tags))if(l.startsWith(s))return a+rv(l.substring(s.length));return l[0]==="!"?l:`!<${l}>`}toString(l){const a=this.yaml.explicit?[`%YAML ${this.yaml.version||"1.2"}`]:[],s=Object.entries(this.tags);let r;if(l&&s.length>0&&ze(l.contents)){const o={};Vn(l.contents,(h,d)=>{ze(d)&&d.tag&&(o[d.tag]=!0)}),r=Object.keys(o)}else r=[];for(const[o,h]of s)o==="!!"&&h==="tag:yaml.org,2002:"||(!l||r.some(d=>d.startsWith(h)))&&a.push(`%TAG ${o} ${h}`);return a.join(`
`)}}ct.defaultYaml={explicit:!1,version:"1.2"};ct.defaultTags={"!!":"tag:yaml.org,2002:"};function jm(u){if(/[\x00-\x19\s,[\]{}]/.test(u)){const a=`Anchor must not contain whitespace or control characters: ${JSON.stringify(u)}`;throw new Error(a)}return!0}function Hm(u){const l=new Set;return Vn(u,{Value(a,s){s.anchor&&l.add(s.anchor)}}),l}function $m(u,l){for(let a=1;;++a){const s=`${u}${a}`;if(!l.has(s))return s}}function fv(u,l){const a=[],s=new Map;let r=null;return{onAnchor:o=>{a.push(o),r||(r=Hm(u));const h=$m(l,r);return r.add(h),h},setAnchors:()=>{for(const o of a){const h=s.get(o);if(typeof h=="object"&&h.anchor&&(_e(h.node)||Le(h.node)))h.node.anchor=h.anchor;else{const d=new Error("Failed to resolve repeated object (this should not happen)");throw d.source=o,d}}},sourceObjects:s}}function oi(u,l,a,s){if(s&&typeof s=="object")if(Array.isArray(s))for(let r=0,o=s.length;r<o;++r){const h=s[r],d=oi(u,s,String(r),h);d===void 0?delete s[r]:d!==h&&(s[r]=d)}else if(s instanceof Map)for(const r of Array.from(s.keys())){const o=s.get(r),h=oi(u,s,r,o);h===void 0?s.delete(r):h!==o&&s.set(r,h)}else if(s instanceof Set)for(const r of Array.from(s)){const o=oi(u,s,r,r);o===void 0?s.delete(r):o!==r&&(s.delete(r),s.add(o))}else for(const[r,o]of Object.entries(s)){const h=oi(u,s,r,o);h===void 0?delete s[r]:h!==o&&(s[r]=h)}return u.call(l,a,s)}function qt(u,l,a){if(Array.isArray(u))return u.map((s,r)=>qt(s,String(r),a));if(u&&typeof u.toJSON=="function"){if(!a||!uv(u))return u.toJSON(l,a);const s={aliasCount:0,count:1,res:void 0};a.anchors.set(u,s),a.onCreate=o=>{s.res=o,delete a.onCreate};const r=u.toJSON(l,a);return a.onCreate&&a.onCreate(r),r}return typeof u=="bigint"&&!(a!=null&&a.keep)?Number(u):u}class Af{constructor(l){Object.defineProperty(this,jt,{value:l})}clone(){const l=Object.create(Object.getPrototypeOf(this),Object.getOwnPropertyDescriptors(this));return this.range&&(l.range=this.range.slice()),l}toJS(l,{mapAsMap:a,maxAliasCount:s,onAnchor:r,reviver:o}={}){if(!bl(l))throw new TypeError("A document argument is required");const h={anchors:new Map,doc:l,keep:!0,mapAsMap:a===!0,mapKeyWarned:!1,maxAliasCount:typeof s=="number"?s:100},d=qt(this,"",h);if(typeof r=="function")for(const{count:m,res:p}of h.anchors.values())r(p,m);return typeof o=="function"?oi(o,{"":d},"",d):d}}class Su extends Af{constructor(l){super(Ef),this.source=l,Object.defineProperty(this,"tag",{set(){throw new Error("Alias nodes cannot have tags")}})}resolve(l){let a;return Vn(l,{Node:(s,r)=>{if(r===this)return Vn.BREAK;r.anchor===this.source&&(a=r)}}),a}toJSON(l,a){if(!a)return{source:this.source};const{anchors:s,doc:r,maxAliasCount:o}=a,h=this.resolve(r);if(!h){const m=`Unresolved alias (the anchor must be set before the alias): ${this.source}`;throw new ReferenceError(m)}let d=s.get(h);if(d||(qt(h,null,a),d=s.get(h)),!d||d.res===void 0){const m="This should not happen: Alias anchor was not resolved?";throw new ReferenceError(m)}if(o>=0&&(d.count+=1,d.aliasCount===0&&(d.aliasCount=su(r,h,s)),d.count*d.aliasCount>o)){const m="Excessive alias count indicates a resource exhaustion attack";throw new ReferenceError(m)}return d.res}toString(l,a,s){const r=`*${this.source}`;if(l){if(jm(this.source),l.options.verifyAliasOrder&&!l.anchors.has(this.source)){const o=`Unresolved alias (the anchor must be set before the alias): ${this.source}`;throw new Error(o)}if(l.implicitKey)return`${r} `}return r}}function su(u,l,a){if(vl(l)){const s=l.resolve(u),r=a&&s&&a.get(s);return r?r.count*r.aliasCount:0}else if(Le(l)){let s=0;for(const r of l.items){const o=su(u,r,a);o>s&&(s=o)}return s}else if(Re(l)){const s=su(u,l.key,a),r=su(u,l.value,a);return Math.max(s,r)}return 1}const Ym=u=>!u||typeof u!="function"&&typeof u!="object";class re extends Af{constructor(l){super(Ft),this.value=l}toJSON(l,a){return a!=null&&a.keep?this.value:qt(this.value,l,a)}toString(){return String(this.value)}}re.BLOCK_FOLDED="BLOCK_FOLDED";re.BLOCK_LITERAL="BLOCK_LITERAL";re.PLAIN="PLAIN";re.QUOTE_DOUBLE="QUOTE_DOUBLE";re.QUOTE_SINGLE="QUOTE_SINGLE";const ov="tag:yaml.org,2002:";function hv(u,l,a){if(l){const s=a.filter(o=>o.tag===l),r=s.find(o=>!o.format)??s[0];if(!r)throw new Error(`Tag ${l} not found`);return r}return a.find(s=>{var r;return((r=s.identify)==null?void 0:r.call(s,u))&&!s.format})}function Ma(u,l,a){var S,N,E;if(bl(u)&&(u=u.contents),ze(u))return u;if(Re(u)){const D=(N=(S=a.schema[Kn]).createNode)==null?void 0:N.call(S,a.schema,null,a);return D.items.push(u),D}(u instanceof String||u instanceof Number||u instanceof Boolean||typeof BigInt<"u"&&u instanceof BigInt)&&(u=u.valueOf());const{aliasDuplicateObjects:s,onAnchor:r,onTagObj:o,schema:h,sourceObjects:d}=a;let m;if(s&&u&&typeof u=="object"){if(m=d.get(u),m)return m.anchor||(m.anchor=r(u)),new Su(m.anchor);m={anchor:null,node:null},d.set(u,m)}l!=null&&l.startsWith("!!")&&(l=ov+l.slice(2));let p=hv(u,l,h.tags);if(!p){if(u&&typeof u.toJSON=="function"&&(u=u.toJSON()),!u||typeof u!="object"){const D=new re(u);return m&&(m.node=D),D}p=u instanceof Map?h[Kn]:Symbol.iterator in Object(u)?h[bi]:h[Kn]}o&&(o(p),delete a.onTagObj);const b=p!=null&&p.createNode?p.createNode(a.schema,u,a):typeof((E=p==null?void 0:p.nodeClass)==null?void 0:E.from)=="function"?p.nodeClass.from(a.schema,u,a):new re(u);return l?b.tag=l:p.default||(b.tag=p.tag),m&&(m.node=b),b}function hu(u,l,a){let s=a;for(let r=l.length-1;r>=0;--r){const o=l[r];if(typeof o=="number"&&Number.isInteger(o)&&o>=0){const h=[];h[o]=s,s=h}else s=new Map([[o,s]])}return Ma(s,void 0,{aliasDuplicateObjects:!1,keepUndefined:!1,onAnchor:()=>{throw new Error("This should not happen, please report a bug.")},schema:u,sourceObjects:new Map})}const Na=u=>u==null||typeof u=="object"&&!!u[Symbol.iterator]().next().done;class Gm extends Af{constructor(l,a){super(l),Object.defineProperty(this,"schema",{value:a,configurable:!0,enumerable:!1,writable:!0})}clone(l){const a=Object.create(Object.getPrototypeOf(this),Object.getOwnPropertyDescriptors(this));return l&&(a.schema=l),a.items=a.items.map(s=>ze(s)||Re(s)?s.clone(l):s),this.range&&(a.range=this.range.slice()),a}addIn(l,a){if(Na(l))this.add(a);else{const[s,...r]=l,o=this.get(s,!0);if(Le(o))o.addIn(r,a);else if(o===void 0&&this.schema)this.set(s,hu(this.schema,r,a));else throw new Error(`Expected YAML collection at ${s}. Remaining path: ${r}`)}}deleteIn(l){const[a,...s]=l;if(s.length===0)return this.delete(a);const r=this.get(a,!0);if(Le(r))return r.deleteIn(s);throw new Error(`Expected YAML collection at ${a}. Remaining path: ${s}`)}getIn(l,a){const[s,...r]=l,o=this.get(s,!0);return r.length===0?!a&&_e(o)?o.value:o:Le(o)?o.getIn(r,a):void 0}hasAllNullValues(l){return this.items.every(a=>{if(!Re(a))return!1;const s=a.value;return s==null||l&&_e(s)&&s.value==null&&!s.commentBefore&&!s.comment&&!s.tag})}hasIn(l){const[a,...s]=l;if(s.length===0)return this.has(a);const r=this.get(a,!0);return Le(r)?r.hasIn(s):!1}setIn(l,a){const[s,...r]=l;if(r.length===0)this.set(s,a);else{const o=this.get(s,!0);if(Le(o))o.setIn(r,a);else if(o===void 0&&this.schema)this.set(s,hu(this.schema,r,a));else throw new Error(`Expected YAML collection at ${s}. Remaining path: ${r}`)}}}const dv=u=>u.replace(/^(?!$)(?: $)?/gm,"#");function pn(u,l){return/^\n+$/.test(u)?u.substring(1):l?u.replace(/^(?! *$)/gm,l):u}const dl=(u,l,a)=>u.endsWith(`
`)?pn(a,l):a.includes(`
`)?`
`+pn(a,l):(u.endsWith(" ")?"":" ")+a,Km="flow",pf="block",uu="quoted";function Tu(u,l,a="flow",{indentAtStart:s,lineWidth:r=80,minContentWidth:o=20,onFold:h,onOverflow:d}={}){if(!r||r<0)return u;r<o&&(o=0);const m=Math.max(1+o,1+r-l.length);if(u.length<=m)return u;const p=[],b={};let S=r-l.length;typeof s=="number"&&(s>r-Math.max(2,o)?p.push(0):S=r-s);let N,E,D=!1,T=-1,w=-1,k=-1;a===pf&&(T=Fg(u,T,l.length),T!==-1&&(S=T+m));for(let H;H=u[T+=1];){if(a===uu&&H==="\\"){switch(w=T,u[T+1]){case"x":T+=3;break;case"u":T+=5;break;case"U":T+=9;break;default:T+=1}k=T}if(H===`
`)a===pf&&(T=Fg(u,T,l.length)),S=T+l.length+m,N=void 0;else{if(H===" "&&E&&E!==" "&&E!==`
`&&E!=="	"){const Q=u[T+1];Q&&Q!==" "&&Q!==`
`&&Q!=="	"&&(N=T)}if(T>=S)if(N)p.push(N),S=N+m,N=void 0;else if(a===uu){for(;E===" "||E==="	";)E=H,H=u[T+=1],D=!0;const Q=T>k+1?T-2:w-1;if(b[Q])return u;p.push(Q),b[Q]=!0,S=Q+m,N=void 0}else D=!0}E=H}if(D&&d&&d(),p.length===0)return u;h&&h();let L=u.slice(0,p[0]);for(let H=0;H<p.length;++H){const Q=p[H],G=p[H+1]||u.length;Q===0?L=`
${l}${u.slice(0,G)}`:(a===uu&&b[Q]&&(L+=`${u[Q]}\\`),L+=`
${l}${u.slice(Q+1,G)}`)}return L}function Fg(u,l,a){let s=l,r=l+1,o=u[r];for(;o===" "||o==="	";)if(l<r+a)o=u[++l];else{do o=u[++l];while(o&&o!==`
`);s=l,r=l+1,o=u[r]}return s}const wu=(u,l)=>({indentAtStart:l?u.indent.length:u.indentAtStart,lineWidth:u.options.lineWidth,minContentWidth:u.options.minContentWidth}),Eu=u=>/^(%|---|\.\.\.)/m.test(u);function gv(u,l,a){if(!l||l<0)return!1;const s=l-a,r=u.length;if(r<=s)return!1;for(let o=0,h=0;o<r;++o)if(u[o]===`
`){if(o-h>s)return!0;if(h=o+1,r-h<=s)return!1}return!0}function _a(u,l){const a=JSON.stringify(u);if(l.options.doubleQuotedAsJSON)return a;const{implicitKey:s}=l,r=l.options.doubleQuotedMinMultiLineLength,o=l.indent||(Eu(u)?"  ":"");let h="",d=0;for(let m=0,p=a[m];p;p=a[++m])if(p===" "&&a[m+1]==="\\"&&a[m+2]==="n"&&(h+=a.slice(d,m)+"\\ ",m+=1,d=m,p="\\"),p==="\\")switch(a[m+1]){case"u":{h+=a.slice(d,m);const b=a.substr(m+2,4);switch(b){case"0000":h+="\\0";break;case"0007":h+="\\a";break;case"000b":h+="\\v";break;case"001b":h+="\\e";break;case"0085":h+="\\N";break;case"00a0":h+="\\_";break;case"2028":h+="\\L";break;case"2029":h+="\\P";break;default:b.substr(0,2)==="00"?h+="\\x"+b.substr(2):h+=a.substr(m,6)}m+=5,d=m+1}break;case"n":if(s||a[m+2]==='"'||a.length<r)m+=1;else{for(h+=a.slice(d,m)+`

`;a[m+2]==="\\"&&a[m+3]==="n"&&a[m+4]!=='"';)h+=`
`,m+=2;h+=o,a[m+2]===" "&&(h+="\\"),m+=1,d=m+1}break;default:m+=1}return h=d?h+a.slice(d):a,s?h:Tu(h,o,uu,wu(l,!1))}function yf(u,l){if(l.options.singleQuote===!1||l.implicitKey&&u.includes(`
`)||/[ \t]\n|\n[ \t]/.test(u))return _a(u,l);const a=l.indent||(Eu(u)?"  ":""),s="'"+u.replace(/'/g,"''").replace(/\n+/g,`$&
${a}`)+"'";return l.implicitKey?s:Tu(s,a,Km,wu(l,!1))}function hi(u,l){const{singleQuote:a}=l.options;let s;if(a===!1)s=_a;else{const r=u.includes('"'),o=u.includes("'");r&&!o?s=yf:o&&!r?s=_a:s=a?yf:_a}return s(u,l)}let vf;try{vf=new RegExp(`(^|(?<!
))
+(?!
|$)`,"g")}catch{vf=/\n+(?!\n|$)/g}function cu({comment:u,type:l,value:a},s,r,o){const{blockQuote:h,commentString:d,lineWidth:m}=s.options;if(!h||/\n[\t ]+$/.test(a)||/^\s*$/.test(a))return hi(a,s);const p=s.indent||(s.forceBlockIndent||Eu(a)?"  ":""),b=h==="literal"?!0:h==="folded"||l===re.BLOCK_FOLDED?!1:l===re.BLOCK_LITERAL?!0:!gv(a,m,p.length);if(!a)return b?`|
`:`>
`;let S,N;for(N=a.length;N>0;--N){const K=a[N-1];if(K!==`
`&&K!=="	"&&K!==" ")break}let E=a.substring(N);const D=E.indexOf(`
`);D===-1?S="-":a===E||D!==E.length-1?(S="+",o&&o()):S="",E&&(a=a.slice(0,-E.length),E[E.length-1]===`
`&&(E=E.slice(0,-1)),E=E.replace(vf,`$&${p}`));let T=!1,w,k=-1;for(w=0;w<a.length;++w){const K=a[w];if(K===" ")T=!0;else if(K===`
`)k=w;else break}let L=a.substring(0,k<w?k+1:w);L&&(a=a.substring(L.length),L=L.replace(/\n+/g,`$&${p}`));let Q=(b?"|":">")+(T?p?"2":"1":"")+S;if(u&&(Q+=" "+d(u.replace(/ ?[\r\n]+/g," ")),r&&r()),b)return a=a.replace(/\n+/g,`$&${p}`),`${Q}
${p}${L}${a}${E}`;a=a.replace(/\n+/g,`
$&`).replace(/(?:^|\n)([\t ].*)(?:([\n\t ]*)\n(?![\n\t ]))?/g,"$1$2").replace(/\n+/g,`$&${p}`);const G=Tu(`${L}${a}${E}`,p,pf,wu(s,!0));return`${Q}
${p}${G}`}function mv(u,l,a,s){const{type:r,value:o}=u,{actualString:h,implicitKey:d,indent:m,indentStep:p,inFlow:b}=l;if(d&&o.includes(`
`)||b&&/[[\]{},]/.test(o))return hi(o,l);if(!o||/^[\n\t ,[\]{}#&*!|>'"%@`]|^[?-]$|^[?-][ \t]|[\n:][ \t]|[ \t]\n|[\n\t ]#|[\n\t :]$/.test(o))return d||b||!o.includes(`
`)?hi(o,l):cu(u,l,a,s);if(!d&&!b&&r!==re.PLAIN&&o.includes(`
`))return cu(u,l,a,s);if(Eu(o)){if(m==="")return l.forceBlockIndent=!0,cu(u,l,a,s);if(d&&m===p)return hi(o,l)}const S=o.replace(/\n+/g,`$&
${m}`);if(h){const N=T=>{var w;return T.default&&T.tag!=="tag:yaml.org,2002:str"&&((w=T.test)==null?void 0:w.test(S))},{compat:E,tags:D}=l.doc.schema;if(D.some(N)||E!=null&&E.some(N))return hi(o,l)}return d?S:Tu(S,m,Km,wu(l,!1))}function Ra(u,l,a,s){const{implicitKey:r,inFlow:o}=l,h=typeof u.value=="string"?u:Object.assign({},u,{value:String(u.value)});let{type:d}=u;d!==re.QUOTE_DOUBLE&&/[\x00-\x08\x0b-\x1f\x7f-\x9f\u{D800}-\u{DFFF}]/u.test(h.value)&&(d=re.QUOTE_DOUBLE);const m=b=>{switch(b){case re.BLOCK_FOLDED:case re.BLOCK_LITERAL:return r||o?hi(h.value,l):cu(h,l,a,s);case re.QUOTE_DOUBLE:return _a(h.value,l);case re.QUOTE_SINGLE:return yf(h.value,l);case re.PLAIN:return mv(h,l,a,s);default:return null}};let p=m(d);if(p===null){const{defaultKeyType:b,defaultStringType:S}=l.options,N=r&&b||S;if(p=m(N),p===null)throw new Error(`Unsupported default string type ${N}`)}return p}function Vm(u,l){const a=Object.assign({blockQuote:!0,commentString:dv,defaultKeyType:null,defaultStringType:"PLAIN",directives:null,doubleQuotedAsJSON:!1,doubleQuotedMinMultiLineLength:40,falseStr:"false",flowCollectionPadding:!0,indentSeq:!0,lineWidth:80,minContentWidth:20,nullStr:"null",simpleKeys:!1,singleQuote:null,trueStr:"true",verifyAliasOrder:!0},u.schema.toStringOptions,l);let s;switch(a.collectionStyle){case"block":s=!1;break;case"flow":s=!0;break;default:s=null}return{anchors:new Set,doc:u,flowCollectionPadding:a.flowCollectionPadding?" ":"",indent:"",indentStep:typeof a.indent=="number"?" ".repeat(a.indent):"  ",inFlow:s,options:a}}function pv(u,l){var r;if(l.tag){const o=u.filter(h=>h.tag===l.tag);if(o.length>0)return o.find(h=>h.format===l.format)??o[0]}let a,s;if(_e(l)){s=l.value;let o=u.filter(h=>{var d;return(d=h.identify)==null?void 0:d.call(h,s)});if(o.length>1){const h=o.filter(d=>d.test);h.length>0&&(o=h)}a=o.find(h=>h.format===l.format)??o.find(h=>!h.format)}else s=l,a=u.find(o=>o.nodeClass&&s instanceof o.nodeClass);if(!a){const o=((r=s==null?void 0:s.constructor)==null?void 0:r.name)??typeof s;throw new Error(`Tag not resolved for ${o} value`)}return a}function yv(u,l,{anchors:a,doc:s}){if(!s.directives)return"";const r=[],o=(_e(u)||Le(u))&&u.anchor;o&&jm(o)&&(a.add(o),r.push(`&${o}`));const h=u.tag?u.tag:l.default?null:l.tag;return h&&r.push(s.directives.tagString(h)),r.join(" ")}function mi(u,l,a,s){var m;if(Re(u))return u.toString(l,a,s);if(vl(u)){if(l.doc.directives)return u.toString(l);if((m=l.resolvedAliases)!=null&&m.has(u))throw new TypeError("Cannot stringify circular structure without alias nodes");l.resolvedAliases?l.resolvedAliases.add(u):l.resolvedAliases=new Set([u]),u=u.resolve(l.doc)}let r;const o=ze(u)?u:l.doc.createNode(u,{onTagObj:p=>r=p});r||(r=pv(l.doc.schema.tags,o));const h=yv(o,r,l);h.length>0&&(l.indentAtStart=(l.indentAtStart??0)+h.length+1);const d=typeof r.stringify=="function"?r.stringify(o,l,a,s):_e(o)?Ra(o,l,a,s):o.toString(l,a,s);return h?_e(o)||d[0]==="{"||d[0]==="["?`${h} ${d}`:`${h}
${l.indent}${d}`:d}function vv({key:u,value:l},a,s,r){const{allNullValues:o,doc:h,indent:d,indentStep:m,options:{commentString:p,indentSeq:b,simpleKeys:S}}=a;let N=ze(u)&&u.comment||null;if(S){if(N)throw new Error("With simple keys, key nodes cannot have comments");if(Le(u)||!ze(u)&&typeof u=="object"){const ee="With simple keys, collection cannot be used as a key value";throw new Error(ee)}}let E=!S&&(!u||N&&l==null&&!a.inFlow||Le(u)||(_e(u)?u.type===re.BLOCK_FOLDED||u.type===re.BLOCK_LITERAL:typeof u=="object"));a=Object.assign({},a,{allNullValues:!1,implicitKey:!E&&(S||!o),indent:d+m});let D=!1,T=!1,w=mi(u,a,()=>D=!0,()=>T=!0);if(!E&&!a.inFlow&&w.length>1024){if(S)throw new Error("With simple keys, single line scalar must not span more than 1024 characters");E=!0}if(a.inFlow){if(o||l==null)return D&&s&&s(),w===""?"?":E?`? ${w}`:w}else if(o&&!S||l==null&&E)return w=`? ${w}`,N&&!D?w+=dl(w,a.indent,p(N)):T&&r&&r(),w;D&&(N=null),E?(N&&(w+=dl(w,a.indent,p(N))),w=`? ${w}
${d}:`):(w=`${w}:`,N&&(w+=dl(w,a.indent,p(N))));let k,L,H;ze(l)?(k=!!l.spaceBefore,L=l.commentBefore,H=l.comment):(k=!1,L=null,H=null,l&&typeof l=="object"&&(l=h.createNode(l))),a.implicitKey=!1,!E&&!N&&_e(l)&&(a.indentAtStart=w.length+1),T=!1,!b&&m.length>=2&&!a.inFlow&&!E&&Ti(l)&&!l.flow&&!l.tag&&!l.anchor&&(a.indent=a.indent.substring(2));let Q=!1;const G=mi(l,a,()=>Q=!0,()=>T=!0);let K=" ";if(N||k||L){if(K=k?`
`:"",L){const ee=p(L);K+=`
${pn(ee,a.indent)}`}G===""&&!a.inFlow?K===`
`&&(K=`

`):K+=`
${a.indent}`}else if(!E&&Le(l)){const ee=G[0],V=G.indexOf(`
`),z=V!==-1,F=a.inFlow??l.flow??l.items.length===0;if(z||!F){let se=!1;if(z&&(ee==="&"||ee==="!")){let U=G.indexOf(" ");ee==="&"&&U!==-1&&U<V&&G[U+1]==="!"&&(U=G.indexOf(" ",U+1)),(U===-1||V<U)&&(se=!0)}se||(K=`
${a.indent}`)}}else(G===""||G[0]===`
`)&&(K="");return w+=K+G,a.inFlow?Q&&s&&s():H&&!Q?w+=dl(w,a.indent,p(H)):T&&r&&r(),w}function Qm(u,l){(u==="debug"||u==="warn")&&(typeof process<"u"&&process.emitWarning?process.emitWarning(l):console.warn(l))}const Ws="<<",yn={identify:u=>u===Ws||typeof u=="symbol"&&u.description===Ws,default:"key",tag:"tag:yaml.org,2002:merge",test:/^<<$/,resolve:()=>Object.assign(new re(Symbol(Ws)),{addToJSMap:Xm}),stringify:()=>Ws},bv=(u,l)=>(yn.identify(l)||_e(l)&&(!l.type||l.type===re.PLAIN)&&yn.identify(l.value))&&(u==null?void 0:u.doc.schema.tags.some(a=>a.tag===yn.tag&&a.default));function Xm(u,l,a){if(a=u&&vl(a)?a.resolve(u.doc):a,Ti(a))for(const s of a.items)nf(u,l,s);else if(Array.isArray(a))for(const s of a)nf(u,l,s);else nf(u,l,a)}function nf(u,l,a){const s=u&&vl(a)?a.resolve(u.doc):a;if(!Si(s))throw new Error("Merge sources must be maps or map aliases");const r=s.toJSON(null,u,Map);for(const[o,h]of r)l instanceof Map?l.has(o)||l.set(o,h):l instanceof Set?l.add(o):Object.prototype.hasOwnProperty.call(l,o)||Object.defineProperty(l,o,{value:h,writable:!0,enumerable:!0,configurable:!0});return l}function Zm(u,l,{key:a,value:s}){if(ze(a)&&a.addToJSMap)a.addToJSMap(u,l,s);else if(bv(u,a))Xm(u,l,s);else{const r=qt(a,"",u);if(l instanceof Map)l.set(r,qt(s,r,u));else if(l instanceof Set)l.add(r);else{const o=Sv(a,r,u),h=qt(s,o,u);o in l?Object.defineProperty(l,o,{value:h,writable:!0,enumerable:!0,configurable:!0}):l[o]=h}}return l}function Sv(u,l,a){if(l===null)return"";if(typeof l!="object")return String(l);if(ze(u)&&(a!=null&&a.doc)){const s=Vm(a.doc,{});s.anchors=new Set;for(const o of a.anchors.keys())s.anchors.add(o.anchor);s.inFlow=!0,s.inStringifyKey=!0;const r=u.toString(s);if(!a.mapKeyWarned){let o=JSON.stringify(r);o.length>40&&(o=o.substring(0,36)+'..."'),Qm(a.doc.options.logLevel,`Keys with collection values will be stringified due to JS Object restrictions: ${o}. Set mapAsMap: true to use object keys.`),a.mapKeyWarned=!0}return r}return JSON.stringify(l)}function Of(u,l,a){const s=Ma(u,void 0,a),r=Ma(l,void 0,a);return new st(s,r)}class st{constructor(l,a=null){Object.defineProperty(this,jt,{value:Lm}),this.key=l,this.value=a}clone(l){let{key:a,value:s}=this;return ze(a)&&(a=a.clone(l)),ze(s)&&(s=s.clone(l)),new st(a,s)}toJSON(l,a){const s=a!=null&&a.mapAsMap?new Map:{};return Zm(a,s,this)}toString(l,a,s){return l!=null&&l.doc?vv(this,l,a,s):JSON.stringify(this)}}function Jm(u,l,a){return(l.inFlow??u.flow?wv:Tv)(u,l,a)}function Tv({comment:u,items:l},a,{blockItemPrefix:s,flowChars:r,itemIndent:o,onChompKeep:h,onComment:d}){const{indent:m,options:{commentString:p}}=a,b=Object.assign({},a,{indent:o,type:null});let S=!1;const N=[];for(let D=0;D<l.length;++D){const T=l[D];let w=null;if(ze(T))!S&&T.spaceBefore&&N.push(""),du(a,N,T.commentBefore,S),T.comment&&(w=T.comment);else if(Re(T)){const L=ze(T.key)?T.key:null;L&&(!S&&L.spaceBefore&&N.push(""),du(a,N,L.commentBefore,S))}S=!1;let k=mi(T,b,()=>w=null,()=>S=!0);w&&(k+=dl(k,o,p(w))),S&&w&&(S=!1),N.push(s+k)}let E;if(N.length===0)E=r.start+r.end;else{E=N[0];for(let D=1;D<N.length;++D){const T=N[D];E+=T?`
${m}${T}`:`
`}}return u?(E+=`
`+pn(p(u),m),d&&d()):S&&h&&h(),E}function wv({items:u},l,{flowChars:a,itemIndent:s}){const{indent:r,indentStep:o,flowCollectionPadding:h,options:{commentString:d}}=l;s+=o;const m=Object.assign({},l,{indent:s,inFlow:!0,type:null});let p=!1,b=0;const S=[];for(let D=0;D<u.length;++D){const T=u[D];let w=null;if(ze(T))T.spaceBefore&&S.push(""),du(l,S,T.commentBefore,!1),T.comment&&(w=T.comment);else if(Re(T)){const L=ze(T.key)?T.key:null;L&&(L.spaceBefore&&S.push(""),du(l,S,L.commentBefore,!1),L.comment&&(p=!0));const H=ze(T.value)?T.value:null;H?(H.comment&&(w=H.comment),H.commentBefore&&(p=!0)):T.value==null&&(L!=null&&L.comment)&&(w=L.comment)}w&&(p=!0);let k=mi(T,m,()=>w=null);D<u.length-1&&(k+=","),w&&(k+=dl(k,s,d(w))),!p&&(S.length>b||k.includes(`
`))&&(p=!0),S.push(k),b=S.length}const{start:N,end:E}=a;if(S.length===0)return N+E;if(!p){const D=S.reduce((T,w)=>T+w.length+2,2);p=l.options.lineWidth>0&&D>l.options.lineWidth}if(p){let D=N;for(const T of S)D+=T?`
${o}${r}${T}`:`
`;return`${D}
${r}${E}`}else return`${N}${h}${S.join(" ")}${h}${E}`}function du({indent:u,options:{commentString:l}},a,s,r){if(s&&r&&(s=s.replace(/^\n+/,"")),s){const o=pn(l(s),u);a.push(o.trimStart())}}function gl(u,l){const a=_e(l)?l.value:l;for(const s of u)if(Re(s)&&(s.key===l||s.key===a||_e(s.key)&&s.key.value===a))return s}class Mt extends Gm{static get tagName(){return"tag:yaml.org,2002:map"}constructor(l){super(Kn,l),this.items=[]}static from(l,a,s){const{keepUndefined:r,replacer:o}=s,h=new this(l),d=(m,p)=>{if(typeof o=="function")p=o.call(a,m,p);else if(Array.isArray(o)&&!o.includes(m))return;(p!==void 0||r)&&h.items.push(Of(m,p,s))};if(a instanceof Map)for(const[m,p]of a)d(m,p);else if(a&&typeof a=="object")for(const m of Object.keys(a))d(m,a[m]);return typeof l.sortMapEntries=="function"&&h.items.sort(l.sortMapEntries),h}add(l,a){var h;let s;Re(l)?s=l:!l||typeof l!="object"||!("key"in l)?s=new st(l,l==null?void 0:l.value):s=new st(l.key,l.value);const r=gl(this.items,s.key),o=(h=this.schema)==null?void 0:h.sortMapEntries;if(r){if(!a)throw new Error(`Key ${s.key} already set`);_e(r.value)&&Ym(s.value)?r.value.value=s.value:r.value=s.value}else if(o){const d=this.items.findIndex(m=>o(s,m)<0);d===-1?this.items.push(s):this.items.splice(d,0,s)}else this.items.push(s)}delete(l){const a=gl(this.items,l);return a?this.items.splice(this.items.indexOf(a),1).length>0:!1}get(l,a){const s=gl(this.items,l),r=s==null?void 0:s.value;return(!a&&_e(r)?r.value:r)??void 0}has(l){return!!gl(this.items,l)}set(l,a){this.add(new st(l,a),!0)}toJSON(l,a,s){const r=s?new s:a!=null&&a.mapAsMap?new Map:{};a!=null&&a.onCreate&&a.onCreate(r);for(const o of this.items)Zm(a,r,o);return r}toString(l,a,s){if(!l)return JSON.stringify(this);for(const r of this.items)if(!Re(r))throw new Error(`Map items must all be pairs; found ${JSON.stringify(r)} instead`);return!l.allNullValues&&this.hasAllNullValues(!1)&&(l=Object.assign({},l,{allNullValues:!0})),Jm(this,l,{blockItemPrefix:"",flowChars:{start:"{",end:"}"},itemIndent:l.indent||"",onChompKeep:s,onComment:a})}}const wi={collection:"map",default:!0,nodeClass:Mt,tag:"tag:yaml.org,2002:map",resolve(u,l){return Si(u)||l("Expected a mapping for this tag"),u},createNode:(u,l,a)=>Mt.from(u,l,a)};class Qn extends Gm{static get tagName(){return"tag:yaml.org,2002:seq"}constructor(l){super(bi,l),this.items=[]}add(l){this.items.push(l)}delete(l){const a=Is(l);return typeof a!="number"?!1:this.items.splice(a,1).length>0}get(l,a){const s=Is(l);if(typeof s!="number")return;const r=this.items[s];return!a&&_e(r)?r.value:r}has(l){const a=Is(l);return typeof a=="number"&&a<this.items.length}set(l,a){const s=Is(l);if(typeof s!="number")throw new Error(`Expected a valid index, not ${l}.`);const r=this.items[s];_e(r)&&Ym(a)?r.value=a:this.items[s]=a}toJSON(l,a){const s=[];a!=null&&a.onCreate&&a.onCreate(s);let r=0;for(const o of this.items)s.push(qt(o,String(r++),a));return s}toString(l,a,s){return l?Jm(this,l,{blockItemPrefix:"- ",flowChars:{start:"[",end:"]"},itemIndent:(l.indent||"")+"  ",onChompKeep:s,onComment:a}):JSON.stringify(this)}static from(l,a,s){const{replacer:r}=s,o=new this(l);if(a&&Symbol.iterator in Object(a)){let h=0;for(let d of a){if(typeof r=="function"){const m=a instanceof Set?d:String(h++);d=r.call(a,m,d)}o.items.push(Ma(d,void 0,s))}}return o}}function Is(u){let l=_e(u)?u.value:u;return l&&typeof l=="string"&&(l=Number(l)),typeof l=="number"&&Number.isInteger(l)&&l>=0?l:null}const Ei={collection:"seq",default:!0,nodeClass:Qn,tag:"tag:yaml.org,2002:seq",resolve(u,l){return Ti(u)||l("Expected a sequence for this tag"),u},createNode:(u,l,a)=>Qn.from(u,l,a)},Au={identify:u=>typeof u=="string",default:!0,tag:"tag:yaml.org,2002:str",resolve:u=>u,stringify(u,l,a,s){return l=Object.assign({actualString:!0},l),Ra(u,l,a,s)}},Ou={identify:u=>u==null,createNode:()=>new re(null),default:!0,tag:"tag:yaml.org,2002:null",test:/^(?:~|[Nn]ull|NULL)?$/,resolve:()=>new re(null),stringify:({source:u},l)=>typeof u=="string"&&Ou.test.test(u)?u:l.options.nullStr},Nf={identify:u=>typeof u=="boolean",default:!0,tag:"tag:yaml.org,2002:bool",test:/^(?:[Tt]rue|TRUE|[Ff]alse|FALSE)$/,resolve:u=>new re(u[0]==="t"||u[0]==="T"),stringify({source:u,value:l},a){if(u&&Nf.test.test(u)){const s=u[0]==="t"||u[0]==="T";if(l===s)return u}return l?a.options.trueStr:a.options.falseStr}};function Kt({format:u,minFractionDigits:l,tag:a,value:s}){if(typeof s=="bigint")return String(s);const r=typeof s=="number"?s:Number(s);if(!isFinite(r))return isNaN(r)?".nan":r<0?"-.inf":".inf";let o=JSON.stringify(s);if(!u&&l&&(!a||a==="tag:yaml.org,2002:float")&&/^\d/.test(o)){let h=o.indexOf(".");h<0&&(h=o.length,o+=".");let d=l-(o.length-h-1);for(;d-- >0;)o+="0"}return o}const Wm={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",test:/^(?:[-+]?\.(?:inf|Inf|INF)|\.nan|\.NaN|\.NAN)$/,resolve:u=>u.slice(-3).toLowerCase()==="nan"?NaN:u[0]==="-"?Number.NEGATIVE_INFINITY:Number.POSITIVE_INFINITY,stringify:Kt},Im={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",format:"EXP",test:/^[-+]?(?:\.[0-9]+|[0-9]+(?:\.[0-9]*)?)[eE][-+]?[0-9]+$/,resolve:u=>parseFloat(u),stringify(u){const l=Number(u.value);return isFinite(l)?l.toExponential():Kt(u)}},Fm={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",test:/^[-+]?(?:\.[0-9]+|[0-9]+\.[0-9]*)$/,resolve(u){const l=new re(parseFloat(u)),a=u.indexOf(".");return a!==-1&&u[u.length-1]==="0"&&(l.minFractionDigits=u.length-a-1),l},stringify:Kt},Nu=u=>typeof u=="bigint"||Number.isInteger(u),_f=(u,l,a,{intAsBigInt:s})=>s?BigInt(u):parseInt(u.substring(l),a);function Pm(u,l,a){const{value:s}=u;return Nu(s)&&s>=0?a+s.toString(l):Kt(u)}const ep={identify:u=>Nu(u)&&u>=0,default:!0,tag:"tag:yaml.org,2002:int",format:"OCT",test:/^0o[0-7]+$/,resolve:(u,l,a)=>_f(u,2,8,a),stringify:u=>Pm(u,8,"0o")},tp={identify:Nu,default:!0,tag:"tag:yaml.org,2002:int",test:/^[-+]?[0-9]+$/,resolve:(u,l,a)=>_f(u,0,10,a),stringify:Kt},np={identify:u=>Nu(u)&&u>=0,default:!0,tag:"tag:yaml.org,2002:int",format:"HEX",test:/^0x[0-9a-fA-F]+$/,resolve:(u,l,a)=>_f(u,2,16,a),stringify:u=>Pm(u,16,"0x")},Ev=[wi,Ei,Au,Ou,Nf,ep,tp,np,Wm,Im,Fm];function Pg(u){return typeof u=="bigint"||Number.isInteger(u)}const Fs=({value:u})=>JSON.stringify(u),Av=[{identify:u=>typeof u=="string",default:!0,tag:"tag:yaml.org,2002:str",resolve:u=>u,stringify:Fs},{identify:u=>u==null,createNode:()=>new re(null),default:!0,tag:"tag:yaml.org,2002:null",test:/^null$/,resolve:()=>null,stringify:Fs},{identify:u=>typeof u=="boolean",default:!0,tag:"tag:yaml.org,2002:bool",test:/^true|false$/,resolve:u=>u==="true",stringify:Fs},{identify:Pg,default:!0,tag:"tag:yaml.org,2002:int",test:/^-?(?:0|[1-9][0-9]*)$/,resolve:(u,l,{intAsBigInt:a})=>a?BigInt(u):parseInt(u,10),stringify:({value:u})=>Pg(u)?u.toString():JSON.stringify(u)},{identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",test:/^-?(?:0|[1-9][0-9]*)(?:\.[0-9]*)?(?:[eE][-+]?[0-9]+)?$/,resolve:u=>parseFloat(u),stringify:Fs}],Ov={default:!0,tag:"",test:/^/,resolve(u,l){return l(`Unresolved plain scalar ${JSON.stringify(u)}`),u}},Nv=[wi,Ei].concat(Av,Ov),Mf={identify:u=>u instanceof Uint8Array,default:!1,tag:"tag:yaml.org,2002:binary",resolve(u,l){if(typeof Buffer=="function")return Buffer.from(u,"base64");if(typeof atob=="function"){const a=atob(u.replace(/[\n\r]/g,"")),s=new Uint8Array(a.length);for(let r=0;r<a.length;++r)s[r]=a.charCodeAt(r);return s}else return l("This environment does not support reading binary tags; either Buffer or atob is required"),u},stringify({comment:u,type:l,value:a},s,r,o){const h=a;let d;if(typeof Buffer=="function")d=h instanceof Buffer?h.toString("base64"):Buffer.from(h.buffer).toString("base64");else if(typeof btoa=="function"){let m="";for(let p=0;p<h.length;++p)m+=String.fromCharCode(h[p]);d=btoa(m)}else throw new Error("This environment does not support writing binary tags; either Buffer or btoa is required");if(l||(l=re.BLOCK_LITERAL),l!==re.QUOTE_DOUBLE){const m=Math.max(s.options.lineWidth-s.indent.length,s.options.minContentWidth),p=Math.ceil(d.length/m),b=new Array(p);for(let S=0,N=0;S<p;++S,N+=m)b[S]=d.substr(N,m);d=b.join(l===re.BLOCK_LITERAL?`
`:" ")}return Ra({comment:u,type:l,value:d},s,r,o)}};function lp(u,l){if(Ti(u))for(let a=0;a<u.items.length;++a){let s=u.items[a];if(!Re(s)){if(Si(s)){s.items.length>1&&l("Each pair must have its own sequence indicator");const r=s.items[0]||new st(new re(null));if(s.commentBefore&&(r.key.commentBefore=r.key.commentBefore?`${s.commentBefore}
${r.key.commentBefore}`:s.commentBefore),s.comment){const o=r.value??r.key;o.comment=o.comment?`${s.comment}
${o.comment}`:s.comment}s=r}u.items[a]=Re(s)?s:new st(s)}}else l("Expected a sequence for this tag");return u}function ip(u,l,a){const{replacer:s}=a,r=new Qn(u);r.tag="tag:yaml.org,2002:pairs";let o=0;if(l&&Symbol.iterator in Object(l))for(let h of l){typeof s=="function"&&(h=s.call(l,String(o++),h));let d,m;if(Array.isArray(h))if(h.length===2)d=h[0],m=h[1];else throw new TypeError(`Expected [key, value] tuple: ${h}`);else if(h&&h instanceof Object){const p=Object.keys(h);if(p.length===1)d=p[0],m=h[d];else throw new TypeError(`Expected tuple with one key, not ${p.length} keys`)}else d=h;r.items.push(Of(d,m,a))}return r}const xf={collection:"seq",default:!1,tag:"tag:yaml.org,2002:pairs",resolve:lp,createNode:ip};class di extends Qn{constructor(){super(),this.add=Mt.prototype.add.bind(this),this.delete=Mt.prototype.delete.bind(this),this.get=Mt.prototype.get.bind(this),this.has=Mt.prototype.has.bind(this),this.set=Mt.prototype.set.bind(this),this.tag=di.tag}toJSON(l,a){if(!a)return super.toJSON(l);const s=new Map;a!=null&&a.onCreate&&a.onCreate(s);for(const r of this.items){let o,h;if(Re(r)?(o=qt(r.key,"",a),h=qt(r.value,o,a)):o=qt(r,"",a),s.has(o))throw new Error("Ordered maps must not include duplicate keys");s.set(o,h)}return s}static from(l,a,s){const r=ip(l,a,s),o=new this;return o.items=r.items,o}}di.tag="tag:yaml.org,2002:omap";const Df={collection:"seq",identify:u=>u instanceof Map,nodeClass:di,default:!1,tag:"tag:yaml.org,2002:omap",resolve(u,l){const a=lp(u,l),s=[];for(const{key:r}of a.items)_e(r)&&(s.includes(r.value)?l(`Ordered maps must not include duplicate keys: ${r.value}`):s.push(r.value));return Object.assign(new di,a)},createNode:(u,l,a)=>di.from(u,l,a)};function ap({value:u,source:l},a){return l&&(u?sp:up).test.test(l)?l:u?a.options.trueStr:a.options.falseStr}const sp={identify:u=>u===!0,default:!0,tag:"tag:yaml.org,2002:bool",test:/^(?:Y|y|[Yy]es|YES|[Tt]rue|TRUE|[Oo]n|ON)$/,resolve:()=>new re(!0),stringify:ap},up={identify:u=>u===!1,default:!0,tag:"tag:yaml.org,2002:bool",test:/^(?:N|n|[Nn]o|NO|[Ff]alse|FALSE|[Oo]ff|OFF)$/,resolve:()=>new re(!1),stringify:ap},_v={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",test:/^(?:[-+]?\.(?:inf|Inf|INF)|\.nan|\.NaN|\.NAN)$/,resolve:u=>u.slice(-3).toLowerCase()==="nan"?NaN:u[0]==="-"?Number.NEGATIVE_INFINITY:Number.POSITIVE_INFINITY,stringify:Kt},Mv={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",format:"EXP",test:/^[-+]?(?:[0-9][0-9_]*)?(?:\.[0-9_]*)?[eE][-+]?[0-9]+$/,resolve:u=>parseFloat(u.replace(/_/g,"")),stringify(u){const l=Number(u.value);return isFinite(l)?l.toExponential():Kt(u)}},xv={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",test:/^[-+]?(?:[0-9][0-9_]*)?\.[0-9_]*$/,resolve(u){const l=new re(parseFloat(u.replace(/_/g,""))),a=u.indexOf(".");if(a!==-1){const s=u.substring(a+1).replace(/_/g,"");s[s.length-1]==="0"&&(l.minFractionDigits=s.length)}return l},stringify:Kt},ka=u=>typeof u=="bigint"||Number.isInteger(u);function _u(u,l,a,{intAsBigInt:s}){const r=u[0];if((r==="-"||r==="+")&&(l+=1),u=u.substring(l).replace(/_/g,""),s){switch(a){case 2:u=`0b${u}`;break;case 8:u=`0o${u}`;break;case 16:u=`0x${u}`;break}const h=BigInt(u);return r==="-"?BigInt(-1)*h:h}const o=parseInt(u,a);return r==="-"?-1*o:o}function Cf(u,l,a){const{value:s}=u;if(ka(s)){const r=s.toString(l);return s<0?"-"+a+r.substr(1):a+r}return Kt(u)}const Dv={identify:ka,default:!0,tag:"tag:yaml.org,2002:int",format:"BIN",test:/^[-+]?0b[0-1_]+$/,resolve:(u,l,a)=>_u(u,2,2,a),stringify:u=>Cf(u,2,"0b")},Cv={identify:ka,default:!0,tag:"tag:yaml.org,2002:int",format:"OCT",test:/^[-+]?0[0-7_]+$/,resolve:(u,l,a)=>_u(u,1,8,a),stringify:u=>Cf(u,8,"0")},Rv={identify:ka,default:!0,tag:"tag:yaml.org,2002:int",test:/^[-+]?[0-9][0-9_]*$/,resolve:(u,l,a)=>_u(u,0,10,a),stringify:Kt},kv={identify:ka,default:!0,tag:"tag:yaml.org,2002:int",format:"HEX",test:/^[-+]?0x[0-9a-fA-F_]+$/,resolve:(u,l,a)=>_u(u,2,16,a),stringify:u=>Cf(u,16,"0x")};class gi extends Mt{constructor(l){super(l),this.tag=gi.tag}add(l){let a;Re(l)?a=l:l&&typeof l=="object"&&"key"in l&&"value"in l&&l.value===null?a=new st(l.key,null):a=new st(l,null),gl(this.items,a.key)||this.items.push(a)}get(l,a){const s=gl(this.items,l);return!a&&Re(s)?_e(s.key)?s.key.value:s.key:s}set(l,a){if(typeof a!="boolean")throw new Error(`Expected boolean value for set(key, value) in a YAML set, not ${typeof a}`);const s=gl(this.items,l);s&&!a?this.items.splice(this.items.indexOf(s),1):!s&&a&&this.items.push(new st(l))}toJSON(l,a){return super.toJSON(l,a,Set)}toString(l,a,s){if(!l)return JSON.stringify(this);if(this.hasAllNullValues(!0))return super.toString(Object.assign({},l,{allNullValues:!0}),a,s);throw new Error("Set items must all have null values")}static from(l,a,s){const{replacer:r}=s,o=new this(l);if(a&&Symbol.iterator in Object(a))for(let h of a)typeof r=="function"&&(h=r.call(a,h,h)),o.items.push(Of(h,null,s));return o}}gi.tag="tag:yaml.org,2002:set";const Rf={collection:"map",identify:u=>u instanceof Set,nodeClass:gi,default:!1,tag:"tag:yaml.org,2002:set",createNode:(u,l,a)=>gi.from(u,l,a),resolve(u,l){if(Si(u)){if(u.hasAllNullValues(!0))return Object.assign(new gi,u);l("Set items must all have null values")}else l("Expected a mapping for this tag");return u}};function kf(u,l){const a=u[0],s=a==="-"||a==="+"?u.substring(1):u,r=h=>l?BigInt(h):Number(h),o=s.replace(/_/g,"").split(":").reduce((h,d)=>h*r(60)+r(d),r(0));return a==="-"?r(-1)*o:o}function cp(u){let{value:l}=u,a=h=>h;if(typeof l=="bigint")a=h=>BigInt(h);else if(isNaN(l)||!isFinite(l))return Kt(u);let s="";l<0&&(s="-",l*=a(-1));const r=a(60),o=[l%r];return l<60?o.unshift(0):(l=(l-o[0])/r,o.unshift(l%r),l>=60&&(l=(l-o[0])/r,o.unshift(l))),s+o.map(h=>String(h).padStart(2,"0")).join(":").replace(/000000\d*$/,"")}const rp={identify:u=>typeof u=="bigint"||Number.isInteger(u),default:!0,tag:"tag:yaml.org,2002:int",format:"TIME",test:/^[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+$/,resolve:(u,l,{intAsBigInt:a})=>kf(u,a),stringify:cp},fp={identify:u=>typeof u=="number",default:!0,tag:"tag:yaml.org,2002:float",format:"TIME",test:/^[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\.[0-9_]*$/,resolve:u=>kf(u,!1),stringify:cp},Mu={identify:u=>u instanceof Date,default:!0,tag:"tag:yaml.org,2002:timestamp",test:RegExp("^([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})(?:(?:t|T|[ \\t]+)([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2}(\\.[0-9]+)?)(?:[ \\t]*(Z|[-+][012]?[0-9](?::[0-9]{2})?))?)?$"),resolve(u){const l=u.match(Mu.test);if(!l)throw new Error("!!timestamp expects a date, starting with yyyy-mm-dd");const[,a,s,r,o,h,d]=l.map(Number),m=l[7]?Number((l[7]+"00").substr(1,3)):0;let p=Date.UTC(a,s-1,r,o||0,h||0,d||0,m);const b=l[8];if(b&&b!=="Z"){let S=kf(b,!1);Math.abs(S)<30&&(S*=60),p-=6e4*S}return new Date(p)},stringify:({value:u})=>u.toISOString().replace(/((T00:00)?:00)?\.000Z$/,"")},em=[wi,Ei,Au,Ou,sp,up,Dv,Cv,Rv,kv,_v,Mv,xv,Mf,yn,Df,xf,Rf,rp,fp,Mu],tm=new Map([["core",Ev],["failsafe",[wi,Ei,Au]],["json",Nv],["yaml11",em],["yaml-1.1",em]]),nm={binary:Mf,bool:Nf,float:Fm,floatExp:Im,floatNaN:Wm,floatTime:fp,int:tp,intHex:np,intOct:ep,intTime:rp,map:wi,merge:yn,null:Ou,omap:Df,pairs:xf,seq:Ei,set:Rf,timestamp:Mu},Lv={"tag:yaml.org,2002:binary":Mf,"tag:yaml.org,2002:merge":yn,"tag:yaml.org,2002:omap":Df,"tag:yaml.org,2002:pairs":xf,"tag:yaml.org,2002:set":Rf,"tag:yaml.org,2002:timestamp":Mu};function lf(u,l,a){const s=tm.get(l);if(s&&!u)return a&&!s.includes(yn)?s.concat(yn):s.slice();let r=s;if(!r)if(Array.isArray(u))r=[];else{const o=Array.from(tm.keys()).filter(h=>h!=="yaml11").map(h=>JSON.stringify(h)).join(", ");throw new Error(`Unknown schema "${l}"; use one of ${o} or define customTags array`)}if(Array.isArray(u))for(const o of u)r=r.concat(o);else typeof u=="function"&&(r=u(r.slice()));return a&&(r=r.concat(yn)),r.reduce((o,h)=>{const d=typeof h=="string"?nm[h]:h;if(!d){const m=JSON.stringify(h),p=Object.keys(nm).map(b=>JSON.stringify(b)).join(", ");throw new Error(`Unknown custom tag ${m}; use one of ${p}`)}return o.includes(d)||o.push(d),o},[])}const zv=(u,l)=>u.key<l.key?-1:u.key>l.key?1:0;class xu{constructor({compat:l,customTags:a,merge:s,resolveKnownTags:r,schema:o,sortMapEntries:h,toStringDefaults:d}){this.compat=Array.isArray(l)?lf(l,"compat"):l?lf(null,l):null,this.name=typeof o=="string"&&o||"core",this.knownTags=r?Lv:{},this.tags=lf(a,this.name,s),this.toStringOptions=d??null,Object.defineProperty(this,Kn,{value:wi}),Object.defineProperty(this,Ft,{value:Au}),Object.defineProperty(this,bi,{value:Ei}),this.sortMapEntries=typeof h=="function"?h:h===!0?zv:null}clone(){const l=Object.create(xu.prototype,Object.getOwnPropertyDescriptors(this));return l.tags=this.tags.slice(),l}}function Uv(u,l){var m;const a=[];let s=l.directives===!0;if(l.directives!==!1&&u.directives){const p=u.directives.toString(u);p?(a.push(p),s=!0):u.directives.docStart&&(s=!0)}s&&a.push("---");const r=Vm(u,l),{commentString:o}=r.options;if(u.commentBefore){a.length!==1&&a.unshift("");const p=o(u.commentBefore);a.unshift(pn(p,""))}let h=!1,d=null;if(u.contents){if(ze(u.contents)){if(u.contents.spaceBefore&&s&&a.push(""),u.contents.commentBefore){const S=o(u.contents.commentBefore);a.push(pn(S,""))}r.forceBlockIndent=!!u.comment,d=u.contents.comment}const p=d?void 0:()=>h=!0;let b=mi(u.contents,r,()=>d=null,p);d&&(b+=dl(b,"",o(d))),(b[0]==="|"||b[0]===">")&&a[a.length-1]==="---"?a[a.length-1]=`--- ${b}`:a.push(b)}else a.push(mi(u.contents,r));if((m=u.directives)!=null&&m.docEnd)if(u.comment){const p=o(u.comment);p.includes(`
`)?(a.push("..."),a.push(pn(p,""))):a.push(`... ${p}`)}else a.push("...");else{let p=u.comment;p&&h&&(p=p.replace(/^\n+/,"")),p&&((!h||d)&&a[a.length-1]!==""&&a.push(""),a.push(pn(o(p),"")))}return a.join(`
`)+`
`}class Ai{constructor(l,a,s){this.commentBefore=null,this.comment=null,this.errors=[],this.warnings=[],Object.defineProperty(this,jt,{value:mf});let r=null;typeof a=="function"||Array.isArray(a)?r=a:s===void 0&&a&&(s=a,a=void 0);const o=Object.assign({intAsBigInt:!1,keepSourceTokens:!1,logLevel:"warn",prettyErrors:!0,strict:!0,stringKeys:!1,uniqueKeys:!0,version:"1.2"},s);this.options=o;let{version:h}=o;s!=null&&s._directives?(this.directives=s._directives.atDocument(),this.directives.yaml.explicit&&(h=this.directives.yaml.version)):this.directives=new ct({version:h}),this.setSchema(h,s),this.contents=l===void 0?null:this.createNode(l,r,s)}clone(){const l=Object.create(Ai.prototype,{[jt]:{value:mf}});return l.commentBefore=this.commentBefore,l.comment=this.comment,l.errors=this.errors.slice(),l.warnings=this.warnings.slice(),l.options=Object.assign({},this.options),this.directives&&(l.directives=this.directives.clone()),l.schema=this.schema.clone(),l.contents=ze(this.contents)?this.contents.clone(l.schema):this.contents,this.range&&(l.range=this.range.slice()),l}add(l){ai(this.contents)&&this.contents.add(l)}addIn(l,a){ai(this.contents)&&this.contents.addIn(l,a)}createAlias(l,a){if(!l.anchor){const s=Hm(this);l.anchor=!a||s.has(a)?$m(a||"a",s):a}return new Su(l.anchor)}createNode(l,a,s){let r;if(typeof a=="function")l=a.call({"":l},"",l),r=a;else if(Array.isArray(a)){const w=L=>typeof L=="number"||L instanceof String||L instanceof Number,k=a.filter(w).map(String);k.length>0&&(a=a.concat(k)),r=a}else s===void 0&&a&&(s=a,a=void 0);const{aliasDuplicateObjects:o,anchorPrefix:h,flow:d,keepUndefined:m,onTagObj:p,tag:b}=s??{},{onAnchor:S,setAnchors:N,sourceObjects:E}=fv(this,h||"a"),D={aliasDuplicateObjects:o??!0,keepUndefined:m??!1,onAnchor:S,onTagObj:p,replacer:r,schema:this.schema,sourceObjects:E},T=Ma(l,b,D);return d&&Le(T)&&(T.flow=!0),N(),T}createPair(l,a,s={}){const r=this.createNode(l,null,s),o=this.createNode(a,null,s);return new st(r,o)}delete(l){return ai(this.contents)?this.contents.delete(l):!1}deleteIn(l){return Na(l)?this.contents==null?!1:(this.contents=null,!0):ai(this.contents)?this.contents.deleteIn(l):!1}get(l,a){return Le(this.contents)?this.contents.get(l,a):void 0}getIn(l,a){return Na(l)?!a&&_e(this.contents)?this.contents.value:this.contents:Le(this.contents)?this.contents.getIn(l,a):void 0}has(l){return Le(this.contents)?this.contents.has(l):!1}hasIn(l){return Na(l)?this.contents!==void 0:Le(this.contents)?this.contents.hasIn(l):!1}set(l,a){this.contents==null?this.contents=hu(this.schema,[l],a):ai(this.contents)&&this.contents.set(l,a)}setIn(l,a){Na(l)?this.contents=a:this.contents==null?this.contents=hu(this.schema,Array.from(l),a):ai(this.contents)&&this.contents.setIn(l,a)}setSchema(l,a={}){typeof l=="number"&&(l=String(l));let s;switch(l){case"1.1":this.directives?this.directives.yaml.version="1.1":this.directives=new ct({version:"1.1"}),s={resolveKnownTags:!1,schema:"yaml-1.1"};break;case"1.2":case"next":this.directives?this.directives.yaml.version=l:this.directives=new ct({version:l}),s={resolveKnownTags:!0,schema:"core"};break;case null:this.directives&&delete this.directives,s=null;break;default:{const r=JSON.stringify(l);throw new Error(`Expected '1.1', '1.2' or null as first argument, but found: ${r}`)}}if(a.schema instanceof Object)this.schema=a.schema;else if(s)this.schema=new xu(Object.assign(s,a));else throw new Error("With a null YAML version, the { schema: Schema } option is required")}toJS({json:l,jsonArg:a,mapAsMap:s,maxAliasCount:r,onAnchor:o,reviver:h}={}){const d={anchors:new Map,doc:this,keep:!l,mapAsMap:s===!0,mapKeyWarned:!1,maxAliasCount:typeof r=="number"?r:100},m=qt(this.contents,a??"",d);if(typeof o=="function")for(const{count:p,res:b}of d.anchors.values())o(b,p);return typeof h=="function"?oi(h,{"":m},"",m):m}toJSON(l,a){return this.toJS({json:!0,jsonArg:l,mapAsMap:!1,onAnchor:a})}toString(l={}){if(this.errors.length>0)throw new Error("Document with errors cannot be stringified");if("indent"in l&&(!Number.isInteger(l.indent)||Number(l.indent)<=0)){const a=JSON.stringify(l.indent);throw new Error(`"indent" option must be a positive integer, not ${a}`)}return Uv(this,l)}}function ai(u){if(Le(u))return!0;throw new Error("Expected a YAML collection as document contents")}class Lf extends Error{constructor(l,a,s,r){super(),this.name=l,this.code=s,this.message=r,this.pos=a}}class ml extends Lf{constructor(l,a,s){super("YAMLParseError",l,a,s)}}class op extends Lf{constructor(l,a,s){super("YAMLWarning",l,a,s)}}const gu=(u,l)=>a=>{if(a.pos[0]===-1)return;a.linePos=a.pos.map(d=>l.linePos(d));const{line:s,col:r}=a.linePos[0];a.message+=` at line ${s}, column ${r}`;let o=r-1,h=u.substring(l.lineStarts[s-1],l.lineStarts[s]).replace(/[\n\r]+$/,"");if(o>=60&&h.length>80){const d=Math.min(o-39,h.length-79);h="…"+h.substring(d),o-=d-1}if(h.length>80&&(h=h.substring(0,79)+"…"),s>1&&/^ *$/.test(h.substring(0,o))){let d=u.substring(l.lineStarts[s-2],l.lineStarts[s-1]);d.length>80&&(d=d.substring(0,79)+`…
`),h=d+h}if(/[^ ]/.test(h)){let d=1;const m=a.linePos[1];m&&m.line===s&&m.col>r&&(d=Math.max(1,Math.min(m.col-r,80-o)));const p=" ".repeat(o)+"^".repeat(d);a.message+=`:

${h}
${p}
`}};function pi(u,{flow:l,indicator:a,next:s,offset:r,onError:o,parentIndent:h,startOnNewline:d}){let m=!1,p=d,b=d,S="",N="",E=!1,D=!1,T=null,w=null,k=null,L=null,H=null,Q=null,G=null;for(const V of u)switch(D&&(V.type!=="space"&&V.type!=="newline"&&V.type!=="comma"&&o(V.offset,"MISSING_CHAR","Tags and anchors must be separated from the next token by white space"),D=!1),T&&(p&&V.type!=="comment"&&V.type!=="newline"&&o(T,"TAB_AS_INDENT","Tabs are not allowed as indentation"),T=null),V.type){case"space":!l&&(a!=="doc-start"||(s==null?void 0:s.type)!=="flow-collection")&&V.source.includes("	")&&(T=V),b=!0;break;case"comment":{b||o(V,"MISSING_CHAR","Comments must be separated from other tokens by white space characters");const z=V.source.substring(1)||" ";S?S+=N+z:S=z,N="",p=!1;break}case"newline":p?S?S+=V.source:m=!0:N+=V.source,p=!0,E=!0,(w||k)&&(L=V),b=!0;break;case"anchor":w&&o(V,"MULTIPLE_ANCHORS","A node can have at most one anchor"),V.source.endsWith(":")&&o(V.offset+V.source.length-1,"BAD_ALIAS","Anchor ending in : is ambiguous",!0),w=V,G===null&&(G=V.offset),p=!1,b=!1,D=!0;break;case"tag":{k&&o(V,"MULTIPLE_TAGS","A node can have at most one tag"),k=V,G===null&&(G=V.offset),p=!1,b=!1,D=!0;break}case a:(w||k)&&o(V,"BAD_PROP_ORDER",`Anchors and tags must be after the ${V.source} indicator`),Q&&o(V,"UNEXPECTED_TOKEN",`Unexpected ${V.source} in ${l??"collection"}`),Q=V,p=a==="seq-item-ind"||a==="explicit-key-ind",b=!1;break;case"comma":if(l){H&&o(V,"UNEXPECTED_TOKEN",`Unexpected , in ${l}`),H=V,p=!1,b=!1;break}default:o(V,"UNEXPECTED_TOKEN",`Unexpected ${V.type} token`),p=!1,b=!1}const K=u[u.length-1],ee=K?K.offset+K.source.length:r;return D&&s&&s.type!=="space"&&s.type!=="newline"&&s.type!=="comma"&&(s.type!=="scalar"||s.source!=="")&&o(s.offset,"MISSING_CHAR","Tags and anchors must be separated from the next token by white space"),T&&(p&&T.indent<=h||(s==null?void 0:s.type)==="block-map"||(s==null?void 0:s.type)==="block-seq")&&o(T,"TAB_AS_INDENT","Tabs are not allowed as indentation"),{comma:H,found:Q,spaceBefore:m,comment:S,hasNewline:E,anchor:w,tag:k,newlineAfterProp:L,end:ee,start:G??ee}}function xa(u){if(!u)return null;switch(u.type){case"alias":case"scalar":case"double-quoted-scalar":case"single-quoted-scalar":if(u.source.includes(`
`))return!0;if(u.end){for(const l of u.end)if(l.type==="newline")return!0}return!1;case"flow-collection":for(const l of u.items){for(const a of l.start)if(a.type==="newline")return!0;if(l.sep){for(const a of l.sep)if(a.type==="newline")return!0}if(xa(l.key)||xa(l.value))return!0}return!1;default:return!0}}function bf(u,l,a){if((l==null?void 0:l.type)==="flow-collection"){const s=l.end[0];s.indent===u&&(s.source==="]"||s.source==="}")&&xa(l)&&a(s,"BAD_INDENT","Flow end indicator should be more indented than parent",!0)}}function hp(u,l,a){const{uniqueKeys:s}=u.options;if(s===!1)return!1;const r=typeof s=="function"?s:(o,h)=>o===h||_e(o)&&_e(h)&&o.value===h.value;return l.some(o=>r(o.key,a))}const lm="All mapping items must start at the same column";function Bv({composeNode:u,composeEmptyNode:l},a,s,r,o){var b;const h=(o==null?void 0:o.nodeClass)??Mt,d=new h(a.schema);a.atRoot&&(a.atRoot=!1);let m=s.offset,p=null;for(const S of s.items){const{start:N,key:E,sep:D,value:T}=S,w=pi(N,{indicator:"explicit-key-ind",next:E??(D==null?void 0:D[0]),offset:m,onError:r,parentIndent:s.indent,startOnNewline:!0}),k=!w.found;if(k){if(E&&(E.type==="block-seq"?r(m,"BLOCK_AS_IMPLICIT_KEY","A block sequence may not be used as an implicit map key"):"indent"in E&&E.indent!==s.indent&&r(m,"BAD_INDENT",lm)),!w.anchor&&!w.tag&&!D){p=w.end,w.comment&&(d.comment?d.comment+=`
`+w.comment:d.comment=w.comment);continue}(w.newlineAfterProp||xa(E))&&r(E??N[N.length-1],"MULTILINE_IMPLICIT_KEY","Implicit keys need to be on a single line")}else((b=w.found)==null?void 0:b.indent)!==s.indent&&r(m,"BAD_INDENT",lm);a.atKey=!0;const L=w.end,H=E?u(a,E,w,r):l(a,L,N,null,w,r);a.schema.compat&&bf(s.indent,E,r),a.atKey=!1,hp(a,d.items,H)&&r(L,"DUPLICATE_KEY","Map keys must be unique");const Q=pi(D??[],{indicator:"map-value-ind",next:T,offset:H.range[2],onError:r,parentIndent:s.indent,startOnNewline:!E||E.type==="block-scalar"});if(m=Q.end,Q.found){k&&((T==null?void 0:T.type)==="block-map"&&!Q.hasNewline&&r(m,"BLOCK_AS_IMPLICIT_KEY","Nested mappings are not allowed in compact mappings"),a.options.strict&&w.start<Q.found.offset-1024&&r(H.range,"KEY_OVER_1024_CHARS","The : indicator must be at most 1024 chars after the start of an implicit block mapping key"));const G=T?u(a,T,Q,r):l(a,m,D,null,Q,r);a.schema.compat&&bf(s.indent,T,r),m=G.range[2];const K=new st(H,G);a.options.keepSourceTokens&&(K.srcToken=S),d.items.push(K)}else{k&&r(H.range,"MISSING_CHAR","Implicit map keys need to be followed by map values"),Q.comment&&(H.comment?H.comment+=`
`+Q.comment:H.comment=Q.comment);const G=new st(H);a.options.keepSourceTokens&&(G.srcToken=S),d.items.push(G)}}return p&&p<m&&r(p,"IMPOSSIBLE","Map comment with trailing content"),d.range=[s.offset,m,p??m],d}function qv({composeNode:u,composeEmptyNode:l},a,s,r,o){const h=(o==null?void 0:o.nodeClass)??Qn,d=new h(a.schema);a.atRoot&&(a.atRoot=!1),a.atKey&&(a.atKey=!1);let m=s.offset,p=null;for(const{start:b,value:S}of s.items){const N=pi(b,{indicator:"seq-item-ind",next:S,offset:m,onError:r,parentIndent:s.indent,startOnNewline:!0});if(!N.found)if(N.anchor||N.tag||S)S&&S.type==="block-seq"?r(N.end,"BAD_INDENT","All sequence items must start at the same column"):r(m,"MISSING_CHAR","Sequence item without - indicator");else{p=N.end,N.comment&&(d.comment=N.comment);continue}const E=S?u(a,S,N,r):l(a,N.end,b,null,N,r);a.schema.compat&&bf(s.indent,S,r),m=E.range[2],d.items.push(E)}return d.range=[s.offset,m,p??m],d}function La(u,l,a,s){let r="";if(u){let o=!1,h="";for(const d of u){const{source:m,type:p}=d;switch(p){case"space":o=!0;break;case"comment":{a&&!o&&s(d,"MISSING_CHAR","Comments must be separated from other tokens by white space characters");const b=m.substring(1)||" ";r?r+=h+b:r=b,h="";break}case"newline":r&&(h+=m),o=!0;break;default:s(d,"UNEXPECTED_TOKEN",`Unexpected ${p} at node end`)}l+=m.length}}return{comment:r,offset:l}}const af="Block collections are not allowed within flow collections",sf=u=>u&&(u.type==="block-map"||u.type==="block-seq");function jv({composeNode:u,composeEmptyNode:l},a,s,r,o){const h=s.start.source==="{",d=h?"flow map":"flow sequence",m=(o==null?void 0:o.nodeClass)??(h?Mt:Qn),p=new m(a.schema);p.flow=!0;const b=a.atRoot;b&&(a.atRoot=!1),a.atKey&&(a.atKey=!1);let S=s.offset+s.start.source.length;for(let w=0;w<s.items.length;++w){const k=s.items[w],{start:L,key:H,sep:Q,value:G}=k,K=pi(L,{flow:d,indicator:"explicit-key-ind",next:H??(Q==null?void 0:Q[0]),offset:S,onError:r,parentIndent:s.indent,startOnNewline:!1});if(!K.found){if(!K.anchor&&!K.tag&&!Q&&!G){w===0&&K.comma?r(K.comma,"UNEXPECTED_TOKEN",`Unexpected , in ${d}`):w<s.items.length-1&&r(K.start,"UNEXPECTED_TOKEN",`Unexpected empty item in ${d}`),K.comment&&(p.comment?p.comment+=`
`+K.comment:p.comment=K.comment),S=K.end;continue}!h&&a.options.strict&&xa(H)&&r(H,"MULTILINE_IMPLICIT_KEY","Implicit keys of flow sequence pairs need to be on a single line")}if(w===0)K.comma&&r(K.comma,"UNEXPECTED_TOKEN",`Unexpected , in ${d}`);else if(K.comma||r(K.start,"MISSING_CHAR",`Missing , between ${d} items`),K.comment){let ee="";e:for(const V of L)switch(V.type){case"comma":case"space":break;case"comment":ee=V.source.substring(1);break e;default:break e}if(ee){let V=p.items[p.items.length-1];Re(V)&&(V=V.value??V.key),V.comment?V.comment+=`
`+ee:V.comment=ee,K.comment=K.comment.substring(ee.length+1)}}if(!h&&!Q&&!K.found){const ee=G?u(a,G,K,r):l(a,K.end,Q,null,K,r);p.items.push(ee),S=ee.range[2],sf(G)&&r(ee.range,"BLOCK_IN_FLOW",af)}else{a.atKey=!0;const ee=K.end,V=H?u(a,H,K,r):l(a,ee,L,null,K,r);sf(H)&&r(V.range,"BLOCK_IN_FLOW",af),a.atKey=!1;const z=pi(Q??[],{flow:d,indicator:"map-value-ind",next:G,offset:V.range[2],onError:r,parentIndent:s.indent,startOnNewline:!1});if(z.found){if(!h&&!K.found&&a.options.strict){if(Q)for(const U of Q){if(U===z.found)break;if(U.type==="newline"){r(U,"MULTILINE_IMPLICIT_KEY","Implicit keys of flow sequence pairs need to be on a single line");break}}K.start<z.found.offset-1024&&r(z.found,"KEY_OVER_1024_CHARS","The : indicator must be at most 1024 chars after the start of an implicit flow sequence key")}}else G&&("source"in G&&G.source&&G.source[0]===":"?r(G,"MISSING_CHAR",`Missing space after : in ${d}`):r(z.start,"MISSING_CHAR",`Missing , or : between ${d} items`));const F=G?u(a,G,z,r):z.found?l(a,z.end,Q,null,z,r):null;F?sf(G)&&r(F.range,"BLOCK_IN_FLOW",af):z.comment&&(V.comment?V.comment+=`
`+z.comment:V.comment=z.comment);const se=new st(V,F);if(a.options.keepSourceTokens&&(se.srcToken=k),h){const U=p;hp(a,U.items,V)&&r(ee,"DUPLICATE_KEY","Map keys must be unique"),U.items.push(se)}else{const U=new Mt(a.schema);U.flow=!0,U.items.push(se);const te=(F??V).range;U.range=[V.range[0],te[1],te[2]],p.items.push(U)}S=F?F.range[2]:z.end}}const N=h?"}":"]",[E,...D]=s.end;let T=S;if(E&&E.source===N)T=E.offset+E.source.length;else{const w=d[0].toUpperCase()+d.substring(1),k=b?`${w} must end with a ${N}`:`${w} in block collection must be sufficiently indented and end with a ${N}`;r(S,b?"MISSING_CHAR":"BAD_INDENT",k),E&&E.source.length!==1&&D.unshift(E)}if(D.length>0){const w=La(D,T,a.options.strict,r);w.comment&&(p.comment?p.comment+=`
`+w.comment:p.comment=w.comment),p.range=[s.offset,T,w.offset]}else p.range=[s.offset,T,T];return p}function uf(u,l,a,s,r,o){const h=a.type==="block-map"?Bv(u,l,a,s,o):a.type==="block-seq"?qv(u,l,a,s,o):jv(u,l,a,s,o),d=h.constructor;return r==="!"||r===d.tagName?(h.tag=d.tagName,h):(r&&(h.tag=r),h)}function Hv(u,l,a,s,r){var N;const o=s.tag,h=o?l.directives.tagName(o.source,E=>r(o,"TAG_RESOLVE_FAILED",E)):null;if(a.type==="block-seq"){const{anchor:E,newlineAfterProp:D}=s,T=E&&o?E.offset>o.offset?E:o:E??o;T&&(!D||D.offset<T.offset)&&r(T,"MISSING_CHAR","Missing newline after block sequence props")}const d=a.type==="block-map"?"map":a.type==="block-seq"?"seq":a.start.source==="{"?"map":"seq";if(!o||!h||h==="!"||h===Mt.tagName&&d==="map"||h===Qn.tagName&&d==="seq")return uf(u,l,a,r,h);let m=l.schema.tags.find(E=>E.tag===h&&E.collection===d);if(!m){const E=l.schema.knownTags[h];if(E&&E.collection===d)l.schema.tags.push(Object.assign({},E,{default:!1})),m=E;else return E!=null&&E.collection?r(o,"BAD_COLLECTION_TYPE",`${E.tag} used for ${d} collection, but expects ${E.collection}`,!0):r(o,"TAG_RESOLVE_FAILED",`Unresolved tag: ${h}`,!0),uf(u,l,a,r,h)}const p=uf(u,l,a,r,h,m),b=((N=m.resolve)==null?void 0:N.call(m,p,E=>r(o,"TAG_RESOLVE_FAILED",E),l.options))??p,S=ze(b)?b:new re(b);return S.range=p.range,S.tag=h,m!=null&&m.format&&(S.format=m.format),S}function dp(u,l,a){const s=l.offset,r=$v(l,u.options.strict,a);if(!r)return{value:"",type:null,comment:"",range:[s,s,s]};const o=r.mode===">"?re.BLOCK_FOLDED:re.BLOCK_LITERAL,h=l.source?Yv(l.source):[];let d=h.length;for(let T=h.length-1;T>=0;--T){const w=h[T][1];if(w===""||w==="\r")d=T;else break}if(d===0){const T=r.chomp==="+"&&h.length>0?`
`.repeat(Math.max(1,h.length-1)):"";let w=s+r.length;return l.source&&(w+=l.source.length),{value:T,type:o,comment:r.comment,range:[s,w,w]}}let m=l.indent+r.indent,p=l.offset+r.length,b=0;for(let T=0;T<d;++T){const[w,k]=h[T];if(k===""||k==="\r")r.indent===0&&w.length>m&&(m=w.length);else{w.length<m&&a(p+w.length,"MISSING_CHAR","Block scalars with more-indented leading empty lines must use an explicit indentation indicator"),r.indent===0&&(m=w.length),b=T,m===0&&!u.atRoot&&a(p,"BAD_INDENT","Block scalar values in collections must be indented");break}p+=w.length+k.length+1}for(let T=h.length-1;T>=d;--T)h[T][0].length>m&&(d=T+1);let S="",N="",E=!1;for(let T=0;T<b;++T)S+=h[T][0].slice(m)+`
`;for(let T=b;T<d;++T){let[w,k]=h[T];p+=w.length+k.length+1;const L=k[k.length-1]==="\r";if(L&&(k=k.slice(0,-1)),k&&w.length<m){const Q=`Block scalar lines must not be less indented than their ${r.indent?"explicit indentation indicator":"first line"}`;a(p-k.length-(L?2:1),"BAD_INDENT",Q),w=""}o===re.BLOCK_LITERAL?(S+=N+w.slice(m)+k,N=`
`):w.length>m||k[0]==="	"?(N===" "?N=`
`:!E&&N===`
`&&(N=`

`),S+=N+w.slice(m)+k,N=`
`,E=!0):k===""?N===`
`?S+=`
`:N=`
`:(S+=N+k,N=" ",E=!1)}switch(r.chomp){case"-":break;case"+":for(let T=d;T<h.length;++T)S+=`
`+h[T][0].slice(m);S[S.length-1]!==`
`&&(S+=`
`);break;default:S+=`
`}const D=s+r.length+l.source.length;return{value:S,type:o,comment:r.comment,range:[s,D,D]}}function $v({offset:u,props:l},a,s){if(l[0].type!=="block-scalar-header")return s(l[0],"IMPOSSIBLE","Block scalar header not found"),null;const{source:r}=l[0],o=r[0];let h=0,d="",m=-1;for(let N=1;N<r.length;++N){const E=r[N];if(!d&&(E==="-"||E==="+"))d=E;else{const D=Number(E);!h&&D?h=D:m===-1&&(m=u+N)}}m!==-1&&s(m,"UNEXPECTED_TOKEN",`Block scalar header includes extra characters: ${r}`);let p=!1,b="",S=r.length;for(let N=1;N<l.length;++N){const E=l[N];switch(E.type){case"space":p=!0;case"newline":S+=E.source.length;break;case"comment":a&&!p&&s(E,"MISSING_CHAR","Comments must be separated from other tokens by white space characters"),S+=E.source.length,b=E.source.substring(1);break;case"error":s(E,"UNEXPECTED_TOKEN",E.message),S+=E.source.length;break;default:{const D=`Unexpected token in block scalar header: ${E.type}`;s(E,"UNEXPECTED_TOKEN",D);const T=E.source;T&&typeof T=="string"&&(S+=T.length)}}}return{mode:o,indent:h,chomp:d,comment:b,length:S}}function Yv(u){const l=u.split(/\n( *)/),a=l[0],s=a.match(/^( *)/),o=[s!=null&&s[1]?[s[1],a.slice(s[1].length)]:["",a]];for(let h=1;h<l.length;h+=2)o.push([l[h],l[h+1]]);return o}function gp(u,l,a){const{offset:s,type:r,source:o,end:h}=u;let d,m;const p=(N,E,D)=>a(s+N,E,D);switch(r){case"scalar":d=re.PLAIN,m=Gv(o,p);break;case"single-quoted-scalar":d=re.QUOTE_SINGLE,m=Kv(o,p);break;case"double-quoted-scalar":d=re.QUOTE_DOUBLE,m=Vv(o,p);break;default:return a(u,"UNEXPECTED_TOKEN",`Expected a flow scalar value, but found: ${r}`),{value:"",type:null,comment:"",range:[s,s+o.length,s+o.length]}}const b=s+o.length,S=La(h,b,l,a);return{value:m,type:d,comment:S.comment,range:[s,b,S.offset]}}function Gv(u,l){let a="";switch(u[0]){case"	":a="a tab character";break;case",":a="flow indicator character ,";break;case"%":a="directive indicator character %";break;case"|":case">":{a=`block scalar indicator ${u[0]}`;break}case"@":case"`":{a=`reserved character ${u[0]}`;break}}return a&&l(0,"BAD_SCALAR_START",`Plain value cannot start with ${a}`),mp(u)}function Kv(u,l){return(u[u.length-1]!=="'"||u.length===1)&&l(u.length,"MISSING_CHAR","Missing closing 'quote"),mp(u.slice(1,-1)).replace(/''/g,"'")}function mp(u){let l,a;try{l=new RegExp(`(.*?)(?<![ 	])[ 	]*\r?
`,"sy"),a=new RegExp(`[ 	]*(.*?)(?:(?<![ 	])[ 	]*)?\r?
`,"sy")}catch{l=/(.*?)[ \t]*\r?\n/sy,a=/[ \t]*(.*?)[ \t]*\r?\n/sy}let s=l.exec(u);if(!s)return u;let r=s[1],o=" ",h=l.lastIndex;for(a.lastIndex=h;s=a.exec(u);)s[1]===""?o===`
`?r+=o:o=`
`:(r+=o+s[1],o=" "),h=a.lastIndex;const d=/[ \t]*(.*)/sy;return d.lastIndex=h,s=d.exec(u),r+o+((s==null?void 0:s[1])??"")}function Vv(u,l){let a="";for(let s=1;s<u.length-1;++s){const r=u[s];if(!(r==="\r"&&u[s+1]===`
`))if(r===`
`){const{fold:o,offset:h}=Qv(u,s);a+=o,s=h}else if(r==="\\"){let o=u[++s];const h=Xv[o];if(h)a+=h;else if(o===`
`)for(o=u[s+1];o===" "||o==="	";)o=u[++s+1];else if(o==="\r"&&u[s+1]===`
`)for(o=u[++s+1];o===" "||o==="	";)o=u[++s+1];else if(o==="x"||o==="u"||o==="U"){const d={x:2,u:4,U:8}[o];a+=Zv(u,s+1,d,l),s+=d}else{const d=u.substr(s-1,2);l(s-1,"BAD_DQ_ESCAPE",`Invalid escape sequence ${d}`),a+=d}}else if(r===" "||r==="	"){const o=s;let h=u[s+1];for(;h===" "||h==="	";)h=u[++s+1];h!==`
`&&!(h==="\r"&&u[s+2]===`
`)&&(a+=s>o?u.slice(o,s+1):r)}else a+=r}return(u[u.length-1]!=='"'||u.length===1)&&l(u.length,"MISSING_CHAR",'Missing closing "quote'),a}function Qv(u,l){let a="",s=u[l+1];for(;(s===" "||s==="	"||s===`
`||s==="\r")&&!(s==="\r"&&u[l+2]!==`
`);)s===`
`&&(a+=`
`),l+=1,s=u[l+1];return a||(a=" "),{fold:a,offset:l}}const Xv={0:"\0",a:"\x07",b:"\b",e:"\x1B",f:"\f",n:`
`,r:"\r",t:"	",v:"\v",N:"",_:" ",L:"\u2028",P:"\u2029"," ":" ",'"':'"',"/":"/","\\":"\\","	":"	"};function Zv(u,l,a,s){const r=u.substr(l,a),h=r.length===a&&/^[0-9a-fA-F]+$/.test(r)?parseInt(r,16):NaN;if(isNaN(h)){const d=u.substr(l-2,a+2);return s(l-2,"BAD_DQ_ESCAPE",`Invalid escape sequence ${d}`),d}return String.fromCodePoint(h)}function pp(u,l,a,s){const{value:r,type:o,comment:h,range:d}=l.type==="block-scalar"?dp(u,l,s):gp(l,u.options.strict,s),m=a?u.directives.tagName(a.source,S=>s(a,"TAG_RESOLVE_FAILED",S)):null;let p;u.options.stringKeys&&u.atKey?p=u.schema[Ft]:m?p=Jv(u.schema,r,m,a,s):l.type==="scalar"?p=Wv(u,r,l,s):p=u.schema[Ft];let b;try{const S=p.resolve(r,N=>s(a??l,"TAG_RESOLVE_FAILED",N),u.options);b=_e(S)?S:new re(S)}catch(S){const N=S instanceof Error?S.message:String(S);s(a??l,"TAG_RESOLVE_FAILED",N),b=new re(r)}return b.range=d,b.source=r,o&&(b.type=o),m&&(b.tag=m),p.format&&(b.format=p.format),h&&(b.comment=h),b}function Jv(u,l,a,s,r){var d;if(a==="!")return u[Ft];const o=[];for(const m of u.tags)if(!m.collection&&m.tag===a)if(m.default&&m.test)o.push(m);else return m;for(const m of o)if((d=m.test)!=null&&d.test(l))return m;const h=u.knownTags[a];return h&&!h.collection?(u.tags.push(Object.assign({},h,{default:!1,test:void 0})),h):(r(s,"TAG_RESOLVE_FAILED",`Unresolved tag: ${a}`,a!=="tag:yaml.org,2002:str"),u[Ft])}function Wv({atKey:u,directives:l,schema:a},s,r,o){const h=a.tags.find(d=>{var m;return(d.default===!0||u&&d.default==="key")&&((m=d.test)==null?void 0:m.test(s))})||a[Ft];if(a.compat){const d=a.compat.find(m=>{var p;return m.default&&((p=m.test)==null?void 0:p.test(s))})??a[Ft];if(h.tag!==d.tag){const m=l.tagString(h.tag),p=l.tagString(d.tag),b=`Value may be parsed as either ${m} or ${p}`;o(r,"TAG_RESOLVE_FAILED",b,!0)}}return h}function Iv(u,l,a){if(l){a===null&&(a=l.length);for(let s=a-1;s>=0;--s){let r=l[s];switch(r.type){case"space":case"comment":case"newline":u-=r.source.length;continue}for(r=l[++s];(r==null?void 0:r.type)==="space";)u+=r.source.length,r=l[++s];break}}return u}const Fv={composeNode:yp,composeEmptyNode:zf};function yp(u,l,a,s){const r=u.atKey,{spaceBefore:o,comment:h,anchor:d,tag:m}=a;let p,b=!0;switch(l.type){case"alias":p=Pv(u,l,s),(d||m)&&s(l,"ALIAS_PROPS","An alias node must not specify any properties");break;case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":case"block-scalar":p=pp(u,l,m,s),d&&(p.anchor=d.source.substring(1));break;case"block-map":case"block-seq":case"flow-collection":p=Hv(Fv,u,l,a,s),d&&(p.anchor=d.source.substring(1));break;default:{const S=l.type==="error"?l.message:`Unsupported token (type: ${l.type})`;s(l,"UNEXPECTED_TOKEN",S),p=zf(u,l.offset,void 0,null,a,s),b=!1}}return d&&p.anchor===""&&s(d,"BAD_ALIAS","Anchor cannot be an empty string"),r&&u.options.stringKeys&&(!_e(p)||typeof p.value!="string"||p.tag&&p.tag!=="tag:yaml.org,2002:str")&&s(m??l,"NON_STRING_KEY","With stringKeys, all keys must be strings"),o&&(p.spaceBefore=!0),h&&(l.type==="scalar"&&l.source===""?p.comment=h:p.commentBefore=h),u.options.keepSourceTokens&&b&&(p.srcToken=l),p}function zf(u,l,a,s,{spaceBefore:r,comment:o,anchor:h,tag:d,end:m},p){const b={type:"scalar",offset:Iv(l,a,s),indent:-1,source:""},S=pp(u,b,d,p);return h&&(S.anchor=h.source.substring(1),S.anchor===""&&p(h,"BAD_ALIAS","Anchor cannot be an empty string")),r&&(S.spaceBefore=!0),o&&(S.comment=o,S.range[2]=m),S}function Pv({options:u},{offset:l,source:a,end:s},r){const o=new Su(a.substring(1));o.source===""&&r(l,"BAD_ALIAS","Alias cannot be an empty string"),o.source.endsWith(":")&&r(l+a.length-1,"BAD_ALIAS","Alias ending in : is ambiguous",!0);const h=l+a.length,d=La(s,h,u.strict,r);return o.range=[l,h,d.offset],d.comment&&(o.comment=d.comment),o}function eb(u,l,{offset:a,start:s,value:r,end:o},h){const d=Object.assign({_directives:l},u),m=new Ai(void 0,d),p={atKey:!1,atRoot:!0,directives:m.directives,options:m.options,schema:m.schema},b=pi(s,{indicator:"doc-start",next:r??(o==null?void 0:o[0]),offset:a,onError:h,parentIndent:0,startOnNewline:!0});b.found&&(m.directives.docStart=!0,r&&(r.type==="block-map"||r.type==="block-seq")&&!b.hasNewline&&h(b.end,"MISSING_CHAR","Block collection cannot start on same line with directives-end marker")),m.contents=r?yp(p,r,b,h):zf(p,b.end,s,null,b,h);const S=m.contents.range[2],N=La(o,S,!1,h);return N.comment&&(m.comment=N.comment),m.range=[a,S,N.offset],m}function Oa(u){if(typeof u=="number")return[u,u+1];if(Array.isArray(u))return u.length===2?u:[u[0],u[1]];const{offset:l,source:a}=u;return[l,l+(typeof a=="string"?a.length:1)]}function im(u){var r;let l="",a=!1,s=!1;for(let o=0;o<u.length;++o){const h=u[o];switch(h[0]){case"#":l+=(l===""?"":s?`

`:`
`)+(h.substring(1)||" "),a=!0,s=!1;break;case"%":((r=u[o+1])==null?void 0:r[0])!=="#"&&(o+=1),a=!1;break;default:a||(s=!0),a=!1}}return{comment:l,afterEmptyLine:s}}class Uf{constructor(l={}){this.doc=null,this.atDirectives=!1,this.prelude=[],this.errors=[],this.warnings=[],this.onError=(a,s,r,o)=>{const h=Oa(a);o?this.warnings.push(new op(h,s,r)):this.errors.push(new ml(h,s,r))},this.directives=new ct({version:l.version||"1.2"}),this.options=l}decorate(l,a){const{comment:s,afterEmptyLine:r}=im(this.prelude);if(s){const o=l.contents;if(a)l.comment=l.comment?`${l.comment}
${s}`:s;else if(r||l.directives.docStart||!o)l.commentBefore=s;else if(Le(o)&&!o.flow&&o.items.length>0){let h=o.items[0];Re(h)&&(h=h.key);const d=h.commentBefore;h.commentBefore=d?`${s}
${d}`:s}else{const h=o.commentBefore;o.commentBefore=h?`${s}
${h}`:s}}a?(Array.prototype.push.apply(l.errors,this.errors),Array.prototype.push.apply(l.warnings,this.warnings)):(l.errors=this.errors,l.warnings=this.warnings),this.prelude=[],this.errors=[],this.warnings=[]}streamInfo(){return{comment:im(this.prelude).comment,directives:this.directives,errors:this.errors,warnings:this.warnings}}*compose(l,a=!1,s=-1){for(const r of l)yield*this.next(r);yield*this.end(a,s)}*next(l){switch(l.type){case"directive":this.directives.add(l.source,(a,s,r)=>{const o=Oa(l);o[0]+=a,this.onError(o,"BAD_DIRECTIVE",s,r)}),this.prelude.push(l.source),this.atDirectives=!0;break;case"document":{const a=eb(this.options,this.directives,l,this.onError);this.atDirectives&&!a.directives.docStart&&this.onError(l,"MISSING_CHAR","Missing directives-end/doc-start indicator line"),this.decorate(a,!1),this.doc&&(yield this.doc),this.doc=a,this.atDirectives=!1;break}case"byte-order-mark":case"space":break;case"comment":case"newline":this.prelude.push(l.source);break;case"error":{const a=l.source?`${l.message}: ${JSON.stringify(l.source)}`:l.message,s=new ml(Oa(l),"UNEXPECTED_TOKEN",a);this.atDirectives||!this.doc?this.errors.push(s):this.doc.errors.push(s);break}case"doc-end":{if(!this.doc){const s="Unexpected doc-end without preceding document";this.errors.push(new ml(Oa(l),"UNEXPECTED_TOKEN",s));break}this.doc.directives.docEnd=!0;const a=La(l.end,l.offset+l.source.length,this.doc.options.strict,this.onError);if(this.decorate(this.doc,!0),a.comment){const s=this.doc.comment;this.doc.comment=s?`${s}
${a.comment}`:a.comment}this.doc.range[2]=a.offset;break}default:this.errors.push(new ml(Oa(l),"UNEXPECTED_TOKEN",`Unsupported token ${l.type}`))}}*end(l=!1,a=-1){if(this.doc)this.decorate(this.doc,!0),yield this.doc,this.doc=null;else if(l){const s=Object.assign({_directives:this.directives},this.options),r=new Ai(void 0,s);this.atDirectives&&this.onError(a,"MISSING_CHAR","Missing directives-end indicator line"),r.range=[0,a,a],this.decorate(r,!1),yield r}}}function tb(u,l=!0,a){if(u){const s=(r,o,h)=>{const d=typeof r=="number"?r:Array.isArray(r)?r[0]:r.offset;if(a)a(d,o,h);else throw new ml([d,d+1],o,h)};switch(u.type){case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":return gp(u,l,s);case"block-scalar":return dp({options:{strict:l}},u,s)}}return null}function nb(u,l){const{implicitKey:a=!1,indent:s,inFlow:r=!1,offset:o=-1,type:h="PLAIN"}=l,d=Ra({type:h,value:u},{implicitKey:a,indent:s>0?" ".repeat(s):"",inFlow:r,options:{blockQuote:!0,lineWidth:-1}}),m=l.end??[{type:"newline",offset:-1,indent:s,source:`
`}];switch(d[0]){case"|":case">":{const p=d.indexOf(`
`),b=d.substring(0,p),S=d.substring(p+1)+`
`,N=[{type:"block-scalar-header",offset:o,indent:s,source:b}];return vp(N,m)||N.push({type:"newline",offset:-1,indent:s,source:`
`}),{type:"block-scalar",offset:o,indent:s,props:N,source:S}}case'"':return{type:"double-quoted-scalar",offset:o,indent:s,source:d,end:m};case"'":return{type:"single-quoted-scalar",offset:o,indent:s,source:d,end:m};default:return{type:"scalar",offset:o,indent:s,source:d,end:m}}}function lb(u,l,a={}){let{afterKey:s=!1,implicitKey:r=!1,inFlow:o=!1,type:h}=a,d="indent"in u?u.indent:null;if(s&&typeof d=="number"&&(d+=2),!h)switch(u.type){case"single-quoted-scalar":h="QUOTE_SINGLE";break;case"double-quoted-scalar":h="QUOTE_DOUBLE";break;case"block-scalar":{const p=u.props[0];if(p.type!=="block-scalar-header")throw new Error("Invalid block scalar header");h=p.source[0]===">"?"BLOCK_FOLDED":"BLOCK_LITERAL";break}default:h="PLAIN"}const m=Ra({type:h,value:l},{implicitKey:r||d===null,indent:d!==null&&d>0?" ".repeat(d):"",inFlow:o,options:{blockQuote:!0,lineWidth:-1}});switch(m[0]){case"|":case">":ib(u,m);break;case'"':cf(u,m,"double-quoted-scalar");break;case"'":cf(u,m,"single-quoted-scalar");break;default:cf(u,m,"scalar")}}function ib(u,l){const a=l.indexOf(`
`),s=l.substring(0,a),r=l.substring(a+1)+`
`;if(u.type==="block-scalar"){const o=u.props[0];if(o.type!=="block-scalar-header")throw new Error("Invalid block scalar header");o.source=s,u.source=r}else{const{offset:o}=u,h="indent"in u?u.indent:-1,d=[{type:"block-scalar-header",offset:o,indent:h,source:s}];vp(d,"end"in u?u.end:void 0)||d.push({type:"newline",offset:-1,indent:h,source:`
`});for(const m of Object.keys(u))m!=="type"&&m!=="offset"&&delete u[m];Object.assign(u,{type:"block-scalar",indent:h,props:d,source:r})}}function vp(u,l){if(l)for(const a of l)switch(a.type){case"space":case"comment":u.push(a);break;case"newline":return u.push(a),!0}return!1}function cf(u,l,a){switch(u.type){case"scalar":case"double-quoted-scalar":case"single-quoted-scalar":u.type=a,u.source=l;break;case"block-scalar":{const s=u.props.slice(1);let r=l.length;u.props[0].type==="block-scalar-header"&&(r-=u.props[0].source.length);for(const o of s)o.offset+=r;delete u.props,Object.assign(u,{type:a,source:l,end:s});break}case"block-map":case"block-seq":{const r={type:"newline",offset:u.offset+l.length,indent:u.indent,source:`
`};delete u.items,Object.assign(u,{type:a,source:l,end:[r]});break}default:{const s="indent"in u?u.indent:-1,r="end"in u&&Array.isArray(u.end)?u.end.filter(o=>o.type==="space"||o.type==="comment"||o.type==="newline"):[];for(const o of Object.keys(u))o!=="type"&&o!=="offset"&&delete u[o];Object.assign(u,{type:a,indent:s,source:l,end:r})}}}const ab=u=>"type"in u?mu(u):ru(u);function mu(u){switch(u.type){case"block-scalar":{let l="";for(const a of u.props)l+=mu(a);return l+u.source}case"block-map":case"block-seq":{let l="";for(const a of u.items)l+=ru(a);return l}case"flow-collection":{let l=u.start.source;for(const a of u.items)l+=ru(a);for(const a of u.end)l+=a.source;return l}case"document":{let l=ru(u);if(u.end)for(const a of u.end)l+=a.source;return l}default:{let l=u.source;if("end"in u&&u.end)for(const a of u.end)l+=a.source;return l}}}function ru({start:u,key:l,sep:a,value:s}){let r="";for(const o of u)r+=o.source;if(l&&(r+=mu(l)),a)for(const o of a)r+=o.source;return s&&(r+=mu(s)),r}const Sf=Symbol("break visit"),sb=Symbol("skip children"),bp=Symbol("remove item");function yl(u,l){"type"in u&&u.type==="document"&&(u={start:u.start,value:u.value}),Sp(Object.freeze([]),u,l)}yl.BREAK=Sf;yl.SKIP=sb;yl.REMOVE=bp;yl.itemAtPath=(u,l)=>{let a=u;for(const[s,r]of l){const o=a==null?void 0:a[s];if(o&&"items"in o)a=o.items[r];else return}return a};yl.parentCollection=(u,l)=>{const a=yl.itemAtPath(u,l.slice(0,-1)),s=l[l.length-1][0],r=a==null?void 0:a[s];if(r&&"items"in r)return r;throw new Error("Parent collection not found")};function Sp(u,l,a){let s=a(l,u);if(typeof s=="symbol")return s;for(const r of["key","value"]){const o=l[r];if(o&&"items"in o){for(let h=0;h<o.items.length;++h){const d=Sp(Object.freeze(u.concat([[r,h]])),o.items[h],a);if(typeof d=="number")h=d-1;else{if(d===Sf)return Sf;d===bp&&(o.items.splice(h,1),h-=1)}}typeof s=="function"&&r==="key"&&(s=s(l,u))}}return typeof s=="function"?s(l,u):s}const Du="\uFEFF",Cu="",Ru="",Da="",ub=u=>!!u&&"items"in u,cb=u=>!!u&&(u.type==="scalar"||u.type==="single-quoted-scalar"||u.type==="double-quoted-scalar"||u.type==="block-scalar");function rb(u){switch(u){case Du:return"<BOM>";case Cu:return"<DOC>";case Ru:return"<FLOW_END>";case Da:return"<SCALAR>";default:return JSON.stringify(u)}}function Tp(u){switch(u){case Du:return"byte-order-mark";case Cu:return"doc-mode";case Ru:return"flow-error-end";case Da:return"scalar";case"---":return"doc-start";case"...":return"doc-end";case"":case`
`:case`\r
`:return"newline";case"-":return"seq-item-ind";case"?":return"explicit-key-ind";case":":return"map-value-ind";case"{":return"flow-map-start";case"}":return"flow-map-end";case"[":return"flow-seq-start";case"]":return"flow-seq-end";case",":return"comma"}switch(u[0]){case" ":case"	":return"space";case"#":return"comment";case"%":return"directive-line";case"*":return"alias";case"&":return"anchor";case"!":return"tag";case"'":return"single-quoted-scalar";case'"':return"double-quoted-scalar";case"|":case">":return"block-scalar-header"}return null}const fb=Object.freeze(Object.defineProperty({__proto__:null,BOM:Du,DOCUMENT:Cu,FLOW_END:Ru,SCALAR:Da,createScalarToken:nb,isCollection:ub,isScalar:cb,prettyToken:rb,resolveAsScalar:tb,setScalarValue:lb,stringify:ab,tokenType:Tp,visit:yl},Symbol.toStringTag,{value:"Module"}));function Gt(u){switch(u){case void 0:case" ":case`
`:case"\r":case"	":return!0;default:return!1}}const am=new Set("0123456789ABCDEFabcdef"),ob=new Set("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-#;/?:@&=+$_.!~*'()"),Ps=new Set(",[]{}"),hb=new Set(` ,[]{}
\r	`),rf=u=>!u||hb.has(u);class wp{constructor(){this.atEnd=!1,this.blockScalarIndent=-1,this.blockScalarKeep=!1,this.buffer="",this.flowKey=!1,this.flowLevel=0,this.indentNext=0,this.indentValue=0,this.lineEndPos=null,this.next=null,this.pos=0}*lex(l,a=!1){if(l){if(typeof l!="string")throw TypeError("source is not a string");this.buffer=this.buffer?this.buffer+l:l,this.lineEndPos=null}this.atEnd=!a;let s=this.next??"stream";for(;s&&(a||this.hasChars(1));)s=yield*this.parseNext(s)}atLineEnd(){let l=this.pos,a=this.buffer[l];for(;a===" "||a==="	";)a=this.buffer[++l];return!a||a==="#"||a===`
`?!0:a==="\r"?this.buffer[l+1]===`
`:!1}charAt(l){return this.buffer[this.pos+l]}continueScalar(l){let a=this.buffer[l];if(this.indentNext>0){let s=0;for(;a===" ";)a=this.buffer[++s+l];if(a==="\r"){const r=this.buffer[s+l+1];if(r===`
`||!r&&!this.atEnd)return l+s+1}return a===`
`||s>=this.indentNext||!a&&!this.atEnd?l+s:-1}if(a==="-"||a==="."){const s=this.buffer.substr(l,3);if((s==="---"||s==="...")&&Gt(this.buffer[l+3]))return-1}return l}getLine(){let l=this.lineEndPos;return(typeof l!="number"||l!==-1&&l<this.pos)&&(l=this.buffer.indexOf(`
`,this.pos),this.lineEndPos=l),l===-1?this.atEnd?this.buffer.substring(this.pos):null:(this.buffer[l-1]==="\r"&&(l-=1),this.buffer.substring(this.pos,l))}hasChars(l){return this.pos+l<=this.buffer.length}setNext(l){return this.buffer=this.buffer.substring(this.pos),this.pos=0,this.lineEndPos=null,this.next=l,null}peek(l){return this.buffer.substr(this.pos,l)}*parseNext(l){switch(l){case"stream":return yield*this.parseStream();case"line-start":return yield*this.parseLineStart();case"block-start":return yield*this.parseBlockStart();case"doc":return yield*this.parseDocument();case"flow":return yield*this.parseFlowCollection();case"quoted-scalar":return yield*this.parseQuotedScalar();case"block-scalar":return yield*this.parseBlockScalar();case"plain-scalar":return yield*this.parsePlainScalar()}}*parseStream(){let l=this.getLine();if(l===null)return this.setNext("stream");if(l[0]===Du&&(yield*this.pushCount(1),l=l.substring(1)),l[0]==="%"){let a=l.length,s=l.indexOf("#");for(;s!==-1;){const o=l[s-1];if(o===" "||o==="	"){a=s-1;break}else s=l.indexOf("#",s+1)}for(;;){const o=l[a-1];if(o===" "||o==="	")a-=1;else break}const r=(yield*this.pushCount(a))+(yield*this.pushSpaces(!0));return yield*this.pushCount(l.length-r),this.pushNewline(),"stream"}if(this.atLineEnd()){const a=yield*this.pushSpaces(!0);return yield*this.pushCount(l.length-a),yield*this.pushNewline(),"stream"}return yield Cu,yield*this.parseLineStart()}*parseLineStart(){const l=this.charAt(0);if(!l&&!this.atEnd)return this.setNext("line-start");if(l==="-"||l==="."){if(!this.atEnd&&!this.hasChars(4))return this.setNext("line-start");const a=this.peek(3);if((a==="---"||a==="...")&&Gt(this.charAt(3)))return yield*this.pushCount(3),this.indentValue=0,this.indentNext=0,a==="---"?"doc":"stream"}return this.indentValue=yield*this.pushSpaces(!1),this.indentNext>this.indentValue&&!Gt(this.charAt(1))&&(this.indentNext=this.indentValue),yield*this.parseBlockStart()}*parseBlockStart(){const[l,a]=this.peek(2);if(!a&&!this.atEnd)return this.setNext("block-start");if((l==="-"||l==="?"||l===":")&&Gt(a)){const s=(yield*this.pushCount(1))+(yield*this.pushSpaces(!0));return this.indentNext=this.indentValue+1,this.indentValue+=s,yield*this.parseBlockStart()}return"doc"}*parseDocument(){yield*this.pushSpaces(!0);const l=this.getLine();if(l===null)return this.setNext("doc");let a=yield*this.pushIndicators();switch(l[a]){case"#":yield*this.pushCount(l.length-a);case void 0:return yield*this.pushNewline(),yield*this.parseLineStart();case"{":case"[":return yield*this.pushCount(1),this.flowKey=!1,this.flowLevel=1,"flow";case"}":case"]":return yield*this.pushCount(1),"doc";case"*":return yield*this.pushUntil(rf),"doc";case'"':case"'":return yield*this.parseQuotedScalar();case"|":case">":return a+=yield*this.parseBlockScalarHeader(),a+=yield*this.pushSpaces(!0),yield*this.pushCount(l.length-a),yield*this.pushNewline(),yield*this.parseBlockScalar();default:return yield*this.parsePlainScalar()}}*parseFlowCollection(){let l,a,s=-1;do l=yield*this.pushNewline(),l>0?(a=yield*this.pushSpaces(!1),this.indentValue=s=a):a=0,a+=yield*this.pushSpaces(!0);while(l+a>0);const r=this.getLine();if(r===null)return this.setNext("flow");if((s!==-1&&s<this.indentNext&&r[0]!=="#"||s===0&&(r.startsWith("---")||r.startsWith("..."))&&Gt(r[3]))&&!(s===this.indentNext-1&&this.flowLevel===1&&(r[0]==="]"||r[0]==="}")))return this.flowLevel=0,yield Ru,yield*this.parseLineStart();let o=0;for(;r[o]===",";)o+=yield*this.pushCount(1),o+=yield*this.pushSpaces(!0),this.flowKey=!1;switch(o+=yield*this.pushIndicators(),r[o]){case void 0:return"flow";case"#":return yield*this.pushCount(r.length-o),"flow";case"{":case"[":return yield*this.pushCount(1),this.flowKey=!1,this.flowLevel+=1,"flow";case"}":case"]":return yield*this.pushCount(1),this.flowKey=!0,this.flowLevel-=1,this.flowLevel?"flow":"doc";case"*":return yield*this.pushUntil(rf),"flow";case'"':case"'":return this.flowKey=!0,yield*this.parseQuotedScalar();case":":{const h=this.charAt(1);if(this.flowKey||Gt(h)||h===",")return this.flowKey=!1,yield*this.pushCount(1),yield*this.pushSpaces(!0),"flow"}default:return this.flowKey=!1,yield*this.parsePlainScalar()}}*parseQuotedScalar(){const l=this.charAt(0);let a=this.buffer.indexOf(l,this.pos+1);if(l==="'")for(;a!==-1&&this.buffer[a+1]==="'";)a=this.buffer.indexOf("'",a+2);else for(;a!==-1;){let o=0;for(;this.buffer[a-1-o]==="\\";)o+=1;if(o%2===0)break;a=this.buffer.indexOf('"',a+1)}const s=this.buffer.substring(0,a);let r=s.indexOf(`
`,this.pos);if(r!==-1){for(;r!==-1;){const o=this.continueScalar(r+1);if(o===-1)break;r=s.indexOf(`
`,o)}r!==-1&&(a=r-(s[r-1]==="\r"?2:1))}if(a===-1){if(!this.atEnd)return this.setNext("quoted-scalar");a=this.buffer.length}return yield*this.pushToIndex(a+1,!1),this.flowLevel?"flow":"doc"}*parseBlockScalarHeader(){this.blockScalarIndent=-1,this.blockScalarKeep=!1;let l=this.pos;for(;;){const a=this.buffer[++l];if(a==="+")this.blockScalarKeep=!0;else if(a>"0"&&a<="9")this.blockScalarIndent=Number(a)-1;else if(a!=="-")break}return yield*this.pushUntil(a=>Gt(a)||a==="#")}*parseBlockScalar(){let l=this.pos-1,a=0,s;e:for(let o=this.pos;s=this.buffer[o];++o)switch(s){case" ":a+=1;break;case`
`:l=o,a=0;break;case"\r":{const h=this.buffer[o+1];if(!h&&!this.atEnd)return this.setNext("block-scalar");if(h===`
`)break}default:break e}if(!s&&!this.atEnd)return this.setNext("block-scalar");if(a>=this.indentNext){this.blockScalarIndent===-1?this.indentNext=a:this.indentNext=this.blockScalarIndent+(this.indentNext===0?1:this.indentNext);do{const o=this.continueScalar(l+1);if(o===-1)break;l=this.buffer.indexOf(`
`,o)}while(l!==-1);if(l===-1){if(!this.atEnd)return this.setNext("block-scalar");l=this.buffer.length}}let r=l+1;for(s=this.buffer[r];s===" ";)s=this.buffer[++r];if(s==="	"){for(;s==="	"||s===" "||s==="\r"||s===`
`;)s=this.buffer[++r];l=r-1}else if(!this.blockScalarKeep)do{let o=l-1,h=this.buffer[o];h==="\r"&&(h=this.buffer[--o]);const d=o;for(;h===" ";)h=this.buffer[--o];if(h===`
`&&o>=this.pos&&o+1+a>d)l=o;else break}while(!0);return yield Da,yield*this.pushToIndex(l+1,!0),yield*this.parseLineStart()}*parsePlainScalar(){const l=this.flowLevel>0;let a=this.pos-1,s=this.pos-1,r;for(;r=this.buffer[++s];)if(r===":"){const o=this.buffer[s+1];if(Gt(o)||l&&Ps.has(o))break;a=s}else if(Gt(r)){let o=this.buffer[s+1];if(r==="\r"&&(o===`
`?(s+=1,r=`
`,o=this.buffer[s+1]):a=s),o==="#"||l&&Ps.has(o))break;if(r===`
`){const h=this.continueScalar(s+1);if(h===-1)break;s=Math.max(s,h-2)}}else{if(l&&Ps.has(r))break;a=s}return!r&&!this.atEnd?this.setNext("plain-scalar"):(yield Da,yield*this.pushToIndex(a+1,!0),l?"flow":"doc")}*pushCount(l){return l>0?(yield this.buffer.substr(this.pos,l),this.pos+=l,l):0}*pushToIndex(l,a){const s=this.buffer.slice(this.pos,l);return s?(yield s,this.pos+=s.length,s.length):(a&&(yield""),0)}*pushIndicators(){switch(this.charAt(0)){case"!":return(yield*this.pushTag())+(yield*this.pushSpaces(!0))+(yield*this.pushIndicators());case"&":return(yield*this.pushUntil(rf))+(yield*this.pushSpaces(!0))+(yield*this.pushIndicators());case"-":case"?":case":":{const l=this.flowLevel>0,a=this.charAt(1);if(Gt(a)||l&&Ps.has(a))return l?this.flowKey&&(this.flowKey=!1):this.indentNext=this.indentValue+1,(yield*this.pushCount(1))+(yield*this.pushSpaces(!0))+(yield*this.pushIndicators())}}return 0}*pushTag(){if(this.charAt(1)==="<"){let l=this.pos+2,a=this.buffer[l];for(;!Gt(a)&&a!==">";)a=this.buffer[++l];return yield*this.pushToIndex(a===">"?l+1:l,!1)}else{let l=this.pos+1,a=this.buffer[l];for(;a;)if(ob.has(a))a=this.buffer[++l];else if(a==="%"&&am.has(this.buffer[l+1])&&am.has(this.buffer[l+2]))a=this.buffer[l+=3];else break;return yield*this.pushToIndex(l,!1)}}*pushNewline(){const l=this.buffer[this.pos];return l===`
`?yield*this.pushCount(1):l==="\r"&&this.charAt(1)===`
`?yield*this.pushCount(2):0}*pushSpaces(l){let a=this.pos-1,s;do s=this.buffer[++a];while(s===" "||l&&s==="	");const r=a-this.pos;return r>0&&(yield this.buffer.substr(this.pos,r),this.pos=a),r}*pushUntil(l){let a=this.pos,s=this.buffer[a];for(;!l(s);)s=this.buffer[++a];return yield*this.pushToIndex(a,!1)}}class Ep{constructor(){this.lineStarts=[],this.addNewLine=l=>this.lineStarts.push(l),this.linePos=l=>{let a=0,s=this.lineStarts.length;for(;a<s;){const o=a+s>>1;this.lineStarts[o]<l?a=o+1:s=o}if(this.lineStarts[a]===l)return{line:a+1,col:1};if(a===0)return{line:0,col:l};const r=this.lineStarts[a-1];return{line:a,col:l-r+1}}}}function hl(u,l){for(let a=0;a<u.length;++a)if(u[a].type===l)return!0;return!1}function sm(u){for(let l=0;l<u.length;++l)switch(u[l].type){case"space":case"comment":case"newline":break;default:return l}return-1}function Ap(u){switch(u==null?void 0:u.type){case"alias":case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":case"flow-collection":return!0;default:return!1}}function eu(u){switch(u.type){case"document":return u.start;case"block-map":{const l=u.items[u.items.length-1];return l.sep??l.start}case"block-seq":return u.items[u.items.length-1].start;default:return[]}}function si(u){var a;if(u.length===0)return[];let l=u.length;e:for(;--l>=0;)switch(u[l].type){case"doc-start":case"explicit-key-ind":case"map-value-ind":case"seq-item-ind":case"newline":break e}for(;((a=u[++l])==null?void 0:a.type)==="space";);return u.splice(l,u.length)}function um(u){if(u.start.type==="flow-seq-start")for(const l of u.items)l.sep&&!l.value&&!hl(l.start,"explicit-key-ind")&&!hl(l.sep,"map-value-ind")&&(l.key&&(l.value=l.key),delete l.key,Ap(l.value)?l.value.end?Array.prototype.push.apply(l.value.end,l.sep):l.value.end=l.sep:Array.prototype.push.apply(l.start,l.sep),delete l.sep)}class Bf{constructor(l){this.atNewLine=!0,this.atScalar=!1,this.indent=0,this.offset=0,this.onKeyLine=!1,this.stack=[],this.source="",this.type="",this.lexer=new wp,this.onNewLine=l}*parse(l,a=!1){this.onNewLine&&this.offset===0&&this.onNewLine(0);for(const s of this.lexer.lex(l,a))yield*this.next(s);a||(yield*this.end())}*next(l){if(this.source=l,this.atScalar){this.atScalar=!1,yield*this.step(),this.offset+=l.length;return}const a=Tp(l);if(a)if(a==="scalar")this.atNewLine=!1,this.atScalar=!0,this.type="scalar";else{switch(this.type=a,yield*this.step(),a){case"newline":this.atNewLine=!0,this.indent=0,this.onNewLine&&this.onNewLine(this.offset+l.length);break;case"space":this.atNewLine&&l[0]===" "&&(this.indent+=l.length);break;case"explicit-key-ind":case"map-value-ind":case"seq-item-ind":this.atNewLine&&(this.indent+=l.length);break;case"doc-mode":case"flow-error-end":return;default:this.atNewLine=!1}this.offset+=l.length}else{const s=`Not a YAML token: ${l}`;yield*this.pop({type:"error",offset:this.offset,message:s,source:l}),this.offset+=l.length}}*end(){for(;this.stack.length>0;)yield*this.pop()}get sourceToken(){return{type:this.type,offset:this.offset,indent:this.indent,source:this.source}}*step(){const l=this.peek(1);if(this.type==="doc-end"&&(!l||l.type!=="doc-end")){for(;this.stack.length>0;)yield*this.pop();this.stack.push({type:"doc-end",offset:this.offset,source:this.source});return}if(!l)return yield*this.stream();switch(l.type){case"document":return yield*this.document(l);case"alias":case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":return yield*this.scalar(l);case"block-scalar":return yield*this.blockScalar(l);case"block-map":return yield*this.blockMap(l);case"block-seq":return yield*this.blockSequence(l);case"flow-collection":return yield*this.flowCollection(l);case"doc-end":return yield*this.documentEnd(l)}yield*this.pop()}peek(l){return this.stack[this.stack.length-l]}*pop(l){const a=l??this.stack.pop();if(!a)yield{type:"error",offset:this.offset,source:"",message:"Tried to pop an empty stack"};else if(this.stack.length===0)yield a;else{const s=this.peek(1);switch(a.type==="block-scalar"?a.indent="indent"in s?s.indent:0:a.type==="flow-collection"&&s.type==="document"&&(a.indent=0),a.type==="flow-collection"&&um(a),s.type){case"document":s.value=a;break;case"block-scalar":s.props.push(a);break;case"block-map":{const r=s.items[s.items.length-1];if(r.value){s.items.push({start:[],key:a,sep:[]}),this.onKeyLine=!0;return}else if(r.sep)r.value=a;else{Object.assign(r,{key:a,sep:[]}),this.onKeyLine=!r.explicitKey;return}break}case"block-seq":{const r=s.items[s.items.length-1];r.value?s.items.push({start:[],value:a}):r.value=a;break}case"flow-collection":{const r=s.items[s.items.length-1];!r||r.value?s.items.push({start:[],key:a,sep:[]}):r.sep?r.value=a:Object.assign(r,{key:a,sep:[]});return}default:yield*this.pop(),yield*this.pop(a)}if((s.type==="document"||s.type==="block-map"||s.type==="block-seq")&&(a.type==="block-map"||a.type==="block-seq")){const r=a.items[a.items.length-1];r&&!r.sep&&!r.value&&r.start.length>0&&sm(r.start)===-1&&(a.indent===0||r.start.every(o=>o.type!=="comment"||o.indent<a.indent))&&(s.type==="document"?s.end=r.start:s.items.push({start:r.start}),a.items.splice(-1,1))}}}*stream(){switch(this.type){case"directive-line":yield{type:"directive",offset:this.offset,source:this.source};return;case"byte-order-mark":case"space":case"comment":case"newline":yield this.sourceToken;return;case"doc-mode":case"doc-start":{const l={type:"document",offset:this.offset,start:[]};this.type==="doc-start"&&l.start.push(this.sourceToken),this.stack.push(l);return}}yield{type:"error",offset:this.offset,message:`Unexpected ${this.type} token in YAML stream`,source:this.source}}*document(l){if(l.value)return yield*this.lineEnd(l);switch(this.type){case"doc-start":{sm(l.start)!==-1?(yield*this.pop(),yield*this.step()):l.start.push(this.sourceToken);return}case"anchor":case"tag":case"space":case"comment":case"newline":l.start.push(this.sourceToken);return}const a=this.startBlockValue(l);a?this.stack.push(a):yield{type:"error",offset:this.offset,message:`Unexpected ${this.type} token in YAML document`,source:this.source}}*scalar(l){if(this.type==="map-value-ind"){const a=eu(this.peek(2)),s=si(a);let r;l.end?(r=l.end,r.push(this.sourceToken),delete l.end):r=[this.sourceToken];const o={type:"block-map",offset:l.offset,indent:l.indent,items:[{start:s,key:l,sep:r}]};this.onKeyLine=!0,this.stack[this.stack.length-1]=o}else yield*this.lineEnd(l)}*blockScalar(l){switch(this.type){case"space":case"comment":case"newline":l.props.push(this.sourceToken);return;case"scalar":if(l.source=this.source,this.atNewLine=!0,this.indent=0,this.onNewLine){let a=this.source.indexOf(`
`)+1;for(;a!==0;)this.onNewLine(this.offset+a),a=this.source.indexOf(`
`,a)+1}yield*this.pop();break;default:yield*this.pop(),yield*this.step()}}*blockMap(l){var s;const a=l.items[l.items.length-1];switch(this.type){case"newline":if(this.onKeyLine=!1,a.value){const r="end"in a.value?a.value.end:void 0,o=Array.isArray(r)?r[r.length-1]:void 0;(o==null?void 0:o.type)==="comment"?r==null||r.push(this.sourceToken):l.items.push({start:[this.sourceToken]})}else a.sep?a.sep.push(this.sourceToken):a.start.push(this.sourceToken);return;case"space":case"comment":if(a.value)l.items.push({start:[this.sourceToken]});else if(a.sep)a.sep.push(this.sourceToken);else{if(this.atIndentedComment(a.start,l.indent)){const r=l.items[l.items.length-2],o=(s=r==null?void 0:r.value)==null?void 0:s.end;if(Array.isArray(o)){Array.prototype.push.apply(o,a.start),o.push(this.sourceToken),l.items.pop();return}}a.start.push(this.sourceToken)}return}if(this.indent>=l.indent){const r=!this.onKeyLine&&this.indent===l.indent,o=r&&(a.sep||a.explicitKey)&&this.type!=="seq-item-ind";let h=[];if(o&&a.sep&&!a.value){const d=[];for(let m=0;m<a.sep.length;++m){const p=a.sep[m];switch(p.type){case"newline":d.push(m);break;case"space":break;case"comment":p.indent>l.indent&&(d.length=0);break;default:d.length=0}}d.length>=2&&(h=a.sep.splice(d[1]))}switch(this.type){case"anchor":case"tag":o||a.value?(h.push(this.sourceToken),l.items.push({start:h}),this.onKeyLine=!0):a.sep?a.sep.push(this.sourceToken):a.start.push(this.sourceToken);return;case"explicit-key-ind":!a.sep&&!a.explicitKey?(a.start.push(this.sourceToken),a.explicitKey=!0):o||a.value?(h.push(this.sourceToken),l.items.push({start:h,explicitKey:!0})):this.stack.push({type:"block-map",offset:this.offset,indent:this.indent,items:[{start:[this.sourceToken],explicitKey:!0}]}),this.onKeyLine=!0;return;case"map-value-ind":if(a.explicitKey)if(a.sep)if(a.value)l.items.push({start:[],key:null,sep:[this.sourceToken]});else if(hl(a.sep,"map-value-ind"))this.stack.push({type:"block-map",offset:this.offset,indent:this.indent,items:[{start:h,key:null,sep:[this.sourceToken]}]});else if(Ap(a.key)&&!hl(a.sep,"newline")){const d=si(a.start),m=a.key,p=a.sep;p.push(this.sourceToken),delete a.key,delete a.sep,this.stack.push({type:"block-map",offset:this.offset,indent:this.indent,items:[{start:d,key:m,sep:p}]})}else h.length>0?a.sep=a.sep.concat(h,this.sourceToken):a.sep.push(this.sourceToken);else if(hl(a.start,"newline"))Object.assign(a,{key:null,sep:[this.sourceToken]});else{const d=si(a.start);this.stack.push({type:"block-map",offset:this.offset,indent:this.indent,items:[{start:d,key:null,sep:[this.sourceToken]}]})}else a.sep?a.value||o?l.items.push({start:h,key:null,sep:[this.sourceToken]}):hl(a.sep,"map-value-ind")?this.stack.push({type:"block-map",offset:this.offset,indent:this.indent,items:[{start:[],key:null,sep:[this.sourceToken]}]}):a.sep.push(this.sourceToken):Object.assign(a,{key:null,sep:[this.sourceToken]});this.onKeyLine=!0;return;case"alias":case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":{const d=this.flowScalar(this.type);o||a.value?(l.items.push({start:h,key:d,sep:[]}),this.onKeyLine=!0):a.sep?this.stack.push(d):(Object.assign(a,{key:d,sep:[]}),this.onKeyLine=!0);return}default:{const d=this.startBlockValue(l);if(d){r&&d.type!=="block-seq"&&l.items.push({start:h}),this.stack.push(d);return}}}}yield*this.pop(),yield*this.step()}*blockSequence(l){var s;const a=l.items[l.items.length-1];switch(this.type){case"newline":if(a.value){const r="end"in a.value?a.value.end:void 0,o=Array.isArray(r)?r[r.length-1]:void 0;(o==null?void 0:o.type)==="comment"?r==null||r.push(this.sourceToken):l.items.push({start:[this.sourceToken]})}else a.start.push(this.sourceToken);return;case"space":case"comment":if(a.value)l.items.push({start:[this.sourceToken]});else{if(this.atIndentedComment(a.start,l.indent)){const r=l.items[l.items.length-2],o=(s=r==null?void 0:r.value)==null?void 0:s.end;if(Array.isArray(o)){Array.prototype.push.apply(o,a.start),o.push(this.sourceToken),l.items.pop();return}}a.start.push(this.sourceToken)}return;case"anchor":case"tag":if(a.value||this.indent<=l.indent)break;a.start.push(this.sourceToken);return;case"seq-item-ind":if(this.indent!==l.indent)break;a.value||hl(a.start,"seq-item-ind")?l.items.push({start:[this.sourceToken]}):a.start.push(this.sourceToken);return}if(this.indent>l.indent){const r=this.startBlockValue(l);if(r){this.stack.push(r);return}}yield*this.pop(),yield*this.step()}*flowCollection(l){const a=l.items[l.items.length-1];if(this.type==="flow-error-end"){let s;do yield*this.pop(),s=this.peek(1);while(s&&s.type==="flow-collection")}else if(l.end.length===0){switch(this.type){case"comma":case"explicit-key-ind":!a||a.sep?l.items.push({start:[this.sourceToken]}):a.start.push(this.sourceToken);return;case"map-value-ind":!a||a.value?l.items.push({start:[],key:null,sep:[this.sourceToken]}):a.sep?a.sep.push(this.sourceToken):Object.assign(a,{key:null,sep:[this.sourceToken]});return;case"space":case"comment":case"newline":case"anchor":case"tag":!a||a.value?l.items.push({start:[this.sourceToken]}):a.sep?a.sep.push(this.sourceToken):a.start.push(this.sourceToken);return;case"alias":case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":{const r=this.flowScalar(this.type);!a||a.value?l.items.push({start:[],key:r,sep:[]}):a.sep?this.stack.push(r):Object.assign(a,{key:r,sep:[]});return}case"flow-map-end":case"flow-seq-end":l.end.push(this.sourceToken);return}const s=this.startBlockValue(l);s?this.stack.push(s):(yield*this.pop(),yield*this.step())}else{const s=this.peek(2);if(s.type==="block-map"&&(this.type==="map-value-ind"&&s.indent===l.indent||this.type==="newline"&&!s.items[s.items.length-1].sep))yield*this.pop(),yield*this.step();else if(this.type==="map-value-ind"&&s.type!=="flow-collection"){const r=eu(s),o=si(r);um(l);const h=l.end.splice(1,l.end.length);h.push(this.sourceToken);const d={type:"block-map",offset:l.offset,indent:l.indent,items:[{start:o,key:l,sep:h}]};this.onKeyLine=!0,this.stack[this.stack.length-1]=d}else yield*this.lineEnd(l)}}flowScalar(l){if(this.onNewLine){let a=this.source.indexOf(`
`)+1;for(;a!==0;)this.onNewLine(this.offset+a),a=this.source.indexOf(`
`,a)+1}return{type:l,offset:this.offset,indent:this.indent,source:this.source}}startBlockValue(l){switch(this.type){case"alias":case"scalar":case"single-quoted-scalar":case"double-quoted-scalar":return this.flowScalar(this.type);case"block-scalar-header":return{type:"block-scalar",offset:this.offset,indent:this.indent,props:[this.sourceToken],source:""};case"flow-map-start":case"flow-seq-start":return{type:"flow-collection",offset:this.offset,indent:this.indent,start:this.sourceToken,items:[],end:[]};case"seq-item-ind":return{type:"block-seq",offset:this.offset,indent:this.indent,items:[{start:[this.sourceToken]}]};case"explicit-key-ind":{this.onKeyLine=!0;const a=eu(l),s=si(a);return s.push(this.sourceToken),{type:"block-map",offset:this.offset,indent:this.indent,items:[{start:s,explicitKey:!0}]}}case"map-value-ind":{this.onKeyLine=!0;const a=eu(l),s=si(a);return{type:"block-map",offset:this.offset,indent:this.indent,items:[{start:s,key:null,sep:[this.sourceToken]}]}}}return null}atIndentedComment(l,a){return this.type!=="comment"||this.indent<=a?!1:l.every(s=>s.type==="newline"||s.type==="space")}*documentEnd(l){this.type!=="doc-mode"&&(l.end?l.end.push(this.sourceToken):l.end=[this.sourceToken],this.type==="newline"&&(yield*this.pop()))}*lineEnd(l){switch(this.type){case"comma":case"doc-start":case"doc-end":case"flow-seq-end":case"flow-map-end":case"map-value-ind":yield*this.pop(),yield*this.step();break;case"newline":this.onKeyLine=!1;case"space":case"comment":default:l.end?l.end.push(this.sourceToken):l.end=[this.sourceToken],this.type==="newline"&&(yield*this.pop())}}}function Op(u){const l=u.prettyErrors!==!1;return{lineCounter:u.lineCounter||l&&new Ep||null,prettyErrors:l}}function db(u,l={}){const{lineCounter:a,prettyErrors:s}=Op(l),r=new Bf(a==null?void 0:a.addNewLine),o=new Uf(l),h=Array.from(o.compose(r.parse(u)));if(s&&a)for(const d of h)d.errors.forEach(gu(u,a)),d.warnings.forEach(gu(u,a));return h.length>0?h:Object.assign([],{empty:!0},o.streamInfo())}function Np(u,l={}){const{lineCounter:a,prettyErrors:s}=Op(l),r=new Bf(a==null?void 0:a.addNewLine),o=new Uf(l);let h=null;for(const d of o.compose(r.parse(u),!0,u.length))if(!h)h=d;else if(h.options.logLevel!=="silent"){h.errors.push(new ml(d.range.slice(0,2),"MULTIPLE_DOCS","Source contains multiple documents; please use YAML.parseAllDocuments()"));break}return s&&a&&(h.errors.forEach(gu(u,a)),h.warnings.forEach(gu(u,a))),h}function gb(u,l,a){let s;typeof l=="function"?s=l:a===void 0&&l&&typeof l=="object"&&(a=l);const r=Np(u,a);if(!r)return null;if(r.warnings.forEach(o=>Qm(r.options.logLevel,o)),r.errors.length>0){if(r.options.logLevel!=="silent")throw r.errors[0];r.errors=[]}return r.toJS(Object.assign({reviver:s},a))}function mb(u,l,a){let s=null;if(typeof l=="function"||Array.isArray(l)?s=l:a===void 0&&l&&(a=l),typeof a=="string"&&(a=a.length),typeof a=="number"){const r=Math.round(a);a=r<1?void 0:r>8?{indent:8}:{indent:r}}if(u===void 0){const{keepUndefined:r}=a??l??{};if(!r)return}return bl(u)&&!s?u.toString(a):new Ai(u,s,a).toString(a)}const pb=Object.freeze(Object.defineProperty({__proto__:null,Alias:Su,CST:fb,Composer:Uf,Document:Ai,Lexer:wp,LineCounter:Ep,Pair:st,Parser:Bf,Scalar:re,Schema:xu,YAMLError:Lf,YAMLMap:Mt,YAMLParseError:ml,YAMLSeq:Qn,YAMLWarning:op,isAlias:vl,isCollection:Le,isDocument:bl,isMap:Si,isNode:ze,isPair:Re,isScalar:_e,isSeq:Ti,parse:gb,parseAllDocuments:db,parseDocument:Np,stringify:mb,visit:Vn,visitAsync:bu},Symbol.toStringTag,{value:"Module"}));function yb(u,l,a={}){var N;const s=new u.LineCounter,r={keepSourceTokens:!0,lineCounter:s,...a},o=u.parseDocument(l,r),h=[],d=E=>[s.linePos(E[0]),s.linePos(E[1])],m=E=>{h.push({message:E.message,range:[s.linePos(E.pos[0]),s.linePos(E.pos[1])]})},p=(E,D)=>{for(const T of D.items){if(T instanceof u.Scalar&&typeof T.value=="string"){const L=pu.parse(T,r,h);L&&(E.children=E.children||[],E.children.push(L));continue}if(T instanceof u.YAMLMap){b(E,T);continue}h.push({message:"Sequence items should be strings or maps",range:d(T.range||D.range)})}},b=(E,D)=>{for(const T of D.items){if(E.children=E.children||[],!(T.key instanceof u.Scalar&&typeof T.key.value=="string")){h.push({message:"Only string keys are supported",range:d(T.key.range||D.range)});continue}const k=T.key,L=T.value;if(k.value==="text"){if(!(L instanceof u.Scalar&&typeof L.value=="string")){h.push({message:"Text value should be a string",range:d(T.value.range||D.range)});continue}E.children.push({kind:"text",text:ff(L.value)});continue}if(k.value==="/children"){if(!(L instanceof u.Scalar&&typeof L.value=="string")||L.value!=="contain"&&L.value!=="equal"&&L.value!=="deep-equal"){h.push({message:'Strict value should be "contain", "equal" or "deep-equal"',range:d(T.value.range||D.range)});continue}E.containerMode=L.value;continue}if(k.value.startsWith("/")){if(!(L instanceof u.Scalar&&typeof L.value=="string")){h.push({message:"Property value should be a string",range:d(T.value.range||D.range)});continue}E.props=E.props??{},E.props[k.value.slice(1)]=ff(L.value);continue}const H=pu.parse(k,r,h);if(!H)continue;if(L instanceof u.Scalar){const K=typeof L.value;if(K!=="string"&&K!=="number"&&K!=="boolean"){h.push({message:"Node value should be a string or a sequence",range:d(T.value.range||D.range)});continue}E.children.push({...H,children:[{kind:"text",text:ff(String(L.value))}]});continue}if(L instanceof u.YAMLSeq){E.children.push(H),p(H,L);continue}h.push({message:"Map values should be strings or sequences",range:d(T.value.range||D.range)})}},S={kind:"role",role:"fragment"};return o.errors.forEach(m),h.length?{errors:h,fragment:S}:(o.contents instanceof u.YAMLSeq||h.push({message:'Aria snapshot must be a YAML sequence, elements starting with " -"',range:o.contents?d(o.contents.range):[{line:0,col:0},{line:0,col:0}]}),h.length?{errors:h,fragment:S}:(p(S,o.contents),h.length?{errors:h,fragment:vb}:((N=S.children)==null?void 0:N.length)===1&&(!S.containerMode||S.containerMode==="contain")?{fragment:S.children[0],errors:[]}:{fragment:S,errors:[]}))}const vb={kind:"role",role:"fragment"};function _p(u){return u.replace(/[\u200b\u00ad]/g,"").replace(/[\r\n\s\t]+/g," ").trim()}function ff(u){return{raw:u,normalized:_p(u)}}class pu{static parse(l,a,s){try{return new pu(l.value)._parse()}catch(r){if(r instanceof cm){const o=a.prettyErrors===!1?r.message:r.message+`:

`+l.value+`
`+" ".repeat(r.pos)+`^
`;return s.push({message:o,range:[a.lineCounter.linePos(l.range[0]),a.lineCounter.linePos(l.range[0]+r.pos)]}),null}throw r}}constructor(l){this._input=l,this._pos=0,this._length=l.length}_peek(){return this._input[this._pos]||""}_next(){return this._pos<this._length?this._input[this._pos++]:null}_eof(){return this._pos>=this._length}_isWhitespace(){return!this._eof()&&/\s/.test(this._peek())}_skipWhitespace(){for(;this._isWhitespace();)this._pos++}_readIdentifier(l){this._eof()&&this._throwError(`Unexpected end of input when expecting ${l}`);const a=this._pos;for(;!this._eof()&&/[a-zA-Z]/.test(this._peek());)this._pos++;return this._input.slice(a,this._pos)}_readString(){let l="",a=!1;for(;!this._eof();){const s=this._next();if(a)l+=s,a=!1;else if(s==="\\")a=!0;else{if(s==='"')return l;l+=s}}this._throwError("Unterminated string")}_throwError(l,a=0){throw new cm(l,a||this._pos)}_readRegex(){let l="",a=!1,s=!1;for(;!this._eof();){const r=this._next();if(a)l+=r,a=!1;else if(r==="\\")a=!0,l+=r;else{if(r==="/"&&!s)return{pattern:l};r==="["?(s=!0,l+=r):r==="]"&&s?(l+=r,s=!1):l+=r}}this._throwError("Unterminated regex")}_readStringOrRegex(){const l=this._peek();return l==='"'?(this._next(),_p(this._readString())):l==="/"?(this._next(),this._readRegex()):null}_readAttributes(l){let a=this._pos;for(;this._skipWhitespace(),this._peek()==="[";){this._next(),this._skipWhitespace(),a=this._pos;const s=this._readIdentifier("attribute");this._skipWhitespace();let r="";if(this._peek()==="=")for(this._next(),this._skipWhitespace(),a=this._pos;this._peek()!=="]"&&!this._isWhitespace()&&!this._eof();)r+=this._next();this._skipWhitespace(),this._peek()!=="]"&&this._throwError("Expected ]"),this._next(),this._applyAttribute(l,s,r||"true",a)}}_parse(){this._skipWhitespace();const l=this._readIdentifier("role");this._skipWhitespace();const a=this._readStringOrRegex()||"",s={kind:"role",role:l,name:a};return this._readAttributes(s),this._skipWhitespace(),this._eof()||this._throwError("Unexpected input"),s}_applyAttribute(l,a,s,r){if(a==="checked"){this._assert(s==="true"||s==="false"||s==="mixed",'Value of "checked" attribute must be a boolean or "mixed"',r),l.checked=s==="true"?!0:s==="false"?!1:"mixed";return}if(a==="disabled"){this._assert(s==="true"||s==="false",'Value of "disabled" attribute must be a boolean',r),l.disabled=s==="true";return}if(a==="expanded"){this._assert(s==="true"||s==="false",'Value of "expanded" attribute must be a boolean',r),l.expanded=s==="true";return}if(a==="active"){this._assert(s==="true"||s==="false",'Value of "active" attribute must be a boolean',r),l.active=s==="true";return}if(a==="level"){this._assert(!isNaN(Number(s)),'Value of "level" attribute must be a number',r),l.level=Number(s);return}if(a==="pressed"){this._assert(s==="true"||s==="false"||s==="mixed",'Value of "pressed" attribute must be a boolean or "mixed"',r),l.pressed=s==="true"?!0:s==="false"?!1:"mixed";return}if(a==="selected"){this._assert(s==="true"||s==="false",'Value of "selected" attribute must be a boolean',r),l.selected=s==="true";return}this._assert(!1,`Unsupported attribute [${a}]`,r)}_assert(l,a,s){l||this._throwError(a||"Assertion error",s)}}class cm extends Error{constructor(l,a){super(l),this.pos=a}}const bb=({className:u,style:l,open:a,isModal:s,minWidth:r,verticalOffset:o,requestClose:h,anchor:d,dataTestId:m,children:p})=>{const b=me.useRef(null),[S,N]=me.useState(0),[E]=of(b),[D,T]=of(d),w=Sb(E,D,o);return me.useEffect(()=>{const k=H=>{!b.current||!(H.target instanceof Node)||b.current.contains(H.target)||h==null||h()},L=H=>{H.key==="Escape"&&(h==null||h())};return a?(document.addEventListener("mousedown",k),document.addEventListener("keydown",L),()=>{document.removeEventListener("mousedown",k),document.removeEventListener("keydown",L)}):()=>{}},[a,h]),me.useLayoutEffect(()=>T(),[a,T]),me.useEffect(()=>{const k=()=>N(L=>L+1);return window.addEventListener("resize",k),()=>{window.removeEventListener("resize",k)}},[]),me.useLayoutEffect(()=>{b.current&&(a?s?b.current.showModal():b.current.show():b.current.close())},[a,s]),X.jsx("dialog",{ref:b,style:{position:"fixed",margin:0,zIndex:110,top:w.top,left:w.left,minWidth:r||0,...l},className:u,"data-testid":m,children:p})};function Sb(u,l,a=4,s=4){let r=Math.max(s,l.left);r+u.width>window.innerWidth-s&&(r=window.innerWidth-u.width-s);let o=Math.max(0,l.bottom)+a;return o+u.height>window.innerHeight-a&&(Math.max(0,l.top)>u.height+a?o=Math.max(0,l.top)-u.height-a:o=window.innerHeight-a-u.height),{left:r,top:o}}const Tb=({sources:u,paused:l,log:a,mode:s})=>{const[r,o]=me.useState(),[h,d]=fu("recorderPropertiesTab","log"),[m,p]=me.useState(),[b,S]=me.useState(),[N,E]=me.useState(!1),[D,T]=s1(),[w,k]=fu("autoExpect",!1),L=me.useRef(null);window.playwrightSelectSource=F=>o(F),me.useEffect(()=>{window.dispatch({event:"setAutoExpect",params:{autoExpect:w}})},[w]);const H=me.useMemo(()=>u.find(se=>se.id===r)??D1(),[u,r]),[Q,G]=me.useState("");window.playwrightElementPicked=(F,se)=>{const U=H.language;G(km(U,F.selector)),p(F.ariaSnapshot),S([]),se&&h!=="locator"&&h!=="aria"&&d("locator"),s==="inspecting"&&h==="aria"||window.dispatch({event:"setMode",params:{mode:s==="inspecting"?"standby":"recording"}}).catch(()=>{})};const K=me.useRef(null);me.useLayoutEffect(()=>{var F;(F=K.current)==null||F.scrollIntoView({block:"center",inline:"nearest"})},[K]),me.useLayoutEffect(()=>{const F=se=>{switch(se.key){case"F8":se.preventDefault(),l?window.dispatch({event:"resume"}):window.dispatch({event:"pause"});break;case"F10":se.preventDefault(),l&&window.dispatch({event:"step"});break}};return document.addEventListener("keydown",F),()=>document.removeEventListener("keydown",F)},[l]);const ee=me.useCallback(F=>{(s==="none"||s==="inspecting")&&window.dispatch({event:"setMode",params:{mode:"standby"}}),G(F),window.dispatch({event:"highlightRequested",params:{selector:F}})},[s]),V=me.useCallback(F=>{(s==="none"||s==="inspecting")&&window.dispatch({event:"setMode",params:{mode:"standby"}});const{fragment:se,errors:U}=yb(pb,F,{prettyErrors:!1}),te=U.map(xe=>({message:xe.message,line:xe.range[1].line,column:xe.range[1].col,type:"subtle-error"}));S(te),p(F),U.length||window.dispatch({event:"highlightRequested",params:{ariaTemplate:se}})},[s]),z=s==="recording"||s==="recording-inspecting"||s==="assertingText"||s==="assertingVisibility";return X.jsxs("div",{className:"recorder",children:[X.jsxs(fm,{children:[X.jsx(_t,{icon:z?"stop-circle":"circle-large-filled",title:z?"Stop Recording":"Start Recording",toggled:z,onClick:()=>{window.dispatch({event:"setMode",params:{mode:s==="none"||s==="standby"||s==="inspecting"?"recording":"standby"}})},children:"Record"}),X.jsx(Xg,{}),X.jsx(_t,{icon:"inspect",title:"Pick locator",toggled:s==="inspecting"||s==="recording-inspecting",onClick:()=>{const F={inspecting:"standby",none:"inspecting",standby:"inspecting",recording:"recording-inspecting","recording-inspecting":"recording",assertingText:"recording-inspecting",assertingVisibility:"recording-inspecting",assertingValue:"recording-inspecting",assertingSnapshot:"recording-inspecting"}[s];window.dispatch({event:"setMode",params:{mode:F}}).catch(()=>{})}}),X.jsx(_t,{icon:"eye",title:"Assert visibility",toggled:s==="assertingVisibility",disabled:s==="none"||s==="standby"||s==="inspecting",onClick:()=>{window.dispatch({event:"setMode",params:{mode:s==="assertingVisibility"?"recording":"assertingVisibility"}})}}),X.jsx(_t,{icon:"whole-word",title:"Assert text",toggled:s==="assertingText",disabled:s==="none"||s==="standby"||s==="inspecting",onClick:()=>{window.dispatch({event:"setMode",params:{mode:s==="assertingText"?"recording":"assertingText"}})}}),X.jsx(_t,{icon:"symbol-constant",title:"Assert value",toggled:s==="assertingValue",disabled:s==="none"||s==="standby"||s==="inspecting",onClick:()=>{window.dispatch({event:"setMode",params:{mode:s==="assertingValue"?"recording":"assertingValue"}})}}),X.jsx(_t,{icon:"gist",title:"Assert snapshot",toggled:s==="assertingSnapshot",disabled:s==="none"||s==="standby"||s==="inspecting",onClick:()=>{window.dispatch({event:"setMode",params:{mode:s==="assertingSnapshot"?"recording":"assertingSnapshot"}})}}),X.jsx(Xg,{}),X.jsx(_t,{icon:"files",title:"Copy",disabled:!H||!H.text,onClick:()=>{zg(H.text)}}),X.jsx(_t,{icon:"debug-continue",title:"Resume (F8)",ariaLabel:"Resume",disabled:!l,onClick:()=>{window.dispatch({event:"resume"})}}),X.jsx(_t,{icon:"debug-pause",title:"Pause (F8)",ariaLabel:"Pause",disabled:l,onClick:()=>{window.dispatch({event:"pause"})}}),X.jsx(_t,{icon:"debug-step-over",title:"Step over (F10)",ariaLabel:"Step over",disabled:!l,onClick:()=>{window.dispatch({event:"step"})}}),X.jsx("div",{style:{flex:"auto"}}),X.jsx("div",{children:"Target:"}),X.jsx(M1,{fileId:H.id,sources:u,setFileId:F=>{o(F),window.dispatch({event:"fileChanged",params:{fileId:F}})}}),X.jsx(_t,{icon:"clear-all",title:"Clear",disabled:!H||!H.text,onClick:()=>{window.dispatch({event:"clear"})}}),X.jsx(_t,{ref:L,icon:"settings-gear",title:"Settings",onClick:()=>E(F=>!F)}),X.jsxs(bb,{style:{padding:"4px 8px"},open:N,verticalOffset:8,requestClose:()=>E(!1),anchor:L,dataTestId:"settings-dialog",children:[X.jsxs("div",{className:"setting",children:[X.jsx("input",{type:"checkbox",id:"dark-mode-setting",checked:D,onChange:()=>T(!D)}),X.jsx("label",{htmlFor:"dark-mode-setting",children:"Dark mode"})]},"dark-mode-setting"),X.jsxs("div",{className:"setting",title:"Automatically generate assertions while recording",children:[X.jsx("input",{type:"checkbox",id:"auto-expect-setting",checked:w,onChange:()=>{window.dispatch({event:"setAutoExpect",params:{autoExpect:!w}}),k(!w)}}),X.jsx("label",{htmlFor:"auto-expect-setting",children:"Generate assertions"})]},"auto-expect-setting")]})]}),X.jsx(O1,{sidebarSize:200,main:X.jsx(ef,{text:H.text,highlighter:H.language,highlight:H.highlight,revealLine:H.revealLine,readOnly:!0,lineNumbers:!0}),sidebar:X.jsx(N1,{rightToolbar:h==="locator"||h==="aria"?[X.jsx(_t,{icon:"files",title:"Copy",onClick:()=>zg((h==="locator"?Q:m)||"")},1)]:[],tabs:[{id:"locator",title:"Locator",render:()=>X.jsx(ef,{text:Q,placeholder:"Type locator to inspect",highlighter:H.language,focusOnChange:!0,onChange:ee,wrapLines:!0})},{id:"log",title:"Log",render:()=>X.jsx(av,{language:H.language,log:Array.from(a.values())})},{id:"aria",title:"Aria",render:()=>X.jsx(ef,{text:m||"",placeholder:"Type aria template to match",highlighter:"yaml",onChange:V,highlight:b,wrapLines:!0})}],selectedTab:h,setSelectedTab:d})})]})},wb=({})=>{const[u,l]=me.useState([]),[a,s]=me.useState(!1),[r,o]=me.useState(new Map),[h,d]=me.useState("none");return me.useLayoutEffect(()=>{window.playwrightSetMode=d,window.playwrightSetSources=m=>{l(m),window.playwrightSourcesEchoForTest=m},window.playwrightSetPageURL=m=>{document.title=m?`Playwright Inspector - ${m}`:"Playwright Inspector"},window.playwrightSetPaused=s,window.playwrightUpdateLogs=m=>{o(p=>{const b=new Map(p);for(const S of m)S.reveal=!p.has(S.id),b.set(S.id,S);return b})}},[]),X.jsx(Tb,{sources:u,paused:a,log:r,mode:h})};(async()=>(l1(),d1.createRoot(document.querySelector("#root")).render(X.jsx(wb,{}))))();export{W0 as g};
